/*********************************************************************************

Copyright(c) 2013 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*
 * This is a supplemental code example for Engineer-to-Engineer Note EE-359
 * ADSP-CM40x Boot Time Optimization & Device Initialization.  Refer to the 
 * Engineer-to-Engineer note in the doc folder for more information.
 */

#include <services/gpio/adi_gpio.h>
#include "system/adi_initialize.h"

/*
 * LED 2 - GPIO PORT and PIN settings
 *
 * ADSP-CM403F EZ-Kit:
 *                     ADI_GPIO_PORT_B
 *                     ADI_GPIO_PIN_4
 * ADSP-CM408F EZ-Kit:
 *                     ADI_GPIO_PORT_E
 *                     ADI_GPIO_PIN_6
 */
#define PORT  ADI_GPIO_PORT_E
#define PIN   ADI_GPIO_PIN_6

#define DELAY  (1000000)

/* used for exit timeout */
#define TIMEOUT (2000)

/* increase the data size of the application by adding some dummy data (see app note) */
//__root uint32_t test_data[] = {
uint32_t test_data[] __attribute__ ((used)) = {  /* KEIL:   */
#include "test_data.h"
};

extern void ConfigSoftSwitches(void);

int main()
{
  bool bError = false;
  volatile uint32_t count = 0;
  uint32_t gpioMaxCallbacks;

  /* GPIO service memory */
  uint8_t gpioMemory[ADI_GPIO_CALLBACK_MEM_SIZE];

  /* Initialize drivers and/or services */
  adi_initComponents();

  /* Software Switch Configuration for the ADSP-CM40z EZ-Kit */
  ConfigSoftSwitches();
  
  /* init the GPIO service */
  if (adi_gpio_Init((void*)gpioMemory, ADI_GPIO_CALLBACK_MEM_SIZE, 
                    &gpioMaxCallbacks) != ADI_GPIO_SUCCESS)
  {
      bError = true;
  }

  /* set GPIO output LED 2 */
  if (adi_gpio_SetDirection(PORT, PIN, ADI_GPIO_DIRECTION_OUTPUT) != ADI_GPIO_SUCCESS)
  {
      bError = true;
  }

  /* toggle LED - exit after a few minutes */
  while ((bError == false) && (count < TIMEOUT))
  {
      /* toggle LED 1 */
      adi_gpio_Toggle(PORT, PIN);
      count++;
      
      /* delay loop */
      for(volatile uint32_t i=0; i< DELAY; i++);
  }
    
  return 0;
}
