;/*****************************************************************************
; * @file     startup_CM40z.s
; * @brief    CMSIS Cortex-M4 Core Device Startup File for
; *           Analog Devices ADSP-CM400 Device Series
; * @version  V1.00
; * @date     14. March 2014
; *
; * @note
; * Copyright (C) 2014 ARM Limited. All rights reserved.
; *
; * @par
; * ARM Limited (ARM) is supplying this software for use with Cortex-M
; * processor based microcontrollers.  This file can be freely distributed
; * within development tools that are supporting such ARM based processors.
; *
; * @par
; * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
; * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
; * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
; * ARM SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
; * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
; *
; ******************************************************************************/
;/*
;//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
;*/


; <h> Stack Configuration
;   <o> Stack Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

Stack_Size      EQU     0x00000400

                AREA    STACK, NOINIT, READWRITE, ALIGN=3
Stack_Mem       SPACE   Stack_Size
__initial_sp


; <h> Heap Configuration
;   <o>  Heap Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

Heap_Size       EQU     0x00000200

                AREA    HEAP, NOINIT, READWRITE, ALIGN=3
__heap_base
Heap_Mem        SPACE   Heap_Size
__heap_limit


                PRESERVE8
                THUMB


; Vector Table Mapped to Address 0 at Reset

                AREA    RESET, DATA, READONLY
                EXPORT  __vector_table
                EXPORT  __vector_table_End
                EXPORT  __vector_table_Size

__vector_table  DCD     __initial_sp              ;    Top of Stack
                DCD     Reset_Handler             ;    Reset Handler
                DCD     NMI_Handler               ;    NMI Handler
                DCD     HardFault_Handler         ;    Hard Fault Handler
                DCD     MemManage_Handler         ;    MPU Fault Handler
                DCD     BusFault_Handler          ;    Bus Fault Handler
                DCD     UsageFault_Handler        ;    Usage Fault Handler
                DCD     0                         ;    Reserved
                DCD     0                         ;    Reserved
                DCD     0                         ;    Reserved
                DCD     0                         ;    Reserved
                DCD     SVC_Handler               ;    SVCall Handler
                DCD     DebugMon_Handler          ;    Debug Monitor Handler
                DCD     0                         ;    Reserved
                DCD     PendSV_Handler            ;    PendSV Handler
                DCD     SysTick_Handler           ;    SysTick Handler

                ; External Interrupts
                DCD     OSCW_EVT_Int_Handler               ;   0  Oscillator Watchdog
                DCD     CGU0_EVT_Int_Handler               ;   1  Event
                DCD     M4P_L1CC_PERR_Int_Handler          ;   2  L1 Cache Code Parity Error
                DCD     M4P_CORE_SRAM_PERR_Int_Handler     ;   3  SRAM Core Parity Error
                DCD     M4P_DMA_SRAM_PERR_Int_Handler      ;   4  SRAM DMA Parity Error
                DCD     M4P_BUS_FAULT_Int_Handler          ;   5  Bus Fault
                DCD     M4P_LOCKUP_Int_Handler             ;   6  Lockup Error (Fault only; not an interrupt)
                DCD     M4P_SRAM_PERR_FLT_Int_Handler      ;   7  SRAM Parity Error (Fault only; not an interrupt)
                DCD     WDOG0_EXP_Int_Handler              ;   8  Expiration
                DCD     SEC0_ERR_Int_Handler               ;   9  Fault Interrupt
                DCD     ECT_EVT0_Int_Handler               ;  10  Embedded Cross Trigger Event 0
                DCD     ECT_EVT1_Int_Handler               ;  11  Embedded Cross Trigger Event 1
                DCD     PWM0_TRIP_Int_Handler              ;  12  Trip
                DCD     PWM1_TRIP_Int_Handler              ;  13  Trip
                DCD     PWM2_TRIP_Int_Handler              ;  14  Trip
                DCD     PWM0_SYNC_Int_Handler              ;  15  Sync
                DCD     PWM1_SYNC_Int_Handler              ;  16  Sync
                DCD     PWM2_SYNC_Int_Handler              ;  17  Sync
                DCD     PINT0_BLOCK_Int_Handler            ;  18  Pin Interrupt Block
                DCD     PINT1_BLOCK_Int_Handler            ;  19  Pin Interrupt Block
                DCD     PINT2_BLOCK_Int_Handler            ;  20  Pin Interrupt Block
                DCD     PINT3_BLOCK_Int_Handler            ;  21  Pin Interrupt Block
                DCD     PINT4_BLOCK_Int_Handler            ;  22  Pin Interrupt Block
                DCD     SPORT0_A_DMA_ERR_Int_Handler       ;  23  DMA Channel 0 Error
                DCD     SPORT0_B_DMA_ERR_Int_Handler       ;  24  DMA Channel 1 Error
                DCD     SPI0_TXDMA_ERR_Int_Handler         ;  25  DMA Channel 2 Error
                DCD     SPI0_RXDMA_ERR_Int_Handler         ;  26  DMA Channel 3 Error
                DCD     UART0_TXDMA_ERR_Int_Handler        ;  27  DMA Channel 4 Error
                DCD     UART0_RXDMA_ERR_Int_Handler        ;  28  DMA Channel 5 Error
                DCD     SPORT1_A_DMA_ERR_Int_Handler       ;  29  DMA Channel 6 Error
                DCD     SPORT1_B_DMA_ERR_Int_Handler       ;  30  DMA Channel 7 Error
                DCD     SPI1_TXDMA_ERR_Int_Handler         ;  31  DMA Channel 8 Error
                DCD     SPI1_RXDMA_ERR_Int_Handler         ;  32  DMA Channel 9 Error
                DCD     UART1_TXDMA_ERR_Int_Handler        ;  33  DMA Channel 10 Error
                DCD     UART1_RXDMA_ERR_Int_Handler        ;  34  DMA Channel 11 Error
                DCD     UART2_TXDMA_ERR_Int_Handler        ;  35  DMA Channel 12 Error
                DCD     UART2_RXDMA_ERR_Int_Handler        ;  36  DMA Channel 13 Error
                DCD     HAE0_RXDMA_CH0_ERR_Int_Handler     ;  37  DMA Channel 14 Error
                DCD     HAE0_RXDMA_CH1_ERR_Int_Handler     ;  38  DMA Channel 15 Error
                DCD     HAE0_TXDMA_ERR_Int_Handler         ;  39  DMA Channel 16 Error
                DCD     RESERVED_VECTOR                    ;  40  Reserved
                DCD     SPI0_ERR_Int_Handler               ;  41  Error
                DCD     SPI1_ERR_Int_Handler               ;  42  Error
                DCD     SPI2_ERR_Int_Handler               ;  43  Error
                DCD     ADCC0_ERR_Int_Handler              ;  44  ADC Error
                DCD     DACC0_ERR_Int_Handler              ;  45  DAC Error
                DCD     MDMA0_SRC_CRC0_IN_ERR_Int_Handler  ;  46  DMA Channel 17 Error
                DCD     MDMA0_DST_CRC0_OUT_ERR_Int_Handler ;  47  DMA Channel 18 Error
                DCD     MDMA1_SRC_ERR_Int_Handler          ;  48  DMA Channel 19 Error
                DCD     MDMA1_DST_ERR_Int_Handler          ;  49  DMA Channel 20 Error
                DCD     TIMER0_STAT_Int_Handler            ;  50  Status
                DCD     TIMER0_TMR0_Int_Handler            ;  51  Timer 0
                DCD     TIMER0_TMR1_Int_Handler            ;  52  Timer 1
                DCD     TIMER0_TMR2_Int_Handler            ;  53  Timer 2
                DCD     TIMER0_TMR3_Int_Handler            ;  54  Timer 3
                DCD     TIMER0_TMR4_Int_Handler            ;  55  Timer 4
                DCD     TIMER0_TMR5_Int_Handler            ;  56  Timer 5
                DCD     TIMER0_TMR6_Int_Handler            ;  57  Timer 6
                DCD     TIMER0_TMR7_Int_Handler            ;  58  Timer 7
                DCD     CNT0_STAT_Int_Handler              ;  59  Status
                DCD     CNT1_STAT_Int_Handler              ;  60  Status
                DCD     CNT2_STAT_Int_Handler              ;  61  Status
                DCD     CNT3_STAT_Int_Handler              ;  62  Status
                DCD     SPORT0_A_STAT_Int_Handler          ;  63  Channel A Status
                DCD     SPORT0_A_DMA_Int_Handler           ;  64  Channel A DMA
                DCD     SPORT0_B_STAT_Int_Handler          ;  65  Channel B Status
                DCD     SPORT0_B_DMA_Int_Handler           ;  66  Channel B DMA
                DCD     UART0_STAT_Int_Handler             ;  67  Status
                DCD     UART0_TXDMA_Int_Handler            ;  68  Transmit DMA
                DCD     UART0_RXDMA_Int_Handler            ;  69  Receive DMA
                DCD     SINC0_STAT_Int_Handler             ;  70  Status
                DCD     SPI0_STAT_Int_Handler              ;  71  Status
                DCD     SPI0_TXDMA_Int_Handler             ;  72  TX DMA Channel
                DCD     SPI0_RXDMA_Int_Handler             ;  73  RX DMA Channel
                DCD     SPORT1_A_STAT_Int_Handler          ;  74  Channel A Status
                DCD     SPORT1_A_DMA_Int_Handler           ;  75  Channel A DMA
                DCD     SPORT1_B_STAT_Int_Handler          ;  76  Channel B Status
                DCD     SPORT1_B_DMA_Int_Handler           ;  77  Channel B DMA
                DCD     UART1_STAT_Int_Handler             ;  78  Status
                DCD     UART1_TXDMA_Int_Handler            ;  79  Transmit DMA
                DCD     UART1_RXDMA_Int_Handler            ;  80  Receive DMA
                DCD     SPI1_STAT_Int_Handler              ;  81  Status
                DCD     SPI1_TXDMA_Int_Handler             ;  82  TX DMA Channel
                DCD     SPI1_RXDMA_Int_Handler             ;  83  RX DMA Channel
                DCD     SPI2_TX_Int_Handler                ;  84  TX  Channel (non-DMA)
                DCD     SPI2_RX_Int_Handler                ;  85  RX Channel (non-DMA)
                DCD     EMAC0_STAT_Int_Handler             ;  86  Status
                DCD     UART2_STAT_Int_Handler             ;  87  Status
                DCD     UART2_TXDMA_Int_Handler            ;  88  Transmit DMA
                DCD     UART2_RXDMA_Int_Handler            ;  89  Receive DMA
                DCD     HAE0_STAT_Int_Handler              ;  90  Status
                DCD     HAE0_RXDMA_CH0_Int_Handler         ;  91  RX DMA Channel 0
                DCD     HAE0_RXDMA_CH1_Int_Handler         ;  92  RX DMA Channel 1
                DCD     HAE0_TXDMA_Int_Handler             ;  93  TX DMA Channel 0
                DCD     ADCC0_TMR0_EVT_Int_Handler         ;  94  Timer 0 Event Complete
                DCD     ADCC0_TMR1_EVT_Int_Handler         ;  95  Timer 1 Event Complete
                DCD     DACC0_DAC0_Int_Handler             ;  96  DAC Interrupt 0
                DCD     DACC0_DAC1_Int_Handler             ;  97  DAC Interrupt 1
                DCD     SPI2_STAT_Int_Handler              ;  98  Status
                DCD     TWI0_DATA_Int_Handler              ;  99  Data Interrupt
                DCD     CRC0_DCNTEXP_Int_Handler           ; 100  Datacount expiration
                DCD     CRC0_ERR_Int_Handler               ; 101  Error
                DCD     MDMA0_SRC_CRC0_IN_Int_Handler      ; 102  Memory DMA Stream 0 Source Channel
                DCD     MDMA0_DST_CRC0_OUT_Int_Handler     ; 103  Memory DMA Stream 0 Destination Channel
                DCD     MDMA1_SRC_Int_Handler              ; 104  Memory DMA Stream 1 Source Channel
                DCD     MDMA1_DST_Int_Handler              ; 105  Memory DMA Stream 1 Destination Channel
                DCD     USB0_STAT_Int_Handler              ; 106  Status/FIFO Data Ready
                DCD     USB0_DATA_Int_Handler              ; 107  DMA Status/Transfer Complete
                DCD     TRU0_INT0_Int_Handler              ; 108  Interrupt 0
                DCD     TRU0_INT1_Int_Handler              ; 109  Interrupt 1
                DCD     TRU0_INT2_Int_Handler              ; 110  Interrupt 2
                DCD     TRU0_INT3_Int_Handler              ; 111  Interrupt 3
                DCD     CGU0_ERR_Int_Handler               ; 112  Error
                DCD     DPM0_EVT_Int_Handler               ; 113  Event
                DCD     SOFT0_Int_Handler                  ; 114  Software-Driven Interrupt 0
                DCD     SOFT1_Int_Handler                  ; 115  Software-Driven Interrupt 1
                DCD     SOFT2_Int_Handler                  ; 116  Software-Driven Interrupt 2
                DCD     SOFT3_Int_Handler                  ; 117  Software-Driven Interrupt 3
                DCD     SWU0_EVT_Int_Handler               ; 118  Event
                DCD     SWU1_EVT_Int_Handler               ; 119  Event
                DCD     SWU2_EVT_Int_Handler               ; 120  Event
                DCD     SWU3_EVT_Int_Handler               ; 121  Event
                DCD     SWU4_EVT_Int_Handler               ; 122  Event
                DCD     CAN0_RX_Int_Handler                ; 123  Receive
                DCD     CAN0_TX_Int_Handler                ; 124  Transmit
                DCD     CAN0_STAT_Int_Handler              ; 125  Status
                DCD     CAN1_RX_Int_Handler                ; 126  Receive
                DCD     CAN1_TX_Int_Handler                ; 127  Transmit
                DCD     CAN1_STAT_Int_Handler              ; 128  Status
__vector_table_End

__vector_table_Size  EQU     __vector_table_End - __vector_table

                AREA    |.text|, CODE, READONLY


; __low_level_init is not exactly the same as in IAR.
; see Engineer-to-Engineer Note EE-359 "ADSP-CM40x Boot Time Optimization and Device Initialization"
__low_level_init\
                PROC
                EXPORT  __low_level_init          [WEAK]
                BX      LR
                ENDP

; Reset Handler
Reset_Handler   PROC
                EXPORT  Reset_Handler             [WEAK]
                IMPORT  preSystemInit
                IMPORT  __main

                ; reset the main SP to clean up any leftovers from the boot rom
                LDR     R0, __vector_table
                MSR     MSP, R0
                    
                ; disable oscillator watchdog false positive anomoly
                ; (avoid driving the fault light on false positives)
                LDR     R0, =preSystemInit
                BLX     R0

                ; copy the functions executed in RAM to RAM
                LDR     R0, =__low_level_init
                BLX     R0

                LDR     R0, =__main
                BX      R0
                ENDP


; Dummy Exception Handlers (infinite loops which can be modified)

NMI_Handler     PROC
                EXPORT  NMI_Handler               [WEAK]
                B       .
                ENDP
HardFault_Handler\
                PROC
                EXPORT  HardFault_Handler         [WEAK]
                B       .
                ENDP
MemManage_Handler\
                PROC
                EXPORT  MemManage_Handler         [WEAK]
                B       .
                ENDP
BusFault_Handler\
                PROC
                EXPORT  BusFault_Handler          [WEAK]
                B       .
                ENDP
UsageFault_Handler\
                PROC
                EXPORT  UsageFault_Handler        [WEAK]
                B       .
                ENDP
SVC_Handler     PROC
                EXPORT  SVC_Handler               [WEAK]
                B       .
                ENDP
DebugMon_Handler\
                PROC
                EXPORT  DebugMon_Handler          [WEAK]
                B       .
                ENDP
PendSV_Handler  PROC
                EXPORT  PendSV_Handler            [WEAK]
                B       .
                ENDP
SysTick_Handler PROC
                EXPORT  SysTick_Handler           [WEAK]
                B       .
                ENDP

Default_Handler PROC

                EXPORT  OSCW_EVT_Int_Handler               [WEAK]
                EXPORT  CGU0_EVT_Int_Handler               [WEAK]
                EXPORT  M4P_L1CC_PERR_Int_Handler          [WEAK]
                EXPORT  M4P_CORE_SRAM_PERR_Int_Handler     [WEAK]
                EXPORT  M4P_DMA_SRAM_PERR_Int_Handler      [WEAK]
                EXPORT  M4P_BUS_FAULT_Int_Handler          [WEAK]
                EXPORT  M4P_LOCKUP_Int_Handler             [WEAK]
                EXPORT  M4P_SRAM_PERR_FLT_Int_Handler      [WEAK]
                EXPORT  WDOG0_EXP_Int_Handler              [WEAK]
                EXPORT  SEC0_ERR_Int_Handler               [WEAK]
                EXPORT  ECT_EVT0_Int_Handler               [WEAK]
                EXPORT  ECT_EVT1_Int_Handler               [WEAK]
                EXPORT  PWM0_TRIP_Int_Handler              [WEAK]
                EXPORT  PWM1_TRIP_Int_Handler              [WEAK]
                EXPORT  PWM2_TRIP_Int_Handler              [WEAK]
                EXPORT  PWM0_SYNC_Int_Handler              [WEAK]
                EXPORT  PWM1_SYNC_Int_Handler              [WEAK]
                EXPORT  PWM2_SYNC_Int_Handler              [WEAK]
                EXPORT  PINT0_BLOCK_Int_Handler            [WEAK]
                EXPORT  PINT1_BLOCK_Int_Handler            [WEAK]
                EXPORT  PINT2_BLOCK_Int_Handler            [WEAK]
                EXPORT  PINT3_BLOCK_Int_Handler            [WEAK]
                EXPORT  PINT4_BLOCK_Int_Handler            [WEAK]
                EXPORT  SPORT0_A_DMA_ERR_Int_Handler       [WEAK]
                EXPORT  SPORT0_B_DMA_ERR_Int_Handler       [WEAK]
                EXPORT  SPI0_TXDMA_ERR_Int_Handler         [WEAK]
                EXPORT  SPI0_RXDMA_ERR_Int_Handler         [WEAK]
                EXPORT  UART0_TXDMA_ERR_Int_Handler        [WEAK]
                EXPORT  UART0_RXDMA_ERR_Int_Handler        [WEAK]
                EXPORT  SPORT1_A_DMA_ERR_Int_Handler       [WEAK]
                EXPORT  SPORT1_B_DMA_ERR_Int_Handler       [WEAK]
                EXPORT  SPI1_TXDMA_ERR_Int_Handler         [WEAK]
                EXPORT  SPI1_RXDMA_ERR_Int_Handler         [WEAK]
                EXPORT  UART1_TXDMA_ERR_Int_Handler        [WEAK]
                EXPORT  UART1_RXDMA_ERR_Int_Handler        [WEAK]
                EXPORT  UART2_TXDMA_ERR_Int_Handler        [WEAK]
                EXPORT  UART2_RXDMA_ERR_Int_Handler        [WEAK]
                EXPORT  HAE0_RXDMA_CH0_ERR_Int_Handler     [WEAK]
                EXPORT  HAE0_RXDMA_CH1_ERR_Int_Handler     [WEAK]
                EXPORT  HAE0_TXDMA_ERR_Int_Handler         [WEAK]
                EXPORT  RESERVED_VECTOR                    [WEAK]
                EXPORT  SPI0_ERR_Int_Handler               [WEAK]
                EXPORT  SPI1_ERR_Int_Handler               [WEAK]
                EXPORT  SPI2_ERR_Int_Handler               [WEAK]
                EXPORT  ADCC0_ERR_Int_Handler              [WEAK]
                EXPORT  DACC0_ERR_Int_Handler              [WEAK]
                EXPORT  MDMA0_SRC_CRC0_IN_ERR_Int_Handler  [WEAK]
                EXPORT  MDMA0_DST_CRC0_OUT_ERR_Int_Handler [WEAK]
                EXPORT  MDMA1_SRC_ERR_Int_Handler          [WEAK]
                EXPORT  MDMA1_DST_ERR_Int_Handler          [WEAK]
                EXPORT  TIMER0_STAT_Int_Handler            [WEAK]
                EXPORT  TIMER0_TMR0_Int_Handler            [WEAK]
                EXPORT  TIMER0_TMR1_Int_Handler            [WEAK]
                EXPORT  TIMER0_TMR2_Int_Handler            [WEAK]
                EXPORT  TIMER0_TMR3_Int_Handler            [WEAK]
                EXPORT  TIMER0_TMR4_Int_Handler            [WEAK]
                EXPORT  TIMER0_TMR5_Int_Handler            [WEAK]
                EXPORT  TIMER0_TMR6_Int_Handler            [WEAK]
                EXPORT  TIMER0_TMR7_Int_Handler            [WEAK]
                EXPORT  CNT0_STAT_Int_Handler              [WEAK]
                EXPORT  CNT1_STAT_Int_Handler              [WEAK]
                EXPORT  CNT2_STAT_Int_Handler              [WEAK]
                EXPORT  CNT3_STAT_Int_Handler              [WEAK]
                EXPORT  SPORT0_A_STAT_Int_Handler          [WEAK]
                EXPORT  SPORT0_A_DMA_Int_Handler           [WEAK]
                EXPORT  SPORT0_B_STAT_Int_Handler          [WEAK]
                EXPORT  SPORT0_B_DMA_Int_Handler           [WEAK]
                EXPORT  UART0_STAT_Int_Handler             [WEAK]
                EXPORT  UART0_TXDMA_Int_Handler            [WEAK]
                EXPORT  UART0_RXDMA_Int_Handler            [WEAK]
                EXPORT  SINC0_STAT_Int_Handler             [WEAK]
                EXPORT  SPI0_STAT_Int_Handler              [WEAK]
                EXPORT  SPI0_TXDMA_Int_Handler             [WEAK]
                EXPORT  SPI0_RXDMA_Int_Handler             [WEAK]
                EXPORT  SPORT1_A_STAT_Int_Handler          [WEAK]
                EXPORT  SPORT1_A_DMA_Int_Handler           [WEAK]
                EXPORT  SPORT1_B_STAT_Int_Handler          [WEAK]
                EXPORT  SPORT1_B_DMA_Int_Handler           [WEAK]
                EXPORT  UART1_STAT_Int_Handler             [WEAK]
                EXPORT  UART1_TXDMA_Int_Handler            [WEAK]
                EXPORT  UART1_RXDMA_Int_Handler            [WEAK]
                EXPORT  SPI1_STAT_Int_Handler              [WEAK]
                EXPORT  SPI1_TXDMA_Int_Handler             [WEAK]
                EXPORT  SPI1_RXDMA_Int_Handler             [WEAK]
                EXPORT  SPI2_TX_Int_Handler                [WEAK]
                EXPORT  SPI2_RX_Int_Handler                [WEAK]
                EXPORT  EMAC0_STAT_Int_Handler             [WEAK]
                EXPORT  UART2_STAT_Int_Handler             [WEAK]
                EXPORT  UART2_TXDMA_Int_Handler            [WEAK]
                EXPORT  UART2_RXDMA_Int_Handler            [WEAK]
                EXPORT  HAE0_STAT_Int_Handler              [WEAK]
                EXPORT  HAE0_RXDMA_CH0_Int_Handler         [WEAK]
                EXPORT  HAE0_RXDMA_CH1_Int_Handler         [WEAK]
                EXPORT  HAE0_TXDMA_Int_Handler             [WEAK]
                EXPORT  ADCC0_TMR0_EVT_Int_Handler         [WEAK]
                EXPORT  ADCC0_TMR1_EVT_Int_Handler         [WEAK]
                EXPORT  DACC0_DAC0_Int_Handler             [WEAK]
                EXPORT  DACC0_DAC1_Int_Handler             [WEAK]
                EXPORT  SPI2_STAT_Int_Handler              [WEAK]
                EXPORT  TWI0_DATA_Int_Handler              [WEAK]
                EXPORT  CRC0_DCNTEXP_Int_Handler           [WEAK]
                EXPORT  CRC0_ERR_Int_Handler               [WEAK]
                EXPORT  MDMA0_SRC_CRC0_IN_Int_Handler      [WEAK]
                EXPORT  MDMA0_DST_CRC0_OUT_Int_Handler     [WEAK]
                EXPORT  MDMA1_SRC_Int_Handler              [WEAK]
                EXPORT  MDMA1_DST_Int_Handler              [WEAK]
                EXPORT  USB0_STAT_Int_Handler              [WEAK]
                EXPORT  USB0_DATA_Int_Handler              [WEAK]
                EXPORT  TRU0_INT0_Int_Handler              [WEAK]
                EXPORT  TRU0_INT1_Int_Handler              [WEAK]
                EXPORT  TRU0_INT2_Int_Handler              [WEAK]
                EXPORT  TRU0_INT3_Int_Handler              [WEAK]
                EXPORT  CGU0_ERR_Int_Handler               [WEAK]
                EXPORT  DPM0_EVT_Int_Handler               [WEAK]
                EXPORT  SOFT0_Int_Handler                  [WEAK]
                EXPORT  SOFT1_Int_Handler                  [WEAK]
                EXPORT  SOFT2_Int_Handler                  [WEAK]
                EXPORT  SOFT3_Int_Handler                  [WEAK]
                EXPORT  SWU0_EVT_Int_Handler               [WEAK]
                EXPORT  SWU1_EVT_Int_Handler               [WEAK]
                EXPORT  SWU2_EVT_Int_Handler               [WEAK]
                EXPORT  SWU3_EVT_Int_Handler               [WEAK]
                EXPORT  SWU4_EVT_Int_Handler               [WEAK]
                EXPORT  CAN0_RX_Int_Handler                [WEAK]
                EXPORT  CAN0_TX_Int_Handler                [WEAK]
                EXPORT  CAN0_STAT_Int_Handler              [WEAK]
                EXPORT  CAN1_RX_Int_Handler                [WEAK]
                EXPORT  CAN1_TX_Int_Handler                [WEAK]
                EXPORT  CAN1_STAT_Int_Handler              [WEAK]

OSCW_EVT_Int_Handler
CGU0_EVT_Int_Handler
M4P_L1CC_PERR_Int_Handler
M4P_CORE_SRAM_PERR_Int_Handler
M4P_DMA_SRAM_PERR_Int_Handler
M4P_BUS_FAULT_Int_Handler
M4P_LOCKUP_Int_Handler
M4P_SRAM_PERR_FLT_Int_Handler
WDOG0_EXP_Int_Handler
SEC0_ERR_Int_Handler
ECT_EVT0_Int_Handler
ECT_EVT1_Int_Handler
PWM0_TRIP_Int_Handler
PWM1_TRIP_Int_Handler
PWM2_TRIP_Int_Handler
PWM0_SYNC_Int_Handler
PWM1_SYNC_Int_Handler
PWM2_SYNC_Int_Handler
PINT0_BLOCK_Int_Handler
PINT1_BLOCK_Int_Handler
PINT2_BLOCK_Int_Handler
PINT3_BLOCK_Int_Handler
PINT4_BLOCK_Int_Handler
SPORT0_A_DMA_ERR_Int_Handler
SPORT0_B_DMA_ERR_Int_Handler
SPI0_TXDMA_ERR_Int_Handler
SPI0_RXDMA_ERR_Int_Handler
UART0_TXDMA_ERR_Int_Handler
UART0_RXDMA_ERR_Int_Handler
SPORT1_A_DMA_ERR_Int_Handler
SPORT1_B_DMA_ERR_Int_Handler
SPI1_TXDMA_ERR_Int_Handler
SPI1_RXDMA_ERR_Int_Handler
UART1_TXDMA_ERR_Int_Handler
UART1_RXDMA_ERR_Int_Handler
UART2_TXDMA_ERR_Int_Handler
UART2_RXDMA_ERR_Int_Handler
HAE0_RXDMA_CH0_ERR_Int_Handler
HAE0_RXDMA_CH1_ERR_Int_Handler
HAE0_TXDMA_ERR_Int_Handler
RESERVED_VECTOR
SPI0_ERR_Int_Handler
SPI1_ERR_Int_Handler
SPI2_ERR_Int_Handler
ADCC0_ERR_Int_Handler
DACC0_ERR_Int_Handler
MDMA0_SRC_CRC0_IN_ERR_Int_Handler
MDMA0_DST_CRC0_OUT_ERR_Int_Handler
MDMA1_SRC_ERR_Int_Handler
MDMA1_DST_ERR_Int_Handler
TIMER0_STAT_Int_Handler
TIMER0_TMR0_Int_Handler
TIMER0_TMR1_Int_Handler
TIMER0_TMR2_Int_Handler
TIMER0_TMR3_Int_Handler
TIMER0_TMR4_Int_Handler
TIMER0_TMR5_Int_Handler
TIMER0_TMR6_Int_Handler
TIMER0_TMR7_Int_Handler
CNT0_STAT_Int_Handler
CNT1_STAT_Int_Handler
CNT2_STAT_Int_Handler
CNT3_STAT_Int_Handler
SPORT0_A_STAT_Int_Handler
SPORT0_A_DMA_Int_Handler
SPORT0_B_STAT_Int_Handler
SPORT0_B_DMA_Int_Handler
UART0_STAT_Int_Handler
UART0_TXDMA_Int_Handler
UART0_RXDMA_Int_Handler
SINC0_STAT_Int_Handler
SPI0_STAT_Int_Handler
SPI0_TXDMA_Int_Handler
SPI0_RXDMA_Int_Handler
SPORT1_A_STAT_Int_Handler
SPORT1_A_DMA_Int_Handler
SPORT1_B_STAT_Int_Handler
SPORT1_B_DMA_Int_Handler
UART1_STAT_Int_Handler
UART1_TXDMA_Int_Handler
UART1_RXDMA_Int_Handler
SPI1_STAT_Int_Handler
SPI1_TXDMA_Int_Handler
SPI1_RXDMA_Int_Handler
SPI2_TX_Int_Handler
SPI2_RX_Int_Handler
EMAC0_STAT_Int_Handler
UART2_STAT_Int_Handler
UART2_TXDMA_Int_Handler
UART2_RXDMA_Int_Handler
HAE0_STAT_Int_Handler
HAE0_RXDMA_CH0_Int_Handler
HAE0_RXDMA_CH1_Int_Handler
HAE0_TXDMA_Int_Handler
ADCC0_TMR0_EVT_Int_Handler
ADCC0_TMR1_EVT_Int_Handler
DACC0_DAC0_Int_Handler
DACC0_DAC1_Int_Handler
SPI2_STAT_Int_Handler
TWI0_DATA_Int_Handler
CRC0_DCNTEXP_Int_Handler
CRC0_ERR_Int_Handler
MDMA0_SRC_CRC0_IN_Int_Handler
MDMA0_DST_CRC0_OUT_Int_Handler
MDMA1_SRC_Int_Handler
MDMA1_DST_Int_Handler
USB0_STAT_Int_Handler
USB0_DATA_Int_Handler
TRU0_INT0_Int_Handler
TRU0_INT1_Int_Handler
TRU0_INT2_Int_Handler
TRU0_INT3_Int_Handler
CGU0_ERR_Int_Handler
DPM0_EVT_Int_Handler
SOFT0_Int_Handler
SOFT1_Int_Handler
SOFT2_Int_Handler
SOFT3_Int_Handler
SWU0_EVT_Int_Handler
SWU1_EVT_Int_Handler
SWU2_EVT_Int_Handler
SWU3_EVT_Int_Handler
SWU4_EVT_Int_Handler
CAN0_RX_Int_Handler
CAN0_TX_Int_Handler
CAN0_STAT_Int_Handler
CAN1_RX_Int_Handler
CAN1_TX_Int_Handler
CAN1_STAT_Int_Handler
                B       .
                ENDP

                ALIGN


; User Initial Stack & Heap

                IF      :DEF:__MICROLIB

                EXPORT  __initial_sp
                EXPORT  __heap_base
                EXPORT  __heap_limit

                ELSE

                IMPORT  __use_two_region_memory
                EXPORT  __user_initial_stackheap
__user_initial_stackheap

                LDR     R0, =  Heap_Mem
                LDR     R1, =(Stack_Mem + Stack_Size)
                LDR     R2, = (Heap_Mem +  Heap_Size)
                LDR     R3, = Stack_Mem
                BX      LR

                ALIGN

                ENDIF


                END
