/*
** init_cgu.c
**
** Initialize clock generation unit
**
** Copyright (c) 2013 Analog Devices, Inc. All Rights Reserved.
**/

#include "init_cgu.h"

// #pragma location = "hw_init"
// __ramfunc INIT_CGU_RESULT init_cgu(void)

#pragma arm section code = "hw_init"
INIT_CGU_RESULT init_cgu(void)
{
  ADI_CGU_TypeDef *pCGU = pADI_CGU0;

  uint32_t cgu_ctl = MSEL_VAL | DF_VAL;
  uint32_t cgu_div = OSEL_VAL | DSEL_VAL | S0SEL_VAL | SYSSEL_VAL | CSEL_VAL;
  uint32_t cgu_clkoutsel =  CLKOUTSEL_VAL | USBCLKSEL_VAL;

  /* Configure the Oscillator watchdog */
  pCGU->OSCWDCTL &= ~(BITM_CGU_OSCWDCTL_HODEN | BITM_CGU_OSCWDCTL_HODF | BITM_CGU_OSCWDCTL_BOUEN | BITM_CGU_OSCWDCTL_BOUF);

  pCGU->OSCWDCTL |= (HODF | BOUF | BITM_CGU_OSCWDCTL_BOUEN | BITM_CGU_OSCWDCTL_HODEN);

  /* Verify the PLL is enabled
   * Return a lock error if PLL is not enabled and locked
   */
  if((pCGU->STAT & (BITM_CGU_STAT_PLLEN |
                    BITM_CGU_STAT_PLOCK |
                    BITM_CGU_STAT_CLKSALGN))
     != (BITM_CGU_STAT_PLLEN | BITM_CGU_STAT_PLOCK))
  {
    return INIT_CGU_RESULT_LOCK_ERR;
  }

  /* Check to see if a change to MSEL is required,
   * if it is then we need to write the CGU_DIV first,
   * followed by the write to CGU_CTL, alignment will reoccur.
   * If no change to core clock is required then only
   * update CGU_DIV ensuring the update bit is also set.
   */
  if((pCGU->CTL & (BITM_CGU_CTL_DF | BITM_CGU_CTL_MSEL)) !=  ((DF_VAL | MSEL_VAL)))
  {
    pCGU->DIV = cgu_div;
    pCGU->CTL = cgu_ctl;
  }
  else
  {
    pCGU->DIV = (cgu_div | BITM_CGU_DIV_UPDT);
  }

  /* Wait for the clocks to all realign and lock */
  while((pCGU->STAT & (BITM_CGU_STAT_PLLEN |
                       BITM_CGU_STAT_PLOCK |
                       BITM_CGU_STAT_PLLBP |
                       BITM_CGU_STAT_CLKSALGN))
        != (BITM_CGU_STAT_PLLEN | BITM_CGU_STAT_PLOCK))
  {
    /* Return with failure if we get an error of any kind */
    if((pCGU->STAT & (BITM_CGU_STAT_WDIVERR |
                      BITM_CGU_STAT_WDFMSERR |
                      BITM_CGU_STAT_DIVERR |
                      BITM_CGU_STAT_LWERR |
                      BITM_CGU_STAT_ADDRERR)) != 0u)
    {
      return INIT_CGU_RESULT_FAILURE;
    }
  }

  /* Configue the CLKOUTSEL register */
  pCGU->CLKOUTSEL = cgu_clkoutsel;

  return INIT_CGU_RESULT_SUCCESS;
}
