/*
** init_mem.c
**
** Initialize memory partitioning
**
** Copyright (c) 2013 Analog Devices, Inc. All Rights Reserved.
**/

#include "init_mem.h"

static INIT_MEM_RESULT init_mem_memfill(void * pSource, uint32_t count);

INIT_MEM_RESULT init_mem(const INIT_MEM_CONFIG config, const INIT_MEM_POSTED_WRITES posted)
{
  /* Configures the memory partitioning for the code and data sections.
   * This software can be kept in the spi flash, no need to copy to the SRAM
   * for execution.
   *
   * The boot code initializes all memory for 50/50 settings, if section is added
   * it is recommended to initialize the newly added section to initialize the parity
   * information
   */


  uint32_t cfgVal = ENUM_M4P_SRAM_CFG_SRAM_CODE_3;
  uint32_t postedVal = 0u;
  void * pSource = 0u;
  uint32_t count = 0u;

  if(posted == INIT_MEM_POSTED_WRITES_ENABLED)
  {
    postedVal = BITM_M4P_SRAM_CFG_POSTWR;
  }

  switch(config)
  {
  case INIT_MEM_CODE3_DATA3:
    if(posted == INIT_MEM_POSTED_WRITES_ENABLED)
    {
      *pREG_M4P0_SRAM_CFG |= BITM_M4P_SRAM_CFG_POSTWR;
    }
    else
    {
      *pREG_M4P0_SRAM_CFG &= (~BITM_M4P_SRAM_CFG_POSTWR);
    }
    return INIT_MEM_RESULT_SUCCESS;
  case INIT_MEM_CODE2_DATA4:
    cfgVal = ENUM_M4P_SRAM_CFG_SRAM_CODE_2 | postedVal;
    pSource = (void *)0x20020000;
    count = 0x00010000;
    break;
  case INIT_MEM_CODE1_DATA5:
    cfgVal = ENUM_M4P_SRAM_CFG_SRAM_CODE_1  | postedVal;
    pSource = (void *)0x20010000;
    count = 0x00020000;
    break;
  case INIT_MEM_CODE0_DATA6:
    cfgVal = ENUM_M4P_SRAM_CFG_SRAM_CODE_0  | postedVal;
    pSource = (void *)0x20000000;
    count = 0x00030000;
    break;
  case INIT_MEM_CODE4_DATA2:
    cfgVal = ENUM_M4P_SRAM_CFG_SRAM_CODE_4  | postedVal;
    pSource = (void *)0x10030000;
    count = 0x00010000;
    break;
  default:
    if(posted == INIT_MEM_POSTED_WRITES_ENABLED)
    {
      *pREG_M4P0_SRAM_CFG |= BITM_M4P_SRAM_CFG_POSTWR;
    }
    else
    {
      *pREG_M4P0_SRAM_CFG &= (~BITM_M4P_SRAM_CFG_POSTWR);
    }
    return INIT_MEM_RESULT_SUCCESS;
  }

  *pREG_M4P0_SRAM_CFG =  (*pREG_M4P0_SRAM_CFG & (~(BITM_M4P_SRAM_CFG_CDBANKS | BITM_M4P_SRAM_CFG_POSTWR ))) | cfgVal;

  init_mem_memfill(pSource, count);

  return INIT_MEM_RESULT_SUCCESS;
}

static INIT_MEM_RESULT init_mem_memfill(void * pSource, uint32_t count)
{
  uint32_t *pMem = (uint32_t *)pSource;

  for(uint32_t i=0; i<(count/4); i++)
  {
    pMem[i] = 0u;
  }

  return INIT_MEM_RESULT_SUCCESS;
}
