/*
** proc_init.c
**
** Configure and optimize the processor
**
** Copyright (c) 2013 Analog Devices, Inc. All Rights Reserved.
**/

#include "device.h"
#include "init_cgu.h"
#include "init_dpm.h"
#include "init_spi.h"
#include "init_mem.h"

void call_app(uint32_t address);
/* __ramfunc */ void error_handler(void);

#if 0
inline void call_app(uint32_t address)
{
  __ASM("ldr r1, [r0, #0]");
  __ASM("msr msp, r1");
  __ASM("ldr r1, [r0, #4]");
  __ASM volatile ("bx      r1");
}
#endif
__INLINE __ASM void call_app(uint32_t address)
{
  ldr r1, [r0, #0]
  msr msp, r1
  ldr r1, [r0, #4]
  bx  r1
}

// #pragma location = "hw_init"
// __ramfunc void proc_init(void)

#pragma arm section code = "hw_init"
void proc_init(void)
{
  /* Bring the PLL out of bypass mode */
  if(init_dpm() != INIT_DPM_RESULT_SUCCESS)
  {
    /* Call the error handler */
    error_handler();
  }

  if(init_cgu() != INIT_CGU_RESULT_SUCCESS)
  {
    /* Call the error handler */
    error_handler();
  }


  if(init_spi() != INIT_SPI_RESULT_SUCCESS)
  {
    /* Call the error handler */
    error_handler();
  }

  if(init_mem(INIT_MEM_CODE3_DATA3, INIT_MEM_POSTED_WRITES_ENABLED) != INIT_MEM_RESULT_SUCCESS)
  {
    /* Call the error handler */
    error_handler();
  }

}

// #pragma location = "hw_init"
// __ramfunc void error_handler(void)

#pragma arm section code = "hw_init"
void error_handler(void)
{
  while(1);
}
