/*!
 *****************************************************************************
 * @file:    device.h
 * @brief:   CMSIS Cortex-M4 Core Peripheral Access Layer Header File for
 *           ADI ADSP-CM40z Device Series
 * @version: $Revision: 21692 $
 * @date:    $Date: 2013-05-31 14:47:25 -0400 (Fri, 31 May 2013) $
 *-----------------------------------------------------------------------------
 *
Copyright (c) 2010 Analog Devices, Inc.

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  - Modified versions of the software must be conspicuously marked as such.
  - This software is licensed solely and exclusively for use with processors
    manufactured by or for Analog Devices, Inc.
  - This software may not be combined or merged with other code in any manner
    that would cause the software to become subject to terms and conditions
    which differ from those listed here.
  - Neither the name of Analog Devices, Inc. nor the names of its
    contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.
  - The use of this software may or may not infringe the patent rights of one
    or more patent holders.  This license does not release you from the
    requirement that you obtain separate licenses from these patent holders
    to use this software.

THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES, INC. AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
TITLE, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
NO EVENT SHALL ANALOG DEVICES, INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, PUNITIVE OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, DAMAGES ARISING OUT OF CLAIMS OF INTELLECTUAL
PROPERTY RIGHTS INFRINGEMENT; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*!  \addtogroup MMR_Layout MMR/ISR Mappings
 *  @{
 */

#ifndef __DEVICE_H__
#define __DEVICE_H__

// verify presence of a supported compiler (Keil or IAR at this time)
#if !defined(__ICCARM__) && !defined(__CC_ARM)
#error "No supported compiler found, please install one of the minimim required compilers."
#endif

// verify minimum required IAR compiler
#ifdef __ICCARM__
#define MIN_CC_VERSION 6010001  // known value for release 6.10.1
#if (MIN_CC_VERSION > __VER__)
#error "Unsupported compiler version, please install minimum required compiler version."
#endif
#endif

// verify minimum required Keil compiler
#ifdef __CC_ARM
#define MIN_CC_VERSION 410713  // known value for release 4.21
#if (MIN_CC_VERSION > __ARMCC_VERSION)
#error "Unsupported compiler version, please install minimum required compiler version."
#endif
#endif

/* Feature macro for conditional compilation */
#define ADI_ADSP_CM40Z  /*!< PreProcessor feature macro */

/* include global MISRA suppressions */
//#include "misra.h"

/* pickup register bitfield and bit mask macros */
//#include "macros.h"

/* pickup system interrupt ID (SID) defines */
#include <services/int/adi_sid_cm40z.h>

/* pickup NVIC SID Mappings and interrup hooking prototypes */
#include "adi_nvic.h"

/* C++ linkage */
#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/* The below code segment is required to disable the interrupt at the NVIC level.
 * If the program is already running from Flash the interrupt gets invoked as soon
 * as we install the handler. Below code acknowledges any pending interrupts at the
 * NVIC level.
 */
#if defined(ADI_ADSP_CM40Z)
#define ADI_ACK_PENDINT(irq)       \
    do {                           \
    if (NVIC_GetPendingIRQ(irq))   \
    {                              \
        NVIC_ClearPendingIRQ(irq); \
    }                              \
    }while(0)
#else
#define ADI_ACK_PENDINT(irq)
#endif

/*!
 * \enum IRQn_Type
 * Interrupt Number Assignments
 */
typedef enum
{
  RESET_IRQn                   =  ADI_INT_RESET                   ,  // M4CORE Reset
  NonMaskableInt_IRQn          =  ADI_INT_NMI                     ,  // M4CORE Non-maskable Interrupt
  HardFault_IRQn               =  ADI_INT_HARD_FAULT              ,  // M4CORE Unmanaged Fault
  MemoryManagement_IRQn        =  ADI_INT_MEM_MGMT                ,  // M4CORE MPU Fault
  BusFault_IRQn                =  ADI_INT_BUS_FAULT               ,  // M4CORE Crossbar Fault
  UsageFault_IRQn              =  ADI_INT_USAGE_FAULT             ,  // M4CORE Instruction/Privilege Fault
  //                           =                                  ,     Reserved
  //                           =                                  ,     Reserved
  //                           =                                  ,     Reserved
  //                           =                                  ,     Reserved
  SVCall_IRQn                  =  ADI_INT_SVCALL                  ,  // M4CORE Service Call
  DebugMonitor_IRQn            =  ADI_INT_DEBUG_MON               ,  // M4CORE Debug Monitor
  //                           =                                  ,     Reserved
  PendSV_IRQn                  =  ADI_INT_PENDSV                  ,  // M4CORE Pending Service
  SysTick_IRQn                 =  ADI_INT_SYSTICK                 ,  // M4CORE System Time Tick

  OSCW_EVT_IRQn                =  ADI_INT_OSCW_EVT                ,  // Oscillator Watchdog
  CGU0_EVT_IRQn                =  ADI_INT_CGU0_EVT                ,  // Event
  M4P_L1CC_PERR_IRQn           =  ADI_INT_M4P_L1CC_PERR           ,  // L1 Cache Code Parity Error
  M4P_CORE_SRAM_PERR_IRQn      =  ADI_INT_M4P_CORE_SRAM_PERR      ,  // SRAM Core Parity Error
  M4P_DMA_SRAM_PERR_IRQn       =  ADI_INT_M4P_DMA_SRAM_PERR       ,  // SRAM DMA Parity Error
  M4P_BUS_FAULT_IRQn           =  ADI_INT_M4P_BUS_FAULT           ,  // Bus Fault
  M4P_LOCKUP_IRQn              =  ADI_INT_M4P_LOCKUP              ,  // Lockup Error (Fault only; not an interrupt)
  M4P_SRAM_PERR_FLT_IRQn       =  ADI_INT_M4P_SRAM_PERR_FLT       ,  // SRAM Parity Error (Fault only; not an interrupt)
  WDOG0_EXP_IRQn               =  ADI_INT_WDOG0_EXP               ,  // Expiration
  SEC0_ERR_IRQn                =  ADI_INT_SEC0_ERR                ,  // Fault Interrupt
  ECT_EVT0_IRQn                =  ADI_INT_ECT_EVT0                ,  // Embedded Cross Trigger Event 0
  ECT_EVT1_IRQn                =  ADI_INT_ECT_EVT1                ,  // Embedded Cross Trigger Event 1
  PWM0_TRIP_IRQn               =  ADI_INT_PWM0_TRIP               ,  // Trip
  PWM1_TRIP_IRQn               =  ADI_INT_PWM1_TRIP               ,  // Trip
  PWM2_TRIP_IRQn               =  ADI_INT_PWM2_TRIP               ,  // Trip
  PWM0_SYNC_IRQn               =  ADI_INT_PWM0_SYNC               ,  // Sync
  PWM1_SYNC_IRQn               =  ADI_INT_PWM1_SYNC               ,  // Sync
  PWM2_SYNC_IRQn               =  ADI_INT_PWM2_SYNC               ,  // Sync
  PINT0_BLOCK_IRQn             =  ADI_INT_PINT0_BLOCK             ,  // Pin Interrupt Block
  PINT1_BLOCK_IRQn             =  ADI_INT_PINT1_BLOCK             ,  // Pin Interrupt Block
  PINT2_BLOCK_IRQn             =  ADI_INT_PINT2_BLOCK             ,  // Pin Interrupt Block
  PINT3_BLOCK_IRQn             =  ADI_INT_PINT3_BLOCK             ,  // Pin Interrupt Block
  PINT4_BLOCK_IRQn             =  ADI_INT_PINT4_BLOCK             ,  // Pin Interrupt Block
  SPORT0_A_DMA_ERR_IRQn        =  ADI_INT_SPORT0_A_DMA_ERR        ,  // DMA Channel 0 Error
  SPORT0_B_DMA_ERR_IRQn        =  ADI_INT_SPORT0_B_DMA_ERR        ,  // DMA Channel 1 Error
  SPI0_TXDMA_ERR_IRQn          =  ADI_INT_SPI0_TXDMA_ERR          ,  // DMA Channel 2 Error
  SPI0_RXDMA_ERR_IRQn          =  ADI_INT_SPI0_RXDMA_ERR          ,  // DMA Channel 3 Error
  UART0_TXDMA_ERR_IRQn         =  ADI_INT_UART0_TXDMA_ERR         ,  // DMA Channel 4 Error
  UART0_RXDMA_ERR_IRQn         =  ADI_INT_UART0_RXDMA_ERR         ,  // DMA Channel 5 Error
  SPORT1_A_DMA_ERR_IRQn        =  ADI_INT_SPORT1_A_DMA_ERR        ,  // DMA Channel 6 Error
  SPORT1_B_DMA_ERR_IRQn        =  ADI_INT_SPORT1_B_DMA_ERR        ,  // DMA Channel 7 Error
  SPI1_TXDMA_ERR_IRQn          =  ADI_INT_SPI1_TXDMA_ERR          ,  // DMA Channel 8 Error
  SPI1_RXDMA_ERR_IRQn          =  ADI_INT_SPI1_RXDMA_ERR          ,  // DMA Channel 9 Error
  UART1_TXDMA_ERR_IRQn         =  ADI_INT_UART1_TXDMA_ERR         ,  // DMA Channel 10 Error
  UART1_RXDMA_ERR_IRQn         =  ADI_INT_UART1_RXDMA_ERR         ,  // DMA Channel 11 Error
  UART2_TXDMA_ERR_IRQn         =  ADI_INT_UART2_TXDMA_ERR         ,  // DMA Channel 12 Error
  UART2_RXDMA_ERR_IRQn         =  ADI_INT_UART2_RXDMA_ERR         ,  // DMA Channel 13 Error
  HAE0_RXDMA_CH0_ERR_IRQn      =  ADI_INT_HAE0_RXDMA_CH0_ERR      ,  // DMA Channel 14 Error
  HAE0_RXDMA_CH1_ERR_IRQn      =  ADI_INT_HAE0_RXDMA_CH1_ERR      ,  // DMA Channel 15 Error
  HAE0_TXDMA_ERR_IRQn          =  ADI_INT_HAE0_TXDMA_ERR          ,  // DMA Channel 16 Error
  //                           =                                  ,     Reserved
  SPI0_ERR_IRQn                =  ADI_INT_SPI0_ERR                ,  // Error
  SPI1_ERR_IRQn                =  ADI_INT_SPI1_ERR                ,  // Error
  SPI2_ERR_IRQn                =  ADI_INT_SPI2_ERR                ,  // Error
  ADCC0_ERR_IRQn               =  ADI_INT_ADCC0_ERR               ,  // ADC Error
  DACC0_ERR_IRQn               =  ADI_INT_DACC0_ERR               ,  // DAC Error
  MDMA0_SRC_CRC0_IN_ERR_IRQn   =  ADI_INT_MDMA0_SRC_CRC0_IN_ERR   ,  // DMA Channel 17 Error
  MDMA0_DST_CRC0_OUT_ERR_IRQn  =  ADI_INT_MDMA0_DST_CRC0_OUT_ERR  ,  // DMA Channel 18 Error
  MDMA1_SRC_ERR_IRQn           =  ADI_INT_MDMA1_SRC_ERR           ,  // DMA Channel 19 Error
  MDMA1_DST_ERR_IRQn           =  ADI_INT_MDMA1_DST_ERR           ,  // DMA Channel 20 Error
  TIMER0_STAT_IRQn             =  ADI_INT_TIMER0_STAT             ,  // Status
  TIMER0_TMR0_IRQn             =  ADI_INT_TIMER0_TMR0             ,  // Timer 0
  TIMER0_TMR1_IRQn             =  ADI_INT_TIMER0_TMR1             ,  // Timer 1
  TIMER0_TMR2_IRQn             =  ADI_INT_TIMER0_TMR2             ,  // Timer 2
  TIMER0_TMR3_IRQn             =  ADI_INT_TIMER0_TMR3             ,  // Timer 3
  TIMER0_TMR4_IRQn             =  ADI_INT_TIMER0_TMR4             ,  // Timer 4
  TIMER0_TMR5_IRQn             =  ADI_INT_TIMER0_TMR5             ,  // Timer 5
  TIMER0_TMR6_IRQn             =  ADI_INT_TIMER0_TMR6             ,  // Timer 6
  TIMER0_TMR7_IRQn             =  ADI_INT_TIMER0_TMR7             ,  // Timer 7
  CNT0_STAT_IRQn               =  ADI_INT_CNT0_STAT               ,  // Status
  CNT1_STAT_IRQn               =  ADI_INT_CNT1_STAT               ,  // Status
  CNT2_STAT_IRQn               =  ADI_INT_CNT2_STAT               ,  // Status
  CNT3_STAT_IRQn               =  ADI_INT_CNT3_STAT               ,  // Status
  SPORT0_A_STAT_IRQn           =  ADI_INT_SPORT0_A_STAT           ,  // Channel A Status
  SPORT0_A_DMA_IRQn            =  ADI_INT_SPORT0_A_DMA            ,  // Channel A DMA
  SPORT0_B_STAT_IRQn           =  ADI_INT_SPORT0_B_STAT           ,  // Channel B Status
  SPORT0_B_DMA_IRQn            =  ADI_INT_SPORT0_B_DMA            ,  // Channel B DMA
  UART0_STAT_IRQn              =  ADI_INT_UART0_STAT              ,  // Status
  UART0_TXDMA_IRQn             =  ADI_INT_UART0_TXDMA             ,  // Transmit DMA
  UART0_RXDMA_IRQn             =  ADI_INT_UART0_RXDMA             ,  // Receive DMA
  SINC0_STAT_IRQn              =  ADI_INT_SINC0_STAT              ,  // Status
  SPI0_STAT_IRQn               =  ADI_INT_SPI0_STAT               ,  // Status
  SPI0_TXDMA_IRQn              =  ADI_INT_SPI0_TXDMA              ,  // TX DMA Channel
  SPI0_RXDMA_IRQn              =  ADI_INT_SPI0_RXDMA              ,  // RX DMA Channel
  SPORT1_A_STAT_IRQn           =  ADI_INT_SPORT1_A_STAT           ,  // Channel A Status
  SPORT1_A_DMA_IRQn            =  ADI_INT_SPORT1_A_DMA            ,  // Channel A DMA
  SPORT1_B_STAT_IRQn           =  ADI_INT_SPORT1_B_STAT           ,  // Channel B Status
  SPORT1_B_DMA_IRQn            =  ADI_INT_SPORT1_B_DMA            ,  // Channel B DMA
  UART1_STAT_IRQn              =  ADI_INT_UART1_STAT              ,  // Status
  UART1_TXDMA_IRQn             =  ADI_INT_UART1_TXDMA             ,  // Transmit DMA
  UART1_RXDMA_IRQn             =  ADI_INT_UART1_RXDMA             ,  // Receive DMA
  SPI1_STAT_IRQn               =  ADI_INT_SPI1_STAT               ,  // Status
  SPI1_TXDMA_IRQn              =  ADI_INT_SPI1_TXDMA              ,  // TX DMA Channel
  SPI1_RXDMA_IRQn              =  ADI_INT_SPI1_RXDMA              ,  // RX DMA Channel
  SPI2_TX_IRQn                 =  ADI_INT_SPI2_TX                 ,  // TX  Channel (non-DMA)
  SPI2_RX_IRQn                 =  ADI_INT_SPI2_RX                 ,  // RX Channel (non-DMA)
  EMAC0_STAT_IRQn              =  ADI_INT_EMAC0_STAT              ,  // Status
  UART2_STAT_IRQn              =  ADI_INT_UART2_STAT              ,  // Status
  UART2_TXDMA_IRQn             =  ADI_INT_UART2_TXDMA             ,  // Transmit DMA
  UART2_RXDMA_IRQn             =  ADI_INT_UART2_RXDMA             ,  // Receive DMA
  HAE0_STAT_IRQn               =  ADI_INT_HAE0_STAT               ,  // Status
  HAE0_RXDMA_CH0_IRQn          =  ADI_INT_HAE0_RXDMA_CH0          ,  // RX DMA Channel 0
  HAE0_RXDMA_CH1_IRQn          =  ADI_INT_HAE0_RXDMA_CH1          ,  // RX DMA Channel 1
  HAE0_TXDMA_IRQn              =  ADI_INT_HAE0_TXDMA              ,  // TX DMA Channel 0
  ADCC0_TMR0_EVT_IRQn          =  ADI_INT_ADCC0_TMR0_EVT          ,  // Timer 0 Event Complete
  ADCC0_TMR1_EVT_IRQn          =  ADI_INT_ADCC0_TMR1_EVT          ,  // Timer 1 Event Complete
  DACC0_DAC0_IRQn              =  ADI_INT_DACC0_DAC0              ,  // DAC Interrupt 0
  DACC0_DAC1_IRQn              =  ADI_INT_DACC0_DAC1              ,  // DAC Interrupt 1
  SPI2_STAT_IRQn               =  ADI_INT_SPI2_STAT               ,  // Status
  TWI0_DATA_IRQn               =  ADI_INT_TWI0_DATA               ,  // Data Interrupt
  CRC0_DCNTEXP_IRQn            =  ADI_INT_CRC0_DCNTEXP            ,  // Datacount expiration
  CRC0_ERR_IRQn                =  ADI_INT_CRC0_ERR                ,  // Error
  MDMA0_SRC_CRC0_IN_IRQn       =  ADI_INT_MDMA0_SRC_CRC0_IN       ,  // Memory DMA Stream 0 Source Channel
  MDMA0_DST_CRC0_OUT_IRQn      =  ADI_INT_MDMA0_DST_CRC0_OUT      ,  // Memory DMA Stream 0 Destination Channel
  MDMA1_SRC_IRQn               =  ADI_INT_MDMA1_SRC               ,  // Memory DMA Stream 1 Source Channel
  MDMA1_DST_IRQn               =  ADI_INT_MDMA1_DST               ,  // Memory DMA Stream 1 Destination Channel
  USB0_STAT_IRQn               =  ADI_INT_USB0_STAT               ,  // Status/FIFO Data Ready
  USB0_DATA_IRQn               =  ADI_INT_USB0_DATA               ,  // DMA Status/Transfer Complete
  TRU0_INT0_IRQn               =  ADI_INT_TRU0_INT0               ,  // Interrupt 0
  TRU0_INT1_IRQn               =  ADI_INT_TRU0_INT1               ,  // Interrupt 1
  TRU0_INT2_IRQn               =  ADI_INT_TRU0_INT2               ,  // Interrupt 2
  TRU0_INT3_IRQn               =  ADI_INT_TRU0_INT3               ,  // Interrupt 3
  CGU0_ERR_IRQn                =  ADI_INT_CGU0_ERR                ,  // Error
  DPM0_EVT_IRQn                =  ADI_INT_DPM0_EVT                ,  // Event
  SOFT0_IRQn                   =  ADI_INT_SOFT0                   ,  // Software-Driven Interrupt 0
  SOFT1_IRQn                   =  ADI_INT_SOFT1                   ,  // Software-Driven Interrupt 1
  SOFT2_IRQn                   =  ADI_INT_SOFT2                   ,  // Software-Driven Interrupt 2
  SOFT3_IRQn                   =  ADI_INT_SOFT3                   ,  // Software-Driven Interrupt 3
  SWU0_EVT_IRQn                =  ADI_INT_SWU0_EVT                ,  // Event
  SWU1_EVT_IRQn                =  ADI_INT_SWU1_EVT                ,  // Event
  SWU2_EVT_IRQn                =  ADI_INT_SWU2_EVT                ,  // Event
  SWU3_EVT_IRQn                =  ADI_INT_SWU3_EVT                ,  // Event
  SWU4_EVT_IRQn                =  ADI_INT_SWU4_EVT                ,  // Event
  CAN0_RX_IRQn                 =  ADI_INT_CAN0_RX                 ,  // Receive
  CAN0_TX_IRQn                 =  ADI_INT_CAN0_TX                 ,  // Transmit
  CAN0_STAT_IRQn               =  ADI_INT_CAN0_STAT               ,  // Status
  CAN1_RX_IRQn                 =  ADI_INT_CAN1_RX                 ,  // Receive
  CAN1_TX_IRQn                 =  ADI_INT_CAN1_TX                 ,  // Transmit
  CAN1_STAT_IRQn               =  ADI_INT_CAN1_STAT               ,  // Status
} IRQn_Type;  // typedef name for fixed interrupt numbers


/*!
 * \enum DMA_CHANn_TypeDef
 * DMA Channel Assignments
 */
typedef enum
{
  SPORT0_A_DMA_CHANn             =     0,    // Channel A DMA
  SPORT0_B_DMA_CHANn             =     1,    // Channel B DMA
  SPI0_TXDMA_CHANn               =     2,    // TX DMA Channel
  SPI0_RXDMA_CHANn               =     3,    // RX DMA Channel
  UART0_TXDMA_CHANn              =     4,    // Transmit DMA
  UART0_RXDMA_CHANn              =     5,    // Receive DMA
  SPORT1_A_DMA_CHANn             =     6,    // Channel A DMA
  SPORT1_B_DMA_CHANn             =     7,    // Channel B DMA
  SPI1_TXDMA_CHANn               =     8,    // TX DMA Channel
  SPI1_RXDMA_CHANn               =     9,    // RX DMA Channel
  UART1_TXDMA_CHANn              =    10,    // Transmit DMA
  UART1_RXDMA_CHANn              =    11,    // Receive DMA
  UART2_TXDMA_CHANn              =    12,    // Transmit DMA
  UART2_RXDMA_CHANn              =    13,    // Receive DMA
  HAE0_RXDMA_CH0_CHANn           =    14,    // RX DMA Channel 0
  HAE0_RXDMA_CH1_CHANn           =    15,    // RX DMA Channel 1
  HAE0_TXDMA_CHANn               =    16,    // TX DMA Channel 0
  MDMA0_SRC_CRC0_IN_CHANn        =    17,    // Memory DMA Stream 0 Source Channel
  MDMA0_DST_CRC0_OUT_CHANn       =    18,    // Memory DMA Stream 0 Destination Channel
  MDMA1_SRC_CHANn                =    19,    // Memory DMA Stream 1 Source Channel
  MDMA1_DST_CHANn                =    20,    // Memory DMA Stream 1 Destination Channel
} DMA_CHANn_TypeDef;  // typedef name for fixed DMA channel assignments


/*!
 * Device Error Code Offsets NEEDS REVISION FOR 350
 */
#define ADI_APP_USERERROR_OFFSET    0x00000100 /*!< Generic Device API Result Error Message Offset  */
#define ADI_DEV_GPT_ERROR_OFFSET    0x00000200 /*!< Error Offset for GP Timer API                   */
#define ADI_DEV_SYS_ERROR_OFFSET    0x00000300 /*!< Error Offset for System API                     */
#define ADI_DEV_PWR_ERROR_OFFSET    0x00000400 /*!< Error Offset for Power API                      */
#define ADI_DEV_EIR_ERROR_OFFSET    0x00000500 /*!< Error Offset for External Interrupt API         */
#define ADI_DEV_RST_ERROR_OFFSET    0x00000600 /*!< Error Offset for Reset API                      */
#define ADI_DEV_LFE_ERROR_OFFSET    0x00000700 /*!< Error Offset for Low Frequency Engine API       */
#define ADI_DEV_WUT_ERROR_OFFSET    0x00000800 /*!< Error Offset for Wakeup Timer API               */
#define ADI_DEV_WDT_ERROR_OFFSET    0x00000900 /*!< Error Offset for Watchdog Timer API             */
#define ADI_DEV_FLASH_ERROR_OFFSET  0x00000A00 /*!< Error Offset for Flash/EE API                   */
#define ADI_DEV_I2C_ERROR_OFFSET    0x00000B00 /*!< Error Offset for I2C API                        */
#define ADI_DEV_SPI_ERROR_OFFSET    0x00000C00 /*!< Error Offset for SPI API                        */
#define ADI_DEV_UART_ERROR_OFFSET   0x00000D00 /*!< Error Offset for UART API                       */
#define ADI_DEV_GPIO_ERROR_OFFSET   0x00000E00 /*!< Error Offset for GPIO API                       */
#define ADI_DEV_ANALOG_ERROR_OFFSET 0x00000F00 /*!< Error Offset for Misc. Analog API               */
#define ADI_DEV_UHF_ERROR_OFFSET    0x00001000 /*!< Error Offset for UHF API                        */
#define ADI_DEV_LFT_ERROR_OFFSET    0x00001100 /*!< Error Offset for Low Frerquency Transmitter API */
#define ADI_DEV_DMA_ERROR_OFFSET    0x00001200 /*!< Error Offset for DMA API                        */
#define ADI_DEV_ADC_ERROR_OFFSET    0x00001300 /*!< Error Offset for ADC API                        */
#define ADI_DEV_PWM_ERROR_OFFSET    0x00001400 /*!< Error Offset for PWM API                        */
#define ADI_DEV_AFE_ERROR_OFFSET    0x00001500 /*!< Error Offset for AFE API                        */

/******************************************************************************
 * Processor and Core Peripheral Section
 *****************************************************************************/
#define __FPU_PRESENT          1  /*!< floating-point unit present or not    */
#define __MPU_PRESENT          1  /*!< MPU present or not                    */
#define __NVIC_PRIO_BITS       4  /*!< Number of Bits for Priority Levels    */
#define __Vendor_SysTickConfig 0  /*!< 1 if different SysTick Config is used */


/******************************************************************************
 * Includes
 *****************************************************************************/

#include <core_cm4.h>    /* Cortex-M3 processor and core peripherals*/

/* SysTick Cycle Count Macros... max 24-bit measure (no wraparound handeling) */
#define CYCLES_INIT {                                                     \
    /* enable with internal clock and no interrupts */                    \
    SysTick->CTRL = SysTick_CTRL_CLKSOURCE_Msk | SysTick_CTRL_ENABLE_Msk; \
    SysTick->LOAD = 0x00FFFFFF;}                                     /*!< SysTick instruction count macro */
#define CYCLES_CLR     {SysTick->VAL = 0;}                           /*!< SysTick instruction count macro */
#define CYCLES_GET     (0x00ffffff - 5 - SysTick->VAL)               /*!< SysTick instruction count macro */
#define CYCLES_SUSPEND {SysTick->CTRL &= ~SysTick_CTRL_ENABLE_Msk;}  /*!< SysTick instruction count macro */
#define CYCLES_RESUME  {SysTick->CTRL |= SysTick_CTRL_ENABLE_Msk;}   /*!< SysTick instruction count macro */

/* need bool_t ahead of system.h */
#ifndef BOOL_TYPE
#define BOOL_TYPE /*!< Create a boolean type */

/*!
 * \typedef bool_t
 * boolean type, since use of C99 "bool" is not MISRA compliant
 */
typedef uint8_t  bool_t;

/* simple true/false defines for functional APIs */
#ifndef false
#define false 0u /*!< Create a boolean 'false' value */
#endif
#ifndef true
#define true 1u /*!< Create a boolean 'true' value */
#endif

#endif /* BOOL_TYPE */

#include "system.h"      /* System Header                           */

/* MIN */
#ifndef MIN
#define MIN(a,b) ((a) < (b) ? (a) : (b)) /*!< Define a handy 'MIN' macro */
#endif

/* MAX */
#ifndef MAX
#define MAX(a,b) ((a) > (b) ? (a) : (b)) /*!< Define a handy 'MAX' macro */
#endif


/*
 * Bit-Banding Macros for atomic RAM/MMR bit operations.
 * Compute the incoming RAM/MMR address and bit position
 * bit-banded address alias and set/clear the bit.
 */
#define RAM_ADR_BASE  0x20000000                                                                                      /*!< BitBanded address translation macro */
#define RAM_BB_ALIAS  0x22000000                                                                                      /*!< BitBanded address translation macro */
#define MMR_ADR_BASE  0x40000000                                                                                      /*!< BitBanded address translation macro */
#define MMR_BB_ALIAS  0x42000000                                                                                      /*!< BitBanded address translation macro */
#define MMR_BB_SET(a,b) ( *(volatile uint32_t *)(MMR_BB_ALIAS | (((uint32_t)a - MMR_ADR_BASE) << 5) | (b << 2)) = 1 ) /*!< BitBanded address translation macro */
#define MMR_BB_CLR(a,b) ( *(volatile uint32_t *)(MMR_BB_ALIAS | (((uint32_t)a - MMR_ADR_BASE) << 5) | (b << 2)) = 0 ) /*!< BitBanded address translation macro */
#define RAM_BB_SET(a,b) ( *(volatile uint32_t *)(RAM_BB_ALIAS | (((uint32_t)a - RAM_ADR_BASE) << 5) | (b << 2)) = 1 ) /*!< BitBanded address translation macro */
#define RAM_BB_CLR(a,b) ( *(volatile uint32_t *)(RAM_BB_ALIAS | (((uint32_t)a - RAM_ADR_BASE) << 5) | (b << 2)) = 0 ) /*!< BitBanded address translation macro */

/******************************************************************************
 * ADuM350 Device Specific Peripheral registers structures
 *****************************************************************************/
#if defined ( __CC_ARM   )
#pragma anon_unions
#endif


/******************************************************************************
 *  Access Modifiers used in the device templates following:                  *
 *    __IO = read/write register                                              *
 *    __I  = read-only register                                               *
 *    __O  = write-only register                                              *
 *****************************************************************************/


/* include CODA-generated device memory maps */
#include "CM40z_device.h"

/* missing CODA defines */
#define ENUM_SEC_SSTAT_SOVFERR               (_ADI_MSK(0x00000000,uint32_t))  /* ERRC: Source Overflow Error */
#define ENUM_SEC_SSTAT_ENDERR                (_ADI_MSK(0x00000020,uint32_t))  /* ERRC: End Error */
#define ENUM_SEC_CSTAT_ACKERR                (_ADI_MSK(0x00000010,uint32_t))  /* ERRC: Acknowledge Error */

/* C++ linkage */
#ifdef __cplusplus
}
#endif

#endif /* include guard */

/*
** EOF
*/

/*@}*/
