/*********************************************************************************

Copyright(c) 2010 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*!
* @file      adi_twi_common.h
*
* @brief     BF50x specific TWI driver header file
*
* @details
*            This is the BF50x specific header file, which contains
*            data and constant definitions used in the APIs
*/

#ifndef __ADI_TWI_COMMON_H__
#define __ADI_TWI_COMMON_H__

#ifndef  __ADI_TWI_H__
#error "This file may not be directly included. The file <drivers/twi/adi_twi.h> should be included instead."
#endif

#define NUM_TWI_DEVICES (1u) /* number of TWI devices */

/* field masks of the TWI_CLKDIV register */
#define BITM_TWI_CLKDIV_CLKHI    (0xFF00)    /* CLK HI */
#define BITM_TWI_CLKDIV_CLKLO    (0x00FF)    /* CLK LO */
#define BITP_TWI_CLKDIV_CLKHI    (0x8)       /* CLK HI position */

/* field masks of the TWI_CTL register */
#define BITM_TWI_CTL_SCCB        (SCCB)      /* SCCB Compatibility */
#define BITM_TWI_CTL_EN          (TWI_ENA)   /* TWI enable */
#define BITM_TWI_CTL_PRESCALE    (PRESCALE)  /* SCLK Prescale Value */

/* field masks of the TWI_SLVCTL register */
#define BITM_TWI_SLVCTL_GEN      (GEN)       /* General Call Enable */
#define BITM_TWI_SLVCTL_NAK      (NAK)       /* Not Acknowledge */
#define BITM_TWI_SLVCTL_TDVAL    (STDVAL)    /* Slave Transmit Data Valid */
#define BITM_TWI_SLVCTL_EN       (SEN)       /* Slave Enable */

/* field masks of the TWI_SLVSTAT register */
#define BITM_TWI_SLVSTAT_GCALL   (GCALL)     /* General Call */
#define BITM_TWI_SLVSTAT_DIR     (SDIR)      /* Slave Transfer Direction */

/* field masks of the TWI_SLVADDR register */
#define BITM_TWI_SLVADDR_ADDR    (0x7F)    /* Slave Mode Address */

/* field masks of the TWI_MSTRCTL register */
#define BITM_TWI_MSTRCTL_SCLOVR  (SCLOVR)    /* Serial Clock Override */
#define BITM_TWI_MSTRCTL_SDAOVR  (SDAOVR)    /* Serial Data Override */
#define BITM_TWI_MSTRCTL_DCNT    (DCNT)      /* Data Transfer Count */
#define BITM_TWI_MSTRCTL_RSTART  (RSTART)    /* Repeat Start */
#define BITM_TWI_MSTRCTL_STOP    (STOP)      /* Issue Stop Condition */
#define BITM_TWI_MSTRCTL_FAST    (FAST)      /* Fast Mode */
#define BITM_TWI_MSTRCTL_DIR     (MDIR)      /* Master Transfer Direction */
#define BITM_TWI_MSTRCTL_EN      (MEN)       /* Master Mode Enable */
#define BITP_TWI_MSTRCTL_DCNT    (0x6)       /* Data Transfer Count */

/* field masks of the TWI_MSTRSTAT register */
#define BITM_TWI_MSTRSTAT_BUSBUSY   (BUSBUSY)   /* Bus Busy */
#define BITM_TWI_MSTRSTAT_SCLSEN    (SCLSEN)    /* Serial Clock Sense */
#define BITM_TWI_MSTRSTAT_SDASEN    (SDASEN)    /* Serial Data Sense */
#define BITM_TWI_MSTRSTAT_BUFWRERR  (BUFWRERR)  /* Buffer Write Error */
#define BITM_TWI_MSTRSTAT_BUFRDERR  (BUFRDERR)  /* Buffer Read Error */
#define BITM_TWI_MSTRSTAT_DNAK      (DNAK)      /* Data Not Acknowledged */
#define BITM_TWI_MSTRSTAT_ANAK      (ANAK)      /* Address Not Acknowledged */
#define BITM_TWI_MSTRSTAT_LOSTARB   (LOSTARB)   /* Lost Arbitration */
#define BITM_TWI_MSTRSTAT_MPROG     (MPROG)     /* Master Transfer in Progress */

/* field masks of the TWI_MSTRADDR register */
#define BITM_TWI_MSTRADDR_ADDR     (0x7F)    /* Master Mode Address */

/* field masks of the TWI_ISTAT register */
#define BITM_TWI_ISTAT_SCLI     (0)         /* Serial Clock Interrupt */
#define BITM_TWI_ISTAT_SDAI     (0)         /* Serial Data Interrupt */
#define BITM_TWI_ISTAT_RXSERV   (RCVSERV)   /* Receive FIFO Service */
#define BITM_TWI_ISTAT_TXSERV   (XMTSERV)   /* Transmit FIFO Service */
#define BITM_TWI_ISTAT_MERR     (MERR)      /* Master Transfer Error */
#define BITM_TWI_ISTAT_MCOMP    (MCOMP)     /* Master Transfer Complete */
#define BITM_TWI_ISTAT_SOVF     (SOVF)      /* Slave Overflow */
#define BITM_TWI_ISTAT_SERR     (SERR)      /* Slave Transfer Error */
#define BITM_TWI_ISTAT_SCOMP    (SCOMP)     /* Slave Transfer Complete */
#define BITM_TWI_ISTAT_SINIT    (SINIT)     /* Slave Transfer Initiated */

/* field masks of the TWI_IMSK register */
#define BITM_TWI_IMSK_SCLI     (0)          /* Serial Clock Interrupt Mask */
#define BITM_TWI_IMSK_SDAI     (0)          /* Serial Data Interrupt Mask */
#define BITM_TWI_IMSK_RXSERV   (0x80)       /* Receive FIFO Service Interrupt Mask */
#define BITM_TWI_IMSK_TXSERV   (0x40)       /* Transmit FIFO Service Interrupt Mask */
#define BITM_TWI_IMSK_MERR     (0x20)       /* Master Transfer Error Interrupt Mask */
#define BITM_TWI_IMSK_MCOMP    (0x10)       /* Master Transfer Complete Interrupt Mask */
#define BITM_TWI_IMSK_SOVF     (0x08)       /* Slave Overflow Interrupt Mask */
#define BITM_TWI_IMSK_SERR     (0x04)       /* Slave Transfer Error Interrupt Mask */
#define BITM_TWI_IMSK_SCOMP    (0x02)       /* Slave Transfer Complete Interrupt Mask */
#define BITM_TWI_IMSK_SINIT    (0x01)       /* Slave Transfer Initiated Interrupt Mask */

/* field masks of the TWI_FIFOCTL register */
#define BITM_TWI_FIFOCTL_RXILEN    (RCVINTLEN)   /* Receive Buffer Interrupt Length */
#define BITM_TWI_FIFOCTL_TXILEN    (XMTINTLEN)   /* Transmit Buffer Interrupt Length */
#define BITM_TWI_FIFOCTL_RXFLUSH   (RCVFLUSH)    /* Receive Buffer Flush */
#define BITM_TWI_FIFOCTL_TXFLUSH   (XMTFLUSH)    /* Transmit Buffer Flush */

/* field masks of the TWI_FIFOSTAT register */
#define BITM_TWI_FIFOSTAT_RXSTAT   (RCVSTAT)   /* Receive FIFO Status */
#define BITM_TWI_FIFOSTAT_TXSTAT   (XMTSTAT)   /* Transmit FIFO Status */
#define BITP_TWI_FIFOSTAT_RXSTAT   (0x2)       /* Receive FIFO Status position */


/* field masks of the TWI_TXDATA8 register */
#define BITM_TWI_TXDATA8_VALUE     (0xFF)   /* Transmit FIFO 8-Bit Data */

/* field masks of the TWI_RXDATA8 register */
#define BITM_TWI_RXDATA8_VALUE     (0xFF)   /* Receive FIFO 8-Bit Data */

#define pREG_TWI0_CLKDIV    (pTWI_CLKDIV)
#define pREG_TWI0_CTL       (pTWI_CONTROL)
#define pREG_TWI0_SLVCTL    (pTWI_SLAVE_CTL)
#define pREG_TWI0_SLVSTAT   (pTWI_SLAVE_STAT)
#define pREG_TWI0_SLVADDR   (pTWI_SLAVE_ADDR)
#define pREG_TWI0_MSTRCTL   (pTWI_MASTER_CTL)
#define pREG_TWI0_MSTRSTAT  (pTWI_MASTER_STAT)
#define pREG_TWI0_MSTRADDR  (pTWI_MASTER_ADDR)
#define pREG_TWI0_ISTAT     (pTWI_INT_STAT)
#define pREG_TWI0_IMSK      (pTWI_INT_MASK)
#define pREG_TWI0_FIFOCTL   (pTWI_FIFO_CTL)
#define pREG_TWI0_FIFOSTAT  (pTWI_FIFO_STAT)
#define pREG_TWI0_TXDATA8   (pTWI_XMT_DATA8)
#define pREG_TWI0_TXDATA16  (pTWI_XMT_DATA16)
#define pREG_TWI0_RXDATA8   (pTWI_RCV_DATA8)
#define pREG_TWI0_RXDATA16  (pTWI_RCV_DATA16)

#endif
