/** *****************************************************************************
 * \file:    drivers/usb/controller/device/adi_usbd_dev.h
 * \brief:   USB Device Mode Controller device driver definitions for all processors
 * \details: Header File for the USB Device Mode Controller driver API functions and definitions
 * \version: $Revision:
 * \date:    $Date:
 *
 * \details: This is the public header file for the Mentor Graphics MUSBMHDRC USB controller driver.
 *           It contains the API declarations, data and constant definitions used in the APIs.
 *
 *****************************************************************************

Copyright(c) 2012 Analog Devices, Inc. All Rights Reserved.
This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/** @addtogroup USBD_Driver_BF6xx USBD Driver
 *  @{
 */

#ifndef __ADI_USBD_DEV_H__
#define __ADI_USBD_DEV_H__


/*
 ************************************************************************************************************
 *      MISRA EXCEPTIONS
 ************************************************************************************************************
*/
#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_1:"Assembly language shall be encapsulated and isolated.")

#pragma diag(suppress:misra_rule_5_1:"")
#pragma diag(suppress:misra_rule_5_6:"")
#pragma diag(suppress:misra_rule_5_7:"")

#pragma diag(suppress:misra_rule_10_1_a:"The value of an expression of integer type shall not be implicitly converted to a different underlying type if it is not a conversion to a wider integer type of the same signedness.")
#pragma diag(suppress:misra_rule_10_3:"The value of a complex expression of integer type shall only be cast to a type of the same signedness that is no wider than the underlying type of the expression.")

#pragma diag(suppress:misra_rule_6_3:"")

#pragma diag(suppress:misra_rule_8_5:"There shall be no definitions of objects or functions in a header file.")

#pragma diag(suppress:misra_rule_11_3:"")
#pragma diag(suppress:misra_rule_11_4:"")

#pragma diag(suppress:misra_rule_12_1:"")
#pragma diag(suppress:misra_rule_12_6:"")
#pragma diag(suppress:misra_rule_12_7:"Bitwise operators shall not be applied to operands whose underlying type is signed")

#pragma diag(suppress:misra_rule_13_2:"")
#pragma diag(suppress:misra_rule_13_7:"Allow to add asserts for memory size macros")

#pragma diag(suppress:misra_rule_14_7:"A function shall have a single point of exit at the end of the function.")
#pragma diag(suppress:misra_rule_16_7:"")

#pragma diag(suppress:misra_rule_17_4:"Array indexing shall be the only allowed form of pointer arithmetic.")

#pragma diag(suppress:misra_rule_18_4:"Unions shall not be used.")

#pragma diag(suppress:misra_rule_19_4:"C macros shall only expand to a braced initialiser, a constant, a string literal, a parenthesised expression, a type qualifier, a storage class specifier, or a do-while-zero construct.")
#pragma diag(suppress:misra_rule_19_7:"")
#endif /* _MISRA_RULES */

/*
 ************************************************************************************************************
 *      MACROS
 ************************************************************************************************************
*/

/* Selection macros */
/** \def ADI_USBD_DEV_NO
 * Provides boolean (0) value for use with configuration macros.
 */
#define ADI_USBD_DEV_NO  (0)

/** \def ADI_USBD_DEV_YES
 * Provides boolean (1) value for use with configuration macros.
 */
#define ADI_USBD_DEV_YES (1)

/*
 ************************************************************************************************************
 *      INCLUDES
 ************************************************************************************************************
*/

/*
 * Analog Devices Inc Tool chain specific includes
 */
#ifdef __ECC__

#include <adi_types.h>
#include <string.h>

/*
 * IAR Tool chain specific includes
 */
#elif defined ( __ICCARM__ )
#define __ADSPARM_FAMILY__

#include <stdint.h>
#ifndef bool
typedef int8_t bool;
#endif 

#ifndef false
#define false (0)
#endif
#ifndef true
#define true (1)
#endif

#endif

#if defined(__ADSPBLACKFIN__)
#include <sys/platform.h>
#include <sys/anomaly_macros_rtl.h>
#include <ccblkfn.h>
#elif defined(__ADSPARM_FAMILY__)

#include <device.h>

#if defined(ADI_ADUCM350)
#include <cdefADuCM350.h>

#endif
#endif

#if defined(ADI_ADSP_CM40Z)
#define RELOCATE_IVT
#endif

#include <drivers/usb/controller/device/config/adi_usbd_dev_cfg.h>

/* Set macro for device driver implementation
*/

#define ADI_USBD_DEV_BF_MUSBHDRC    (1)
#define ADI_USBD_DEV_BF_MUSBMHDRC   (2)
#define ADI_USBD_DEV_ARM_MUSBMHDRC  (3)
#define ADI_USBD_DEV_BF_NET2272     (4)
#define ADI_USBD_DEV_SH_NET2272     (5)

/* *************************************************************************************************
 * BF52x/BF54x MUSBHDRC implementation
 */
#if defined(__ADSPBF527_FAMILY__) || defined(__ADSPBF526_FAMILY__)  || defined(__ADSPBF548_FAMILY__)

#define ADI_USBD_DEV_DRVR (ADI_USBD_DEV_BF_MUSBHDRC)
#include <drivers/usb/controller/device/musbhdrc/adi_usbd_dev_musbhdrc.h>

/* *************************************************************************************************
 * BF609 MUSBMHDRC implementation
 */
#elif defined(__ADSPBF609_FAMILY__)

#define ADI_USBD_DEV_DRVR (ADI_USBD_DEV_BF_MUSBMHDRC)
#include <drivers/usb/controller/device/musbmhdrc/adi_usbd_dev_musbmhdrc.h>

/* *************************************************************************************************
 * ARM MUSBMHDRC implementation
 */
#elif defined(__ADSPARM_FAMILY__)

#include <device.h>
#define ADI_USBD_DEV_DRVR (ADI_USBD_DEV_ARM_MUSBMHDRC)

/* include name mapping macros to resolve CODA name changes */
#include "adi_resolve_coda_changes.h"

#include <drivers/usb/controller/device/musbmhdrc/adi_usbd_dev_musbmhdrc.h>

/* *************************************************************************************************
 * BLACKFIN NET2272 implementation
 */
#elif defined(__ADSPBF533_FAMILY__) || defined(__ADSPBF537_FAMILY__) || defined(__ADSPBF538_FAMILY__) || defined(__ADSPBF518_FAMILY__) || defined(__ADSPBF561_FAMILY__)

#define ADI_USBD_DEV_DRVR (ADI_USBD_DEV_BF_NET2272)
#ifdef BF_USBLAN_EXTENDER
#include <drivers/usb/controller/device/net2272/adi_usbd_dev_net2272.h>
#else
#warning "Blackfin USB-LAN EXTENDER BSP required."
#endif

/* *************************************************************************************************
 * SHARC NET2272 implementation
 */
#elif defined(__2146x__)

#define ADI_USBD_DEV_DRVR (ADI_USBD_DEV_SH_NET2272)
#ifdef SH_USB_EXTENDER
#include <adi_usbd_dev_net2272.h>
#else
#warning "SHARC USB EXTENDER BSP required."
#endif

#endif



#ifdef __ECC__
#pragma pack(4)
#endif
/** Endpoint structure for basic information */
typedef struct {
    bool      bInUse;       /*!< Boolean flag: true if the EP is in use; false if not.  */
    uint8_t   EpAddr;       /*!< Logical EP address                                     */
    void      *pClient;     /*!< Client handle (internal use)                           */
} ADI_USBD_DEV_EP_DEF;

/** Endpoint table structure */
typedef struct {
    uint8_t                 NumberEntries;  /*!< Number of endpoints in table.          */
    ADI_USBD_DEV_EP_DEF     **pTable;       /*!< Array of pointers to endpoint entries. */
} ADI_USBD_DEV_EP_TABLE_DEF;

/** Enumeration for the Endpoint types  */
typedef enum {
    ADI_USBD_DEV_EP_TYPE_INVALID=-1,    /*!< Invalid Endpoint       */
    ADI_USBD_DEV_EP_TYPE_CTRL=0,        /*!< Control Endpoint       */
    ADI_USBD_DEV_EP_TYPE_ISOC=1,        /*!< Isochronous Endpoint   */
    ADI_USBD_DEV_EP_TYPE_BULK=2,        /*!< Bulk Endpoint          */
    ADI_USBD_DEV_EP_TYPE_INTR=3         /*!< Interrupt Endpoint     */
} ADI_USBD_DEV_EP_TYPE;

/** Enumeration for the Endpoint Direction */
typedef enum {
    ADI_USBD_DEV_EP_DIR_TX,         /*!< Device to Host (TX).                   */
    ADI_USBD_DEV_EP_DIR_RX,         /*!< Host to Device (RX).                   */
    ADI_USBD_DEV_EP_DIR_EP0         /*!< Control transfer, either direction.    */

} ADI_USBD_DEV_EP_DIR;


/** Enumeration for the Bus Speed */
typedef enum {
    ADI_USBD_DEV_SPEED_FS,      /*!< Full Speed Device (USB 1.1)            */
    ADI_USBD_DEV_SPEED_HS       /*!< High Speed Device (USB 2.0)            */
}ADI_USBD_DEV_SPEED;


/** Enumeration for Result Codes */
enum {
    ADI_USBD_DEV_RESULT_SUCCESS = 0,                /*!< Successful completion              */
    ADI_USBD_DEV_RESULT_FAIL,                       /*!< Generic failure */
    ADI_USBD_DEV_RESULT_NO_MEMORY,                  /*!< Insufficient memory for operation  */
    ADI_USBD_DEV_RESULT_INVALID_EP,                 /*!< Invalid endpoint                   */
    ADI_USBD_DEV_RESULT_NO_BUFFER,                  /*!< No buffer available                */
    ADI_USBD_DEV_RESULT_NO_DMA,                     /*!< No DMA channel available           */
    ADI_USBD_DEV_RESULT_DEVICE_NOT_OPEN         /*!< Device driver not opened           */
};


/** Enumeration for Events */
enum {
    ADI_USBD_DEV_EVENT_NONE = 0,                    /*!< No event                               */
    ADI_USBD_DEV_EVENT_SETUP,                       /*!< Setup Request received                 */
    ADI_USBD_DEV_EVENT_RX,                          /*!< RX transfer complete                   */
    ADI_USBD_DEV_EVENT_TX,                          /*!< TX transfer complete                   */
    ADI_USBD_DEV_EVENT_BUS_CONN,                    /*!< Bus connect event                      */
    ADI_USBD_DEV_EVENT_BUS_DISCONN,                 /*!< Bus Disconnect event                   */
    ADI_USBD_DEV_EVENT_BUS_SUSPEND,                 /*!< Bus suspend event                      */
    ADI_USBD_DEV_EVENT_BUS_RESUME,                  /*!< Bus resume event                       */
    ADI_USBD_DEV_EVENT_BUS_RESET,                   /*!< Bus reset event                        */
    ADI_USBD_DEV_EVENT_BUS_HS_DETECTED,             /*!< High Speed detection event             */
    ADI_USB_EVENT_PKT_RCVD_NO_BUFFER                /*!< Packet received with no buffer queued  */
};

/** Hardware information structure */
typedef struct {
    uint8_t HWVERS_RC;                              /*!< Release candidate flag (1 if RC, 0 if not).    */
    uint8_t HWVERS_Major;                           /*!< major revision number of controller            */
    uint8_t HWVERS_Minor;                           /*!< minor revision number of controller.           */
    uint8_t NumRxEndpoints;                         /*!< Maximum number of RX endpoints.                */
    uint8_t NumTxEndpoints;                         /*!< Maximum number of TX endpoints.                */
    uint8_t NumDMAChannels;                         /*!< Number of available DMA channels.              */
    uint8_t RamAddressWidth;                        /*!< RAM address width                              */
} ADI_USBD_DEV_MUSBMHDRC_HWINFO;

/** Endpoint buffer table entry structure */
typedef struct _ADI_USBD_DEV_EP_BUFFER {
    bool                            bInUse;             /*!< Buffer in use flag: true if in use; false if not.  */
    void                            *pData;             /*!< Start address of data buffer.                      */
    uint32_t                        nTransferSize;      /*!< Number of bytes to transfer.                       */
    uint32_t                        nBytesProcessed;    /*!< Number of bytes processed.                         */
    uint32_t                        nBytesThisPass;     /*!< Number of bytes transfered in current pass.        */
    uint32_t                        nBytesRemaining;    /*!< Number of bytes remaining.                         */
    struct _ADI_USBD_DEV_EP_BUFFER   *pNext;

} ADI_USBD_DEV_EP_BUFFER;

/** Buffer table structure */
typedef struct {
    uint8_t                 nMaxBuffers;            /*!< Maximum number of buffers for each endpoint*/
    ADI_USBD_DEV_EP_BUFFER  *pNextFreeBuffer;       /*!< Pointer to next free buffer to which adi_usbd_dev_EpQueueBuffer can submit data. */
    ADI_USBD_DEV_EP_BUFFER  *pNextQueued;           /*!< Pointer to next queued buffer*/
    ADI_USBD_DEV_EP_BUFFER  **ppList;               /*!< Array of pointers to buffer entries, ppList[i=0,nMaxBuffers]. */
} ADI_USBD_DEV_EP_BUFFER_LIST;

/** Callback Function typedef */
typedef void (*ADI_USBD_DEV_CB_FN)(void *, uint32_t, void *);   /*!< arguments: client handle, event, pointer */

/** opaque data handle */
typedef void *ADI_USBD_DEV_HANDLE;

/** Result code typedef */
typedef uint32_t ADI_USBD_DEV_RESULT;

/*
 ************************************************************************************************************
 *              PUBLIC API
 ************************************************************************************************************
*/
/** Public API Entrypoints' Structure */
typedef struct {
   ADI_USBD_DEV_RESULT (*Open)(                                                                         /*!< @see ADI_USBD_DEV_Open.            */
                                        uint32_t                   DeviceNumber,
                                        void *const                pDeviceMemory,
                                        uint32_t                   nDeviceMemorySize,
                                        uint8_t                    nEndpointsInUse,
                                        ADI_USBD_DEV_CB_FN         pfBusCallback,
                                        void                       *CallbackHandle,
                                        ADI_USBD_DEV_HANDLE        *phDevice
                            );

   ADI_USBD_DEV_RESULT (*Close)(                                                                        /*!< @see ADI_USBD_DEV_Close.           */
                                        ADI_USBD_DEV_HANDLE        hDevice
                            );

   ADI_USBD_DEV_RESULT (*EnableUSB)(                                                                    /*!< @see ADI_USBD_DEV_EnableUSB.       */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        bool                       bEnable
                                );

   ADI_USBD_DEV_RESULT (*Connect)(                                                                      /*!< @see ADI_USBD_DEV_Connect.         */
                                        ADI_USBD_DEV_HANDLE        hDevice
                              );

   ADI_USBD_DEV_RESULT (*Disconnect)(                                                                   /*!< @see ADI_USBD_DEV_Disconnect.      */
                                        ADI_USBD_DEV_HANDLE        hDevice
                                );

/* Control Functions */
   ADI_USBD_DEV_RESULT (*SetAddress)(                                                                   /*!<@see ADI_USBD_DEV_SetAddress.       */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        uint8_t                    DevAddr
                                );

   ADI_USBD_DEV_RESULT (*SetSpeed)(                                                                     /*!< @see ADI_USBD_DEV_SetSpeed.        */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        ADI_USBD_DEV_SPEED         Speed
                               );

   ADI_USBD_DEV_RESULT (*GetSpeed)(                                                                     /*!< @see ADI_USBD_DEV_GetSpeed.        */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        ADI_USBD_DEV_SPEED         *pSpeed
                               );

   ADI_USBD_DEV_RESULT (*GetFrameNumber)(                                                               /*!< @see ADI_USBD_DEV_GetFrameNumber.  */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        uint16_t                   *pnFrameNumber
                                     );

/* Endpoint Functions */
   ADI_USBD_DEV_RESULT (*BindEndpoint) (                                                                /*!< @see ADI_USBD_DEV_BindEndpoint.    */
                                        ADI_USBD_DEV_HANDLE             hDevice,
                                        uint8_t                         EpAddr,
                                        uint8_t                         EpType,
                                        uint16_t                        nMaxPktSize,
                                        uint8_t                         nHighCapacityMultiplier,
                                        ADI_USBD_DEV_EP_DEF **          ppEndpoint
                                    );

   ADI_USBD_DEV_RESULT (*ReleaseEndpoint)  (                                                            /*!< @see ADI_USBD_DEV_ReleaseEndpoint. */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        uint8_t     EpAddress
                                        );

   ADI_USBD_DEV_RESULT (*EpRegisterCallback)(                                                           /*!< @see ADI_USBD_DEV_EpRegisterCallback.  */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        uint8_t     EpAddress,
                                        ADI_USBD_DEV_CB_FN         pfCallback,
                                        void                       *pCBParam
                                        );

   ADI_USBD_DEV_RESULT (*EpSubmitRxBuffer)(                                                             /*!< @see ADI_USBD_DEV_EpSubmitRxBuffer.    */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        uint8_t     EpAddress,
                                        void                       *pBuffer,
                                        uint32_t                   nBufSize
                                        );

   ADI_USBD_DEV_RESULT (*EpGetRxBuffer)(                                                                /*!< @see ADI_USBD_DEV_EpGetRxBuffer.       */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        uint8_t     EpAddress,
                                        void ** const              ppBuffer,
                                        uint32_t                   *pnBytes
                                    );

   ADI_USBD_DEV_RESULT (*IsRxBufferAvailable)(                                                          /*!< @see ADI_USBD_DEV_IsRxBufferAvailable. */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        uint8_t     EpAddress,
                                        bool * const               pbAvailable
                                        );

   ADI_USBD_DEV_RESULT (*EpRead)(                                                                       /*!< @see ADI_USBD_DEV_EpRead.              */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        uint8_t     EpAddress,
                                        void                       *pBuffer,
                                        uint32_t                   nBufSize
                                        );

   ADI_USBD_DEV_RESULT (*EpReadZLP)(                                                                    /*!< @see ADI_USBD_DEV_EpReadZLP.           */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        uint8_t     EpAddress
                                );

   ADI_USBD_DEV_RESULT (*EpSubmitTxBuffer)(                                                             /*!< @see ADI_USBD_DEV_EpSubmitTxBuffer.    */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        uint8_t     EpAddress,
                                        void                       *pBuffer,
                                        uint32_t                   nBufSize
                                        );

   ADI_USBD_DEV_RESULT (*EpGetTxBuffer)(                                                                /*!< @see ADI_USBD_DEV_EpGetTxBuffer.       */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        uint8_t     EpAddress,
                                        void ** const              ppBuffer,
                                        uint32_t                   *pnBytes
                                    );

   ADI_USBD_DEV_RESULT (*IsTxBufferAvailable)(                                                          /*!< @see ADI_USBD_DEV_IsTxBufferAvailable. */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        uint8_t     EpAddress,
                                        bool * const               pbAvailable
                                        );

   ADI_USBD_DEV_RESULT (*EpWrite)(                                                                      /*!< @see ADI_USBD_DEV_EpWrite.             */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        uint8_t     EpAddress,
                                        void                       *pBuffer,
                                        uint32_t                   nBufSize
                                        );

   ADI_USBD_DEV_RESULT (*EpWriteZLP)(                                                                   /*!< @see ADI_USBD_DEV_EpWriteZLP.          */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        uint8_t     EpAddress
                                        );

   ADI_USBD_DEV_RESULT (*EpSetStall)(                                                                   /*!< @see ADI_USBD_DEV_EpSetStall.          */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        uint8_t     EpAddress
                                        );

   ADI_USBD_DEV_RESULT (*EpClearStall)(                                                                 /*!< @see ADI_USBD_DEV_EpClearStall.            */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        uint8_t     EpAddress
                                    );

   ADI_USBD_DEV_RESULT (*EpAbortTransfer)(                                                              /*!< @see ADI_USBD_DEV_EpAbortTransfer.         */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        uint8_t     EpAddress
                                        );

   ADI_USBD_DEV_RESULT (*EpGetMaxRxTransferSize)(                                                       /*!< @see ADI_USBD_DEV_EpGetMaxRxTransferSize.  */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        uint8_t                    EpAddress,
                                        uint8_t                    *pData,
                                        uint32_t                   *pMaxTransferSize
                                        );

   ADI_USBD_DEV_RESULT (*EpGetMaxTxTransferSize)(                                                       /*!< @see ADI_USBD_DEV_EpGetMaxTxTransferSize.  */
                                        ADI_USBD_DEV_HANDLE        hDevice,
                                        uint8_t                    EpAddress,
                                        uint8_t                    *pData,
                                        uint32_t                   *pMaxTransferSize
                                        );

   ADI_USBD_DEV_RESULT (*SetCallbackHandle) (                                                           /*!< @see ADI_USBD_DEV_SetCallbackHandle. */
                                        ADI_USBD_DEV_HANDLE hDevice,
                                        void *CallbackHandle
                            );

   ADI_USBD_DEV_RESULT (*SetConfig) (                                                           /*!< @see ADI_USBD_DEV_SetConfig. */
                                        ADI_USBD_DEV_HANDLE hDevice,
                                        uint8_t ConfigID
                            );

   ADI_USBD_DEV_RESULT (*ClearConfig) (                                                         /*!< @see ADI_USBD_DEV_ClearConfig. */
                                        ADI_USBD_DEV_HANDLE hDevice,
                                        uint8_t ConfigID
                            );


} ADI_USBD_DEV_API;

/*
 ************************************************************************************************************
 *      DEVICE DRIVER API ENTRY POINTS
 ************************************************************************************************************
*/
/* define for non controller module sources only
*/
#ifndef _ADI_USBD_DEV_MODULE_

#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC) || (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC)
extern ADI_USBD_DEV_API ADI_USBD_DEV_BF_MUSBMHDRC_API;
static const ADI_USBD_DEV_API *pADI_USBD_Drv = &ADI_USBD_DEV_BF_MUSBMHDRC_API;

#elif (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC)
extern ADI_USBD_DEV_API ADI_USBD_DEV_ARM_MUSBMHDRC_API;
static const ADI_USBD_DEV_API *pADI_USBD_Drv = &ADI_USBD_DEV_ARM_MUSBMHDRC_API;

#elif (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_NET2272)
extern ADI_USBD_DEV_API ADI_USBD_DEV_BF_NET2272_API;
static const ADI_USBD_DEV_API *pADI_USBD_Drv = &ADI_USBD_DEV_BF_NET2272_API;

#elif (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_SH_NET2272)
extern ADI_USBD_DEV_API ADI_USBD_DEV_SH_NET2272_API;
static const ADI_USBD_DEV_API *pADI_USBD_Drv = &ADI_USBD_DEV_SH_NET2272_API;

#else
#error "USB Controller not supported on this processor"
#endif

#endif

/*
 ************************************************************************************************************
 ************************************************************************************************************
 *              C O M M O N  F U N C T I O N S
 ************************************************************************************************************
 ************************************************************************************************************
*/
/*
 ************************************************************************************************************
 *              ENDPOINT TABLE MANAGEMENT
 ************************************************************************************************************
*/

void adi_usbd_dev_EpCreateTable (
        uint8_t                     NumberEntries,
        void                        *pDataMemory,
        ADI_USBD_DEV_EP_TABLE_DEF   **ppEpTableDef,
        uint32_t                    EntrySize

);

ADI_USBD_DEV_EP_DEF *adi_usbd_dev_EpAddEntry(
        ADI_USBD_DEV_EP_TABLE_DEF *pEpTable
);

void adi_usbd_dev_EpRemoveEntry(
        ADI_USBD_DEV_EP_TABLE_DEF   *pEpTable,
        uint8_t                     EpAddr
);
ADI_USBD_DEV_EP_DEF *adi_usbd_dev_EpGetEntry(
        ADI_USBD_DEV_EP_TABLE_DEF   *pEpTable,
        uint8_t                     EpAddr
);


void adi_usbd_dev_EpCreateBufferList (
        uint8_t                         NumberEntries,
        void                            *pDataMemory,
        ADI_USBD_DEV_EP_BUFFER_LIST     **ppEpBufferDef
);

void adi_usbd_dev_EpResetBufferList(
                    ADI_USBD_DEV_EP_BUFFER_LIST *pBufferList
);

ADI_USBD_DEV_EP_BUFFER* adi_usbd_dev_EpQueueBuffer(
            ADI_USBD_DEV_EP_BUFFER_LIST     *pBufferList,
            void                            *pData,
            uint32_t                        nTransferSize
);

void adi_usbd_dev_EpFreeBuffer(
                                 ADI_USBD_DEV_EP_BUFFER_LIST      *pBufferList,
                                 ADI_USBD_DEV_EP_BUFFER          *pBuffer
);

ADI_USBD_DEV_EP_BUFFER *adi_usbd_dev_EpGetQueuedBuffer(
                                 ADI_USBD_DEV_EP_BUFFER_LIST      *pBufferList
);


#if defined(__ADSPBLACKFIN__) && (ADI_USBD_DEV_USE_DMA == ADI_USBD_DEV_YES)
void adi_usbd_dev_FlushCachedBuffer(
                ADI_USBD_DEV_EP_DIR      Direction,
                void                     *pData,
                uint32_t                 DataLen
);

#endif

/* Miscellaneous API functions */
bool adi_usbd_dev_IsDataCacheEnabled(void);
void adi_usbd_dev_musbmhdrc_GetHardwareConfig(ADI_USBD_DEV_MUSBMHDRC_HWINFO *pHwInfo);

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

/*@}*/

#endif /* __ADI_USBD_DEV_H__ */
