/*********************************************************************************

Copyright(c) 2013 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential. By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_usbd_dev_os.h ($Rev: 21484 $)
*
* @brief     USB Device Driver (MUSBMHDRC) RTOS interface header file.
*
* @details
*            This is the RTOS interface header file for the device driver
*            for the Mentor Graphics MUSBMHDRC Controller.
*/

#ifndef __ADI_USBD_DEV_OS_H__
#define __ADI_USBD_DEV_OS_H__

#include <drivers/usb/controller/device/adi_usbd_dev.h>

#if defined(__ADSPBF609_FAMILY__)

#include <services/int/adi_sec.h>
#include <services/int/adi_int.h>

#elif defined(__ADSPBF526_FAMILY__) || defined(__ADSPBF527_FAMILY__) || defined(__ADSPBF548_FAMILY__)

#include <services/int/adi_int.h>

#define INTR_USB0_STAT (ADI_SID_USB_INT0)
#define INTR_USB0_DATA (ADI_SID_USB_DMAINT)

#elif defined(__ADSPARM_FAMILY__)

#if defined(ADI_ADUCM350)

#if defined(RELOCATE_IVT)

#include <adi_int.h>
#include <adi_sid_cm350.h>

#define INTR_USB0_STAT	(ADI_INT_USB)
#define INTR_USB0_DATA	(ADI_INT_USB_DMA)

#else

#include <adi_nvic.h>
#define INTR_USB0_STAT	(37)
#define INTR_USB0_DATA	(38)

#endif /* RELOCATE_IVT */

#endif /* ADI_ADUCM350 */


#endif /* __ADSPARM_FAMILY__ */
#if defined(ADI_ADUCM350) && !defined(RELOCATE_IVT)


void _adi_usbd_dev_os_init(ADI_USBD_DEV_HANDLE pMUSBMHDRC);
void _adi_usbd_dev_os_EnableInt( uint32_t iid, bool bEnable);
void _adi_usbd_dev_os_isr_begin(void);
void _adi_usbd_dev_os_isr_end(void);

#endif

/*
 * Critical regions
*/
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC) || (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC) || ( defined(__ADSPARM_FAMILY__) && defined(RELOCATE_IVT))

#define ADI_USBD_ENTER_CRITICAL()  do {                                             \
	                                  adi_int_EnableInt(INTR_USB0_STAT, false); \
	                                  adi_int_EnableInt(INTR_USB0_DATA, false); \
                                      } while(0)

#elif defined(ADI_ADUCM350) && !defined(RELOCATE_IVT)

#define ADI_USBD_ENTER_CRITICAL()  do {                                             \
                                      adi_nvic_EnableInt(INTR_USB0_STAT, false); \
	                                  adi_nvic_EnableInt(INTR_USB0_DATA, false); \
                                      } while(0)

#endif

#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC) || (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC) || ( defined(__ADSPARM_FAMILY__) && defined(RELOCATE_IVT))

#define ADI_USBD_EXIT_CRITICAL()   do {                                            \
	                                  adi_int_EnableInt(INTR_USB0_STAT, true); \
	                                  adi_int_EnableInt(INTR_USB0_DATA, true); \
	                              } while(0)

#elif defined(__ADSPARM_FAMILY__) && !defined(RELOCATE_IVT)

#define ADI_USBD_EXIT_CRITICAL()   do {                                            \
	                                  adi_nvic_EnableInt(INTR_USB0_STAT, true); \
	                                  adi_nvic_EnableInt(INTR_USB0_DATA, true); \
	                              } while(0)

#endif




#endif /* __ADI_USBD_DEV_OS_H__ */
