/*********************************************************************************

Copyright(c) 2012 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/**
* @file      adi_usbd_dev_musbmhdrc_cfg.h
*
* @brief     USB Device Driver (MUSBMHDRC) Configuration header
*
* @details
*            This is a header file which defines the selective build macros
*            for the USB Device Driver (MUSBMHDRC).
*/

/** @addtogroup USBD_Driver_BF6xx USBD Driver
 *  @{
 */

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_8_5:"There shall be no definitions of objects or functions in a header file.")
#pragma diag(suppress:misra_rule_13_7:"Allow to add asserts for memory size macros")
#pragma diag(suppress:misra_rule_14_7:"A function shall have a single point of exit at the end of the function.")
#pragma diag(suppress:misra_rule_18_4:"Unions shall not be used.")
#pragma diag(suppress:misra_rule_19_4:"C macros shall only expand to a braced initialiser, a constant, a string literal, a parenthesised expression, a type qualifier, a storage class specifier, or a do-while-zero construct.")
#endif /* _MISRA_RULES */

#ifndef __ADI_USBD_DEV_MUSBMHDRC_CFG_H__
#define __ADI_USBD_DEV_MUSBMHDRC_CFG_H__

#include <drivers/usb/controller/device/adi_usbd_dev.h>
/** \def ADI_USBD_DEV_USE_PIO
 * When this macro is set to #ADI_USBD_DEV_YES Programmed I/O (PIO) will be used for all transfers and all DMA
 * source code will be removed from the build. The recommended value is #ADI_USBD_DEV_NO to maximize throughput
 * performance. (Defined in adi_usbd_dev_musbmhdrc_cfg.h).
 */
#define ADI_USBD_DEV_USE_PIO (ADI_USBD_DEV_NO)

#if (ADI_USBD_DEV_USE_PIO == ADI_USBD_DEV_NO)

#define ADI_USBD_DEV_USE_DMA (ADI_USBD_DEV_YES)

#else

#define ADI_USBD_DEV_USE_DMA (ADI_USBD_DEV_NO)

#endif

/*
 * The following should only be changed in standalone mode if a silicon anomlay demands it.
 * FOR MUSBMHDRC this should not be an issue as DMA Mode 1 should be functional.
 */

/** \def ADI_USBD_DEV_RX_DMA_MODE0
 * Enforces the DMA behavior for host to device (OUT) transfers.
 * Valid values are #ADI_USBD_DEV_YES to enforce single packet DMA mode or #ADI_USBD_DEV_NO to enforce multi-packet
 * DMA mode. ordinarily the controller is capable of multi-packet DMA and thus leaving the value set to #ADI_USBD_DEV_NO
 * is highly recommended to considerable improve throughput. (Defined in adi_usbd_dev_musbmhdrc_cfg.h).
 */
#define ADI_USBD_DEV_RX_DMA_MODE0   (ADI_USBD_DEV_YES)

/** \def ADI_USBD_DEV_TX_DMA_MODE0
 * Enforces the DMA behavior for device to host (IN) transfers.
 * Valid values are #ADI_USBD_DEV_YES to enforce single packet DMA mode or #ADI_USBD_DEV_NO to enforce multi-packet
 * DMA mode. Ordinarily the controller is capable of multi-packet DMA and thus leaving the value set to #ADI_USBD_DEV_NO
 * is highly recommended to considerable improve throughput. (Defined in adi_usbd_dev_musbmhdrc_cfg.h).
 */
#define ADI_USBD_DEV_TX_DMA_MODE0   (ADI_USBD_DEV_YES)

#define ADI_USBD_DEV_RX_SNGL_BUFFER (ADI_USBD_DEV_YES)

/** \def ADI_USBD_DEV_TX_SNGL_BUFFER
 * Enforces single buffer mode on the EP TX (IN) FIFO
 * Valid values are #ADI_USBD_DEV_YES to enforce single buffer mode or #ADI_USBD_DEV_NO to enforce double buffer
 * mode. Ordinarily the controller is capable of double buffer mode and thus leaving the value set to #ADI_USBD_DEV_NO
 * is highly recommended to considerable improve throughput. (Defined in adi_usbd_dev_musbmhdrc_cfg.h).
 */
#define ADI_USBD_DEV_TX_SNGL_BUFFER (ADI_USBD_DEV_YES)

/** \def ADI_USBD_DEV_HS_ENABLE
 * Enforces whether the device should operate in High Speed mode (USB 2.0)
 * Valid values are #ADI_USBD_DEV_YES to enforce High Speed or #ADI_USBD_DEV_NO enforce full speed mode. Ordinarily,
 * the ADSP-BF609 USB controller is capable of High Speed. (Defined in adi_usbd_dev_musbmhdrc_cfg.h).
 */
#define ADI_USBD_DEV_HS_ENABLE      (ADI_USBD_DEV_YES)

/** \def _ADI_USBD_ALGNMNT_CHECK_
 * Determines whether data buffers should be tested for 32 byte alignment. This is only relevant when processor
 * data cache is enabled; in such cases problems may occur when the buffers are not aligned with the data cache
 * lines. If the test is enabled (#ADI_USBD_DEV_YES) then non-compliant buffers will be handled using PIO mode.
 * (Defined in adi_usbd_dev_musbmhdrc_cfg.h).
 */
#define _ADI_USBD_ALGNMNT_CHECK_ (ADI_USBD_DEV_NO)

/*
 * Define the number of buffers to use for each endpoint
 * (a value of 2 complies with the ADI SSL & DD version 2.0 specification).
 */
/** \def  ADI_USBD_DEV_MAX_BUFS
 *  Defines the number of buffers to use for each endpoint. Applies to all Controller drivers.
 *  (Defined in adi_usbd_dev_cfg.h). A value of 2 is normally sufficient.
 * */
#define ADI_USBD_DEV_MAX_BUFS               2u

#ifdef __VDSP5__

#define IVG_USB_INT0    10
#define IVG_USB_DMAINT  11
#else
#define IVG_USB_INT0    10
#define IVG_USB_DMAINT  11

#endif
#endif /* __ADI_USBD_DEV_MUSBMHDRC_CFG_H__ */

/*@}*/

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

