
/** *****************************************************************************
 * \file:    drivers/usb/controller/device/adi_usbd_dev.h
 * \brief:   USB Device Mode Controller device driver definitions for all processors
 * \details: Header File for the USB Device Mode Controller driver API functions and definitions
 * \version: $Revision:
 * \date:    $Date:
 *****************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.
This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.
*********************************************************************************/

/** @addtogroup USBD_Driver_BF6xx ADSP-BF6xx USBD Driver
 *  @{
 */


/*
 ************************************************************************************************************
 *      MISRA EXCEPTIONS
 ************************************************************************************************************
*/
#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_8_5:"There shall be no definitions of objects or functions in a header file.")
#pragma diag(suppress:misra_rule_13_7:"Allow to add asserts for memory size macros")
#pragma diag(suppress:misra_rule_14_7:"A function shall have a single point of exit at the end of the function.")
#pragma diag(suppress:misra_rule_18_4:"Unions shall not be used.")
#pragma diag(suppress:misra_rule_19_4:"C macros shall only expand to a braced initialiser, a constant, a string literal, a parenthesised expression, a type qualifier, a storage class specifier, or a do-while-zero construct.")
#endif /* _MISRA_RULES */

#ifndef __ADI_USBD_DEV_MUSBMHDRC_H__
#define __ADI_USBD_DEV_MUSBMHDRC_H__

#if defined ( __ICCARM__ )
#define __ADSPARM_FAMILY__
#endif

#include <drivers/usb/controller/device/adi_usbd_dev.h>
#include <sys/platform.h>

#if defined(__ADSPBF609_FAMILY__)

#define ADI_USBD_DEV_MAX_DEVICES        (1)
#define ADI_USBD_DEV_MAX_REGS           (35)
#define ADI_USBD_DEV_MAX_EP             (12)
#define ADI_USBD_DEV_MAX_DMA_CHANNELS   (8)

#define ADI_USBD_DEV_DYN_FIFO               (ADI_USBD_DEV_YES)
#define ADI_USBD_DEV_DYN_PKT_SPLIT          (ADI_USBD_DEV_YES)
#define ADI_USBD_DEV_DYN_HI_BNDWDTH_ISO     (ADI_USBD_DEV_YES)
#define ADI_USBD_DEV_DBL_BUFFER_FIFO        (ADI_USBD_DEV_YES)


#elif defined(__ADSPARM_FAMILY__)

#if defined(ADI_ADSP_CM40Z) /* ADSP-CM40x family */
  #include <services/int/adi_int.h>
  #include <services/int/adi_sid_cm40z.h>
  #include <cdefCM40z.h>
  #define ADI_USBD_DEV_MAX_DEVICES        (1)
  #define ADI_USBD_DEV_MAX_REGS           (35)
  #define ADI_USBD_DEV_MAX_EP             (7) 
  #define ADI_USBD_DEV_MAX_DMA_CHANNELS   (7)

  #define ADI_USBD_DEV_DYN_FIFO               (ADI_USBD_DEV_NO)
  #define ADI_USBD_DEV_DYN_PKT_SPLIT          (ADI_USBD_DEV_NO)
  #define ADI_USBD_DEV_DYN_HI_BNDWDTH_ISO     (ADI_USBD_DEV_NO)
  #define ADI_USBD_DEV_DBL_BUFFER_FIFO        (ADI_USBD_DEV_YES)

#ifdef ADI_USBD_DEV_HS_ENABLE
#undef ADI_USBD_DEV_HS_ENABLE
#define ADI_USBD_DEV_HS_ENABLE              (ADI_USBD_DEV_NO)
#endif
#else /* ADuCM350 */

#define __ADI_ADUCM350__
#define ADI_USBD_DEV_MAX_DEVICES        (1)
#define ADI_USBD_DEV_MAX_REGS           (35)
#define ADI_USBD_DEV_MAX_EP             (4)
#define ADI_USBD_DEV_MAX_DMA_CHANNELS   (2)

#define ADI_USBD_DEV_DYN_FIFO               (ADI_USBD_DEV_NO)
#define ADI_USBD_DEV_DYN_PKT_SPLIT          (ADI_USBD_DEV_NO)
#define ADI_USBD_DEV_DYN_HI_BNDWDTH_ISO     (ADI_USBD_DEV_NO)
#define ADI_USBD_DEV_DBL_BUFFER_FIFO        (ADI_USBD_DEV_YES)

#ifdef ADI_USBD_DEV_HS_ENABLE
#undef ADI_USBD_DEV_HS_ENABLE
#define ADI_USBD_DEV_HS_ENABLE              (ADI_USBD_DEV_NO)
#endif

#endif /* ADI_ADSP_CM40Z */
#endif /* __ADSPBF609_FAMILY__ */

typedef uint16_t ADI_USBD_8BIT_REG_TYPE;
typedef uint16_t ADI_USBD_FIFO_REG_TYPE;

typedef struct {

   volatile uint8_t  *pFADDR;
   volatile uint8_t  *pPOWER;

   volatile uint8_t  *pDEV_CTL;
   volatile uint16_t *pVBUS_CTL;

   volatile uint16_t *pPLL_OSC;
   volatile uint8_t  *pPHY_CTL;

   volatile uint16_t *pINTRTX;
   volatile uint16_t *pINTRRX;
   volatile uint16_t *pINTRTXE;
   volatile uint16_t *pINTRRXE;
   volatile uint8_t  *pIRQ;
   volatile uint8_t  *pIEN;
   volatile uint16_t *pFRAME;
   volatile uint8_t  *pINDEX;
   volatile uint8_t  *pTXFIFOSZ;
   volatile uint8_t  *pRXFIFOSZ;

   volatile uint8_t  *pTESTMODE;

   /* EP indexed registers (used with INDEX = 1,2,....)*/
   volatile uint8_t  *pEPI_TXTYPE;
   volatile uint16_t *pEPI_TXMAXP;
   volatile uint8_t  *pEPI_TXINTERVAL;
   volatile uint16_t *pEPI_TXCSR;
   /* TXCNT is of no use */

   volatile uint8_t  *pEPI_RXTYPE;
   volatile uint16_t *pEPI_RXMAXP;
   volatile uint8_t  *pEPI_RXINTERVAL;
   volatile uint16_t *pEPI_RXCSR;
   volatile uint16_t *pEPI_RXCNT;

  /* EP0 registers (used with INDEX = 0)*/
   volatile uint8_t  *pEP0I_TYPE0;
   volatile uint16_t *pEP0_CSR0;
   volatile uint16_t *pEP0_CNT0;
   volatile uint8_t  *pEP0I_NAKLIMIT0;

   volatile uint8_t  *pEP0I_CFGDATA0;

   /* DMA channels */
   volatile uint8_t  *pDMA_IRQ;
   volatile uint16_t *pDMA_CTL[ADI_USBD_DEV_MAX_DMA_CHANNELS];
   volatile uint32_t *pDMA_ADDR[ADI_USBD_DEV_MAX_DMA_CHANNELS];
   volatile uint32_t *pDMA_CNT[ADI_USBD_DEV_MAX_DMA_CHANNELS];

   volatile uint32_t *pEP_FIFO[ADI_USBD_DEV_MAX_EP];

} ADI_USBD_DEV_REGS_DEF;

/* Union type to enable FIFO to accept dword, word and byte values
*/
typedef union {
    uint32_t dword;
    uint16_t word;
    uint8_t  byte;
} ADI_USBD_DEV_FIFO_ENTRY;


#if (ADI_USBD_DEV_USE_DMA == ADI_USBD_DEV_YES)
typedef struct {
    bool    bAvailable;
    uint8_t IRQBitmask;
} ADI_USBD_DEV_DMA_DEF;
#endif

/** \def ADI_USBD_DEV_DEF_DATA_SIZE
 *  Device driver definition structure size. This is required to calculate the size of memory which needs to
 *  be supplied to the MUSBMHDRC driver in the call to #ADI_USBD_DEV_Open.
 */
#define ADI_USBD_DEV_DEF_DATA_SIZE       (64)

/** \def ADI_USBD_DEV_EP_DATA_SIZE
 *  End point definition structure size. This is required to calculate the size of memory which needs to
 *  be supplied to the MUSBMHDRC driver in the call to #ADI_USBD_DEV_Open.
 */
#define ADI_USBD_DEV_EP_DATA_SIZE        (0x100)

#define __EP_DEBUG
/* The max number of available endpoints
*/
#if defined(__ADSPBF609_FAMILY__) || defined(__ADSPARM_FAMILY__)

/** \def ADI_USBD_DEV_MAX_EP_COUNT
 * Defines the maximum number of endpoints in teh MUSBMHDRC controller driver.
 */
#define ADI_USBD_DEV_MAX_EP_COUNT  (12)

#endif


/*@}*/

#endif /* __ADI_USBD_DEV_MUSBMHDRC_H__ */

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */


