/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*!
 * @file:    adi_pdma_channels_bf60x.h
 * @brief:   Peripheral DMA Channel ID definitions for ADSP-BF60x processor family
 * @version: $Revision: 18384 $
 * @date:    $Date: 2013-04-04 17:17:34 -0400 (Thu, 04 Apr 2013) $
 *
 * @details
 *           This is an additional header file for the Peripheral DMA Manager service of
 *           ADSP-BF60x processor family, which contains the Peripheral DMA Channel ID
 *           definitions used in the APIs
 */

#ifndef __ADI_PDMA_CHANNELS_BF60X_H__
#define __ADI_PDMA_CHANNELS_BF60X_H__

/*==============  D E F I N E S  ===============*/

/* ADSP-BF608/609 Channel IDs */
#if defined(__ADSPBF609_FAMILY__)

/** @ingroup PDMA_Manager_bf60x
 *  DMA Channel IDs for ADSP-BF608/609 processors
 */
typedef enum
{
    ADI_DMA_CHNL_SPORT0_A    = CHID_SPORT0_A_DMA,    /*!< DMA  0 - SPORT0 Channel A */
    ADI_DMA_CHNL_SPORT0_B    = CHID_SPORT0_B_DMA,    /*!< DMA  1 - SPORT0 Channel B */
    ADI_DMA_CHNL_SPORT1_A    = CHID_SPORT1_A_DMA,    /*!< DMA  2 - SPORT1 Channel A */
    ADI_DMA_CHNL_SPORT1_B    = CHID_SPORT1_B_DMA,    /*!< DMA  3 - SPORT1 Channel B */
    ADI_DMA_CHNL_SPORT2_A    = CHID_SPORT2_A_DMA,    /*!< DMA  4 - SPORT2 Channel A */
    ADI_DMA_CHNL_SPORT2_B    = CHID_SPORT2_B_DMA,    /*!< DMA  5 - SPORT2 Channel B */
    ADI_DMA_CHNL_SPI0_TX     = CHID_SPI0_TXDMA,      /*!< DMA  6 - SPI0 TX */
    ADI_DMA_CHNL_SPI0_RX     = CHID_SPI0_RXDMA,      /*!< DMA  7 - SPI0 RX */
    ADI_DMA_CHNL_SPI1_TX     = CHID_SPI1_TXDMA,      /*!< DMA  8 - SPI1 TX */
    ADI_DMA_CHNL_SPI1_RX     = CHID_SPI1_RXDMA,      /*!< DMA  9 - SPI1 RX */
    ADI_DMA_CHNL_RSI0        = CHID_RSI0_DMA,        /*!< DMA 10 - RSI0 */
    ADI_DMA_CHNL_SDU0        = CHID_SDU0_DMA,        /*!< DMA 11 - System Debug Unit 0 (SDU0) */
    ADI_DMA_CHNL_RESERVED0   = (12u),                /*!< DMA 12 - Reserved, Not available for programmer */
    ADI_DMA_CHNL_LP0         = CHID_LP0_DMA,         /*!< DMA 13 - Link Port 0 (LP0) */
    ADI_DMA_CHNL_LP1         = CHID_LP1_DMA,         /*!< DMA 14 - Link Port 1 (LP1) */
    ADI_DMA_CHNL_LP2         = CHID_LP2_DMA,         /*!< DMA 15 - Link Port 2 (LP2) */
    ADI_DMA_CHNL_LP3         = CHID_LP3_DMA,         /*!< DMA 16 - Link Port 3 (LP3) */
    ADI_DMA_CHNL_UART0_TX    = CHID_UART0_TXDMA,     /*!< DMA 17 - UART0 TX */
    ADI_DMA_CHNL_UART0_RX    = CHID_UART0_RXDMA,     /*!< DMA 18 - UART0 RX */
    ADI_DMA_CHNL_UART1_TX    = CHID_UART1_TXDMA,     /*!< DMA 19 - UART1 TX */
    ADI_DMA_CHNL_UART1_RX    = CHID_UART1_RXDMA,     /*!< DMA 20 - UART1 RX */
    ADI_DMA_CHNL_MEMDMA_S0   = CHID_MDMA0_SRC,       /*!< DMA 21 - MDMA0 Source */
    ADI_DMA_CHNL_MEMDMA_D0   = CHID_MDMA0_DST,       /*!< DMA 22 - MDMA0 Destination */
    ADI_DMA_CHNL_MEMDMA_S1   = CHID_MDMA1_SRC,       /*!< DMA 23 - MDMA1 Source */
    ADI_DMA_CHNL_MEMDMA_D1   = CHID_MDMA1_DST,       /*!< DMA 24 - MDMA1 Destination */
    ADI_DMA_CHNL_MEMDMA_S2   = CHID_MDMA2_SRC,       /*!< DMA 25 - MDMA2 Source */
    ADI_DMA_CHNL_MEMDMA_D2   = CHID_MDMA2_DST,       /*!< DMA 26 - MDMA2 Destination */
    ADI_DMA_CHNL_MEMDMA_S3   = CHID_MDMA3_SRC,       /*!< DMA 27 - MDMA3 Source */
    ADI_DMA_CHNL_MEMDMA_D3   = CHID_MDMA3_DST,       /*!< DMA 28 - MDMA3 Destination */
    ADI_DMA_CHNL_EPPI0_CH0   = CHID_EPPI0_CH0_DMA,   /*!< DMA 29 - EPPI0 Channel 0 */
    ADI_DMA_CHNL_EPPI0_CH1   = CHID_EPPI0_CH1_DMA,   /*!< DMA 30 - EPPI0 Channel 1 */
    ADI_DMA_CHNL_EPPI2_CH0   = CHID_EPPI2_CH0_DMA,   /*!< DMA 31 - EPPI2 Channel 0 */
    ADI_DMA_CHNL_EPPI2_CH1   = CHID_EPPI2_CH1_DMA,   /*!< DMA 32 - EPPI2 Channel 1 */
    ADI_DMA_CHNL_EPPI1_CH0   = CHID_EPPI1_CH0_DMA,   /*!< DMA 33 - EPPI1 Channel 0 */
    ADI_DMA_CHNL_EPPI1_CH1   = CHID_EPPI1_CH1_DMA,   /*!< DMA 34 - EPPI1 Channel 1 */
    
#if defined (__ADI_HAS_PIXC__)    
    ADI_DMA_CHNL_PIXC0_CH0   = CHID_PIXC0_CH0_DMA,   /*!< DMA 35 - PIXC0 Channel 0 */
    ADI_DMA_CHNL_PIXC0_CH1   = CHID_PIXC0_CH1_DMA,   /*!< DMA 36 - PIXC0 Channel 1 */
    ADI_DMA_CHNL_PIXC0_CH2   = CHID_PIXC0_CH2_DMA,   /*!< DMA 37 - PIXC0 Channel 2 */
#endif /* __ADI_HAS_PIXC__ */

#if defined (__ADI_HAS_PVP__)     
    ADI_DMA_CHNL_PVP0_CPDOB  = CHID_PVP0_CPDOB_DMA,  /*!< DMA 38 - PVP0 Camera Pipe Data Out B */
    ADI_DMA_CHNL_PVP0_CPDOC  = CHID_PVP0_CPDOC_DMA,  /*!< DMA 39 - PVP0 Camera Pipe Data Out C */
    ADI_DMA_CHNL_PVP0_CPSTAT = CHID_PVP0_CPSTAT_DMA, /*!< DMA 40 - PVP0 Camera Pipe Status Out */
    ADI_DMA_CHNL_PVP0_CPCI   = CHID_PVP0_CPCI_DMA,   /*!< DMA 41 - PVP0 Camera Pipe Control In */
    ADI_DMA_CHNL_PVP0_MPDO   = CHID_PVP0_MPDO_DMA,   /*!< DMA 42 - PVP0 Memory Pipe Data Out */
    ADI_DMA_CHNL_PVP0_MPDI   = CHID_PVP0_MPDI_DMA,   /*!< DMA 43 - PVP0 Memory Pipe Data In */
    ADI_DMA_CHNL_PVP0_MPSTAT = CHID_PVP0_MPSTAT_DMA, /*!< DMA 44 - PVP0 Memory Pipe Status Out */
    ADI_DMA_CHNL_PVP0_MPCI   = CHID_PVP0_MPCI_DMA,   /*!< DMA 45 - PVP0 Memory Pipe Control In */
    ADI_DMA_CHNL_PVP0_CPDOA  = CHID_PVP0_CPDOA_DMA,  /*!< DMA 46 - PVP0 Camera Pipe Data Out A */
#endif /*  __ADI_HAS_PVP__ */   

    ADI_DMA_CHANNEL_END		 = (47u)                 /* End of DMA Channel listing */

} ADI_DMA_CHANNEL_ID;

#endif  /* __ADSPBF609_FAMILY__ */

#endif  /* __ADI_PDMA_CHANNELS_BF60X_H__ */

/*****/

/*@}*/

