/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*!
 * @file:    adi_pdma_channels_cm40z.h
 * @brief:   Peripheral DMA Channel ID definitions for ADSP-CM40z processor family
 * @version: $Revision: 18384 $
 * @date:    $Date: 2013-04-04 17:17:34 -0400 (Thu, 04 Apr 2013) $
 *
 * @details
 *           This is an additional header file for the Peripheral DMA Manager service of
 *           ADSP-CM40z processor family, which contains the Peripheral DMA Channel ID
 *           definitions used in the APIs
 */

#ifndef __ADI_PDMA_CHANNELS_CM40Z_H__
#define __ADI_PDMA_CHANNELS_CM40Z_H__

/*==============  D E F I N E S  ===============*/

/* ADSP-CM402/CM403/CM407/CM408 processors */
#if defined(ADI_ADSP_CM40Z)

/** @ingroup PDMA_Manager_cm40x
 *  DMA Channel IDs for ADSP_CM40Z processors
 */
typedef enum
{
    ADI_DMA_CHNL_SPORT0_A    = CHID_SPORT0_A_DMA,       /*!< DMA  0 - SPORT0 Channel A */
    ADI_DMA_CHNL_SPORT0_B    = CHID_SPORT0_B_DMA,       /*!< DMA  1 - SPORT0 Channel B */
    ADI_DMA_CHNL_SPI0_TX     = CHID_SPI0_TXDMA,         /*!< DMA  2 - SPI0 TX */
    ADI_DMA_CHNL_SPI0_RX     = CHID_SPI0_RXDMA,         /*!< DMA  3 - SPI0 RX */
    ADI_DMA_CHNL_UART0_TX    = CHID_UART0_TXDMA,        /*!< DMA  4 - UART0 TX */
    ADI_DMA_CHNL_UART0_RX    = CHID_UART0_RXDMA,        /*!< DMA  5 - UART0 RX */
    ADI_DMA_CHNL_SPORT1_A    = CHID_SPORT1_A_DMA,       /*!< DMA  6 - SPORT1 Channel A */
    ADI_DMA_CHNL_SPORT1_B    = CHID_SPORT1_B_DMA,       /*!< DMA  7 - SPORT1 Channel B */
    ADI_DMA_CHNL_SPI1_TX     = CHID_SPI1_TXDMA,         /*!< DMA  8 - SPI1 TX */
    ADI_DMA_CHNL_SPI1_RX     = CHID_SPI1_RXDMA,         /*!< DMA  9 - SPI1 RX */
    ADI_DMA_CHNL_UART1_TX    = CHID_UART1_TXDMA,        /*!< DMA 10 - UART1 TX */
    ADI_DMA_CHNL_UART1_RX    = CHID_UART1_RXDMA,        /*!< DMA 11 - UART1 RX */
    ADI_DMA_CHNL_UART2_TX    = CHID_UART2_TXDMA,        /*!< DMA 12 - UART2 TX */
    ADI_DMA_CHNL_UART2_RX    = CHID_UART2_RXDMA,        /*!< DMA 13 - UART2 RX */
    ADI_DMA_CHNL_HAE0_RX_CH0 = CHID_HAE0_RXDMA_CH0,     /*!< DMA 14 - Harmonic Analysis Engine (HAE) RX DMA Channel 0 */
    ADI_DMA_CHNL_HAE0_RX_CH1 = CHID_HAE0_RXDMA_CH1,     /*!< DMA 15 - Harmonic Analysis Engine (HAE) RX DMA Channel 1 */
    ADI_DMA_CHNL_HAE0_TX     = CHID_HAE0_TXDMA,         /*!< DMA 16 - Harmonic Analysis Engine (HAE) TX DMA */
    ADI_DMA_CHNL_MEMDMA_S0   = CHID_MDMA0_SRC,          /*!< DMA 17 - MDMA0 Source/CRC0 Input */
    ADI_DMA_CHNL_MEMDMA_D0   = CHID_MDMA0_DST,          /*!< DMA 18 - MDMA0 Destination/CRC0 Output */
    ADI_DMA_CHNL_MEMDMA_S1   = CHID_MDMA1_SRC,          /*!< DMA 19 - MDMA1 Source */
    ADI_DMA_CHNL_MEMDMA_D1   = CHID_MDMA1_DST,          /*!< DMA 20 - MDMA1 Destination */
    ADI_DMA_CHANNEL_END	     = (21u)                    /* End of DMA Channel listing */

} ADI_DMA_CHANNEL_ID;

#endif  /* ADI_ADSP_CM40Z  */

#endif  /* __ADI_PDMA_CHANNELS_CM40Z_H__ */

/*****/

/*@}*/

