/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_int.h
*
* @brief     Interrupt Handler API header file
*
* @details
*            This is the primary header file for installing interrupt handlers
*/

#ifndef ADI_INT_H_
#define ADI_INT_H_


/**
 *  @defgroup INTERRUPTS Interrupt Related APIs
 *  System Interrupt controllers will vary from platform to platform. Their
 *  corresponding device driver APIs will vary as well.
 */

/** @defgroup Interrupt_Install Interrupt Installation
 *  @ingroup INTERRUPTS
 */



/*! @addtogroup Interrupt_Install Interrupt Installation
 *  @{
 */

#include <adi_types.h>
#include "adi_int_id.h"

/*! Enumeration of different error codes returned by adi_int* APIs. */
typedef enum
{
    ADI_INT_SUCCESS, /*!< The operation is completed successfully. */
    ADI_INT_FAILURE  /*!< The operation failed. */

} ADI_INT_STATUS;

#ifdef __cplusplus 
extern "C" { 
#endif /* __cplusplus */ 

/**
 * @brief       Interrupt Handler
 *
 * @details     Interrupt handler to be registered by the application.
 *
 * @param [in]  iid         Interrupt ID for the interrupt.
 * @param [in]  handlerArg  Callback parameter given by the application while
 *                          registering the handler.
 *
 * @return      None
 */

typedef void (*ADI_INT_HANDLER_PTR)(
    uint32_t            iid, 
    void*               handlerArg
);

ADI_INT_STATUS  adi_int_InstallHandler (
    uint32_t            iid,
    ADI_INT_HANDLER_PTR pfHandler,
    void*               pCBParam,
    bool                bEnable
               );

ADI_INT_STATUS adi_int_EnableInt(
    uint32_t            iid,
    bool                bEnable
               );
ADI_INT_STATUS  adi_int_UninstallHandler (uint32_t iid);

/**
 * @enum ADI_SSL_CALLBACK
 * @brief Device Drivers/System Services Callback function definition
 */
typedef void (* ADI_CALLBACK) (  /*!< Callback function pointer */
    void      *pCBParam,         /*!< Client supplied callback param */
    uint32_t   Event,            /*!< Event ID specific to the Driver/Service */
    void      *pArg);            /*!< Pointer to the event specific argument */

#ifdef __cplusplus 
}
#endif /* __cplusplus */ 
	
/*@}*/

#endif /* ADI_INT_H_ */
