/*!
 *****************************************************************************
 * @file:    adi_sid_cm40z.h
 * @brief:   Defenition of interrupt IDs using which application/drivers
             register/unregister interrupt handlers.
 * @version: $Revision: 21680 $
 * @date:    $Date: 2013-05-31 10:42:26 -0400 (Fri, 31 May 2013) $
 *-----------------------------------------------------------------------------
 *
Copyright (c) 2010 Analog Devices, Inc.

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  - Modified versions of the software must be conspicuously marked as such.
  - This software is licensed solely and exclusively for use with processors
    manufactured by or for Analog Devices, Inc.
  - This software may not be combined or merged with other code in any manner
    that would cause the software to become subject to terms and conditions
    which differ from those listed here.
  - Neither the name of Analog Devices, Inc. nor the names of its
    contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.
  - The use of this software may or may not infringe the patent rights of one
    or more patent holders.  This license does not release you from the
    requirement that you obtain separate licenses from these patent holders
    to use this software.

THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES, INC. AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
TITLE, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
NO EVENT SHALL ANALOG DEVICES, INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, PUNITIVE OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, DAMAGES ARISING OUT OF CLAIMS OF INTELLECTUAL
PROPERTY RIGHTS INFRINGEMENT; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

**********************************************************************************/


/* Cortex-M4 processor exceptions */
#define ADI_INT_RESET                     -15  // Reset Interrupt
#define ADI_INT_NMI                       -14  // Non Maskable Interrupt
#define ADI_INT_HARD_FAULT                -13  // Hard Fault Interrupt
#define ADI_INT_MEM_MGMT                  -12  // Memory Management Interrupt
#define ADI_INT_BUS_FAULT                 -11  // Bus Fault Interrupt
#define ADI_INT_USAGE_FAULT               -10  // Usage Fault Interrupt
//                                         -9     Reserved
//                                         -8     Reserved
//                                         -7     Reserved
//                                         -6     Reserved
#define ADI_INT_SVCALL                     -5  // System Service Call Interrupt
#define ADI_INT_DEBUG_MON                  -4  // Debug Monitor Interrupt
//                                         -3     Reserved
#define ADI_INT_PENDSV                     -2  // Pendable request for system device Interrupt
#define ADI_INT_SYSTICK                    -1  // System Tick Interrupt

/* ADSP-CM40z Specific Interrupts */
#define ADI_INT_OSCW_EVT                    0  // Oscillator Watchdog
#define ADI_INT_CGU0_EVT                    1  // Event
#define ADI_INT_M4P_L1CC_PERR               2  // L1 Cache Code Parity Error
#define ADI_INT_M4P_CORE_SRAM_PERR          3  // SRAM Core Parity Error
#define ADI_INT_M4P_DMA_SRAM_PERR           4  // SRAM DMA Parity Error
#define ADI_INT_M4P_BUS_FAULT               5  // Bus Fault
#define ADI_INT_M4P_LOCKUP                  6  // Lockup Error (Fault only; not an interrupt)
#define ADI_INT_M4P_SRAM_PERR_FLT           7  // SRAM Parity Error (Fault only; not an interrupt)
#define ADI_INT_WDOG0_EXP                   8  // Expiration
#define ADI_INT_SEC0_ERR                    9  // Fault Interrupt
#define ADI_INT_ECT_EVT0                   10  // Embedded Cross Trigger Event 0
#define ADI_INT_ECT_EVT1                   11  // Embedded Cross Trigger Event 1
#define ADI_INT_PWM0_TRIP                  12  // Trip
#define ADI_INT_PWM1_TRIP                  13  // Trip
#define ADI_INT_PWM2_TRIP                  14  // Trip
#define ADI_INT_PWM0_SYNC                  15  // Sync
#define ADI_INT_PWM1_SYNC                  16  // Sync
#define ADI_INT_PWM2_SYNC                  17  // Sync
#define ADI_INT_PINT0_BLOCK                18  // Pin Interrupt Block
#define ADI_INT_PINT1_BLOCK                19  // Pin Interrupt Block
#define ADI_INT_PINT2_BLOCK                20  // Pin Interrupt Block
#define ADI_INT_PINT3_BLOCK                21  // Pin Interrupt Block
#define ADI_INT_PINT4_BLOCK                22  // Pin Interrupt Block
#define ADI_INT_SPORT0_A_DMA_ERR           23  // DMA Channel 0 Error
#define ADI_INT_SPORT0_B_DMA_ERR           24  // DMA Channel 1 Error
#define ADI_INT_SPI0_TXDMA_ERR             25  // DMA Channel 2 Error
#define ADI_INT_SPI0_RXDMA_ERR             26  // DMA Channel 3 Error
#define ADI_INT_UART0_TXDMA_ERR            27  // DMA Channel 4 Error
#define ADI_INT_UART0_RXDMA_ERR            28  // DMA Channel 5 Error
#define ADI_INT_SPORT1_A_DMA_ERR           29  // DMA Channel 6 Error
#define ADI_INT_SPORT1_B_DMA_ERR           30  // DMA Channel 7 Error
#define ADI_INT_SPI1_TXDMA_ERR             31  // DMA Channel 8 Error
#define ADI_INT_SPI1_RXDMA_ERR             32  // DMA Channel 9 Error
#define ADI_INT_UART1_TXDMA_ERR            33  // DMA Channel 10 Error
#define ADI_INT_UART1_RXDMA_ERR            34  // DMA Channel 11 Error
#define ADI_INT_UART2_TXDMA_ERR            35  // DMA Channel 12 Error
#define ADI_INT_UART2_RXDMA_ERR            36  // DMA Channel 13 Error
#define ADI_INT_HAE0_RXDMA_CH0_ERR         37  // DMA Channel 14 Error
#define ADI_INT_HAE0_RXDMA_CH1_ERR         38  // DMA Channel 15 Error
#define ADI_INT_HAE0_TXDMA_ERR             39  // DMA Channel 16 Error
//                                         40     Reserved
#define ADI_INT_SPI0_ERR                   41  // Error
#define ADI_INT_SPI1_ERR                   42  // Error
#define ADI_INT_SPI2_ERR                   43  // Error
#define ADI_INT_ADCC0_ERR                  44  // ADC Error
#define ADI_INT_DACC0_ERR                  45  // DAC Error
#define ADI_INT_MDMA0_SRC_CRC0_IN_ERR      46  // DMA Channel 17 Error
#define ADI_INT_MDMA0_DST_CRC0_OUT_ERR     47  // DMA Channel 18 Error
#define ADI_INT_MDMA1_SRC_ERR              48  // DMA Channel 19 Error
#define ADI_INT_MDMA1_DST_ERR              49  // DMA Channel 20 Error
#define ADI_INT_TIMER0_STAT                50  // Status
#define ADI_INT_TIMER0_TMR0                51  // Timer 0
#define ADI_INT_TIMER0_TMR1                52  // Timer 1
#define ADI_INT_TIMER0_TMR2                53  // Timer 2
#define ADI_INT_TIMER0_TMR3                54  // Timer 3
#define ADI_INT_TIMER0_TMR4                55  // Timer 4
#define ADI_INT_TIMER0_TMR5                56  // Timer 5
#define ADI_INT_TIMER0_TMR6                57  // Timer 6
#define ADI_INT_TIMER0_TMR7                58  // Timer 7
#define ADI_INT_CNT0_STAT                  59  // Status
#define ADI_INT_CNT1_STAT                  60  // Status
#define ADI_INT_CNT2_STAT                  61  // Status
#define ADI_INT_CNT3_STAT                  62  // Status
#define ADI_INT_SPORT0_A_STAT              63  // Channel A Status
#define ADI_INT_SPORT0_A_DMA               64  // Channel A DMA
#define ADI_INT_SPORT0_B_STAT              65  // Channel B Status
#define ADI_INT_SPORT0_B_DMA               66  // Channel B DMA
#define ADI_INT_UART0_STAT                 67  // Status
#define ADI_INT_UART0_TXDMA                68  // Transmit DMA
#define ADI_INT_UART0_RXDMA                69  // Receive DMA
#define ADI_INT_SINC0_STAT                 70  // Status
#define ADI_INT_SPI0_STAT                  71  // Status
#define ADI_INT_SPI0_TXDMA                 72  // TX DMA Channel
#define ADI_INT_SPI0_RXDMA                 73  // RX DMA Channel
#define ADI_INT_SPORT1_A_STAT              74  // Channel A Status
#define ADI_INT_SPORT1_A_DMA               75  // Channel A DMA
#define ADI_INT_SPORT1_B_STAT              76  // Channel B Status
#define ADI_INT_SPORT1_B_DMA               77  // Channel B DMA
#define ADI_INT_UART1_STAT                 78  // Status
#define ADI_INT_UART1_TXDMA                79  // Transmit DMA
#define ADI_INT_UART1_RXDMA                80  // Receive DMA
#define ADI_INT_SPI1_STAT                  81  // Status
#define ADI_INT_SPI1_TXDMA                 82  // TX DMA Channel
#define ADI_INT_SPI1_RXDMA                 83  // RX DMA Channel
#define ADI_INT_SPI2_TX                    84  // TX  Channel (non-DMA)
#define ADI_INT_SPI2_RX                    85  // RX Channel (non-DMA)
#define ADI_INT_EMAC0_STAT                 86  // Status
#define ADI_INT_UART2_STAT                 87  // Status
#define ADI_INT_UART2_TXDMA                88  // Transmit DMA
#define ADI_INT_UART2_RXDMA                89  // Receive DMA
#define ADI_INT_HAE0_STAT                  90  // Status
#define ADI_INT_HAE0_RXDMA_CH0             91  // RX DMA Channel 0
#define ADI_INT_HAE0_RXDMA_CH1             92  // RX DMA Channel 1
#define ADI_INT_HAE0_TXDMA                 93  // TX DMA Channel 0
#define ADI_INT_ADCC0_TMR0_EVT             94  // Timer 0 Event Complete
#define ADI_INT_ADCC0_TMR1_EVT             95  // Timer 1 Event Complete
#define ADI_INT_DACC0_DAC0                 96  // DAC Interrupt 0
#define ADI_INT_DACC0_DAC1                 97  // DAC Interrupt 1
#define ADI_INT_SPI2_STAT                  98  // Status
#define ADI_INT_TWI0_DATA                  99  // Data Interrupt
#define ADI_INT_CRC0_DCNTEXP              100  // Datacount expiration
#define ADI_INT_CRC0_ERR                  101  // Error
#define ADI_INT_MDMA0_SRC_CRC0_IN         102  // Memory DMA Stream 0 Source Channel
#define ADI_INT_MDMA0_DST_CRC0_OUT        103  // Memory DMA Stream 0 Destination Channel
#define ADI_INT_MDMA1_SRC                 104  // Memory DMA Stream 1 Source Channel
#define ADI_INT_MDMA1_DST                 105  // Memory DMA Stream 1 Destination Channel
#define ADI_INT_USB0_STAT                 106  // Status/FIFO Data Ready
#define ADI_INT_USB0_DATA                 107  // DMA Status/Transfer Complete
#define ADI_INT_TRU0_INT0                 108  // Interrupt 0
#define ADI_INT_TRU0_INT1                 109  // Interrupt 1
#define ADI_INT_TRU0_INT2                 110  // Interrupt 2
#define ADI_INT_TRU0_INT3                 111  // Interrupt 3
#define ADI_INT_CGU0_ERR                  112  // Error
#define ADI_INT_DPM0_EVT                  113  // Event
#define ADI_INT_SOFT0                     114  // Software-Driven Interrupt 0
#define ADI_INT_SOFT1                     115  // Software-Driven Interrupt 1
#define ADI_INT_SOFT2                     116  // Software-Driven Interrupt 2
#define ADI_INT_SOFT3                     117  // Software-Driven Interrupt 3
#define ADI_INT_SWU0_EVT                  118  // Event
#define ADI_INT_SWU1_EVT                  119  // Event
#define ADI_INT_SWU2_EVT                  120  // Event
#define ADI_INT_SWU3_EVT                  121  // Event
#define ADI_INT_SWU4_EVT                  122  // Event
#define ADI_INT_CAN0_RX                   123  // Receive
#define ADI_INT_CAN0_TX                   124  // Transmit
#define ADI_INT_CAN0_STAT                 125  // Status
#define ADI_INT_CAN1_RX                   126  // Receive
#define ADI_INT_CAN1_TX                   127  // Transmit
#define ADI_INT_CAN1_STAT                 128  // Status

/* Update this based on the number of interrupts */
#define ADI_INT_NUM_INT         129          /* Number of Interrupts */

