/*********************************************************************************

Copyright(c) 2010 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*!
* @file      adi_pwm_6xx.h
*
* @brief     PWM driver defintions API Header file
*
* @details
*            Header File for the PWM service API functions and definitions
*/

/** @addtogroup PWM_Service_BF6xx PWM Service
*  @{
*/

#ifndef __ADI_PWM_6XX_H__
#define __ADI_PWM_6XX_H__

#ifndef  __ADI_PWM_H__
#error "This file may not be directly included. The file <services/pwm/adi_pwm.h> should be included instead"
#endif

/* disable misra diagnostics as necessary */
#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"Identifiers shall not rely on the significance of more than 31 characters")
#endif /* _MISRA_RULES */

#ifdef __cplusplus 
extern "C" { 
#endif /* __cplusplus */ 

#if defined (__ADSPBF609_FAMILY__)
#include <services/int/adi_sec.h>
#endif
  
/*==============  D E F I N E S  ===============*/

/* Size of PWM memory */
#define ADI_PWM_MEMORY_SIZE 8u

/*! PWM Device handle */
typedef void* ADI_PWM_HANDLE;

#define ADI_PWM_TRIP_ALL		 (ENUM_PWM_STAT_TRIP0       | \
    							  ENUM_PWM_STAT_TRIP1       | \
    							  ENUM_PWM_STAT_FLTA_TRIP   | \
    							  ENUM_PWM_STAT_FLTB_TRIP   | \
    							  ENUM_PWM_STAT_FLTC_TRIP   | \
    							  ENUM_PWM_STAT_FLTD_TRIP)

#define ADI_PWM_TRIP_ONLY  		 (ENUM_PWM_STAT_TRIP0       | \
    							  ENUM_PWM_STAT_TRIP1)

#define ADI_PWM_TRIP_FAULT_ONLY	 (ENUM_PWM_STAT_FLTA_TRIP   | \
    							  ENUM_PWM_STAT_FLTB_TRIP   | \
    							  ENUM_PWM_STAT_FLTC_TRIP   | \
    							  ENUM_PWM_STAT_FLTD_TRIP)


#define  ADI_PWM_PERIOD_ALL		 (ENUM_PWM_STAT_PER0        | \
    					    	  ENUM_PWM_STAT_PER1        | \
    							  ENUM_PWM_STAT_PER2        | \
    							  ENUM_PWM_STAT_PER3        | \
    							  ENUM_PWM_STAT_PER4)

/*! Enumeration for the PWM events */
typedef enum
{
	ADI_PWM_HW_ERR_NONE         = 0,                   	        /*!< No event             					    */
	ADI_PWM_TRIP0               = ENUM_PWM_STAT_TRIP0,          /*!< PWM trip0 event                            */
	ADI_PWM_TRIP1               = ENUM_PWM_STAT_TRIP1,          /*!< PWM trip1 event                            */
	ADI_PWM_HW_ERR_TRIP_FAULTA  = ENUM_PWM_STAT_FLTA_TRIP,      /*!< PWM trip fault A event                     */
    ADI_PWM_HW_ERR_TRIP_FAULTB  = ENUM_PWM_STAT_FLTB_TRIP,      /*!< PWM trip fault B event                     */
    ADI_PWM_HW_ERR_TRIP_FAULTC  = ENUM_PWM_STAT_FLTC_TRIP,      /*!< PWM trip fault C event                     */
    ADI_PWM_HW_ERR_TRIP_FAULTD  = ENUM_PWM_STAT_FLTD_TRIP,      /*!< PWM trip fault D event                     */
    ADI_PWM_TMR0_BOUNDARY       = ENUM_PWM_STAT_PER0,           /*!< PWM timer 0 period boundary                */
    ADI_PWM_TMR1_BOUNDARY       = ENUM_PWM_STAT_PER1,           /*!< PWM timer 1 period boundary                */
    ADI_PWM_TMR2_BOUNDARY       = ENUM_PWM_STAT_PER2,           /*!< PWM timer 2 period boundary                */
    ADI_PWM_TMR3_BOUNDARY       = ENUM_PWM_STAT_PER3,           /*!< PWM timer 3 period boundary				*/
    ADI_PWM_TMR4_BOUNDARY       = ENUM_PWM_STAT_PER4            /*!< PWM timer 4 period boundary                */

} ADI_PWM_EVENT;

/*! Enumeration for the PWM return codes */
typedef enum
{
    ADI_PWM_SUCCESS = 0,                                        /*!< A generic success                          */
    ADI_PWM_FAILURE,                                            /*!< A generic failure                          */
    ADI_PWM_INVALID_DEVICE_NUM,                                 /*!< PWM number is invalid                      */
    ADI_PWM_INVALID_HANDLE,                                     /*!< PWM handle is invalid                      */    
    ADI_PWM_INSUFFICIENT_MEMORY,                                /*!< Insufficient PWM driver memory          	*/
    ADI_PWM_CONFIGURATION_ERROR,                                /*!< There is a configuration conflict          */
    ADI_PWM_IN_USE,                                             /*!< PWM resource is already in use             */
    ADI_PWM_BAD_PARAMETER,                                      /*!< The parameter passed is invalid            */
    ADI_PWM_INTERRUPT_FAILED        						    /*!< PWM interrupt problem    					*/
}ADI_PWM_RESULT;

 /*! Enumeration for the field masks of the PWM_IMASK register */
typedef enum
{
    ADI_PWM_INTERRUPT_TRIP0     = BITM_PWM_IMSK_TRIP0,		/*!< PWM TRIP0  interrupt mask                          */
    ADI_PWM_INTERRUPT_TRIP1     = BITM_PWM_IMSK_TRIP1,     	/*!< PWM TRIP1  interrupt mask                          */
    ADI_PWM_INTERRUPT_TIMER0    = BITM_PWM_IMSK_TMR0PER,   	/*!< PWM TIMER0 interrupt mask                          */
    ADI_PWM_INTERRUPT_TIMER1    = BITM_PWM_IMSK_TMR1PER,   	/*!< PWM TIMER1 interrupt mask                          */
    ADI_PWM_INTERRUPT_TIMER2    = BITM_PWM_IMSK_TMR2PER,   	/*!< PWM TIMER2 interrupt mask                          */
    ADI_PWM_INTERRUPT_TIMER3    = BITM_PWM_IMSK_TMR3PER,   	/*!< PWM TIMER3 interrupt mask                          */
    ADI_PWM_INTERRUPT_TIMER4    = BITM_PWM_IMSK_TMR4PER    	/*!< PWM TIMER4 interrupt mask                          */
} ADI_PWM_INTERRUPT_ID;

 /*! Enumeration for the field masks of the PWM_STATUS register */
typedef enum
{
    ADI_PWM_STATUS_RAWTRIP0      = BITM_PWM_STAT_RAWTRIP0, 	/*!< PWM TRIP0 level                                    */
    ADI_PWM_STATUS_RAWTRIP1      = BITM_PWM_STAT_RAWTRIP1, 	/*!< PWM TRIP1 level                                    */
    ADI_PWM_STATUS_RESTARTA      = BITM_PWM_STAT_SRTRIPA,  	/*!< PWM TRIP restart channel A                         */
    ADI_PWM_STATUS_RESTARTB      = BITM_PWM_STAT_SRTRIPA,  	/*!< PWM TRIP restart channel B                         */
    ADI_PWM_STATUS_RESTARTC      = BITM_PWM_STAT_SRTRIPA,  	/*!< PWM TRIP restart channel C                         */
    ADI_PWM_STATUS_RESTARTD      = BITM_PWM_STAT_SRTRIPA,  	/*!< PWM TRIP restart channel D                         */
    ADI_PWM_STATUS_TIMER0_PHASE  = BITM_PWM_STAT_TMR0PHASE,	/*!< PWM TIMER1 phase                                   */
    ADI_PWM_STATUS_TIMER1_PHASE  = BITM_PWM_STAT_TMR1PHASE,	/*!< PWM TIMER1 phase                                   */
    ADI_PWM_STATUS_TIMER2_PHASE  = BITM_PWM_STAT_TMR2PHASE,	/*!< PWM TIMER2 phase                                   */
    ADI_PWM_STATUS_TIMER3_PHASE  = BITM_PWM_STAT_TMR3PHASE,	/*!< PWM TIMER3 phase                                   */
    ADI_PWM_STATUS_TIMER4_PHASE  = BITM_PWM_STAT_TMR4PHASE 	/*!< PWM TIMER4 phase                                   */
/*  ADI_PWM_STATUS_HP_READY      = BITM_PWM_STAT_HPRDY */	/*!< PWM high precision ready                        	*/
} ADI_PWM_STATUS_ID;

/*! Enumeration for PWM channel pairs */
typedef enum
{
    ADI_PWM_CHANNEL_A,                                     /*!< PWM channel pair A                                 	*/
    ADI_PWM_CHANNEL_B,                                     /*!< PWM channel pair B                                 	*/
    ADI_PWM_CHANNEL_C,                                     /*!< PWM channel pair C                                 	*/
    ADI_PWM_CHANNEL_D                                      /*!< PWM channel pair D                                 	*/
} ADI_PWM_CHANNEL;

/*! Enumeration for the high side and low side outputs of a PWM channel pair */
typedef enum
{
    ADI_PWM_HIGH_SIDE,                                     /*!< High Side PWM Output                               	*/
    ADI_PWM_LOW_SIDE                                       /*!< Low Side PWM Output                                	*/
} ADI_PWM_CHANNEL_OUTPUT;

/*! Enumeration for the reference timers */
typedef enum
{
    ADI_PWM_TIMER0,                                        /*!< TIMER0, main reference timer                        */
    ADI_PWM_TIMER1,                                        /*!< TIMER1, PWM Channel A timer                         */
    ADI_PWM_TIMER2,                                        /*!< TIMER2, PWM Channel B timer                         */
    ADI_PWM_TIMER3,                                        /*!< TIMER3, PWM Channel C timer                         */
    ADI_PWM_TIMER4                                         /*!< TIMER4, PWM Channel D timer                         */
} ADI_PWM_TIMER;

/*! Enumeration for the pulse types; symmetrical, asymmetrical, left half or right half */
typedef enum
{
    ADI_PWM_PULSE_SYMMETRICAL   = ENUM_PWM_SYM_HI,        /*!< symmetrical pulse                                    */
    ADI_PWM_PULSE_ASYMMETRICAL  = ENUM_PWM_ASYM_HI  >>    /*!< asymmetrical pulse                                   */
                                  BITP_PWM_ACTL_PULSEMODEHI,
    ADI_PWM_PULSE_LEFT_HALF     = ENUM_PWM_LEFT_HI  >>    /*!< left half pulse                                      */
                                  BITP_PWM_ACTL_PULSEMODEHI,
    ADI_PWM_PULSE_RIGHT_HALF    = ENUM_PWM_RIGHT_HI >>    /*!< right half pulse                                     */
                                  BITP_PWM_ACTL_PULSEMODEHI
} ADI_PWM_PULSE_TYPE;

/*! Enumeration for the PWM trip source */
typedef enum
{
    ADI_PWM_TRIP0_SRC,                                     /*!< TRIP0                                              */
    ADI_PWM_TRIP1_SRC                                      /*!< TRIP1                                              */
} ADI_PWM_TRIP_SOURCE;


/*************************************
  PWM API
*************************************/
ADI_PWM_RESULT adi_pwm_Open(
    uint32_t        DeviceNumber,
    void            *pDeviceMemory,
	uint32_t		MemorySize,
    ADI_PWM_HANDLE  *phDevice,	
    ADI_CALLBACK    const pfCallback,
    void            *const pCBParam
);

ADI_PWM_RESULT adi_pwm_Close(
    ADI_PWM_HANDLE  const hDevice
);

/************************************/
/* PWM callback registration        */
/************************************/
/* This function enables / disables the interrupts of the PWM */
ADI_PWM_RESULT adi_pwm_InterruptEnable(
    ADI_PWM_HANDLE       const hDevice,
    ADI_PWM_INTERRUPT_ID eInterrupt,
    bool                 bEnable
);

/* This function sets the reference period of the main timer (TIMER0) */
ADI_PWM_RESULT adi_pwm_SetReferencePeriod(
    ADI_PWM_HANDLE       const hDevice,
    uint32_t             Period
);

/* This function sets the period of TIMER_x for the PWM_x channel */
ADI_PWM_RESULT adi_pwm_SetPeriod(
    ADI_PWM_HANDLE       const hDevice,
    ADI_PWM_CHANNEL      eChannel,
    uint32_t             Period
);

/* This function sets the delay time between main timer (TIMER0) and the start of TIMER_x of the PWM_x channel. */
ADI_PWM_RESULT adi_pwm_SetDelay(
    ADI_PWM_HANDLE       const hDevice,
    ADI_PWM_CHANNEL      eChannel,
    uint32_t             Delay
);

/* This function sets the deadtime between the high side and low side outputs. */
ADI_PWM_RESULT adi_pwm_SetDeadtime(
    ADI_PWM_HANDLE       const hDevice,
    uint32_t             Deadtime
);

/* This function sets the period of the high-frequency chopping signal. */
ADI_PWM_RESULT adi_pwm_SetChopFrequency(
    ADI_PWM_HANDLE       const hDevice,
    uint32_t             Chop
);

/* This function sets the width of the internal SYNC pulse. */
ADI_PWM_RESULT adi_pwm_SetSyncWidth(
    ADI_PWM_HANDLE       const hDevice,
    uint32_t             SyncWidth
);

/* This function inconjunction with adi_PWM_SetOutputDutyCycle */
ADI_PWM_RESULT adi_pwm_SetOutputDutyCycle(
    ADI_PWM_HANDLE       	const hDevice,
    ADI_PWM_CHANNEL      	eChannel,
    ADI_PWM_CHANNEL_OUTPUT	eChannelOutput,
    uint32_t             	Duty,
    uint32_t             	Duty1
);

/* This function This function is the global PWM enable. It enables/disables all PWM outputs */
ADI_PWM_RESULT adi_pwm_Enable(
    ADI_PWM_HANDLE       const hDevice,
    bool                 bEnable
);

/* This function enables / disables the timer delay of a PWM channel. */
ADI_PWM_RESULT adi_pwm_DelayEnable(
    ADI_PWM_HANDLE       const hDevice,
    ADI_PWM_CHANNEL      eChannel,
    bool                 bEnable
);

/* This function enables / disables the input of an external SYNC pulse */
ADI_PWM_RESULT adi_pwm_ExtSyncEnable(
    ADI_PWM_HANDLE       const hDevice,
    bool                 bSync,
    bool                 bEnable
);

/*This function sets the reference timer for the generation of the internal SYNC pulse output */
ADI_PWM_RESULT adi_pwm_SetIntSyncTimerMode(
    ADI_PWM_HANDLE       const hDevice,
    ADI_PWM_TIMER        eTimer
);


/* This function sets the reference timer for each PWM channel pair. */
ADI_PWM_RESULT adi_pwm_SetTimerMode(
    ADI_PWM_HANDLE       const hDevice,
    ADI_PWM_CHANNEL      eChannel,
    bool                 bTimer
);

/* This function sets the mode of operation of the PWM channel low side output  */
/* The low side output can be the inversion of the high side or be independent. */
ADI_PWM_RESULT adi_pwm_SetLowSideMode(
    ADI_PWM_HANDLE       const hDevice,
    ADI_PWM_CHANNEL      eChannel,
    bool                 bMode
);

/* This function sets the polarity of either the high side or low side output of a PWM channel. */
ADI_PWM_RESULT adi_pwm_SetOutputPolarity(
    ADI_PWM_HANDLE       	const hDevice,
    ADI_PWM_CHANNEL      	eChannel,
    ADI_PWM_CHANNEL_OUTPUT  eChannelOutput,
    bool                    bPolarity
);

/* This function enables / disables gated chopping of either the high side or low side output of a PWM channel. */
ADI_PWM_RESULT adi_pwm_OutputChoppingEnable(
    ADI_PWM_HANDLE       	const hDevice,
    ADI_PWM_CHANNEL      	eChannel,
    ADI_PWM_CHANNEL_OUTPUT  eChannelOutput,
    bool                    bEnable
);

/*****************************************************************************/
/* TRIP CONFIGURATION REGISTER
*/
/*****************************************************************************/

/* This function enables / disables the PWM TRIP source for each channel. */
ADI_PWM_RESULT adi_pwm_SetTripEnable(
    ADI_PWM_HANDLE       const hDevice,
    ADI_PWM_CHANNEL      eChannel,
    ADI_PWM_TRIP_SOURCE  eTripSource,
    bool                 bEnable
);

/* This function configures the operation of the PWM TRIP. */
ADI_PWM_RESULT adi_pwm_SetTripMode(
    ADI_PWM_HANDLE       const hDevice,
    ADI_PWM_CHANNEL      eChannel,
    ADI_PWM_TRIP_SOURCE  eTripSource,
    bool                 bMode
);

/* This function disables either the high side or low side output of a PWM channel.*/
ADI_PWM_RESULT adi_pwm_OutputEnable(
    ADI_PWM_HANDLE       	const hDevice,
    ADI_PWM_CHANNEL         eChannel,
    ADI_PWM_CHANNEL_OUTPUT  eChannelOutput,
    bool                    bEnable
);

/* This function enables / disables the crossover or switching of the outputs from the high side to the low side and vice versa. */
ADI_PWM_RESULT adi_pwm_HiLoCrossoverEnable(
    ADI_PWM_HANDLE       const hDevice,
    ADI_PWM_CHANNEL      eChannel,
    bool                 bEnable
);

/* This function sets the type of pulses generated on the outputs of a PWM channel */
ADI_PWM_RESULT adi_pwm_SetPulseMode(
    ADI_PWM_HANDLE       	const hDevice,
    ADI_PWM_CHANNEL      	eChannel,
    ADI_PWM_CHANNEL_OUTPUT  eChannelOutput,
    ADI_PWM_PULSE_TYPE      ePulse
);

/* This function enables application to clear the status register and recovery from a trip condition */
ADI_PWM_RESULT adi_pwm_ClearTrip(
    ADI_PWM_HANDLE const hDevice
);

/************************************/
/* Status register access          */
/************************************/
ADI_PWM_RESULT adi_pwm_GetStatusRegister(
	ADI_PWM_HANDLE hDevice,
    uint32_t       *pStatus
);

#ifdef __cplusplus 
} 
#endif /* __cplusplus */ 

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

#endif /* _ADI_PWM_BF6XX_H_ */
