/*********************************************************************************

Copyright(c) 2010 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_ctmr.h
*
* @brief     API header file for core timer service
*
* @details
*            This is the primary API header file for core timer service.
*
*/

/** @addtogroup Timer_Service Core Timer Service
*  @{
*/

#ifndef _ADI_CTMR_H_
#define _ADI_CTMR_H_

#include <stdint.h>
#include <stdbool.h>

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rules_all:"Suppress all rules for other header files")
#endif
#include <services/int/adi_int.h>
#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

/*! Core timer handle. */
typedef void*   ADI_CTMR_HANDLE;


/*! enumeration of possible return codes from core timer service. */
typedef enum _ADI_CTMR_RESULT_
{
    ADI_CTMR_SUCCESS  = 0,           /*!< Success */
    ADI_CTMR_FAILURE,                /*!< Generic Failure */
    ADI_CTMR_INT_FAILURE,            /*!< Core timer interrupt related failure. */
    ADI_CTMR_INVALID_HANDLE,         /*!< The given timer handle is invalid */
    ADI_CTMR_INVALID_TMR_ID,	     /*!< The given TMR ID is invalid */
    ADI_CTMR_IN_USE,                 /*!< Trying to open a timer which is already in use */
    ADI_CTMR_INVALID_POINTER         /*!< If any of the given pointers are invalid */

} ADI_CTMR_RESULT;

/*! enumeration of available core timers in the processor */
typedef enum _ADI_CTMR_DEVICE_
{
    ADI_CTMR_DEV0 = 0,              /*!< Timer 0 */

    ADI_MAX_CTMRS                   /*!< Maximum number of core timers available */

} ADI_CTMR_DEVICE;


/*! Enumeration of different events generated by core timer service */
typedef enum _ADI_CTMR_EVENT_
{
    ADI_CTIMER_EXPIRED             /*!< Core timer expired */


} ADI_CTMR_EVENT;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

ADI_CTMR_RESULT  adi_ctmr_Open (
    ADI_CTMR_DEVICE             eDevice,
    ADI_CALLBACK                pfCallback,
    void               *const   pCBParam,
    ADI_CTMR_HANDLE            *phCTimer
);


ADI_CTMR_RESULT  adi_ctmr_Close (
    ADI_CTMR_HANDLE      const hTimer
);

ADI_CTMR_RESULT  adi_ctmr_EnableAutoReload (
    ADI_CTMR_HANDLE      const  hTimer,
    bool                        bEnable
);

ADI_CTMR_RESULT  adi_ctmr_Enable (
    ADI_CTMR_HANDLE      const hTimer,
    bool                       bEnable
);

ADI_CTMR_RESULT  adi_ctmr_SetPeriod (
    ADI_CTMR_HANDLE      const hTimer,
    uint32_t                   nPeriod
);

ADI_CTMR_RESULT  adi_ctmr_SetScale (
    ADI_CTMR_HANDLE      const hTimer,
    uint32_t                   nScale
);

ADI_CTMR_RESULT  adi_ctmr_GetCount (
    ADI_CTMR_HANDLE      const hTimer,
    uint32_t                  *pCount
);

ADI_CTMR_RESULT  adi_ctmr_Activate (
    ADI_CTMR_HANDLE      const hTimer,
    bool                       bActive
);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _ADI_CTMR_H_ */

/* @} */


