/*********************************************************************************

Copyright(c) 2012 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*!
 * @file:    adi_tru.h
 * @brief:   Trigger Routing Unit Manager definitions and API for ADSP-BF60x processor family
 * @version: $Revision: 18272 $
 * @date:    $Date: 2013-03-06 01:16:36 -0500 (Wed, 06 Mar 2013) $
 *
 * @details
 *           This is the primary header file for the Trigger Routing Unit Manager
 *           for ADSP-BF60x and ADSP-CM40Z processor family, which contains the
 *           API declarations, data and constant definitions used in the APIs
 */

/** @addtogroup TRU_Manager Trigger Routing Unit Manager
 *  @{
 */

#ifndef __ADI_TRU_H__
#define __ADI_TRU_H__

/*=============  I N C L U D E S   =============*/

/* Interrupt Manager includes */
#include <services/int/adi_int.h>

/*==============  D E F I N E S  ===============*/

#if !defined(_LANGUAGE_ASM)

/**
 *  Memory size (in bytes) required to handle a TRU device instance
 */
#define ADI_TRU_REQ_MEMORY         (4u)

/**
 *  Trigger Routing Unit Handle
 */
typedef void* ADI_TRU_HANDLE;

/**
 *  TRU Manager return codes
 */
typedef enum
{
    ADI_TRU_SUCCESS = 0,        /*!< 0x00 - Generic success */
    ADI_TRU_FAILURE,            /*!< 0x01 - Generic failure */
    ADI_TRU_IN_USE,			    /*!< 0x02 - TRU Device already in use */
    ADI_TRU_BAD_HANDLE          /*!< 0x03 - TRU Device handle is invalid */
} ADI_TRU_RESULT;

/*=======  P U B L I C   P R O T O T Y P E S  ========*/
/*            (globally-scoped functions)             */

#ifdef __cplusplus
extern "C" {
#endif

/* Opens a Trigger Routing Unit */
ADI_TRU_RESULT  adi_tru_Open(
    uint32_t            DeviceNumber,
    void *const         pMemory,
    uint32_t            MemorySize,
    ADI_TRU_HANDLE      *phDevice);

/* Closes a Trigger Routing Unit */
ADI_TRU_RESULT  adi_tru_Close(
    ADI_TRU_HANDLE      hDevice);

/* Routes/Maps a trigger master to a slave */
ADI_TRU_RESULT  adi_tru_TriggerRoute(
    ADI_TRU_HANDLE      hDevice,
    uint32_t            TriggerSlave,
    uint32_t            TriggerMaster);

/* Enables/Disables Trigger Routing Unit */
ADI_TRU_RESULT  adi_tru_Enable(
    ADI_TRU_HANDLE      hDevice,
    bool                bEnable);

/* Configures Trigger Master register to generate outgoing Trigger(s) */
ADI_TRU_RESULT adi_tru_ConfigTriggerMaster(
    ADI_TRU_HANDLE      hDevice,
    uint32_t            MasterTriggerVal);

#ifdef __cplusplus
}
#endif

#endif

#endif  /* __ADI_TRU_H__ */

/*****/

/*@}*/
