/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_wd_bf6xx.h
*
* @brief     Watchdog timer API header file.
*
* @details   This is a header file which contains the APIs of watchdog timer for
*            BF6xx processor families.
**********************************************************************************/

/** @addtogroup Watchdog_Service_BF6xx Watchdog Service
*  @{
*/

#ifndef _ADI_WD_BF6XX_H_
#define _ADI_WD_BF6XX_H_

#ifndef  __ADI_WD_H__
#error "This file may not be directly included. The file <services/wd/adi_wd.h> should be included instead"
#endif

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"Allows doxygen addtogroup block")
#endif

#ifdef __cplusplus 
extern "C" { 
#endif /* __cplusplus */ 

#include <services/int/adi_int.h>

/*! Watchdog timer handle. */
typedef void*   ADI_WD_HANDLE;

/*! Enumeration of possible return codes from watchdog timer service. */
typedef enum _ADI_WD_RESULT_
{
    ADI_WD_SUCCESS  = 0u,          /*!< Success. */
    ADI_WD_FAILURE,                /*!< Generic failure. */
    ADI_WD_INVALID_HANDLE,         /*!< If the given watchdog timer handle is invalid. */
    ADI_WD_INVALID_POINTER,		   /*!< If the given watchdog pointer is invalid. */
    ADI_WD_IN_USE,                 /*!< If trying to open a watchdog timer which is already in use. */
    ADI_WD_INVALID_TIMERID,        /*!< Invalid watchdog timer ID. */
    ADI_WD_BAD_PARAMETER           /*!< Bad parameter. */
} ADI_WD_RESULT;

#define ADI_WD_ENABLE   0x00000FF0
#define ADI_WD_DISABLE  0x00000AD0
/*! Enumeration of possible watchdog timer event modes */
typedef enum _ADI_WD_EVENT_
{
    ADI_WD_EVENT_GP_INT            /*!< Generate a general purpose interrupt event. */
} ADI_WD_EVENT;


ADI_WD_RESULT  adi_wd_Open (
    uint32_t        Device,
    ADI_WD_HANDLE   *const phWDTimer,
    ADI_CALLBACK    const pfCallback,
    void            *const pCBParam
);

ADI_WD_RESULT  adi_wd_Close (
    ADI_WD_HANDLE    const hWDTimer
);

ADI_WD_RESULT  adi_wd_SetCounter(
    ADI_WD_HANDLE    const hWDTimer,
    uint32_t         WDCounter
);

ADI_WD_RESULT  adi_wd_GetCounter(
    ADI_WD_HANDLE    const hWDTimer,
    uint32_t         *pCounter
);

ADI_WD_RESULT  adi_wd_Reset(
    ADI_WD_HANDLE    const hWDTimer,
    uint32_t         WDCounter
);

ADI_WD_RESULT  adi_wd_Enable(
    ADI_WD_HANDLE    const hWDTimer,
    bool             bEnable
);

ADI_WD_RESULT  adi_wd_GetRollOverStatus(
    ADI_WD_HANDLE    const hWDTimer,
    bool             *pRolloverStatus
);


/* size of watch dog  instance */
#define ADI_WD_INSTANCE_SIZE (sizeof(ADI_WD))
/* number of EPPI instances */
#define ADI_WD_NUM_DEVICES (sizeof(gaWDTimers)/sizeof(ADI_WD))


#ifdef __cplusplus 
} 
#endif /* __cplusplus */ 

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

#endif /* _ADI_WD_BF6XX_H_ */

