/*********************************************************************************

Copyright(c) 2012 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

#ifndef _ADI_ADCC_DEF_H_
#define _ADI_ADCC_DEF_H_

/* Event registers for each event */
typedef struct
{
    uint32_t EVT;                          /* Event Time Register */
    uint32_t EVCTL;                        /* Event Control Register */
} ADI_ADCC_EVT_REG;

/* Timer DMA registers and timing registers */
typedef struct
{
    uint32_t BPTR;                         /* TIMER DMA base pointer               */
    uint32_t FRINC;                        /* TIMER DMA frame increment            */
    uint32_t CBSIZ;                        /* TIMER DMA Circular buffer size       */
    uint32_t TCA;                          /* ADC Timing control A register        */
    uint32_t TCB;                          /* ADC Timing control B register        */
    uint32_t BWMON;                        /* TIMER0 Bandwidth monitor register    */

} ADI_ADCC_TMR_DMA_REG;

/* Timer status registers */
typedef struct
{
    uint32_t STAT;                         /* Timer Status register                */
    uint32_t CNTR;                         /* Timer Current Count                  */

} ADI_ADCC_TMR_STAT_REG;


/* ADC Controller registers */
typedef struct
{
    uint32_t CTL;                           /* ADCC Control register               */
    uint32_t ERRSTAT;                       /* Error status register               */
    uint32_t ERRMSK;                        /* Error Mask register                 */
    uint32_t ERRMSK_SET;                    /* Error Mask Set register             */
    uint32_t ERRMSK_CLR;                    /* Error Mask Clear regsiter           */
    uint32_t EISTAT;                        /* Event interrupt status              */
    uint32_t EIMSK;                         /* Event interrupt Mask register       */
    uint32_t EIMSK_SET;                     /* Event interrupt Mask Set register   */
    uint32_t EIMSK_CLR;                     /* Event interrupt Mask Clear register */
    uint32_t FISTAT;                        /* Frame interrupt status register     */
    uint32_t FIMSK;                         /* Frame interrupt Mask register       */
    uint32_t FIMSK_SET;                     /* Frame interrupt Mask Set register   */
    uint32_t FIMSK_CLR;                     /* Frame interrupt Mask Clear register */
    uint32_t EVTEN;                         /* Event enable register               */
    uint32_t EVTEN_SET;                     /* Event Enable Set register           */
    uint32_t EVTEN_CLR;                     /* Event Enable Clear Register         */
    uint32_t ECOL;                          /* Event Collision status register     */
    uint32_t EMISS;                         /* Event Miss status register          */
    uint32_t BPTR0;                         /* TIMER0 DMA base pointer             */
    uint32_t FRINC0;                        /* TIMER0 DMA frame increment          */
    uint32_t CBSIZ0;                        /* TIMER0 DMA Circular buffer size     */
    uint32_t TCA0;                          /* ADC0 Timing control A register      */
    uint32_t TCB0;                          /* ADC0 Timing control B register      */
    uint32_t BWMON0;                        /* TIMER0 Bandwidth monitor register   */
    uint32_t CFG;                           /* ADC Configuration register          */
    uint32_t BPTR1;                         /* TIMER0 DMA base pointer             */
    uint32_t FRINC1;                        /* TIMER0 DMA frame increment          */
    uint32_t CBSIZ1;                        /* TIMER0 DMA Circular buffer size     */
    uint32_t TCA1;                          /* ADC0 Timing control A register      */
    uint32_t TCB1;                          /* ADC0 Timing control B register      */
    uint32_t BWMON1;                        /* TIMER0 Bandwidth monitor register   */
    ADI_ADCC_EVT_REG EVT_REG[24];           /* Event registers for each event      */
    uint8_t  RESERVED0[196];                /* reserved                            */
    uint32_t EPND;                          /* Pending events status               */
    ADI_ADCC_TMR_STAT_REG TMRSTAT[2];       /* Timer status registers              */
    uint32_t EVDAT[24];                     /* Event n Data register               */
    uint32_t EVSTAT[24];                    /* Event n Status register             */
} ADI_ADCC_REG;

/* enumeration of device states */
typedef enum
{
    
    ADI_ADCC_STATE_NOT_OPENED,                                          /* The device is not yet opened */
    ADI_ADCC_STATE_OPENED,                                              /* The device is opened */
    ADI_ADCC_STATE_ENABLED                                              /* The data flow is enabled. */

} ADI_ADCC_STATE;


/* enumeration of timer states */
typedef enum
{
    ADI_ADCC_TMR_STATE_NOT_OPENED,                                      /* The timer is not yet opened */
    ADI_ADCC_TMR_STATE_OPENED,                                          /* The device is opened */
    ADI_ADCC_TMR_STATE_ENABLED                                          /* The data flow is enabled. */

} ADI_ADCC_TMR_STATE;

/* enumeration of different IO modes */
typedef enum
{
    ADI_ADCC_TMR_IOMODE_LINEAR,                                         /* Linear DMA mode */
    ADI_ADCC_TMR_IOMODE_CIRC                                            /* Circular DMA Mode */

} ADI_ADCC_TMR_IOMODE;

/* enumeration of buffer states */
typedef enum
{
    ADI_ADCC_BUF_STATE_FREE,                                            /* No buffer submittted (Buffer taken back by application) */
    ADI_ADCC_BUF_STATE_INUSE,                                           /* Buffer is being processed */
    ADI_ADCC_BUF_STATE_PROCESSED                                        /* Buffer processed and ready to be taken by application */

} ADI_ADCC_BUF_STATE;


/* Structure which holds the buffer related information */
typedef struct
{
    ADI_ADCC_BUF_STATE              eBufState;                          /* Buffer state */
    void *                          pBuffer;                            /* Buffer pointer */
    uint32_t                        nBufSize;                           /* Buffer size in bytes */
} ADI_ADCC_BUF_INFO;

/* Structure which holds the device data */
typedef struct
{
    ADI_CALLBACK                    pfCallback;                         /* Pointer to the device callback */
    void *                          pCBParam;                           /* Callback parameter */
    bool                            bDMAMode;                           /* Flag which indicates whether DMA mode is enabled */
} ADI_ADCC_DEV_DATA;

/* forward declaration of ADCC device */
typedef struct _ADI_ADCC_DEVICE_*   ADI_ADCC_DEVICEP;

/* Structure which timer specific data */
typedef struct
{
    ADI_CALLBACK                    pfCallback;                         /* Pointer to the timer callback */
    void *                          pCBParam;                           /* Callback parameter */
    ADI_ADCC_TMR_IOMODE             eIOMode;                            /* IO mode */
    ADI_ADCC_TMR_STATE              eTmrState;                          /* Timer state */
    uint32_t                        TmrMask;                            /* Mask for the timers that are configured */
    ADI_OSAL_SEM_HANDLE             hTmrSem;                            /* Handle to the semaphore */

    /* Memory required to open semaphore */
    uint8_t                         SemMemory[ADI_OSAL_MAX_SEM_SIZE_CHAR];
    ADI_ADCC_DEVICEP                pADCCDevice;                        /* Pionter to ADCC device */
    ADI_ADCC_BUF_INFO               BufInfo;                            /* Holds the buffer information */
    volatile uint32_t               HwErrStatus;                        /* Hardware error status */
} ADI_ADCC_TMR_DATA;

/* Structure which holds the timer related data */
typedef struct
{
    const uint32_t                          nTimerID;                   /* Timer ID */
    const uint32_t                          nTmrIntID;                  /* Timer Interrupt ID */
    volatile ADI_ADCC_REG *const            pADCCRegs;                  /* Pointer to ADCC registers */
    volatile ADI_ADCC_TMR_DMA_REG * const   pTmrDMARegs;                /* Timer DMA related registers */
    ADI_ADCC_TMR_DATA *                     pTmrData;                   /* Pointer to timer Data */
} ADI_ADCC_TMR_DEVICE;

/* Structure which holds mostly static information */
typedef struct _ADI_ADCC_DEVICE_
{   
    ADI_ADCC_STATE                  eADCCState;                         /* ADCC device state */
    volatile ADI_ADCC_REG *const    pADCCRegs;                          /* Pointer to ADCC registers */
    const uint32_t                  nErrorIntID;                        /* Error interrupt ID */
    ADI_ADCC_DEV_DATA *             pADCCData;                          /* ADCC device data */
    ADI_ADCC_TMR_DEVICE             TmrDevice[ADI_ADCC_MAX_TIMERS];     /* pointer to the timer data structure */
	
} ADI_ADCC_DEVICE;

/* Number of ADCC Devices in the processor */
#define ADI_ADCC_NUM_DEVICES            (sizeof(gADCCDevices)/sizeof(ADI_ADCC_DEVICE))

/*
 *  Local APIs
 */

/* Error interrupt handler */
static void ErrorIntHandler(uint32_t iid, void *pCBParam);

/* Timer related interrupt handler */
static void TimerIntHandler(uint32_t iid, void *pCBParam);

#ifdef ADI_DEBUG

/* Validate the device */
static ADI_ADCC_RESULT ValidateDevice(ADI_ADCC_DEVICE     *pDevice);

/* Validate the timer */
static ADI_ADCC_RESULT ValidateTimer(ADI_ADCC_TMR_DEVICE     *pTmrDevice);

#endif /* ADI_DEBUG */

#endif /* _ADI_ADCC_DEF_H_ */
