/*********************************************************************************

Copyright(c) 2013 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

#if defined(ADI_ADSP_CM40Z)
/*
 * Enable DMA support. Note that programmed i/o and dma support can be 
 * dynamically controlled. If driver is expected to be used only in DMA
 * mode you can turn off the PIO mode to save space and execution time.
 */
#ifndef ADI_DACC_CFG_DMA
#define ADI_DACC_CFG_DMA   (1)
#endif

/*
 * Enable PIO support. Note that programmed i/o and dma support can be 
 * dynamically controlled. If driver is expected to be used only in DMA
 * mode you can turn off the DMA mode to save space and execution time.
 */
#ifndef ADI_DACC_CFG_PIO
#define ADI_DACC_CFG_PIO   (1)
#endif

/*
 * Enable it to use the APIs to run the system in blocking mode.Note that
 * in Blocking mode callbacks are not allowed. Once callback DACC buffer 
 * callback is installed driver goes into non-blocking mode.If your application
 * is not using blocking calls at all then you can set this flag to false to
 * save space and execution time.
 */
#ifndef ADI_DACC_CFG_BIO
#define ADI_DACC_CFG_BIO  (1) 
#endif 


/* This flag gets sets if both DMA and PIO modes are enabled. This enables both
 * DMA and PIO features of the driver and applications can set the mode at the
 * runtime. In circular buffer mode once DACC is enabled application can not change
 * the mode at runtime. 
 */ 
#ifndef ADI_DACC_CFG_ALL
#define ADI_DACC_CFG_ALL (ADI_DACC_CFG_DMA & ADI_DACC_CFG_PIO)
#endif

/* Collect statistical information */
#ifndef ADI_DACC_CFG_STATS
#define ADI_DACC_CFG_STATS (0)
#endif

/* DACC device initializations */
ADI_DACC_DEVICE     gDACCDevices[] = 
{
        {   
	    ADI_DACC_STATE_NOT_OPENED,                      /* DACC device state            */
	    ADI_DACC_DACIF0,                                /* DACC device interface number */
	    (ADI_DACC_CTRL_REG *) REG_DACC0_CTL0,           /* Pointer to DAC control regs  */
	    (ADI_DACC_REGS *) REG_DACC0_CTL0,               /* Pointer to DAC registers     */ 
	    (ADI_DACC_DMA_REGS *) REG_DACC0_TC0,            /* Pointer to DAC DMA regs      */
            NULL,                                           /* DACC device data             */
	},
        {   
	    ADI_DACC_STATE_NOT_OPENED,                      /* DACC device state            */
	    ADI_DACC_DACIF1,                                /* DACC device interface number */
	    (ADI_DACC_CTRL_REG *) REG_DACC0_CTL1,           /* Pointer to DAC control regs  */
	    (ADI_DACC_REGS *) (REG_DACC0_CTL0),             /* Pointer to DAC registers     */
	    (ADI_DACC_DMA_REGS *) (REG_DACC0_TC1),          /* Pointer to DAC dma regs      */
            NULL,                                           /* DACC device data             */
	}
};

/* DAC statistical collection block */
#if ADI_DACC_CFG_STATS
static ADI_DACC_STATS          gDaccStats = {0};    
#endif

#else
#error "Processor not supported"
#endif
