/*
*********************************************************************************************************
*                                             uC/TCP-IP V2
*                                      The Embedded TCP/IP Suite
*
*                          (c) Copyright 2003-2012; Micrium, Inc.; Weston, FL
*
*               All rights reserved.  Protected by international copyright laws.
*
*               uC/TCP-IP is provided in source form to registered licensees ONLY.  It is
*               illegal to distribute this source code to any third party unless you receive
*               written permission by an authorized Micrium representative.  Knowledge of
*               the source code may NOT be used to develop a similar product.
*
*               Please help us continue to provide the Embedded community with the finest
*               software available.  Your honesty is greatly appreciated.
*
*               You can contact us at www.micrium.com.
*********************************************************************************************************
*/

/********************************************************************************************************
 *
 *              Portions Copyright (c) 2013 Analog Devices, Inc
 *
 *              When this driver is used with uC/TCP-IP stack a proper license of uC/TCP-IP must be 
 *              obtained from Analog Devices or Micrium.
 *
 ********************************************************************************************************
 */

/*
*********************************************************************************************************
*
*                                        NETWORK DEVICE DRIVER
*
*                                        CM408F MAC 10/100
*
* Filename      : net_dev_cm408f.c
* Version       : V2.12.00.00
* Programmer(s) : AF
*                 EHS
*                 AA
*                 AP
*********************************************************************************************************
* Note(s)       : (1) Assumes uC/TCP-IP V2.02 (or more recent version) is included in the project build.
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                            INCLUDE FILES
*********************************************************************************************************
*/
#include  <net.h>
#include  <net_dev_cm408f.h>

/*
*********************************************************************************************************
*                                            LOCAL DEFINES
*********************************************************************************************************
*/
                                                                               /* -------- ETHERNET MAC addresses OFFSET ------- */
#define  ETH_MAC_HASH_TBL_HIGH                             0x00000000
#define  ETH_MAC_HASH_TBL_LOW                              0x00000000
#define  ETH_MAC_VLAN_TAG_COMPARAISON_16BIT                0x00000000
#define  ETH_MAC_VLAN_TAG_COMPARAISON_12BIT                0x00010000
#define  ETH_DMA_DESC_SKIP_LEN                             0x00000000
#define  ETH_VLAN_TAG_IDENTIFIER                           0x00000000
#define  ETH_MACMIIAR_CR_MASK                              0xFFFFFFE3          /* ETHERNET MACMIIAR register Mask                */
#define  ETH_MACCR_CLR_MASK                                0xFF20810F          /* ETHERNET MACCR register Mask                   */
#define  ETH_MACFCR_CLR_MASK                               0x0000FF41          /* ETHERNET MACFCR register Mask                  */
#define  ETH_DMAOMR_CLR_MASK                               0xF8DE3F23          /* ETHERNET DMAOMR register Mask                  */
#define  ETH_DMA_RX_DESC_FRAME_LEN_SHIFT                           16          /* ETHERNET DMA Rx descriptors Frame Length Shift */

                                                                               /* ---------- PHY Read/write Timeouts ----------- */
#define  PHY_RD_TO                                         0x0004FFFF
#define  PHY_WR_TO                                         0x0004FFFF


#define  MAC_CR_DIV42                                      0x00000000          /* HCLK:60-72 MHz; MDC clock= HCLK/42             */
#define  MAC_CR_DIV16                                      0x00000008          /* HCLK:20-35 MHz; MDC clock= HCLK/16             */
#define  MAC_CR_DIV26                                      0x0000000C          /* HCLK:35-60 MHz; MDC clock= HCLK/26             */

#define  RX_BUF_SIZE_MULT                                          16          /* Rx bufs must be a mul   of 16 bytes.           */
#define  RX_DESC_BUF_ALIGN                                         16          /* Rx bufs must be aligned to 16 byte boundary.   */
#define  TX_DESC_BUF_ALIGN                                         16          /* Tx bufs must be aligned to 16 byte boundary.   */


/*$PAGE*/
/*
*********************************************************************************************************
*                                       REGISTER BIT DEFINITIONS
*********************************************************************************************************
*/
                                                                               /* --------- MAC CONFIGURATION REGISTER --------- */
#define  ETH_MACCR_TC                                      0x01000000          /* Transmit Configuration in RGMII/SGMII bit      */
#define  ETH_MACCR_WD                                      0x00800000          /* Watchdog Disable bit                           */
#define  ETH_MACCR_JD                                      0x00400000          /* Jabber Disable bit                             */
#define  ETH_MACCR_BE                                      0x00200000          /* Frame Burst Enable bit                         */
#define  ETH_MACCR_JE                                      0x00100000          /* Jumbo Frame Enable bit                         */
#define  ETH_MACCR_IFG_96BITS                              0x00000000          /* Inter-Frame Gap bits: 96 bit times             */
#define  ETH_MACCR_IFG_88BITS                              0x00020000          /* Inter-Frame Gap bits: 88 bit times             */
#define  ETH_MACCR_IFG_80BITS                              0x00040000          /* Inter-Frame Gap bits: 80 bit times             */
#define  ETH_MACCR_IFG_72BITS                              0x00060000          /* Inter-Frame Gap bits: 72 bit times             */
#define  ETH_MACCR_IFG_64BITS                              0x00080000          /* Inter-Frame Gap bits: 64 bit times             */
#define  ETH_MACCR_IFG_56BITS                              0x000A0000          /* Inter-Frame Gap bits: 56 bit times             */
#define  ETH_MACCR_IFG_48BITS                              0x000C0000          /* Inter-Frame Gap bits: 48 bit times             */
#define  ETH_MACCR_IFG_40BITS                              0x000E0000          /* Inter-Frame Gap bits: 40 bit times             */
#define  ETH_MACCR_DCRS                                    0x00010000          /* Disable Carrier Sense During Transmission bit  */
#define  ETH_MACCR_PS                                      0x00008000          /* Port Select bit                                */
#define  ETH_MACCR_FES                                     0x00004000          /* Speed 0:10Mbps  1:100Mbps                      */
#define  ETH_MACCR_ROD                                     0x00002000          /* Disable Receive Own bit                        */
#define  ETH_MACCR_LM                                      0x00001000          /* Loopback Mode bit                              */
#define  ETH_MACCR_DM                                      0x00000800          /* Duplex Mode bit                                */
#define  ETH_MACCR_IPC                                     0x00000400          /* Checksum Offload bit                           */
#define  ETH_MACCR_DR                                      0x00000200          /* Disable Retry bit                              */
#define  ETH_MACCR_LUD                                     0x00000100          /* Link Up/Down  bit                              */
#define  ETH_MACCR_ACS                                     0x00000080          /* Automatic Pad/CRC Stripping bit                */
#define  ETH_MACCR_BL_10                                   0x00000000          /* Back-Off Limit                                 */
#define  ETH_MACCR_BL_8                                    0x00000020
#define  ETH_MACCR_BL_4                                    0x00000040
#define  ETH_MACCR_BL_1                                    0x00000060
#define  ETH_MACCR_DC                                      0x00000010          /* Back-Off Limit bit                             */
#define  ETH_MACCR_TE                                      0x00000008          /* Transmitter Enable bit                         */
#define  ETH_MACCR_RE                                      0x00000004          /* Receiver Enable bit                            */

                                                                               /* --------- MAC FRAME FILTER REGISTER ---------- */
#define  ETH_MACFFR_RA                                     0x80000000          /* Receive All bit                                */
#define  ETH_MACFFR_HPF                                    0x00000400          /* Hash or Perfect Filter bit                     */
#define  ETH_MACFFR_SAF                                    0x00000200          /* Source Address Filter Enable bit               */
#define  ETH_MACFFR_SAIF                                   0x00000100          /* SA Inverse Filtering bit                       */
#define  ETH_MACFFR_PCF_0x                                 0x00000040          /* Pass Control Frames bit                        */
#define  ETH_MACFFR_PCF_10                                 0x00000080
#define  ETH_MACFFR_PCF_11                                 0x000000C0
#define  ETH_MACFFR_DBF                                    0x00000020          /* Disable Broadcast Frames bit                   */
#define  ETH_MACFFR_PM                                     0x00000010          /* Pass All Multicast bit                         */
#define  ETH_MACFFR_DAIF                                   0x00000008          /* DA Inverse Filtering bit                       */
#define  ETH_MACFFR_HMC                                    0x00000004          /* Hash Multicast bit                             */
#define  ETH_MACFFR_HUC                                    0x00000002          /* Hash Unicast bit                               */
#define  ETH_MACFFR_PR                                     0x00000001          /* Promiscuous Mode bit                           */

                                                                               /* --------- MAC FLOW CONTROL  REGISTER --------- */
#define  ETH_MACFCR_ZQPD                                   0x00000080          /* Disable Zero-Quanta Pause bit                  */
#define  ETH_MACFCR_PLT_00                                 0x00000000          /* Pause Low Threshold : 4 slot times             */
#define  ETH_MACFCR_PLT_01                                 0x00000010          /* Pause Low Threshold : 28 slot times            */
#define  ETH_MACFCR_PLT_10                                 0x00000020          /* Pause Low Threshold : 144 slot times           */
#define  ETH_MACFCR_PLT_11                                 0x00000030          /* Pause Low Threshold : 256 slot times           */
#define  ETH_MACFCR_UP                                     0x00000008          /* Unicast Pause Frame Detect bit                 */
#define  ETH_MACFCR_RFE                                    0x00000004          /* Receive Flow Control Enable bit                */
#define  ETH_MACFCR_TFE                                    0x00000002          /* Transmit Flow Control Enable bit               */
#define  ETH_MACFCR_FCB_BPA                                0x00000001          /* Flow Control Busy/Backpressure Activate bit    */
/*$PAGE*/

                                                                               /* -------- DMA OPERATION MODE REGISTER --------- */
#define  ETH_DMAOMR_DTCEFD                                 0x04000000          /* Disable Dropp. of TCP/IP Checksum Error Frames */
#define  ETH_DMAOMR_RSF                                    0x02000000          /* Receive Store and Forward bit                  */
#define  ETH_DMAOMR_DFF                                    0x01000000          /* Disable Flushing of Received Frames bit        */
#define  ETH_DMAOMR_RFA                                    0x00800000          /* MSB of Threshold for Activating Flow Control   */
#define  ETH_DMAOMR_RFD                                    0x00400000          /* MSB of Threshold for Activating Flow Control   */
#define  ETH_DMAOMR_TSF                                    0x00200000          /* Transmit Store and Forward bit                 */
#define  ETH_DMAOMR_FTF                                    0x00100000          /* Flush Transmit FIFO bit                        */
#define  ETH_DMAOMR_TTC_64                                 0x00000000          /* Transmit FIFO Threshold Control: level 64      */
#define  ETH_DMAOMR_TTC_128                                0x00004000          /* Transmit FIFO Threshold Control: level 128     */
#define  ETH_DMAOMR_TTC_192                                0x00008000          /* Transmit FIFO Threshold Control: level 192     */
#define  ETH_DMAOMR_TTC_256                                0x0000C000          /* Transmit FIFO Threshold Control: level 256     */
#define  ETH_DMAOMR_TTC_40                                 0x00010000          /* Transmit FIFO Threshold Control: level 40      */
#define  ETH_DMAOMR_TTC_32                                 0x00014000          /* Transmit FIFO Threshold Control: level 32      */
#define  ETH_DMAOMR_TTC_24                                 0x00018000          /* Transmit FIFO Threshold Control: level 24      */
#define  ETH_DMAOMR_TTC_16                                 0x0001C000          /* Transmit FIFO Threshold Control: level 16      */
#define  ETH_DMAOMR_ST                                     0x00002000          /* Start/Stop Transmission Command bit            */
#define  ETH_DMAOMR_RFD_1KB                                0x00000000          /* Threshold for deactivating flow control        */
#define  ETH_DMAOMR_RFD_2KB                                0x00000800
#define  ETH_DMAOMR_RFD_3KB                                0x00001000
#define  ETH_DMAOMR_RFD_4KB                                0x00001800
#define  ETH_DMAOMR_RFA_1KB                                0x00000000          /* Threshold for activating flow control          */
#define  ETH_DMAOMR_RFA_2KB                                0x00000200
#define  ETH_DMAOMR_RFA_3KB                                0x00000400
#define  ETH_DMAOMR_RFA_4KB                                0x00000600
#define  ETH_DMAOMR_EFC                                    0x00000100          /* Enable HW flow control bit                     */
#define  ETH_DMAOMR_FEF                                    0x00000080          /* Forward Error Frames bit                       */
#define  ETH_DMAOMR_FUF                                    0x00000040          /* Forward Undersized Good Frames bit             */
#define  ETH_DMAOMR_RTC_64                                 0x00000000          /* Receive FIFO Threshold Control : level 64      */
#define  ETH_DMAOMR_RTC_32                                 0x00000008          /* Receive FIFO Threshold Control : level 32      */
#define  ETH_DMAOMR_RTC_96                                 0x00000010          /* Receive FIFO Threshold Control : level 96      */
#define  ETH_DMAOMR_RTC_128                                0x00000018          /* Receive FIFO Threshold Control : level 128     */
#define  ETH_DMAOMR_OSF                                    0x00000004          /* Operate on Second Frame bit                    */
#define  ETH_DMAOMR_SR                                     0x00000002          /* Start/Stop Receive bit                         */

                                                                               /* ----------- DMA BUS MODE REGISTER ------------ */
#define  ETH_DMABMR_AAB                                    0x02000000          /* Address-Aligned Beats bit                      */
#define  ETH_DMABMR_4xPBL                                  0x01000000          /* 4xPBL Mode bit                                 */
#define  ETH_DMABMR_USP                                    0x00800000          /* Use Separate PBL bit                           */
#define  ETH_DMABMR_RX_DMA_1_BEAT                          0x00020000          /* RxDMA PBL : 1 Beat                             */
#define  ETH_DMABMR_RX_DMA_2_BEAT                          0x00040000          /* RxDMA PBL : 2 Beats                            */
#define  ETH_DMABMR_RX_DMA_4_BEAT                          0x00080000          /* RxDMA PBL : 4 Beats                            */
#define  ETH_DMABMR_RX_DMA_8_BEAT                          0x00100000          /* RxDMA PBL : 8 Beats                            */
#define  ETH_DMABMR_RX_DMA_16_BEAT                         0x00200000          /* RxDMA PBL : 16 Beats                           */
#define  ETH_DMABMR_RX_DMA_32_BEAT                         0x00400000          /* RxDMA PBL : 32 Beats                           */
#define  ETH_DMABMR_TX_DMA_1_BEAT                          0x00000100          /* TxDMA PBL : 1 Beat                             */
#define  ETH_DMABMR_TX_DMA_2_BEAT                          0x00000200          /* TxDMA PBL : 2 Beats                            */
#define  ETH_DMABMR_TX_DMA_4_BEAT                          0x00000400          /* TxDMA PBL : 4 Beats                            */
#define  ETH_DMABMR_TX_DMA_8_BEAT                          0x00000800          /* TxDMA PBL : 8 Beats                            */
#define  ETH_DMABMR_TX_DMA_16_BEAT                         0x00001000          /* TxDMA PBL : 16 Beats                           */
#define  ETH_DMABMR_TX_DMA_32_BEAT                         0x00002000          /* TxDMA PBL : 32 Beats                           */
#define  ETH_DMABMR_RX_TX_DMA_PRIO_1_1                     0x00000000          /* Rx:Tx priority ratio : 1:1                     */
#define  ETH_DMABMR_RX_TX_DMA_PRIO_2_1                     0x00004000          /* Rx:Tx priority ratio : 2:1                     */
#define  ETH_DMABMR_RX_TX_DMA_PRIO_3_1                     0x00008000          /* Rx:Tx priority ratio : 3:1                     */
#define  ETH_DMABMR_RX_TX_DMA_PRIO_4_1                     0x0000C000          /* Rx:Tx priority ratio : 4:1                     */
#define  ETH_DMABMR_ATDS                                   0x00000080          /* Alternate descriptor size                      */
#define  ETH_DMABMR_SR                                     0x00000001          /* Software reset                                 */

                                                                               /* ------------ DMA STATUS REGISTER ------------- */
#define  ETH_DMASR_NIS                                     0x00010000          /* Normal interrupt summary                       */
#define  ETH_DMASR_AIS                                     0x00008000          /* Abnormal interrupt summary                     */
#define  ETH_DMASR_ERS                                     0x00004000          /* Early receive status                           */
#define  ETH_DMASR_FBES                                    0x00002000          /* Fatal bus error status                         */
#define  ETH_DMASR_ETS                                     0x00000400          /* Early transmit status                          */
#define  ETH_DMASR_RWTS                                    0x00000200          /* Receive watchdog timeout status                */
#define  ETH_DMASR_RPSS                                    0x00000100          /* Receive process stopped status                 */
#define  ETH_DMASR_RBUS                                    0x00000080          /* Receive buffer unavailable status              */
#define  ETH_DMASR_RS                                      0x00000040          /* Receive status                                 */
#define  ETH_DMASR_TUS                                     0x00000020          /* Transmit underflow status                      */
#define  ETH_DMASR_ROS                                     0x00000010          /* Receive overflow status                        */
#define  ETH_DMASR_TJTS                                    0x00000008          /* Transmit jabber timeout status                 */
#define  ETH_DMASR_TBUS                                    0x00000004          /* Transmit buffer unavailable status             */
#define  ETH_DMASR_TPSS                                    0x00000002          /* Transmit process stopped status                */
#define  ETH_DMASR_TS                                      0x00000001          /* Transmit status                                */

                                                                               /* ------- DMA INTERRUPT ENBALE  REGISTER ------- */
#define  ETH_DMA_IT_NIS                                    0x00010000          /* Normal interrupt summary                       */
#define  ETH_DMA_IT_AIS                                    0x00008000          /* Abnormal interrupt summary                     */
#define  ETH_DMA_IT_ER                                     0x00004000          /* Early receive interrupt                        */
#define  ETH_DMA_IT_FBE                                    0x00002000          /* Fatal bus error interrupt                      */
#define  ETH_DMA_IT_ET                                     0x00000400          /* Early transmit interrupt                       */
#define  ETH_DMA_IT_RWT                                    0x00000200          /* Receive watchdog timeout interrupt             */
#define  ETH_DMA_IT_RPS                                    0x00000100          /* Receive process stopped interrupt              */
#define  ETH_DMA_IT_RBU                                    0x00000080          /* Receive buffer unavailable interrupt           */
#define  ETH_DMA_IT_R                                      0x00000040          /* Receive interrupt                              */
#define  ETH_DMA_IT_TU                                     0x00000020          /* Underflow interrupt                            */
#define  ETH_DMA_IT_RO                                     0x00000010          /* Overflow interrupt                             */
#define  ETH_DMA_IT_TJT                                    0x00000008          /* Transmit jabber timeout interrupt              */
#define  ETH_DMA_IT_TBU                                    0x00000004          /* Transmit buffer unavailable interrupt          */
#define  ETH_DMA_IT_TPS                                    0x00000002          /* Transmit process stopped interrupt             */
#define  ETH_DMA_IT_T                                      0x00000001          /* Transmit interrupt                             */
/*$PAGE*/

                                                                               /* ---------- MAC MII ADDRESS REGISTER ---------- */
#define  ETH_MACMIIAR_PA                                   0x0000F800          /* Physical layer address                         */
#define  ETH_MACMIIAR_MR                                   0x000007C0          /* MII register in the selected PHY               */
#define  ETH_MACMIIAR_CR                                   0x0000001C          /* CR clock range: 6 cases                        */
#define  ETH_MACMIIAR_CR_DIV42                             0x00000000          /* HCLK:60-72 MHz; MDC clock= HCLK/42             */
#define  ETH_MACMIIAR_CR_DIV16                             0x00000008          /* HCLK:20-35 MHz; MDC clock= HCLK/16             */
#define  ETH_MACMIIAR_CR_DIV26                             0x0000000C          /* HCLK:35-60 MHz; MDC clock= HCLK/26             */
#define  ETH_MACMIIAR_MW                                   0x00000002          /* MII write                                      */
#define  ETH_MACMIIAR_MB                                   0x00000001          /* MII busy                                       */


/*
*********************************************************************************************************
*                                    Ethernet DMA descriptors registers bits definition
*
* Note(s) : (1) Rx Descriptor :
*               -----------------------------------------------------------------------------------------------------------------
*               RDES0 | OWN(31) |                                             Status [30:0]                                      |
*               -----------------------------------------------------------------------------------------------------------------
*               RDES1 | CTRL(31) | Reserved[30:29] | Buffer2 ByteCnt[28:16] | CTRL[15:14] | Reserved(13) | Buffer1 ByteCnt[12:0] |
*               -----------------------------------------------------------------------------------------------------------------
*               RDES2 |                                       Buffer1 Address [31:0]                                             |
*               -----------------------------------------------------------------------------------------------------------------
*               RDES3 |                          Buffer2 Address [31:0] / Next Desciptor Address [31:0]                          |
*               -----------------------------------------------------------------------------------------------------------------
*
*           (2) Tx Descriptor :
*               -----------------------------------------------------------------------------------------------------------------
*               TDES0 | OWN(31) | CTRL[30:26] | Reserved[25:24] | CTRL[23:20] | Reserved[19:17] | Status[16:0]                   |
*               -----------------------------------------------------------------------------------------------------------------
*               TDES1 | Reserved[31:29] | Buffer2 ByteCount[28:16] | Reserved[15:13] | Buffer1 ByteCount[12:0]                   |
*               -----------------------------------------------------------------------------------------------------------------
*               TDES2 |                         Buffer1 Address [31:0]                                                           |
*               -----------------------------------------------------------------------------------------------------------------
*               TDES3 |                   Buffer2 Address [31:0] / Next Desciptor Address [31:0]                                 |
*               -----------------------------------------------------------------------------------------------------------------
*********************************************************************************************************
*/

                                                                               /* ---- TDES0 : DMA Tx descriptor status Reg ---- */
#define  ETH_DMA_TX_DESC_OWN                               0x80000000          /* OWN bit: descriptor is owned by DMA engine     */
#define  ETH_DMA_TX_DESC_IC                                0x40000000          /* Interrupt on Completion                        */
#define  ETH_DMA_TX_DESC_LS                                0x20000000          /* Last Segment                                   */
#define  ETH_DMA_TX_DESC_FS                                0x10000000          /* First Segment                                  */
#define  ETH_DMA_TX_DESC_DC                                0x08000000          /* Disable CRC                                    */
#define  ETH_DMA_TX_DESC_DP                                0x04000000          /* Disable Padding                                */
#define  ETH_DMA_TX_DESC_TTSE                              0x02000000          /* Transmit Time Stamp Enable                     */
#define  ETH_DMA_TX_DESC_CIC                               0x00C00000          /* Checksum Insertion Control: 4 cases            */
#define  ETH_DMA_TX_DESC_CIC_BYPASS                        0x00000000          /* Do Nothing: Checksum Engine is bypassed        */
#define  ETH_DMA_TX_DESC_CIC_IPV4_HEADER                   0x00400000          /* IPV4 header Checksum Insertion                 */
#define  ETH_DMA_TX_DESC_CIC_TCPUDPICMP_SEGMENT            0x00800000          /* TCP/UDP/ICMP Checksum Insertion calculated     */
                                                                               /* over segment only                              */
#define  ETH_DMA_TX_DESC_CIC_TCPUDPICMP_FULL               0x00C00000          /* TCP/UDP/ICMP Checksum Insertion fully          */
                                                                               /* calculated                                     */
#define  ETH_DMA_TX_DESC_TER                               0x00200000          /* Transmit End of Ring                           */
#define  ETH_DMA_TX_DESC_TCH                               0x00100000          /* Second Address Chained                         */
#define  ETH_DMA_TX_DESC_TTSS                              0x00020000          /* Tx Time Stamp Status                           */
#define  ETH_DMA_TX_DESC_IHE                               0x00010000          /* IP Header Error                                */
#define  ETH_DMA_TX_DESC_ES                                0x00008000          /* Error summary: OR of the following bits:       */
                                                                               /* UE || ED || EC || LCO || NC || LCA || FF || JT */
#define  ETH_DMA_TX_DESC_JT                                0x00004000          /* Jabber Timeout                                 */
#define  ETH_DMA_TX_DESC_FF                                0x00002000          /* Frame Flushed: DMA/MTL flushed the frame due   */
                                                                               /* to SW flush                                    */
#define  ETH_DMA_TX_DESC_PCE                               0x00001000          /* Payload Checksum Error                         */
#define  ETH_DMA_TX_DESC_LCA                               0x00000800          /* Loss of Carrier: carrier lost during           */
                                                                               /* tramsmission                                   */
#define  ETH_DMA_TX_DESC_NC                                0x00000400          /* No Carrier: no carrier signal from the         */
                                                                               /* tranceiver                                     */
#define  ETH_DMA_TX_DESC_LCO                               0x00000200          /* Late Collision: transmission aborted due       */
                                                                               /* to collision                                   */
#define  ETH_DMA_TX_DESC_EC                                0x00000100          /* Excessive Collision: transmission aborted      */
                                                                               /* after 16 collisions                            */
#define  ETH_DMA_TX_DESC_VF                                0x00000080          /* VLAN Frame                                     */
#define  ETH_DMA_TX_DESC_CC                                0x00000078          /* Collision Count                                */
#define  ETH_DMA_TX_DESC_ED                                0x00000004          /* Excessive Deferral                             */
#define  ETH_DMA_TX_DESC_UF                                0x00000002          /* Underflow Error: late data arrival from        */
                                                                               /* the memory                                     */
#define  ETH_DMA_TX_DESC_DB                                0x00000001          /* Deferred Bit                                   */

                                                                               /* -------------- TDES1 Register ---------------- */
#define  ETH_DMA_TX_DESC_TBS2                              0x1FFF0000          /* Transmit Buffer2 Size                          */
#define  ETH_DMA_TX_DESC_TBS1                              0x00001FFF          /* Transmit Buffer1 Size                          */

                                                                               /* -------------- TDES2 Register ---------------- */
#define  ETH_DMA_TX_DESC_B1AP                              0xFFFFFFFF          /* Buffer1 Address Pointer                        */

                                                                               /* --------------- TDES3 Register --------------- */
#define  ETH_DMA_TX_DESC_B2AP                              0xFFFFFFFF          /* Buffer2 Address Pointer                        */
/*$PAGE*/

                                                                               /* ---- RDES0 : DMA Rx descriptor status Reg ---- */
#define  ETH_DMA_RX_DESC_OWN                               0x80000000          /* OWN bit: descriptor is owned by DMA engine     */
#define  ETH_DMA_RX_DESC_AFM                               0x40000000          /* DA Filter Fail for the rx frame                */
#define  ETH_DMA_RX_DESC_FL                                0x3FFF0000          /* Receive descriptor frame len                   */
#define  ETH_DMA_RX_DESC_ES                                0x00008000          /* Error summary: OR of the following bits:       */
                                                                               /* DE || OE || IPC || LC || RWT || RE || CE       */
#define  ETH_DMA_RX_DESC_DE                                0x00004000          /* Desciptor error: no more descriptors for       */
                                                                               /* receive frame                                  */
#define  ETH_DMA_RX_DESC_SAF                               0x00002000          /* SA Filter Fail for the received frame          */
#define  ETH_DMA_RX_DESC_LE                                0x00001000          /* Frame size not matching with len field         */
#define  ETH_DMA_RX_DESC_OE                                0x00000800          /* Overflow Error: Frame was damaged due to       */
                                                                               /* buffer overflow                                */
#define  ETH_DMA_RX_DESC_VLAN                              0x00000400          /* VLAN Tag: received frame is a VLAN frame       */
#define  ETH_DMA_RX_DESC_FS                                0x00000200          /* First descriptor of the frame                  */
#define  ETH_DMA_RX_DESC_LS                                0x00000100          /* Last descriptor of the frame                   */
#define  ETH_DMA_RX_DESC_IPV4HCE                           0x00000080          /* IPC Checksum Error/Giant Frame: Rx Ipv4 header */
                                                                               /* checksum error                                 */
#define  ETH_DMA_RX_DESC_RX_LONG_FRAME                     0x00000080          /* (Giant FrameRx - frame is longer than          */
                                                                               /* 1518/1522                                      */
#define  ETH_DMA_RX_DESC_LC                                0x00000040          /* Late collision occurred during reception       */
#define  ETH_DMA_RX_DESC_FT                                0x00000020          /* Frame type - Ethernet, otherwise 802.3         */
#define  ETH_DMA_RX_DESC_RWT                               0x00000010          /* Receive Watchdog Timeout: watchdog timer       */
                                                                               /* expired during reception                       */
#define  ETH_DMA_RX_DESC_RE                                0x00000008          /* Receive error: error reported by MII interface */
#define  ETH_DMA_RX_DESC_DBE                               0x00000004          /* Dribble bit error: frame contains non int      */
                                                                               /* multiple of 8 bits                             */
#define  ETH_DMA_RX_DESC_CE                                0x00000002          /* CRC error                                      */
#define  ETH_DMA_RX_DESC_MAMPCE                            0x00000001          /* Rx MAC Address/Payload Checksum Error: Rx MAC  */
                                                                               /* address matched/ Rx Payload Checksum Error     */

                                                                               /* -------------- RDES1 register ---------------- */
#define  ETH_DMA_RX_DESC_DIC                               0x80000000          /* Disable Interrupt on Completion                */
#define  ETH_DMA_RX_DESC_RBS2                              0x1FFF0000          /* Receive Buffer2 Size                           */
#define  ETH_DMA_RX_DESC_RER                               0x00008000          /* Receive End of Ring                            */
#define  ETH_DMA_RX_DESC_RCH                               0x00004000          /* Second Address Chained                         */
#define  ETH_DMA_RX_DESC_RBS1                              0x00001FFF          /* Receive Buffer1 Size                           */

                                                                               /* -------------- RDES2 register ---------------- */
#define  ETH_DMA_RX_DESC_B1AP                              0xFFFFFFFF          /* Buffer1 Address Pointer                        */

                                                                               /* -------------- RDES3 register ---------------- */
#define  ETH_DMA_RX_DESC_B2AP                              0xFFFFFFFF          /* Buffer2 Address Pointer                        */


/*
*********************************************************************************************************
*                                                 GLOBAL VARIABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                                 MACROS
*********************************************************************************************************
*/

/*$PAGE*/
/*
*********************************************************************************************************
*                                          LOCAL DATA TYPES
*
* Note(s) : (1) Instance specific data area structures should be defined below.  The data area
*               structure typically includes error counters and variables used to track the
*               state of the device.  Variables required for correct operation of the device
*               MUST NOT be defined globally and should instead be included within the instance
*               specific data area structure and referenced as pif->Dev_Data structure members.
*
*           (2) DMA based devices may require more than one type of descriptor.  Each descriptor
*               type should be defined below.  An example descriptor has been provided.
*
*           (3) All device drivers MUST track the addresses of ALL buffers that have been
*               transmitted and not yet acknowledged through transmit complete interrupts.
*********************************************************************************************************
*/

typedef  struct  dev_desc  DEV_DESC;

struct dev_desc {
    CPU_REG32    Status;                                        /* DMA status register.                                 */
    CPU_REG32    Len;                                           /* DMA buffer size.                                     */
    CPU_INT08U  *Addr;                                          /* DMA buffer pointer                                   */
    DEV_DESC    *Next;                                          /* DMA next descriptor pointer.                         */
#ifdef NET_DEV_CM408F_MODULE
    CPU_REG32   ExtendedStatus;                                 /* Extended status */
    CPU_REG32   Reserved;                                       /* Reserved        */ 
    CPU_REG32   RxTimeStampLo;                                  /* Time stamp low  */
    CPU_REG32   RxTimeStampHi;                                  /* Time stamp high */
#endif
};

                                                                /* --------------- DEVICE INSTANCE DATA --------------- */
typedef  struct  net_dev_data {
#if (NET_CTR_CFG_STAT_EN == DEF_ENABLED)
    NET_CTR      StatRxCtrRx;
    NET_CTR      StatRxCtrRxISR;
    NET_CTR      StatTxCtrTxISR;
    NET_CTR      StatRxCtrTaskSignalISR;
    NET_CTR      StatRxCtrTaskSignalSelf;
    NET_CTR      StatRxCtrTaskSignalNone;
#endif

#if (NET_CTR_CFG_ERR_EN == DEF_ENABLED)
    NET_CTR      ErrRxCtrDataAreaAlloc;
    NET_CTR      ErrRxCtrROS;
    NET_CTR      ErrRxCtrRBUS;
    NET_CTR      ErrRxCtrRPSS;
    NET_CTR      ErrRxCtrDescES;
    NET_CTR      ErrRxCtrDescOWN;
    NET_CTR      ErrRxCtrRunt;
    NET_CTR      ErrRxTxCtrFatalBusErr;
    NET_CTR      ErrRxCtrTaskSignalFail;
#endif

    CPU_REG32    IntStatusLast;

    DEV_DESC    *RxBufDescPtrStart;
    DEV_DESC    *RxBufDescPtrCur;
    DEV_DESC    *RxBufDescPtrEnd;

    DEV_DESC    *TxBufDescPtrStart;
    DEV_DESC    *TxBufDescPtrCur;
    DEV_DESC    *TxBufDescPtrComp;                              /* See Note #3.                                         */

#ifdef NET_MULTICAST_PRESENT
    CPU_INT08U   MulticastAddrHashBitCtr[64];
#endif
} NET_DEV_DATA;


/*$PAGE*/
/*
*********************************************************************************************************
*                                      LOCAL FUNCTION PROTOTYPES
*
* Note(s) : (1) Device driver functions may be arbitrarily named.  However, it is recommended that device
*               driver functions be named using the names provided below.  All driver function prototypes
*               should be located within the driver C source file ('net_dev_&&&.c') & be declared as
*               static functions to prevent name clashes with other network protocol suite device drivers.
*********************************************************************************************************
*/

                                                                /* ------------ FNCT'S COMMON TO ALL DRV'S ------------ */
static  void  NetDev_Init               (NET_IF             *pif,
                                         NET_ERR            *perr);

static  void  NetDev_Start              (NET_IF             *pif,
                                         NET_ERR            *perr);


static  void  NetDev_Stop               (NET_IF             *pif,
                                         NET_ERR            *perr);

static  void  NetDev_Rx                 (NET_IF             *pif,
                                         CPU_INT08U        **p_data,
                                         CPU_INT16U         *size,
                                         NET_ERR            *perr);

static  void  NetDev_Tx                 (NET_IF             *pif,
                                         CPU_INT08U         *p_data,
                                         CPU_INT16U          size,
                                         NET_ERR            *perr);

#if (NET_VERSION >= 205u)
static  void  NetDev_AddrMulticastAdd   (NET_IF             *pif,
                                         CPU_INT08U         *paddr_hw,
                                         CPU_INT08U          addr_hw_len,
                                         NET_ERR            *perr);

static  void  NetDev_AddrMulticastRemove(NET_IF             *pif,
                                         CPU_INT08U         *paddr_hw,
                                         CPU_INT08U          addr_hw_len,
                                         NET_ERR            *perr);
#endif

static  void  NetDev_ISR_Handler        (NET_IF             *pif,
                                         NET_DEV_ISR_TYPE    type);

static  void  NetDev_IO_Ctrl            (NET_IF             *pif,
                                         CPU_INT08U          opt,
                                         void               *p_data,
                                         NET_ERR            *perr);

static  void  NetDev_MII_Rd             (NET_IF             *pif,
                                         CPU_INT08U          phy_addr,
                                         CPU_INT08U          reg_addr,
                                         CPU_INT16U         *p_data,
                                         NET_ERR            *perr);

static  void  NetDev_MII_Wr             (NET_IF             *pif,
                                         CPU_INT08U          phy_addr,
                                         CPU_INT08U          reg_addr,
                                         CPU_INT16U          data,
                                         NET_ERR            *perr);
                                                                /* --------- FNCT'S COMMON TO DMA BASED DRV'S --------- */
static  void  NetDev_RxDescInit         (NET_IF             *pif,
                                         NET_ERR            *perr);

static  void  NetDev_RxDescPtrCurInc    (NET_IF             *pif);

static  void  NetDev_TxDescInit         (NET_IF             *pif,
                                         NET_ERR            *perr);

static  void  NetDev_RxDescFreeAll      (NET_IF             *pif,
                                         NET_ERR            *perr);


/*$PAGE*/
/*
*********************************************************************************************************
*                                            LOCAL TABLES
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                      NETWORK DEVICE DRIVER API
*
* Note(s) : (1) Device driver API structures are used by applications during calls to NetIF_Add().  This
*               API structure allows higher layers to call specific device driver functions via function
*               pointer instead of by name.  This enables the network protocol suite to compile & operate
*               with multiple device drivers.
*
*           (2) In most cases, the API structure provided below SHOULD suffice for most device drivers
*               exactly as is with the exception that the API structure's name which MUST be unique &
*               SHOULD clearly identify the device being implemented.  For example, the Cirrus Logic
*               CS8900A Ethernet controller's API structure should be named NetDev_API_CS8900A[].
*
*               The API structure MUST also be externally declared in the device driver header file
*               ('net_dev_&&&.h') with the exact same name & type.
*********************************************************************************************************
*/
                                                                                    /* CM408F dev API fnct ptrs :    */
const  NET_DEV_API_ETHER  NetDev_API_CM408F = { NetDev_Init,                     /*   Init/add                       */
                                                   NetDev_Start,                    /*   Start                          */
                                                   NetDev_Stop,                     /*   Stop                           */
                                                   NetDev_Rx,                       /*   Rx                             */
                                                   NetDev_Tx,                       /*   Tx                             */
#if  (NET_VERSION >= 20600u)
                                                   NetDev_AddrMulticastAdd,         /*   Multicast addr add             */
                                                   NetDev_AddrMulticastRemove,      /*   Multicast addr remove          */
#endif
                                                   NetDev_ISR_Handler,              /*   ISR handler                    */
                                                   NetDev_IO_Ctrl,                  /*   I/O ctrl                       */
#if ((NET_VERSION <  20600u) && \
     (NET_VERSION >= 205u))
                                                   NetDev_AddrMulticastAdd,         /*   Multicast addr add             */
                                                   NetDev_AddrMulticastRemove,      /*   Multicast addr remove          */
#endif
                                                   NetDev_MII_Rd,                   /*   Phy reg rd                     */
                                                   NetDev_MII_Wr                    /*   Phy reg wr                     */
                                                 };


/*
*********************************************************************************************************
*                                        REGISTER DEFINITIONS
*
* Note(s) : (1) The device register definition structure MUST take into account appropriate
*               register offsets and apply reserved space as required.  The registers listed
*               within the register definition structure MUST reflect the exact ordering and
*               data sizes illustrated in the device user guide. An example device register
*               structure is provided below.
*
*           (2) The device register definition structure is mapped over the corresponding base
*               address provided within the device configuration structure.  The device
*               configuration structure is provided by the application developer within
*               net_dev_cfg.c.  Refer to the Network Protocol Suite User Manual for more
*               information related to declaring device configuration structures.
*********************************************************************************************************
*/
typedef  struct  net_dev {
                                                                /* ----------------- MAC REGISTERS -------------------- */
    CPU_REG32  MACCR;                                           /* MAC Configuration Register.                          */
    CPU_REG32  MACFFR;                                          /* MAC Frame Filter Register.                           */
    CPU_REG32  MACHTHR;                                         /* MAC Hash Table High Register.                        */
    CPU_REG32  MACHTLR;                                         /* MAC Hash Table Low Register.                         */
    CPU_REG32  MACMIIAR;                                        /* GMII Address Register.                               */
    CPU_REG32  MACMIIDR;                                        /* gmii Data Register.                                  */
    CPU_REG32  MACFCR;                                          /* MAC Flow Control Register.                           */
    CPU_REG32  MACVLANTR;                                       /* MAC VLAN Tag Register.                               */
    CPU_REG32  RESERVED0[2];
    CPU_REG32  MACRWUFFR;                                       /* MAC Remote Wake-Up Frame Filter.                     */
    CPU_REG32  MACPMTCSR;                                       /* PMT Control and Status.                              */
    CPU_REG32  RESERVED1[2];
    CPU_REG32  MACSR;                                           /* MAC Status Register.                                 */
    CPU_REG32  MACIMR;                                          /* MAC Interrupt Mask Register.                         */
    CPU_REG32  MACA0HR;                                         /* MAC Address0 High Regsiter.                          */
    CPU_REG32  MACA0LR;                                         /* MAC Address0 Low Regsiter.                           */
    CPU_REG08 RESERVED2[184];
/*$PAGE*/
                                                                /* ----------------- MMC REGISTERS -------------------- */
    CPU_REG32  MMCCR;                                           /* MMC Control Register.                                */
    CPU_REG32  MMCRIR;                                          /* MMC Receive Interrupt Register.                      */
    CPU_REG32  MMCTIR;                                          /* MMC Transmit Interrupt Register.                     */
    CPU_REG32  MMCRIMR;                                         /* MMC Receive Interrupt Mask Register.                 */
    CPU_REG32  MMCTIMR;                                         /* MMC Transmit Interrupt Mask Register.                */
#ifdef NET_DEV_CM408F_MODULE
    CPU_REG32    TXOCTCNT_GB;                                   /* Num bytes transmitted exclusive of preamble          */
    CPU_REG32    TXFRMCNT_GB;                                   /* Num frames transmitted exclusive of retired          */
    CPU_REG32    TXBCASTFRM_G;                                  /* Number of good broadcast frames transmitted          */
    CPU_REG32    TXMCASTFRM_G;                                  /* Number of good multicast frames transmitted          */
    CPU_REG32    TX64_GB;                                       /* Number of 64 byte length frames                      */
    CPU_REG32    TX65TO127_GB;                                  /* Number of frames of length b/w 65-127 (inclusive) bytes */
    CPU_REG32    TX128TO255_GB;                                 /* Number of frames of length b/w 128-255 (inclusive) bytes */
    CPU_REG32    TX256TO511_GB;                                 /* Number of frames of length b/w 256-511 (inclusive) bytes */
    CPU_REG32    TX512TO1023_GB;                                /* Number of frames of length b/w 512-1023 (inclusive) bytes */
    CPU_REG32    TX1024TOMAX_GB;                                /* Number of frames of length b/w 1024-max (inclusive) bytes */
    CPU_REG32    TXUCASTFRM_GB;                                 /* Number of good and bad unicast frames transmitted     */
    CPU_REG32    TXMCASTFRM_GB;                                 /* Number of good and bad multicast frames transmitted   */
    CPU_REG32    TXBCASTFRM_GB;                                 /* Number of good and bad broadcast frames transmitted   */
    CPU_REG32    TXUNDR_ERR;                                    /* Number of frames aborted due to frame underflow error */
#endif
    CPU_REG32  MMCTGFSCCR;                                      /* MMC Transmit good frames after single col ctr        */
    CPU_REG32  MMCTGFMSCCR;                                     /* MMC Transmit good frames after mult col ctr          */
#ifdef NET_DEV_CM408F_MODULE
    CPU_REG32    TXDEFERRED;                                    /* Number of transmitted frames after deferral          */
    CPU_REG32    TXLATECOL;                                     /* Number of frames aborted due to late collision error */
    CPU_REG32    TXEXCESSCOL;                                   /* Number of aborted frames due to excessive collisions */
    CPU_REG32    TXCARR_ERR;                                    /* Number of aborted frames due to carrier sense error  */
    CPU_REG32    TXOCTCNT_G;                                    /* Number of bytes transmitted in good frames only      */
    CPU_REG32    TXFRMCNT_G;                                    /* Number of good frames transmitted.                   */
    CPU_REG32    TXEXCESSDEF;                                   /* Number of frames aborted due to excessive deferral   */
    CPU_REG32    TXPAUSEFRM;                                    /* Number of good PAUSE frames transmitted.             */
    CPU_REG32    TXVLANFRM_G;                                   /* Number of VLAN frames transmitted                    */
    CPU_REG08     RESERVED3[8];
    CPU_REG32    RXFRMCNT_GB;                                   /* Number of good and bad frames received.              */
    CPU_REG32    RXOCTCNT_GB;                                   /* Number of bytes received in good and bad frames      */
    CPU_REG32    RXOCTCNT_G;                                    /* Number of bytes received only in good frames         */
    CPU_REG32    RXBCASTFRM_G;                                  /* Number of good broadcast frames received.            */
    CPU_REG32    RXMCASTFRM_G;                                  /* Number of good multicast frames received             */
#endif

    CPU_REG32  MMCRFCECR;                                       /* MMC received frames with err ctr.                    */
    CPU_REG32  MMCRFAECR;                                       /* MMC received frames with alignment err ctr.          */

#ifdef NET_DEV_CM408F_MODULE
    CPU_REG32    RXRUNT_ERR;                                    /* Number of frames received with runt error.            */
    CPU_REG32    RXJAB_ERR;                                     /* Number of frames received with length greater than 1518 */
    CPU_REG32    RXUSIZE_G;                                     /* Number of frames received with length 64              */
    CPU_REG32    RXOSIZE_G;                                     /* Number of frames received with length greater than maxium */
    CPU_REG32    RX64_GB;                                       /* Number of good and bad frames of lengh 64 bytes        */
    CPU_REG32    RX65TO127_GB;                                  /* Number of good and bad frame between 64-127(inclusive) */
    CPU_REG32    RX128TO255_GB;                                 /* Number of good and bad frames received with length between 128 and 255 (inclusive) bytes, exclusive of preamble. */
    CPU_REG32    RX256TO511_GB;                                 /* Number of good and bad frames between 256-511(inclusive) */
    CPU_REG32    RX512TO1023_GB;                                /* Number of good and bad frames received between 512-1023  */
    CPU_REG32    RX1024TOMAX_GB;                                /* Number of frames received between 1024 and maxsize       */
#endif
    CPU_REG32  MMCRGUFCR;                                       /* Number of good unicast frames received.                  */

#ifdef NET_DEV_CM408F_MODULE
    CPU_REG32    RXLEN_ERR;                                     /* Number of frames received with length error              */
    CPU_REG32    RXOORTYPE;                                     /* Number of frames with length not equal to valid frame size */
    CPU_REG32    RXPAUSEFRM;                                    /* Number of good and valid PAUSE frames received.          */
    CPU_REG32    RXFIFO_OVF;                                    /* Number of missed received frames due to FIFO overflow.   */
    CPU_REG32    RXVLANFRM_GB;                                  /* Number of good and bad VLAN frames received.             */
    CPU_REG32    RXWDOG_ERR;                                    /* Frames received with error due to watchdog timeout       */
    CPU_REG08    RESERVED4[32];
    CPU_REG32   IPC_RXIMSK;                                    /* MMC IPC RX Interrupt Mask Register                       */
    CPU_REG08    RESERVED5[4];
    CPU_REG32   IPC_RXINT;                                     /* MMC IPC RX Interrupt Register                            */
    CPU_REG08    RESERVED6[4];
    CPU_REG32    RXIPV4_GD_FRM;                                 /* Number of good IPv4 datagrams                            */
    CPU_REG32    RXIPV4_HDR_ERR_FRM;                            /* Number of IPv4 datagrams with header errors              */
    CPU_REG32    RXIPV4_NOPAY_FRM;                              /* Number of IPv4 datagrams without checksum                */
    CPU_REG32    RXIPV4_FRAG_FRM;                               /* Number of good IPv4 datagrams with fragmentation         */
    CPU_REG32    RXIPV4_UDSBL_FRM;                              /* Number of IPv4 UDP datagrams with disabled checksum      */
    CPU_REG32    RXIPV6_GD_FRM;                                 /* Number of IPv4 datagrams with TCP/UDP/ICMP payloads      */
    CPU_REG32    RXIPV6_HDR_ERR_FRM;                            /* Number of IPv6 datagrams with header errors              */
    CPU_REG32    RXIPV6_NOPAY_FRM;                              /* Number of IPv6 datagrams with no TCP/UDP/ICMP payload    */
    CPU_REG32    RXUDP_GD_FRM;                                  /* Number of good IP datagrames with good UDP payload       */
    CPU_REG32    RXUDP_ERR_FRM;                                 /* Number of good IP datagrams with UDP checksum errors     */
    CPU_REG32    RXTCP_GD_FRM;                                  /* Number of good IP datagrams with a good TCP payload      */
    CPU_REG32    RXTCP_ERR_FRM;                                 /* Number of good IP datagrams with TCP checksum errors     */
    CPU_REG32    RXICMP_GD_FRM;                                 /* Number of good IP datagrams with a good ICMP payload     */
    CPU_REG32    RXICMP_ERR_FRM;                                /* Number of good IP datagrams with ICMP checksum errors    */
    CPU_REG08    RESERVED7[8];
    CPU_REG32    RXIPV4_GD_OCT;                                 /* Bytes received in IPv4 datagrams including tcp,udp or icmp */
    CPU_REG32    RXIPV4_HDR_ERR_OCT;                            /* Bytes received in IPv4 datagrams with header errors        */
    CPU_REG32    RXIPV4_NOPAY_OCT;                              /* Bytes received in IPv4 datagrams without tcp,udp,icmp load */
    CPU_REG32    RXIPV4_FRAG_OCT;                               /* Bytes received in fragmented IPv4 datagrams                */
    CPU_REG32    RXIPV4_UDSBL_OCT;                              /* Bytes received in UDP segment with checksum disabled       */
    CPU_REG32    RXIPV6_GD_OCT;                                 /* Bytes received in good IPv6  including tcp,udp or icmp load */
    CPU_REG32    RXIPV6_HDR_ERR_OCT;                            /* Number of bytes received in IPv6 with header errors        */
    CPU_REG32    RXIPV6_NOPAY_OCT;                              /* Bytes received in IPv6 without tcp,udp or icmp load        */
    CPU_REG32    RXUDP_GD_OCT;                                  /* Number of bytes received in good UDP segments              */
    CPU_REG32    RXUDP_ERR_OCT;                                 /* Number of bytes received in UDP segment with checksum err  */
    CPU_REG32    RXTCP_GD_OCT;                                  /* Number of bytes received in a good TCP segment             */
    CPU_REG32    RXTCP_ERR_OCT;                                 /* Number of bytes received in TCP segment with checksum err  */
    CPU_REG32    RXICMP_GD_OCT;                                 /* Number of bytes received in a good ICMP segment            */
    CPU_REG32    RXICMP_ERR_OCT;                                /* Bytes received in an ICMP segment with checksum errors     */
    CPU_REG08    RESERVED8[1144];
#endif
    CPU_REG32  PTPTSCR;                                         /* PTP time stamp ctrl Register.                             */
    CPU_REG32  PTPSSIR;                                         /* PTP subsecond inc Register.                               */
    CPU_REG32  PTPTSHR;                                         /* PTP time stamp high Register.                             */
    CPU_REG32  PTPTSLR;                                         /* PTP time stamp low  Register.                             */
    CPU_REG32  PTPTSHUR;                                        /* PTP time stamp update high Register.                      */
    CPU_REG32  PTPTSLUR;                                        /* PTP time stamp update low Register.                       */
    CPU_REG32  PTPTSAR;                                         /* PTP time stamp addend Register.                           */
    CPU_REG32  PTPTTHR;                                         /* PTP target time high Register.                            */
    CPU_REG32  PTPTTLR;                                         /* PTP target time low Register.                             */
#ifdef NET_DEV_CM408F_MODULE
    CPU_REG32   TM_HISEC;                                       /* EMAC Time Stamp High Second Register                      */
    CPU_REG32   TM_STMPSTAT;                                    /* EMAC Time Stamp Status Register                           */
    CPU_REG32   TM_PPSCTL;                                      /* EMAC PPS Control Register                                 */ 
    CPU_REG32   TM_AUXSTMP_NSEC;                                /* EMAC Auxillary Time Stamp Nano Register                   */
    CPU_REG32   TM_AUXSTMP_SEC;                                 /* EMAC Auxillary Time Stamp Sec Register                    */
    CPU_REG08    RESERVED9[40];
    CPU_REG32   TM_PPSINTVL;                                    /* PPS Interval Register                                     */
    CPU_REG32   TM_PPSWIDTH;                                    /* PPS Width Register                                        */
    CPU_REG08    RESERVED10[2200];
#endif
    CPU_REG32  DMABMR;                                          /* DMA Bus Mode Register.                                    */
    CPU_REG32  DMATPDR;                                         /* DMA Transmit Poll Demand Register.                        */
    CPU_REG32  DMARPDR;                                         /* DMA Receive Poll Demand Register                          */
    CPU_REG32  DMARDLAR;                                        /* DMA Receive Descriptor List Address Register.             */
    CPU_REG32  DMATDLAR;                                        /* DMA Transmit Descriptor List Address Register.            */
    CPU_REG32  DMASR;                                           /* DMA Status Register.                                      */
    CPU_REG32  DMAOMR;                                          /* DMA Operation Mode Register.                              */
    CPU_REG32  DMAIER;                                          /* DMA Interrupt Enable Register.                            */
    CPU_REG32  DMAMFBOCR;                                       /* DMA Missed Frame and Buffer Overflow Count Register.      */
#ifdef NET_DEV_CM408F_MODULE
    CPU_REG32  DMA_RXIWDOG;                                     /* DMA RX Interrupt Watch Dog timer                          */
    CPU_REG32  DMA_BMMODE;                                      /* SCB Bus Mode Register                                     */
    CPU_REG32  DMA_BMSTAT;                                      /* SCB Status Register                                       */
    CPU_REG08   RESERVED11[24];
#endif
    CPU_REG32  DMACHTDR;                                        /* DMA Current Host Transmit Descriptor Register.            */
    CPU_REG32  DMACHRDR;                                        /* DMA Current Host Receive Descriptor Register.             */
    CPU_REG32  DMACHTBAR;                                       /* DMA Current Host Transmit Buffer Addr Register            */
    CPU_REG32  DMACHRBAR;                                       /* DMA Current Host Receive Buffer Addr Register             */
} NET_DEV;


/*$PAGE*/
/*
*********************************************************************************************************
*                                            NetDev_Init()
*
* Description : (1) Initialize Network Driver Layer :
*
*                   (a) Initialize required clock sources
*                   (b) Initialize external interrupt controller
*                   (c) Initialize external GPIO controller
*                   (d) Initialize driver state variables
*                   (e) Allocate memory for device DMA descriptors
*                   (f) Initialize additional device registers
*                       (1) (R)MII mode / Phy bus type
*                       (2) Disable device interrupts
*                       (3) Disable device receiver and transmitter
*                       (4) Other necessary device initialization
*
* Argument(s) : pif     Pointer to the interface requiring service.
*
*               perr    Pointer to return error code.
*                           NET_DEV_ERR_NONE            No Error.
*                           NET_DEV_ERR_INIT            General initialization error.
*                           NET_BUF_ERR_POOL_MEM_ALLOC  Memory allocation failed.
*                           NET_DEV_ERR_INVALID_CFG     Invalid device configuration.
*
* Return(s)   : none.
*
* Caller(s)   : NetIF_Ether_IF_Add() via 'pdev_api->Init()'.
*
* Note(s)     : (2) The application developer SHOULD define NetDev_CfgClk() within net_bsp.c
*                   in order to properly enable clocks for specified network interface.  In
*                   some cases, a device may require clocks to be enabled for BOTH the device
*                   and accessory peripheral modules such as GPIO.  A call to this function
*                   MAY need to occur BEFORE any device register accesses are made.  In the
*                   event that a device does NOT require any external clocks to be enabled,
*                   it is recommended that the device driver still call the NetBSP fuction
*                   which may in turn leave the section for the specific interface number
*                   empty.
*
*               (3) The application developer SHOULD define NetDev_CfgIntCtrl() within net_bsp.c
*                   in order to properly enable interrupts on an external or CPU integrated
*                   interrupt controller.  Interrupt sources that are specific to the DEVICE
*                   hardware MUST NOT be initialized from within NetDev_CfgIntCtrl() and
*                   SHOULD only be modified from within the device driver.
*
*                   (a) External interrupt sources are cleared within the NetBSP first level
*                       ISR handler either before or after the call to the device driver ISR
*                       handler function.  The device driver ISR handler function SHOULD only
*                       clear the device specific interrupts and NOT external or CPU interrupt
*                       controller interrupt sources.
*
*               (4) The application developer SHOULD define NetDev_CfgGPIO() within net_bsp.c
*                   in order to properly configure any necessary GPIO necessary for the device
*                   to operate properly.  Micrium recommends defining and calling this NetBSP
*                   function even if no additional GPIO initialization is required.
*
*               (5) All functions that require device register access MUST obtain reference
*                   to the device hardware register space PRIOR to attempting to access
*                   any registers.  Register definitions SHOULD NOT be absolute and SHOULD
*                   use the provided base address within the device configuration structure,
*                   as well as the device register definition structure in order to properly
*                   resolve register addresses during run-time.
*
*               (6) All device drivers that store instance specific data MUST declare all
*                   instance specific variables within the device data area defined above.
*$PAGE*
*               (7) Drivers SHOULD validate device configuration values and set *perr to
*                   NET_DEV_ERR_INVALID_CFG if unacceptible values have been specified. Fields
*                   of interest generally include, but are not limited to :
*
*                   (a) pdev_cfg->RxBufAlignOctets
*                   (b) pdev_cfg->RxBufLargeSize
*
*               (8) Many DMA devices may generate only one interrupt for several ready receive
*                   descriptors.  In order to accommodate this, it is recommended that all DMA
*                   based drivers count the number of ready receive descriptors during the
*                   receive event and signal the receive task accordingly ONLY for those
*                   NEW descriptors which have not yet been accounted for.  Each time a
*                   descriptor is processed (or discarded) the count for acknowledged and
*                   unprocessed frames should be decremented by 1.  This function initializes the
*                   acknowledged receive descriptor count to 0.
*
*              (9) Descriptors are typically required to be contiguous in memory.  Allocation of
*                   descriptors MUST occur as a single contigous block of memory.  The driver may
*                   use pointer arithmetic to sub-divide and traverse the descriptor list.
*
*              (10) NetDev_Init() should exit with :
*
*                   (a) All device interrupt source disabled. External interrupt controllers
*                       should however be ready to accept interrupt requests.
*                   (b) All device interrupt sources cleared.
*                   (c) Both the receiver and transmitter disabled.
*
*              (11) Some drivers MAY require knowledge of the Phy configuration in order
*                   to properly configure the MAC with the correct Phy bus mode, speed and
*                   duplex settings.  If a driver requires access to the Phy configuration,
*                   then the driver MUST validate the pif->Phy_Cfg pointer by checking for
*                   a NULL pointer BEFORE attempting to access members of the Phy
*                   configuration structure.  Phy configuration fields of interest generally
*                   include, but are  not limited to :
*
*                   (a) pphy_cfg->Type :
*
*                       (1) NET_PHY_TYPE_INT            Phy integrated with MAC.
*                       (2) NET_PHY_TYPE_EXT            Phy externally attached to MAC.
*
*                   (b) pphy_cfg->BusMode :
*
*                       (1) NET_PHY_BUS_MODE_MII        Phy bus mode configured to MII.
*                       (2) NET_PHY_BUS_MODE_RMII       Phy bus mode configured to RMII.
*                       (3) NET_PHY_BUS_MODE_SMII       Phy bus mode configured to SMII.
*
*                   (c) pphy_cfg->Spd :
*
*                       (1) NET_PHY_SPD_0               Phy link speed unknown or NOT linked.
*                       (2) NET_PHY_SPD_10              Phy link speed configured to  10   mbit/s.
*                       (3) NET_PHY_SPD_100             Phy link speed configured to  100  mbit/s.
*                       (4) NET_PHY_SPD_1000            Phy link speed configured to  1000 mbit/s.
*                       (5) NET_PHY_SPD_AUTO            Phy link speed configured for auto-negotiation.
*
*                   (d) pphy_cfg->Duplex :
*
*                       (1) NET_PHY_DUPLEX_UNKNOWN      Phy link duplex unknown or link not established.
*                       (2) NET_PHY_DUPLEX_HALF         Phy link duplex configured to  half duplex.
*                       (3) NET_PHY_DUPLEX_FULL         Phy link duplex configured to  full duplex.
*                       (4) NET_PHY_DUPLEX_AUTO         Phy link duplex configured for auto-negotiation.
*********************************************************************************************************
*/
/*$PAGE*/
static  void  NetDev_Init (NET_IF   *pif,
                           NET_ERR  *perr)
{
#if (NET_VERSION >= 21000u)
    NET_DEV_BSP_ETHER  *pdev_bsp;
#endif
    NET_DEV_CFG_ETHER  *pdev_cfg;
    NET_DEV_DATA       *pdev_data;
    NET_DEV            *pdev;
    NET_BUF_SIZE        buf_size_max;
    NET_BUF_SIZE        buf_ix;
    CPU_INT32U          input_freq;
    CPU_INT32U          mdc_div;
    CPU_SIZE_T          reqd_octets;
    CPU_SIZE_T          nbytes;
    LIB_ERR             lib_err;


                                                                /* -------- OBTAIN REFERENCE TO CFGs/REGs/BSP --------- */
    pdev_cfg = (NET_DEV_CFG_ETHER *)pif->Dev_Cfg;
    pdev     = (NET_DEV           *)pdev_cfg->BaseAddr;         /* Overlay dev reg struct on top of dev base addr.      */
#if (NET_VERSION >= 21000u)
    pdev_bsp = (NET_DEV_BSP_ETHER *)pif->Dev_BSP;
#endif

                                                                /* --------------- VALIDATE DEVICE CFG ---------------- */
                                                               /* Validate Rx buf size.                                */
    if ((pdev_cfg->RxBufLargeSize   % RX_BUF_SIZE_MULT   != 0u)) {
        *perr = NET_DEV_ERR_INVALID_CFG;                        /* Rx buf's MUST be a mult of 16 octets.                */
         return;
    }

                                                                /* Validate Rx buf alignment.                           */
    if ((pdev_cfg->RxBufAlignOctets % RX_DESC_BUF_ALIGN) != 0u) {
        *perr = NET_DEV_ERR_INVALID_CFG;
         return;
    }
#if (NET_VERSION >= 21000u)                                     /* Validate Rx buf ix offset.                           */
    if (pdev_cfg->RxBufIxOffset != 0u) {
       *perr = NET_DEV_ERR_INVALID_CFG;
        return;
    }
#endif
                                                                /* Validate Rx buf size.                                */
#if (NET_VERSION >= 21000u)
    buf_ix       = NET_IF_IX_RX;
#else
    buf_ix       = NET_IF_RX_IX;
#endif
    buf_size_max = NetBuf_GetMaxSize((NET_IF_NBR     )pif->Nbr,
                                     (NET_TRANSACTION)NET_TRANSACTION_RX,
                                     (NET_BUF       *)0,
                                     (NET_BUF_SIZE   )buf_ix);
#if (NET_VERSION >= 21000u)
    if (buf_size_max < NET_IF_ETHER_FRAME_MAX_SIZE) {
       *perr = NET_DEV_ERR_INVALID_CFG;
        return;
    }
#else
   (void)&buf_size_max;
#endif

                                                                /* Validate Tx buf alignment.                           */
    if ((pdev_cfg->TxBufAlignOctets % TX_DESC_BUF_ALIGN != 0)) {
        *perr = NET_DEV_ERR_INVALID_CFG;                        /* Tx buf's MUST be aligned to 1 16 BYTE boundary.      */
         return;
    }

#if (NET_VERSION >= 21000u)                                     /* Validate Tx buf ix offset.                           */
    if (pdev_cfg->TxBufIxOffset != 0u) {
       *perr = NET_DEV_ERR_INVALID_CFG;
        return;
    }
#endif


                                                                /* ------------- ALLOCATE DEV DATA AREA --------------- */
    pif->Dev_Data = Mem_HeapAlloc((CPU_SIZE_T  ) sizeof(NET_DEV_DATA),
                                  (CPU_SIZE_T  ) 4,
                                  (CPU_SIZE_T *)&reqd_octets,
                                  (LIB_ERR    *)&lib_err);
    if (pif->Dev_Data == (void *)0) {
       *perr = NET_DEV_ERR_MEM_ALLOC;
        return;
    }

    pdev_data = (NET_DEV_DATA *)pif->Dev_Data;


                                                                /* ------------ ALLOCATE DESCRIPTOR LISTS ------------- */

    nbytes = pdev_cfg->RxDescNbr * sizeof(DEV_DESC);            /* Allocate Rx desc buf's.                              */
    pdev_data->RxBufDescPtrStart = (DEV_DESC *)Mem_HeapAlloc((CPU_SIZE_T  ) nbytes,
                                                             (CPU_SIZE_T  ) pdev_cfg->RxBufAlignOctets,
                                                             (CPU_SIZE_T *)&reqd_octets,
                                                             (LIB_ERR    *)&lib_err);
    if (pdev_data->RxBufDescPtrStart == (void *)0) {
       *perr = NET_DEV_ERR_MEM_ALLOC;
        return;
    }

    nbytes = pdev_cfg->TxDescNbr * sizeof(DEV_DESC);            /* Allocate Tx desc's.                                  */
    pdev_data->TxBufDescPtrStart = (DEV_DESC *)Mem_HeapAlloc((CPU_SIZE_T  ) nbytes,
                                                             (CPU_SIZE_T  ) pdev_cfg->TxBufAlignOctets,
                                                             (CPU_SIZE_T *)&reqd_octets,
                                                             (LIB_ERR    *)&lib_err);
    if (pdev_data->TxBufDescPtrStart == (void *)0) {
       *perr = NET_DEV_ERR_MEM_ALLOC;
        return;
    }



#if (NET_VERSION >= 21000u)                                     /* Enable module clks (GPIO & EMAC) [see Note #2].      */
    pdev_bsp->CfgClk(pif, perr);
#else
    NetDev_CfgClk(pif->Nbr, perr);
#endif
    if (*perr != NET_DEV_ERR_NONE) {
        return;
    }


    pdev->DMABMR |= ETH_DMABMR_SR;                              /* Reset MAC reg's to dflt val's.                       */
    pdev->DMAIER  = 0;                                          /* Dis. all    int. src's.                              */
    pdev->DMASR   = 0xFFFFFFFF;                                 /* Clr pending int. src's.                              */

#if (NET_VERSION >= 21000u)                                     /* Configure ext int ctrl'r (see Note #3).              */
    pdev_bsp->CfgIntCtrl(pif, perr);
#else
    NetDev_CfgIntCtrl(pif->Nbr, perr);
#endif
    if (*perr != NET_DEV_ERR_NONE) {
        return;
    }

#if (NET_VERSION >= 21000u)                                     /* Configure MAC / Phy GPIO (see Note #4).              */
    pdev_bsp->CfgGPIO(pif, perr);
#else
    NetDev_CfgGPIO(pif->Nbr, perr);
#endif
    if (*perr != NET_DEV_ERR_NONE) {
        return;
    }
/*$PAGE*/
                                                                /* -------------------- CFG MAC ----------------------- */
    while ((pdev->DMABMR & ETH_DMABMR_SR) != 0) {               /* Wait reset to complete.                              */
        ;
    }

#if (NET_VERSION >= 21000u)                                     /* Get MDC clk freq (in Hz).                            */
    input_freq = pdev_bsp->ClkFreqGet(pif, perr);
#else
    input_freq = NetDev_MDC_ClkFreqGet(pif->Nbr, perr);
#endif
    if (*perr != NET_DEV_ERR_NONE) {
         return;
    }

    if ((input_freq >= 20000000UL) &&                           /* Determine MDC clk div.                               */
        (input_freq <  35000000UL)) {
        mdc_div = (CPU_INT32U)MAC_CR_DIV16;
    } else if ((input_freq >= 35000000UL) &&
               (input_freq <  60000000UL)) {
        mdc_div = (CPU_INT32U)MAC_CR_DIV26;
    } else {
        mdc_div = (CPU_INT32U)MAC_CR_DIV42;
    }

    pdev->MACMIIAR &=  ETH_MACMIIAR_CR_MASK;                    /* Clr MDC clk div bits (div 64).                       */
    pdev->MACMIIAR |=  mdc_div;                                 /* Cfg MDC clk div.                                     */

    pdev->MACCR     = (ETH_MACCR_WD         |                   /* Disable watchdog.                                    */
                       ETH_MACCR_IFG_96BITS |                   /* Cfg Inter-frame gap to 96 bits.                      */
                       ETH_MACCR_FES        |                   /* Cfg initial MAC speed  to 100Mbps.                   */
                       ETH_MACCR_DM         |                   /* Cfg initial MAC duplex to FULL.                      */
                       ETH_MACCR_ROD        |                   /* Disable receive own frames when in half duplex mode. */
                       ETH_MACCR_BL_10);                        /* Cfg back-off limit to MIN(n, 10).                    */

    pdev->MACFFR    = (ETH_MACFFR_HPF       |                   /* Cfg Rx filter for Hash or Perfect match.             */
                       ETH_MACFFR_HMC);                         /* Enable multicast hashing.                            */

    pdev->MACFCR    =  ETH_MACFCR_ZQPD;                         /* Disable automatic generation of pause frames.        */


                                                                /* -------------------- CFG DMA ----------------------- */
    pdev->DMAOMR    = (ETH_DMAOMR_DFF       |                   /* Disable flushing of Rx desc buf's when RBNA.         */
                    //   ETH_DMAOMR_RSF       |                   /* Defer Rx until entire frame is within the Rx FIFO.   */
                       ETH_DMAOMR_TSF);                         /* Defer Tx until entire frame is within the Tx FIFO.   */

    pdev->DMABMR    = (ETH_DMABMR_AAB                 |         /* First DMA transfer is 16 byte aligned.               */
                       ETH_DMABMR_ATDS                |         /* Alternate descriptor size                            */
                       ETH_DMABMR_RX_DMA_1_BEAT       |         /* 1 beat max burst per Rx DMA transfer.                */
                       ETH_DMABMR_TX_DMA_1_BEAT       |         /* 1 beat max burst per Tx DMA transfer.                */
                      (ETH_DMA_DESC_SKIP_LEN << 2)    |         /* If in ring mode, desc's are contiguous in mem.       */
                       ETH_DMABMR_RX_TX_DMA_PRIO_1_1  |         /* Rx transactions always take precedence over Tx.      */
                       ETH_DMABMR_USP);                         /* Use seperate PBL for Rx & Tx burst len.              */

   *perr = NET_DEV_ERR_NONE;
}


/*$PAGE*/
/*
*********************************************************************************************************
*                                            NetDev_Start()
*
* Description : (1) Start network interface hardware :
*
*                   (a) Initialize transmit semaphore count
*                   (b) Initialize hardware address registers
*                   (c) Initialize receive and transmit descriptors
*                   (d) Clear all pending interrupt sources
*                   (e) Enable supported interrupts
*                   (f) Enable the transmitter and receiver
*                   (g) Start / Enable DMA if required
*
*
* Argument(s) : pif         Pointer to a network interface.
*               ---         Argument validated in NetIF_Start().
*
*               perr        Pointer to variable that will receive the return error code from this function :
*
*                               NET_DEV_ERR_NONE                Ethernet device successfully started.
*
*                                                               - RETURNED BY NetIF_AddrHW_SetHandler() : --
*                               NET_IF_ERR_NULL_PTR             Argument(s) passed a NULL pointer.
*                               NET_IF_ERR_NULL_FNCT            Invalid NULL function pointer.
*                               NET_IF_ERR_INVALID_CFG          Invalid/NULL API configuration.
*                               NET_IF_ERR_INVALID_STATE        Invalid network interface state.
*                               NET_IF_ERR_INVALID_ADDR         Invalid hardware address.
*                               NET_IF_ERR_INVALID_ADDR_LEN     Invalid hardware address length.
*
*                                                               - RETURNED BY NetOS_Dev_CfgTxRdySignal() : -
*                               NET_IF_ERR_INVALID_IF           Invalid network interface number.
*                               NET_OS_ERR_INIT_DEV_TX_RDY_VAL  Invalid device transmit ready signal.
*
*                                                               -------- RETURNED BY RxDescInit() : --------
*                          !!!!
*
*                                                               -------- RETURNED BY TxDescInit() : --------
*                          !!!!
*
* Return(s)   : none.
*
* Caller(s)   : NetIF_Ether_IF_Start() via 'pdev_api->Start()'.
*
* Note(s)     : (2) Setting the maximum number of frames queued for transmission is optional.  By
*                   default, all network interfaces are configured to block until the previous frame
*                   has completed transmission.  However, DMA based interfaces may have several
*                   frames configured for transmission before blocking is required. The number
*                   of queued transmit frames depends on the number of configured transmit
*                   descriptors.
*
*               (3) The physical hardware address should not be configured from NetDev_Init(). Instead,
*                   it should be configured from within NetDev_Start() to allow for the proper use
*                   of NetIF_Ether_HW_AddrSet(), hard coded hardware addresses from the device
*                   configuration structure, or auto-loading EEPROM's. Changes to the physical address
*                   only take effect when the device transitions from the DOWN to UP state.
*
*               (4) The device hardware address is set from one of the data sources below. Each source
*                   is listed in the order of precedence.
*
*                   (a) Device Configuration Structure       Configure a valid HW address during
*                                                                compile time.
*
*                                                            Configure either "00:00:00:00:00:00" or
*                                                                an empty string, "", in order to
*                                                                configure the HW address using using
*                                                                method (b).
*
*                   (b) NetIF_Ether_HW_AddrSet()             Call NetIF_Ether_HW_AddrSet() if the HW
*                                                                address needs to be configured via
*                                                                run-time from a different data
*                                                                source. E.g. Non auto-loading
*                                                                memory such as I2C or SPI EEPROM.
*                                                               (see Note #3).
*
*                   (c) Auto-Loading via EEPROM.             If neither options a) or b) are used,
*                                                                the IF layer will use the HW address
*                                                                obtained from the network hardware
*                                                                address registers.
*********************************************************************************************************
*/
/*$PAGE*/
static  void  NetDev_Start (NET_IF   *pif,
                            NET_ERR  *perr)
{
    NET_DEV_CFG_ETHER  *pdev_cfg;
    NET_DEV_DATA       *pdev_data;
    NET_DEV            *pdev;
    CPU_INT08U          hw_addr[NET_IF_ETHER_ADDR_SIZE];
    CPU_INT08U          hw_addr_len;
    CPU_BOOLEAN         hw_addr_cfg;
    NET_ERR             err;


                                                                /* -- OBTAIN REFERENCE TO DEVICE CFG/DATA/REGISTERS --- */
    pdev_cfg  = (NET_DEV_CFG_ETHER *)pif->Dev_Cfg;              /* Obtain ptr to the dev cfg struct.                    */
    pdev_data = (NET_DEV_DATA      *)pif->Dev_Data;             /* Obtain ptr to dev data area.                         */
    pdev      = (NET_DEV           *)pdev_cfg->BaseAddr;        /* Overlay dev reg struct on top of dev base addr.      */


#if (NET_CTR_CFG_STAT_EN == DEF_ENABLED)
    pdev_data->StatRxCtrRx             = 0;
    pdev_data->StatRxCtrRxISR          = 0;
    pdev_data->StatTxCtrTxISR          = 0;
    pdev_data->StatRxCtrTaskSignalISR  = 0;
    pdev_data->StatRxCtrTaskSignalSelf = 0;
    pdev_data->StatRxCtrTaskSignalNone = 0;
#endif

#if (NET_CTR_CFG_ERR_EN  == DEF_ENABLED)
    pdev_data->ErrRxCtrDataAreaAlloc   = 0;
    pdev_data->ErrRxCtrROS             = 0;
    pdev_data->ErrRxCtrRBUS            = 0;
    pdev_data->ErrRxCtrRPSS            = 0;
    pdev_data->ErrRxCtrDescES          = 0;
    pdev_data->ErrRxCtrDescOWN         = 0;
    pdev_data->ErrRxCtrRunt            = 0;
    pdev_data->ErrRxTxCtrFatalBusErr   = 0;
    pdev_data->ErrRxCtrTaskSignalFail  = 0;
#endif


                                                                /* ---------------- CFG TX RDY SIGNAL ----------------- */
    NetOS_Dev_CfgTxRdySignal(pif->Nbr,                          /* See Note #2.                                         */
                             pdev_cfg->TxDescNbr,
                             perr);
    if (*perr != NET_OS_ERR_NONE) {
         return;
    }


                                                                /* ------------------- CFG HW ADDR -------------------- */
    hw_addr_cfg = DEF_NO;                                       /* See Notes #3 & #4.                                   */

    NetASCII_Str_to_MAC(pdev_cfg->HW_AddrStr,                   /* Get configured HW MAC address string, if any ...     */
                       &hw_addr[0],                             /* ... (see Note #4a).                                  */
                       &err);
    if (err == NET_ASCII_ERR_NONE) {
        NetIF_AddrHW_SetHandler((NET_IF_NBR  ) pif->Nbr,
                                (CPU_INT08U *)&hw_addr[0],
                                (CPU_INT08U  ) sizeof(hw_addr),
                                (NET_ERR    *)&err);
    }

    if (err == NET_IF_ERR_NONE) {                               /* If no errors, configure device    HW MAC address.    */
        hw_addr_cfg = DEF_YES;

    } else {                                                    /* Else get  app-configured IF layer HW MAC address, ...*/
                                                                /* ... if any (see Note #4b).                           */
        hw_addr_len = sizeof(hw_addr);
        NetIF_AddrHW_GetHandler(pif->Nbr, &hw_addr[0], &hw_addr_len, &err);
        if (err == NET_IF_ERR_NONE) {
            hw_addr_cfg  = NetIF_AddrHW_IsValidHandler(pif->Nbr, &hw_addr[0], &err);
        } else {
            hw_addr_cfg  = DEF_NO;
        }

        if (hw_addr_cfg != DEF_YES) {                           /* Else attempt to get device's automatically loaded ...*/
                                                                /* ... HW MAC address, if any (see Note #4c).           */
            hw_addr[0] = (pdev->MACA0LR >> (0 * DEF_INT_08_NBR_BITS)) & DEF_INT_08_MASK;
            hw_addr[1] = (pdev->MACA0LR >> (1 * DEF_INT_08_NBR_BITS)) & DEF_INT_08_MASK;
            hw_addr[2] = (pdev->MACA0LR >> (2 * DEF_INT_08_NBR_BITS)) & DEF_INT_08_MASK;
            hw_addr[3] = (pdev->MACA0LR >> (3 * DEF_INT_08_NBR_BITS)) & DEF_INT_08_MASK;

            hw_addr[4] = (pdev->MACA0HR >> (0 * DEF_INT_08_NBR_BITS)) & DEF_INT_08_MASK;
            hw_addr[5] = (pdev->MACA0HR >> (1 * DEF_INT_08_NBR_BITS)) & DEF_INT_08_MASK;

            NetIF_AddrHW_SetHandler((NET_IF_NBR  ) pif->Nbr,    /* Configure IF layer to use automatically-loaded ...   */
                                    (CPU_INT08U *)&hw_addr[0],  /* ... HW MAC address.                                  */
                                    (CPU_INT08U  ) sizeof(hw_addr),
                                    (NET_ERR    *) perr);
            if (*perr != NET_IF_ERR_NONE) {                     /* No valid HW MAC address configured, return error.    */
                 return;
            }
        }
    }
    
      

    if (hw_addr_cfg == DEF_YES) {                               /* If necessary, set device HW MAC address.             */
        pdev->MACA0LR = (((CPU_INT32U)hw_addr[0] << (0 * DEF_INT_08_NBR_BITS)) |
                         ((CPU_INT32U)hw_addr[1] << (1 * DEF_INT_08_NBR_BITS)) |
                         ((CPU_INT32U)hw_addr[2] << (2 * DEF_INT_08_NBR_BITS)) |
                         ((CPU_INT32U)hw_addr[3] << (3 * DEF_INT_08_NBR_BITS)));

        pdev->MACA0HR = (((CPU_INT32U)hw_addr[4] << (0 * DEF_INT_08_NBR_BITS)) |
                         ((CPU_INT32U)hw_addr[5] << (1 * DEF_INT_08_NBR_BITS)));
    }


                                                                /* ------------------ CFG MULTICAST ------------------- */
    pdev->MACHTHR = 0;                                          /* Clr multicast hash filter hi bits.                   */
    pdev->MACHTLR = 0;                                          /* Clr multicast hash filter lo bits.                   */

#ifdef  NET_MULTICAST_PRESENT
                                                                /* Clr multicast addr reference bits.                   */
    Mem_Clr((void     *)       pdev_data->MulticastAddrHashBitCtr,
            (CPU_SIZE_T)sizeof(pdev_data->MulticastAddrHashBitCtr));
#endif


    NetDev_RxDescInit(pif, perr);                               /* Init Rx desc's.                                      */
    if (*perr != NET_DEV_ERR_NONE) {
         return;
    }

    NetDev_TxDescInit(pif, perr);                               /* Init Tx desc'c.                                      */
    if (*perr != NET_DEV_ERR_NONE) {
         return;
    }


    pdev->MACCR  |= (ETH_MACCR_TE   |                           /* Enable MAC Tx.                                       */
                     ETH_MACCR_RE);                             /* Enable MAC Rx.                                       */

    pdev->DMAOMR |= (ETH_DMAOMR_FTF |                           /* Flush  DMA Tx FIFO.                                  */
                     ETH_DMAOMR_ST  |                           /* Enable DMA Tx.                                       */
                     ETH_DMAOMR_SR);                            /* Enable DMA Rx.                                       */

    pdev->DMAIER  = (ETH_DMA_IT_NIS |                           /* Normal int. summary.                                 */
                     ETH_DMA_IT_R   |                           /* Rx     int.                                          */
                     ETH_DMA_IT_T);                             /* Tx     int.                                          */

    pdev->DMAIER |= (ETH_DMA_IT_AIS |                           /* Abnormal int. summary.                               */
                     ETH_DMA_IT_RO  |                           /* Rx (FIFO) overflow.                                  */
                     ETH_DMA_IT_RBU |                           /* Rx buffer unavailable.                               */
                     ETH_DMA_IT_FBE);                           /* Fatal bus err.                                       */

    (void)&pdev_data;                                           /* Prevent compiler warning.                            */

   *perr = NET_DEV_ERR_NONE;
}


/*$PAGE*/
/*
*********************************************************************************************************
*                                            NetDev_Stop()
*
* Description : (1) Shutdown network interface hardware :
*
*                   (a) Disable the receiver and transmitter
*                   (b) Disable receive and transmit interrupts
*                   (c) Clear pending interrupt requests
*                   (d) Free ALL receive descriptors (Return ownership to hardware)
*                   (e) Deallocate ALL transmit buffers
*
* Argument(s) : pif     Pointer to the interface requiring service.
*
*               perr    Pointer to return error code.
*                           NET_DEV_ERR_NONE    No Error
*
* Return(s)   : none.
*
* Caller(s)   : NetIF_Ether_IF_Stop() via 'pdev_api->Stop()'.
*
* Note(s)     : (2) (a) (1) It is recommended that a device driver should only post all currently-used,
*                           i.e. not-fully-transmitted, transmit buffers to the network interface transmit
*                           deallocation queue.
*
*                       (2) However, a driver MAY attempt to post all queued &/or transmitted buffers.
*                           The network interface transmit deallocation task will silently ignore any
*                           unknown or duplicate transmit buffers.  This allows device drivers to
*                           indiscriminately & easily post all transmit buffers without determining
*                           which buffers have NOT yet been transmitted.
*
*                   (b) (1) Device drivers should assume that the network interface transmit deallocation
*                           queue is large enough to post all currently-used transmit buffers.
*
*                       (2) If the transmit deallocation queue is NOT large enough to post all transmit
*                           buffers, some transmit buffers may/will be leaked/lost.
*
*               (3) All functions that require device register access MUST obtain reference
*                   to the device hardware register space PRIOR to attempting to access
*                   any registers.  Register definitions SHOULD NOT be absolute and SHOULD
*                   use the provided base address within the device configuration structure,
*                   as well as the device register definition structure in order to properly
*                   resolve register addresses during run-time.
*********************************************************************************************************
*/

static  void  NetDev_Stop (NET_IF   *pif,
                           NET_ERR  *perr)
{
    NET_DEV_CFG_ETHER  *pdev_cfg;
    NET_DEV_DATA       *pdev_data;
    NET_DEV            *pdev;
    DEV_DESC           *pdesc;
    CPU_INT08U          i;
    NET_ERR             err;


                                                                /* -- OBTAIN REFERENCE TO DEVICE CFG/DATA/REGISTERS --- */
    pdev_cfg  = (NET_DEV_CFG_ETHER *)pif->Dev_Cfg;              /* Obtain ptr to the dev cfg struct.                    */
    pdev_data = (NET_DEV_DATA      *)pif->Dev_Data;             /* Obtain ptr to dev data area.                         */
    pdev      = (NET_DEV           *)pdev_cfg->BaseAddr;        /* Overlay dev reg struct on top of dev base addr.      */

    pdev->MACCR  &= ~ETH_MACCR_TE;                              /* Disable the MAC transmission.                        */
    pdev->DMAOMR |=  ETH_DMAOMR_FTF;                            /* Set the Flush Transmit FIFO bit.                     */
    pdev->MACCR  &= ~ETH_MACCR_RE;                              /* Disable the MAC reception.                           */
    pdev->DMAOMR &= ~ETH_DMAOMR_ST;                             /* Disable the DMA transmission.                        */
    pdev->DMAOMR &= ~ETH_DMAOMR_SR;                             /* Disable the DMA reception.                           */


                                                                /* ------------- DISABLE & CLEAR INT'S ---------------- */
    pdev->DMAIER = 0;                                           /* Disable Rx, Tx and other supported int. src.         */
    pdev->DMASR  = 0xFFFFFFFF;                                  /* Clear all pending int. sources.                      */

                                                                /* --------------- FREE RX DESCRIPTORS ---------------- */
    NetDev_RxDescFreeAll(pif, perr);
    if (*perr != NET_DEV_ERR_NONE) {
         return;
    }

                                                                /* ------------ FREE USED TX DESCRIPTORS -------------- */
    pdesc = &pdev_data->TxBufDescPtrStart[0];
    for (i = 0; i < pdev_cfg->TxDescNbr; i++) {                 /* Dealloc ALL tx bufs (see Note #2a2).                 */
        NetOS_IF_TxDeallocTaskPost((CPU_INT08U *)pdesc->Addr, &err);
       (void)&err;                                              /* Ignore possible dealloc err (see Note #2b2).         */
        pdesc++;
    }


   *perr = NET_DEV_ERR_NONE;
}


/*$PAGE*/
/*
*********************************************************************************************************
*                                            NetDev_Rx()
*
* Description : (1) This function returns a pointer to the received data to the caller :
*                   (a) Determine which receive descriptor caused the interrupt
*                   (b) Obtain pointer to data area to replace existing data area
*                   (c) Reconfigure descriptor with pointer to new data area
*                   (d) Set return values.  Pointer to received data area and size
*                   (e) Update current receive descriptor pointer
*                   (f) Increment counters.
*
* Argument(s) : pif     Pointer to the interface requiring service.
*
*               p_data  Pointer to pointer to received DMA data area. The received data
*                       area address should be returned to the stack by dereferencing
*                       p_data as *p_data = (address of receive data area).
*
*               size    Pointer to size. The number of bytes received should be returned
*                       to the stack by dereferencing size as *size = (number of bytes).
*
*               perr    Pointer to return error code.
*                           NET_DEV_ERR_NONE            No Error
*                           NET_DEV_ERR_RX              Generic Rx error.
*                           NET_DEV_ERR_INVALID_SIZE    Invalid Rx frame size.
*                           NET_BUF error codes         Potential NET_BUF error codes
*
* Return(s)   : none.
*
* Caller(s)   : NetIF_RxPkt() via 'pdev_api->Rx()'.
*
* Note(s)     : (2) All functions that require device register access MUST obtain reference
*                   to the device hardware register space PRIOR to attempting to access
*                   any registers.  Register definitions SHOULD NOT be absolute and SHOULD
*                   use the provided base address within the device configuration structure,
*                   as well as the device register definition structure in order to properly
*                   resolve register addresses during run-time.
*
*               (3) If a receive error occurs and the descriptor is invalid then the function
*                   SHOULD return 0 for the size, a NULL pointer to the data area AND an
*                   error equal to NET_DEV_ERR_RX.
*
*                   (a) If the next expected ready / valid descriptor is NOT owned by
*                       software, then there is descriptor pointer corruption and the
*                       driver should NOT increment the current receive descriptor
*                       pointer.
*
*                   (b) If the descriptor IS valid, but an error is indicated within
*                       the descriptor status bits, or length field, then the driver
*                       MUST increment the current receive descriptor pointer and discard
*                       the received frame.
*
*                   (c) If a new data area is unavailable, the driver MUST increment
*                       the current receive descriptor pointer and discard the received
*                       frame.  This will invoke the DMA to re-use the existing configured
*                       data area.
*********************************************************************************************************
*/
/*$PAGE*/
static  void  NetDev_Rx (NET_IF       *pif,
                         CPU_INT08U  **p_data,
                         CPU_INT16U   *size,
                         NET_ERR      *perr)
{
    NET_DEV_CFG_ETHER  *pdev_cfg;
    NET_DEV_DATA       *pdev_data;
    NET_DEV            *pdev;
    DEV_DESC           *pdesc;
    CPU_INT08U         *pbuf_new;
    CPU_INT16S          len;


                                                                /* -- OBTAIN REFERENCE TO DEVICE CFG/DATA/REGISTERS --- */
    pdev_cfg  = (NET_DEV_CFG_ETHER *)pif->Dev_Cfg;              /* Obtain ptr to the dev cfg struct.                    */
    pdev_data = (NET_DEV_DATA      *)pif->Dev_Data;             /* Obtain ptr to dev data area.                         */
    pdev      = (NET_DEV           *)pdev_cfg->BaseAddr;        /* Overlay dev reg struct on top of dev base addr.      */
    pdesc     = (DEV_DESC          *)pdev_data->RxBufDescPtrCur;/* Obtain ptr to next ready desc.                       */


                                                                /* ------------- CHECK FOR RECEIVE ERRORS ------------- */
    if ((pdesc->Status & ETH_DMA_RX_DESC_OWN) != 0) {           /* Check if the desc is owned by DMA.                   */
        pdev->DMAIER |= (ETH_DMA_IT_R   |                       /* Re-Enable Rx int's.                                  */
                         ETH_DMA_IT_RO  |                       /* Rx (FIFO) overrun.                                   */
                         ETH_DMA_IT_RBU |                       /* Rx buffer unavailable.                               */
                         ETH_DMA_IT_RPS);                       /* Rx process stopped.                                  */

        NET_CTR_ERR_INC(pdev_data->ErrRxCtrDescOWN);
       *size   = (CPU_INT16U  )0;
       *p_data = (CPU_INT08U *)0;
       *perr   = (NET_ERR     )NET_DEV_ERR_RX;
        return;
    }

    if ((pdesc->Status & ETH_DMA_RX_DESC_ES) != 0) {            /* If err Summary reports err's, including Rx overflow. */
        NetDev_RxDescPtrCurInc(pif);                            /* Discard recv'd frame (see Note #3b)                  */
        NET_CTR_ERR_INC(pdev_data->ErrRxCtrDescES);
       *size   = (CPU_INT16U  )0;
       *p_data = (CPU_INT08U *)0;
       *perr   = (NET_ERR     )NET_DEV_ERR_RX;
        return;
    }

                                                                /* Shift and subtract 4 bytes of the CRC.               */
    len = ((pdesc->Status & ETH_DMA_RX_DESC_FL) >> ETH_DMA_RX_DESC_FRAME_LEN_SHIFT) - 4;	

    if (len < NET_IF_ETHER_FRAME_MIN_SIZE) {                    /* If frame is a runt.                                  */
        NetDev_RxDescPtrCurInc(pif);                            /* Discard received frame (see Note #3b).               */
        NET_CTR_ERR_INC(pdev_data->ErrRxCtrRunt);
       *size   = (CPU_INT16U  )0;
       *p_data = (CPU_INT08U *)0;
       *perr   = (NET_ERR     )NET_DEV_ERR_INVALID_SIZE;
        return;
    }

    if(len > 100) {
      volatile int i=0;
      i++;
    }
                                                                /* --------- OBTAIN PTR TO NEW DMA DATA AREA ---------- */
    pbuf_new = NetBuf_GetDataPtr((NET_IF        *)pif,
                                 (NET_TRANSACTION)NET_TRANSACTION_RX,
#if (NET_VERSION >= 21000u)
                                 (NET_BUF_SIZE   )NET_IF_ETHER_FRAME_MAX_SIZE,
                                 (NET_BUF_SIZE   )NET_IF_IX_RX,
                                 (NET_BUF_SIZE  *)0,
#else
                                 (NET_BUF_SIZE   )pdev_cfg->RxBufLargeSize,
                                 (NET_BUF_SIZE   )0u,
#endif
                                 (NET_BUF_SIZE  *)0,
                                 (NET_TYPE      *)0,
                                 (NET_ERR       *)perr);


    if (*perr != NET_BUF_ERR_NONE) {                            /* If unable to get a buffer.                           */
        NetDev_RxDescPtrCurInc(pif);                            /* Free cur desc.                                       */
        NET_CTR_ERR_INC(pdev_data->ErrRxCtrDataAreaAlloc);
       *size   = (CPU_INT16U  )0;
       *p_data = (CPU_INT08U *)0;
        return;
    }

   *size          =  len;                                       /* Return the size of the recv'd frame.                 */
   *p_data        = (CPU_INT08U *)pdesc->Addr;                  /* Return a ptr to the newly recv'd data.               */

    pdesc->Addr   =  pbuf_new;                                  /* Update the desc to point to a new data area          */
    NetDev_RxDescPtrCurInc(pif);                                /* Free the current desc.                               */

    NET_CTR_STAT_INC(pdev_data->StatRxCtrRx);

    *perr = NET_DEV_ERR_NONE;
}


/*$PAGE*/
/*
*********************************************************************************************************
*                                            NetDev_Tx()
*
* Description : (1) This function transmits the specified data :
*
*                   (a) Check if the transmitter is ready.
*                   (b) Configure the next transmit descriptor for pointer to data and data size.
*                   (c) Issue the transmit command.
*                   (d) Increment pointer to next transmit descriptor
*
* Argument(s) : pif     Pointer to the interface requiring service.
*
*               perr    Pointer to return error code.
*                           NET_DEV_ERR_NONE        No Error
*                           NET_DEV_ERR_TX_BUSY     No Tx descriptors available
*
* Return(s)   : none.
*
* Caller(s)   : NetIF_TxPkt() via 'pdev_api->Tx()'.
*
* Note(s)     : (2) All functions that require device register access MUST obtain reference
*                   to the device hardware register space PRIOR to attempting to access
*                   any registers.  Register definitions SHOULD NOT be absolute and SHOULD
*                   use the provided base address within the device configuration structure,
*                   as well as the device register definition structure in order to properly
*                   resolve register addresses during run-time.
*
*               (3) Care should be taken to avoid skipping transmit descriptors while selecting
*                   the next available descriptor.  Software MUST track the descriptor which
*                   is expected to generate the next transmit complete interrupt.  Skipping
*                   descriptors, unless carefully accounted for, may make it difficult to
*                   know which descriptor will complete transmission next.  Some device
*                   drivers may find it useful to adjust pdev_data->TxBufDescCompPtr after
*                   having selected the next available transmit descriptor.
*
*               (4)  The T01 descriptor bit is reserved for software use ONLY. This bit is set by software
*                    during transmission to indicate that the data ready bit for the descriptor is being set.
*                    When the DMA hardware completes transmission of the descriptor data, the ready bit is
*                    cleared and the T01 bit remains set. The transmit complete ISR handler then detects the
*                    cleared ready bit and set T01 bit and determines that the descriptor has completed
*                    transmission. The T01 bit is then cleared by the ISR handler.
*********************************************************************************************************
*/

static  void  NetDev_Tx (NET_IF      *pif,
                         CPU_INT08U  *p_data,
                         CPU_INT16U   size,
                         NET_ERR     *perr)
{
    NET_DEV_CFG_ETHER  *pdev_cfg;
    NET_DEV_DATA       *pdev_data;
    NET_DEV            *pdev;
    DEV_DESC           *pdesc;

                                                                /* ------- OBTAIN REFERENCE TO DEVICE REGISTERS ------- */
    pdev_cfg  = (NET_DEV_CFG_ETHER *)pif->Dev_Cfg;              /* Obtain ptr to the dev cfg struct.                    */
    pdev_data = (NET_DEV_DATA      *)pif->Dev_Data;             /* Obtain ptr to dev data area.                         */
    pdev      = (NET_DEV           *)pdev_cfg->BaseAddr;        /* Overlay dev reg struct on top of dev base addr.      */
    pdesc     = (DEV_DESC          *)pdev_data->TxBufDescPtrCur;

    if ((pdesc->Status & ETH_DMA_TX_DESC_OWN) != 0) {           /* If cur Tx desc owned by software, then return err.   */
       *perr = NET_DEV_ERR_TX_BUSY;
        return;
    }

    pdesc->Addr                =  p_data;                       /* Cfg desc with Tx data area addr.                     */
    pdesc->Len                 = (size & ETH_DMA_TX_DESC_TBS1); /* Cfg frame len: bits[12:0].                           */

    pdesc->Status             |=  ETH_DMA_TX_DESC_OWN;          /* Give desc ownership to HW.                           */

    pdev->DMASR                =  ETH_DMASR_TBUS;
    pdev->DMATPDR              =  1;                            /* Inform hardware that a Tx desc has been made avail.  */
    pdev_data->TxBufDescPtrCur =  pdesc->Next;                  /* Increment for the next transmit                      */

   *perr = NET_DEV_ERR_NONE;
}


/*$PAGE*/
/*
*********************************************************************************************************
*                                        NetDev_RxDescFreeAll()
*
* Description : (1) This function returns the descriptor memory block and descriptor data area
*                   memory blocks back to their respective memory pools :
*
*                   (a) Free Rx descriptor data areas
*                   (b) Free Rx descriptor memory block
*
* Argument(s) : pif     Pointer to the interface requiring service.
*
*               perr    Pointer to return error code.
*                           NET_DEV_ERR_NONE        No error
*                           NET_IF_MGR_ERR_nnnn     Various Network Interface management error codes
*                           NET_BUF_ERR_nnn         Various Network buffer error codes
* Return(s)   : none.
*
* Caller(s)   : NetDev_Stop().
*
* Note(s)     : none.
*********************************************************************************************************
*/

static  void  NetDev_RxDescFreeAll(NET_IF   *pif,
                                   NET_ERR  *perr)
{
    NET_DEV_CFG_ETHER  *pdev_cfg;
    NET_DEV_DATA       *pdev_data;
    DEV_DESC           *pdesc;
    CPU_INT16U          i;
    CPU_INT08U         *pdesc_data;


                                                                /* ------- OBTAIN REFERENCE TO DEVICE CFG/DATA -------- */
    pdev_cfg  = (NET_DEV_CFG_ETHER *)pif->Dev_Cfg;              /* Obtain ptr to the dev cfg struct.                    */
    pdev_data = (NET_DEV_DATA      *)pif->Dev_Data;             /* Obtain ptr to dev data area.                         */

                                                                /* ------------- FREE RX DESC DATA AREAS -------------- */
    pdesc  = pdev_data->RxBufDescPtrStart;
    for (i = 0; i < pdev_cfg->RxDescNbr; i++) {                 /* Free Rx desc ring.                                   */
        pdesc_data = (CPU_INT08U *)(pdesc->Addr);
        NetBuf_FreeBufDataAreaRx(pif->Nbr, pdesc_data);         /* Return data area to Rx data area pool.               */
        pdesc++;
    }

   *perr = NET_DEV_ERR_NONE;
}


/*$PAGE*/
/*
*********************************************************************************************************
*                                          NetDev_RxDescInit()
*
* Description : This function initializes the Rx descriptor list for the specified interface.
*
* Argument(s) : pif     Pointer to the interface requiring service.
*
*               perr    Pointer to return error code.
*                           NET_DEV_ERR_NONE        No error
*                           NET_IF_MGR_ERR_nnnn     Various Network Interface management error codes
*                           NET_BUF_ERR_nnn         Various Network buffer error codes
* Return(s)   : none.
*
* Caller(s)   : NetDev_Init().
*
* Note(s)     : (1) Memory allocation for the descriptors and receive buffers MUST be performed BEFORE
*                   calling this function. This ensures that multiple calls to this function do NOT
*                   allocate additional memory to the interface and that the Rx descriptors may be
*                   safely re-initialized by calling this function.
*********************************************************************************************************
*/

static  void  NetDev_RxDescInit (NET_IF   *pif,
                                 NET_ERR  *perr)
{
    NET_DEV_CFG_ETHER  *pdev_cfg;
    NET_DEV_DATA       *pdev_data;
    NET_DEV            *pdev;
    DEV_DESC           *pdesc;
    CPU_INT16U          i;


                                                                /* -- OBTAIN REFERENCE TO DEVICE CFG/DATA/REGISTERS --- */
    pdev_cfg  = (NET_DEV_CFG_ETHER *)pif->Dev_Cfg;              /* Obtain ptr to the dev cfg struct.                    */
    pdev_data = (NET_DEV_DATA      *)pif->Dev_Data;             /* Obtain ptr to dev data area.                         */
    pdev      = (NET_DEV           *)pdev_cfg->BaseAddr;        /* Overlay dev reg struct on top of dev base addr.      */


                                                                /* ------------- INIT DESCRIPTOR PTRS  ---------------- */
    pdesc                        = (DEV_DESC *)pdev_data->RxBufDescPtrStart;
    pdev_data->RxBufDescPtrStart = (DEV_DESC *)pdesc;
    pdev_data->RxBufDescPtrCur   = (DEV_DESC *)pdesc;
    pdev_data->RxBufDescPtrEnd   = (DEV_DESC *)pdesc + (pdev_cfg->RxDescNbr - 1);

    for (i = 0; i < pdev_cfg->RxDescNbr; i++) {                 /* Init Rx descriptor ring.                             */
        pdesc->Len    = pdev_cfg->RxBufLargeSize |
                        ETH_DMA_RX_DESC_RCH;

        pdesc->Status = ETH_DMA_RX_DESC_OWN;                    /* DMA owns the desc.                                   */

        pdesc->Addr   = NetBuf_GetDataPtr((NET_IF        *)pif,
                                          (NET_TRANSACTION)NET_TRANSACTION_RX,
#if (NET_VERSION >= 21000u)
                                          (NET_BUF_SIZE   )NET_IF_ETHER_FRAME_MAX_SIZE,
                                          (NET_BUF_SIZE   )NET_IF_IX_RX,
                                          (NET_BUF_SIZE  *)0,
#else
                                          (NET_BUF_SIZE   )pdev_cfg->RxBufLargeSize,
                                          (NET_BUF_SIZE   )0u,
#endif
                                          (NET_BUF_SIZE  *)0,
                                          (NET_TYPE      *)0,
                                          (NET_ERR       *)perr);
        if (*perr != NET_BUF_ERR_NONE) {
             return;
        }

        pdesc->Next = (DEV_DESC *)(pdesc + 1);                  /* Point to next desc in list.                          */
        pdesc++;
    }

    pdesc--;
    pdesc->Next    = (DEV_DESC *)pdev_data->RxBufDescPtrStart;  /* Last desc points to first desc.                      */
    pdev->DMARDLAR = (CPU_INT32U)pdev_data->RxBufDescPtrStart;  /* Cfg DMA with addr of first desc.                     */

   *perr = NET_DEV_ERR_NONE;
}


/*$PAGE*/
/*
*********************************************************************************************************
*                                        NetDev_RxDescPtrCurInc()
*
* Description : (1) Increment current descriptor pointer to next receive descriptor :
*
*                   (a) Return ownership of current descriptor back to DMA.
*                   (b) Point to the next descriptor.
*
* Argument(s) : pif     Pointer to interface requiring service.
*               ---
*
* Return(s)   : none.
*
* Caller(s)   : NetDev_Rx().
*
* Note(s)     : none.
*********************************************************************************************************
*/

static  void  NetDev_RxDescPtrCurInc (NET_IF  *pif)
{
    NET_DEV_CFG_ETHER  *pdev_cfg;
    NET_DEV            *pdev;
    NET_DEV_DATA       *pdev_data;
    DEV_DESC           *pdesc;
    NET_ERR             err;
    CPU_INT32U          int_status;


                                                                /* -- OBTAIN REFERENCE TO DEVICE CFG/DATA/REGISTERS --- */
    pdev_cfg  = (NET_DEV_CFG_ETHER *)pif->Dev_Cfg;              /* Obtain ptr to the dev cfg struct.                    */
    pdev_data = (NET_DEV_DATA      *)pif->Dev_Data;             /* Obtain ptr to dev data area.                         */
    pdev      = (NET_DEV           *)pdev_cfg->BaseAddr;        /* Overlay dev reg struct on top of dev base addr.      */



    pdesc                      = pdev_data->RxBufDescPtrCur;
    pdev_data->RxBufDescPtrCur = pdesc->Next;
    pdesc->Status              = ETH_DMA_RX_DESC_OWN;           /* Return desc ownership back to hardware.              */

    pdev->DMARPDR = 1;                                          /* Inform hardware of available Rx desc.                */

    pdesc = pdesc->Next;                                        /* Check if there are add'l frames to Rx.               */
    if ((pdesc->Status & ETH_DMA_RX_DESC_OWN) == 0) {           /* Check if the desc is owned by SW.                    */
        NetOS_IF_RxTaskSignal(pif->Nbr, &err);                  /* Signal Net IF RxQ Task for each new rdy Desc.        */
        switch (err) {
            case NET_IF_ERR_NONE:
                 NET_CTR_STAT_INC(pdev_data->StatRxCtrTaskSignalSelf);
                 break;


            case NET_IF_ERR_RX_Q_FULL:
            case NET_IF_ERR_RX_Q_SIGNAL_FAULT:
            default:
                 NET_CTR_ERR_INC(pdev_data->ErrRxCtrTaskSignalFail);
                 break;
        }
    } else {
        int_status    =  pdev->DMASR;                           /* Clr active int. src's.                               */
        pdev->DMASR   =  int_status;

        pdev->DMAIER |= (ETH_DMA_IT_R   |                       /* Re-Enable Rx int's.                                  */
                         ETH_DMA_IT_RO  |                       /* Rx (FIFO) overrun.                                   */
                         ETH_DMA_IT_RBU |                       /* Rx buffer unavailable.                               */
                         ETH_DMA_IT_RPS);                       /* Rx process stopped.                                  */

        NET_CTR_STAT_INC(pdev_data->StatRxCtrTaskSignalNone);
    }
}


/*$PAGE*/
/*
*********************************************************************************************************
*                                        NetDev_TxDescInit()
*
* Description : (1) This function initializes the Tx descriptor list for the specified interface :
*
*                   (a) Obtain reference to the Tx descriptor(s) memory block
*                   (b) Initialize Tx descriptor pointers
*                   (c) Obtain Tx descriptor data areas
*                   (d) Initialize hardware registers
*
* Argument(s) : pif     Pointer to the interface requiring service.
*
*               perr    Pointer to return error code.
*                           NET_DEV_ERR_NONE        No error
*                           NET_IF_MGR_ERR_nnnn     Various Network Interface management error codes
*                           NET_BUF_ERR_nnn         Various Network buffer error codes
* Return(s)   : none.
*
* Caller(s)   : NetDev_Start().
*
* Note(s)     : (2) All functions that require device register access MUST obtain reference
*                   to the device hardware register space PRIOR to attempting to access
*                   any registers.  Register definitions SHOULD NOT be absolute and SHOULD
*                   use the provided base address within the device configuration structure,
*                   as well as the device register definition structure in order to properly
*                   resolve register addresses during run-time.
*
*               (3) All Tx descriptors are allocated as ONE memory block.  This removes the
*                   necessity to ensure that descriptor blocks are returned to the descriptor
*                   pool in the opposite order in which they were allocated; doing so would
*                   ensure that each memory block address was contiguous to the one before
*                   and after it.  If the descriptors are NOT contiguous, then software
*                   MUST NOT assign a pointer to the pool start address and use pointer
*                   arithmetic to navigate the descriptor list.  Since pointer arithmetic
*                   is a convenient way to navigate the descriptor list, ONE block is allocated
*                   and the driver uses pointer arithmetic to slice the block into descriptor
*                   sized units.
*********************************************************************************************************
*/

static  void  NetDev_TxDescInit (NET_IF   *pif,
                                 NET_ERR  *perr)
{
    NET_DEV_CFG_ETHER  *pdev_cfg;
    NET_DEV_DATA       *pdev_data;
    NET_DEV            *pdev;
    DEV_DESC           *pdesc;
    CPU_INT16U          i;


                                                                /* -- OBTAIN REFERENCE TO DEVICE CFG/DATA/REGISTERS --- */
    pdev_cfg  = (NET_DEV_CFG_ETHER *)pif->Dev_Cfg;              /* Obtain ptr to the dev cfg struct.                    */
    pdev_data = (NET_DEV_DATA      *)pif->Dev_Data;             /* Obtain ptr to dev data area.                         */
    pdev      = (NET_DEV           *)pdev_cfg->BaseAddr;        /* Overlay dev reg struct on top of dev base addr.      */


                                                                /* -------------- INIT DESCRIPTOR PTRS  --------------- */
    pdesc                       = (DEV_DESC *)pdev_data->TxBufDescPtrStart;
    pdev_data->TxBufDescPtrComp = (DEV_DESC *)pdev_data->TxBufDescPtrStart;
    pdev_data->TxBufDescPtrCur  = (DEV_DESC *)pdev_data->TxBufDescPtrStart;

    for (i = 0; i < pdev_cfg->TxDescNbr; i++) {                 /* Init Tx desc ring.                                   */
        pdesc->Addr   =  0;
        pdesc->Len    =  0;

        pdesc->Status =  ETH_DMA_TX_DESC_TCH |                  /* Second Addr Chained field points to next desc.       */
                         ETH_DMA_TX_DESC_FS  |                  /* Desc contains first segment.                         */
                         ETH_DMA_TX_DESC_LS  |                  /* Desc contains last  segment.                         */
                         ETH_DMA_TX_DESC_IC;                    /* Interrupt on completion.                             */

        pdesc->Next   = (DEV_DESC *)(pdesc + 1);                /* Point to next desc in list.                          */

        pdesc++;
    }

    pdesc--;
    pdesc->Next    = (DEV_DESC *)pdev_data->TxBufDescPtrStart;  /* Last desc wraps to first desc.                       */
    pdev->DMATDLAR = (CPU_INT32U)pdev_data->TxBufDescPtrStart;  /* Set Tx desc list addr reg.                           */

    *perr = NET_DEV_ERR_NONE;
}


/*$PAGE*/
/*
*********************************************************************************************************
*                                       NetDev_AddrMulticastAdd()
*
* Description : Configure hardware address filtering to accept specified hardware address.
*
* Argument(s) : pif         Pointer to an Ethernet network interface.
*               ---         Argument validated in NetIF_AddrHW_SetHandler().
*
*               paddr_hw    Pointer to hardware address.
*               --------    Argument checked   in NetIF_AddrHW_SetHandler().
*
*               addr_len    Length  of hardware address.
*
*               perr        Pointer to variable that will receive the return error code from this function :
*
*                               NET_DEV_ERR_NONE                Hardware address successfully configured.
*
*                                                               - RETURNED BY NetUtil_32BitCRC_Calc() : -
*                               NET_UTIL_ERR_NULL_PTR           Argument 'pdata_buf' passed NULL pointer.
*                               NET_UTIL_ERR_NULL_SIZE          Argument 'len' passed equal to 0.
*
* Return(s)   : none.
*
* Caller(s)   : NetIF_Ether_AddrMulticastAdd() via 'pdev_api->AddrMulticastAdd()'.
*
* Note(s)     : (1) The device is capable of the following multicast address filtering techniques :
*                       (a) Perfect        filtering of ONE multicast address.
*                       (b) Imperfect hash filtering of 64  multicast addresses.
*                       (c) Promiscous non filtering.  Disable filtering of all received frames.
*
*               (2) This function implements the filtering mechanism described in 1b.
*********************************************************************************************************
*/

#if (NET_VERSION >= 205u)
static  void  NetDev_AddrMulticastAdd (NET_IF      *pif,
                                       CPU_INT08U  *paddr_hw,
                                       CPU_INT08U   addr_hw_len,
                                       NET_ERR     *perr)
{
#ifdef NET_MULTICAST_PRESENT
    NET_DEV_CFG_ETHER  *pdev_cfg;
    NET_DEV_DATA       *pdev_data;
    NET_DEV            *pdev;
    CPU_INT32U          crc;
    CPU_INT08U          bit_sel;
    CPU_INT08U          hash;
    CPU_INT08U         *paddr_hash_ctrs;
    CPU_BOOLEAN         reg_sel;


                                                                /* -- OBTAIN REFERENCE TO DEVICE CFG/DATA/REGISTERS --- */
    pdev_cfg  = (NET_DEV_CFG_ETHER *)pif->Dev_Cfg;              /* Obtain ptr to the dev cfg struct.                    */
    pdev_data = (NET_DEV_DATA      *)pif->Dev_Data;             /* Obtain ptr to dev data area.                         */
    pdev      = (NET_DEV           *)pdev_cfg->BaseAddr;        /* Overlay dev reg struct on top of dev base addr.      */


    crc = NetUtil_32BitCRC_CalcCpl((CPU_INT08U  *)paddr_hw,        /* Obtain CRC without compliment.                       */
                                   (CPU_INT32U   )addr_hw_len,
                                   (NET_ERR     *)perr);

    if (*perr != NET_UTIL_ERR_NONE) {
        return;
    }

    crc = NetUtil_32BitReflect(crc);                            /* Reflect CRC to produce equivalent hardware result.   */

    hash    = (crc >> 26) & 0x3F;                               /* Obtain hash from 6 most significant bits of CRC.     */
    bit_sel = (crc >> 26) & 0x1F;                               /* Obtain least 5 significant bits for for reg bit sel. */
    reg_sel = (crc >> 31) & 0x01;                               /* Obtain most significant bit for reg sel.             */

    paddr_hash_ctrs = &pdev_data->MulticastAddrHashBitCtr[hash];
    (*paddr_hash_ctrs)++;                                       /* Increment hash bit reference ctr.                    */

    if (reg_sel == 0) {                                         /* Set multicast hash reg bit.                          */
        pdev->MACHTLR |= (1 << bit_sel);
    } else {
        pdev->MACHTHR |= (1 << bit_sel);
    }

#else
   (void)&pif;                                                  /* Prevent 'variable unused' compiler warnings.         */
   (void)&paddr_hw;
   (void)&addr_hw_len;
#endif

   *perr = NET_DEV_ERR_NONE;
}
#endif


/*$PAGE*/
/*
*********************************************************************************************************
*                                     NetDev_AddrMulticastRemove()
*
* Description : Configure hardware address filtering to reject specified hardware address.
*
* Argument(s) : pif         Pointer to an Ethernet network interface.
*               ---         Argument validated in NetIF_AddrHW_SetHandler().
*
*               paddr_hw    Pointer to hardware address.
*               --------    Argument checked   in NetIF_AddrHW_SetHandler().
*
*               addr_len    Length  of hardware address.
*
*               perr        Pointer to variable that will receive the return error code from this function :
*
*                               NET_DEV_ERR_NONE                Hardware address successfully removed.
*
*                                                               - RETURNED BY NetUtil_32BitCRC_Calc() : -
*                               NET_UTIL_ERR_NULL_PTR           Argument 'pdata_buf' passed NULL pointer.
*                               NET_UTIL_ERR_NULL_SIZE          Argument 'len' passed equal to 0.
* Return(s)   : none.
*
* Caller(s)   : NetIF_Ether_AddrMulticastAdd() via 'pdev_api->AddrMulticastRemove()'.
*
* Note(s)     : (1) The device is capable of the following multicast address filtering techniques :
*                       (a) Perfect        filtering of ONE multicast address.
*                       (b) Imperfect hash filtering of 64  multicast addresses.
*                       (c) Promiscous non filtering.  Disable filtering of all received frames.
*
*               (2) This function implements the filtering mechanism described in 1b.
*********************************************************************************************************
*/

#if (NET_VERSION >= 205u)
static  void  NetDev_AddrMulticastRemove (NET_IF      *pif,
                                          CPU_INT08U  *paddr_hw,
                                          CPU_INT08U   addr_hw_len,
                                          NET_ERR     *perr)
{
#ifdef NET_MULTICAST_PRESENT
    NET_DEV_CFG_ETHER  *pdev_cfg;
    NET_DEV_DATA       *pdev_data;
    NET_DEV            *pdev;
    CPU_INT32U          crc;
    CPU_INT08U          bit_sel;
    CPU_INT08U          hash;
    CPU_INT08U         *paddr_hash_ctrs;
    CPU_BOOLEAN         reg_sel;


                                                                /* -- OBTAIN REFERENCE TO DEVICE CFG/DATA/REGISTERS --- */
    pdev_cfg  = (NET_DEV_CFG_ETHER *)pif->Dev_Cfg;              /* Obtain ptr to the dev cfg struct.                    */
    pdev_data = (NET_DEV_DATA      *)pif->Dev_Data;             /* Obtain ptr to dev data area.                         */
    pdev      = (NET_DEV           *)pdev_cfg->BaseAddr;        /* Overlay dev reg struct on top of dev base addr.      */

    crc = NetUtil_32BitCRC_CalcCpl((CPU_INT08U  *)paddr_hw,     /* Obtain CRC without compliment.                       */
                                   (CPU_INT32U   )addr_hw_len,
                                   (NET_ERR     *)perr);

    if (*perr != NET_UTIL_ERR_NONE) {
        return;
    }

    crc = NetUtil_32BitReflect(crc);                            /* Reflect CRC to produce equivalent hardware result.   */

    hash    = (crc >> 26) & 0x3F;                               /* Obtain hash from 6 most significant bits of CRC.     */
    bit_sel = (crc >> 26) & 0x1F;                               /* Obtain least 5 significant bits for for reg bit sel. */
    reg_sel = (crc >> 31) & 0x01;                               /* Obtain most significant bit for reg sel.             */

    paddr_hash_ctrs = &pdev_data->MulticastAddrHashBitCtr[hash];
    if (*paddr_hash_ctrs > 1) {                                 /* If hash bit reference ctr not zero.                  */
       (*paddr_hash_ctrs)--;                                    /* Decrement hash bit reference ctr.                    */
        *perr = NET_DEV_ERR_NONE;
         return;
    }

   *paddr_hash_ctrs = 0;                                        /* Zero hash bit references remaining.                  */

    if (reg_sel == 0) {                                         /* Clr multicast hash reg bit.                          */
        pdev->MACHTLR &= ~(1 << bit_sel);
    } else {
        pdev->MACHTHR &= ~(1 << bit_sel);
    }

#else
   (void)&pif;                                                  /* Prevent 'variable unused' compiler warnings.         */
   (void)&paddr_hw;
   (void)&addr_hw_len;
#endif

   *perr = NET_DEV_ERR_NONE;
}
#endif


/*$PAGE*/
/*
*********************************************************************************************************
*                                            NetDev_ISR_Handler()
*
* Description : This function serves as the device Interrupt Service Routine Handler. This ISR
*               handler MUST service and clear all necessary and enabled interrupt events for
*               the device.
*
* Argument(s) : pif     Pointer to interface requiring service.
*               ---
*
*               type    Network Interface defined argument representing the type of ISR in progress. Codes
*                       for Rx, Tx, Overrun, Jabber, etc... are defined within net_if.h and are passed
*                       into this function by the corresponding Net BSP ISR handler function. The Net
*                       BSP ISR handler function may be called by a specific ISR vector and therefore
*                       know which ISR type code to pass.  Otherwise, the Net BSP may pass
*                       NET_DEV_ISR_TYPE_UNKNOWN and the device driver MAY ignore the parameter when
*                       the ISR type can be deduced by reading an available interrupt status register.
*
*                       Type codes that are defined within net_if.c include but are not limited to :
*                           NET_DEV_ISR_TYPE_RX
*                           NET_DEV_ISR_TYPE_TX_COMPLETE
*                           NET_DEV_ISR_TYPE_UNKNOWN
*
* Return(s)   : none.
*
* Caller(s)   : Specific first- or second-level BSP ISR handler.
*
* Note(s)     : (1) This function is called by name from the context of an ISR.
*
*               (2) In the case of an interrupt occurring prior to Network Protocol Stack initialization,
*                   the device driver should ensure that the interrupt source is cleared in order
*                   to prevent the potential for an infinite interrupt loop during system initialization.
*
*               (3) Many DMA devices generate only one interrupt event for several ready receive
*                   descriptors.  In order to accommodate this, it is recommended that all DMA based
*                   drivers count the number of ready receive descriptors during the receive event
*                   and signal the receive task for ONLY newly received descriptors which have not
*                   yet been signaled for during the last receive interrupt event.
*
*               (4) Many DMA devices generate only one interrupt event for several transmit
*                   complete descriptors.  In this case, the driver MUST determine which descriptors
*                   have completed transmission and post each descriptor data area address to
*                   the transmit deallocation task.  The code provided below assumes one
*                   interrupt per transmit event which may not necessarily be the case for all
*                   devices.
*
*               (5) Receive complete interrupts are disabled until all received frames have been
*                   processed.  This action reduces unnecessary interrupt overhead.  Rx interrupts
*                   are restored from NetDev_RxDescPtrCurInc() when the 'next' descriptor is
*                   found to be owned by the DMA.
*
*               (6) The CM408F reference manual, section 28.6.6, states the following :
*                   (a) For any data transfer initiated by a DMA channel, if the slave replies with
*                       an error response, that DMA stops all operations and updates the error bits
*                       and the fatal bus error bit in the Status register (ETH_DMASR register).
*                       That DMA controller can resume operation only after soft- or hard-resetting
*                       the peripheral and re-initializing the DMA.
*********************************************************************************************************
*/
/*$PAGE*/
static  void  NetDev_ISR_Handler (NET_IF            *pif,
                                  NET_DEV_ISR_TYPE   type)
{
    NET_DEV_DATA       *pdev_data;
    NET_DEV_CFG_ETHER  *pdev_cfg;
    DEV_DESC           *pdesc;
    NET_DEV            *pdev;
    NET_ERR             err;
    CPU_INT32U          int_status;
    CPU_BOOLEAN         tx_dealloc_done;

   (void)&type;                                                 /* Prevent compiler warning for arg type unused.        */


                                                                /* -- OBTAIN REFERENCE TO DEVICE CFG/DATA/REGISTERS --- */
    pdev_cfg  = (NET_DEV_CFG_ETHER *)pif->Dev_Cfg;              /* Obtain ptr to the dev cfg struct.                    */
    pdev_data = (NET_DEV_DATA      *)pif->Dev_Data;             /* Obtain ptr to dev data area.                         */
    pdev      = (NET_DEV           *)pdev_cfg->BaseAddr;        /* Overlay dev reg struct on top of dev base addr.      */



    pdev->DMAIER &= ~(ETH_DMA_IT_NIS |                          /* Disable normal & abnormal int. summary src's to ...  */
                      ETH_DMA_IT_AIS);                          /* ... prevent int. nesting.                            */

    int_status    =   pdev->DMASR;                              /* Clr active int. src's.                               */
    pdev->DMASR   =   int_status;

    if ((int_status & ETH_DMASR_FBES) > 0) {                    /* Fatal bus err (see Note #6).                         */
        NET_CTR_ERR_INC(pdev_data->ErrRxTxCtrFatalBusErr);
    }

    if ((int_status & ETH_DMASR_TS) > 0) {                      /* Tx complete?                                         */
        tx_dealloc_done = DEF_NO;
        while (tx_dealloc_done == DEF_NO) {                     /* Dealloc all buf tx'd (1 int for 2 & more buf tx'd).  */
            NET_CTR_STAT_INC(pdev_data->StatTxCtrTxISR);
            pdesc = pdev_data->TxBufDescPtrComp;
            pdev_data->TxBufDescPtrComp = pdesc->Next;
            NetOS_IF_TxDeallocTaskPost(pdesc->Addr, &err);
            NetOS_Dev_TxRdySignal(pif->Nbr);                    /* Signal Net IF that Tx resources are available.       */

                                                                /* if all tx'd buff dealloc'd.                          */
            if (pdev_data->TxBufDescPtrComp == pdev_data->TxBufDescPtrCur) {
                tx_dealloc_done = DEF_YES;
            }
        }
    }

    if ((int_status & (ETH_DMASR_RS   |                         /* Rx int.                                              */
                       ETH_DMASR_ROS  |                         /* Rx (FIFO) overflow.                                  */
                       ETH_DMASR_RBUS |                         /* Rx buffer unavailable.                               */
                       ETH_DMASR_RPSS)) > 0) {                  /* Rx process stopped.                                  */

        if ((int_status & ETH_DMASR_RS) > 0) {
            NET_CTR_STAT_INC(pdev_data->StatRxCtrRxISR);
        }

        if ((int_status & ETH_DMASR_ROS) > 0) {
            NET_CTR_ERR_INC(pdev_data->ErrRxCtrROS);
        }

        if ((int_status & ETH_DMASR_RBUS) > 0) {
            NET_CTR_ERR_INC(pdev_data->ErrRxCtrRBUS);
        }

        if ((int_status & ETH_DMASR_RPSS) > 0) {
            NET_CTR_ERR_INC(pdev_data->ErrRxCtrRPSS);
        }

        pdev_data->IntStatusLast = int_status;                  /* Record last ISR status for debug.                    */

        pdev->DMAIER &= ~(ETH_DMA_IT_R   |                      /* Dis. future Rx int's (see Note #5).                  */
                          ETH_DMA_IT_RO  |                      /* Rx (FIFO) overrun.                                   */
                          ETH_DMA_IT_RBU |                      /* Rx buffer unavailable.                               */
                          ETH_DMA_IT_RPS);                      /* Rx process stopped.                                  */

        NetOS_IF_RxTaskSignal(pif->Nbr, &err);                  /* Signal Net IF RxQ Task for each new rdy Desc.        */
        switch (err) {
            case NET_IF_ERR_NONE:
                 NET_CTR_STAT_INC(pdev_data->StatRxCtrTaskSignalISR);
                 break;

            case NET_IF_ERR_RX_Q_FULL:
            case NET_IF_ERR_RX_Q_SIGNAL_FAULT:
            default:
                 pdev->DMAIER |= ETH_DMA_IT_R;                  /* Re-enable int's and signal on next Rx.               */
                 NET_CTR_ERR_INC(pdev_data->ErrRxCtrTaskSignalFail);
                 break;
        }
    }

    pdev->DMAIER |= (ETH_DMA_IT_NIS |                           /* Re-Enable normal & abnormal int. src's.              */
                     ETH_DMA_IT_AIS);
}


/*$PAGE*/
/*
*********************************************************************************************************
*                                            NetDev_IO_Ctrl()
*
* Description : This function provides a mechanism for the Phy driver to update the MAC link
*               and duplex settings, as well as a method for the application and link state
*               timer to obtain the current link status.  Additional user specified driver
*               functionality MAY be added if necessary.
*
* Argument(s) : pif     Pointer to interface requiring service.
*
*               opt     Option code representing desired function to perform. The Network Protocol Suite
*                       specifies the option codes below. Additional option codes may be defined by the
*                       driver developer in the driver's header file.
*                           NET_IF_IO_CTRL_LINK_STATE_GET
*                           NET_IF_IO_CTRL_LINK_STATE_UPDATE
*
*                       Driver defined operation codes MUST be defined starting from 20 or higher
*                       to prevent clashing with the pre-defined operation code types. See the
*                       device driver header file for more details.
*
*               data    Pointer to optional data for either sending or receiving additional function
*                       arguments or return data.
*
*               perr    Pointer to return error code.
*                           NET_IF_ERR_INVALID_IO_CTRL_OPT      Invalid option number specified.
*                           NET_IF_ERR_NULL_FNCT                Null interface function pointer encountered.
*
*                           NET_DEV_ERR_NONE                    IO Ctrl operation completed successfully.
*                           NET_DEV_ERR_NULL_PTR                Null argument pointer passed.
*
*                           NET_PHY_ERR_NULL_PTR                Pointer argument(s) passed NULL pointer(s).
*                           NET_PHY_ERR_TIMEOUT_RESET           Phy reset          time-out.
*                           NET_PHY_ERR_TIMEOUT_AUTO_NEG        Auto-Negotiation   time-out.
*                           NET_PHY_ERR_TIMEOUT_REG_RD          Phy register read  time-out.
*                           NET_PHY_ERR_TIMEOUT_REG_WR          Phy register write time-out.
*
* Return(s)   : none.
*
* Caller(s)   : NetIF_Ether_IO_CtrlHandler() via 'pdev_api->IO_Ctrl()',
*               NetPhy_LinkStateGet()        via 'pdev_api->IO_Ctrl()'.
*
* Note(s)     : (1) All functions that require device register access MUST obtain reference
*                   to the device hardware register space PRIOR to attempting to access
*                   any registers.  Register definitions SHOULD NOT be absolute and SHOULD
*                   use the provided base address within the device configuration structure,
*                   as well as the device register definition structure in order to properly
*                   resolve register addresses during run-time.
*********************************************************************************************************
*/
/*$PAGE*/
static  void  NetDev_IO_Ctrl (NET_IF      *pif,
                              CPU_INT08U   opt,
                              void        *p_data,
                              NET_ERR     *perr)
{
    NET_DEV_CFG_ETHER   *pdev_cfg;
    NET_DEV_LINK_ETHER  *plink_state;
    NET_PHY_API_ETHER   *pphy_api;
    NET_DEV             *pdev;


                                                                /* ------- OBTAIN REFERENCE TO DEVICE REGISTERS ------- */
    pdev_cfg = (NET_DEV_CFG_ETHER *)pif->Dev_Cfg;               /* Obtain ptr to the dev cfg struct.                    */
    pdev     = (NET_DEV           *)pdev_cfg->BaseAddr;         /* Overlay dev reg struct on top of dev base addr.      */

    switch (opt) {
        case NET_IF_IO_CTRL_LINK_STATE_GET_INFO:
             plink_state      = (NET_DEV_LINK_ETHER *)p_data;
             if (plink_state == (NET_DEV_LINK_ETHER *)0) {
                *perr = NET_DEV_ERR_NULL_PTR;
                 return;
             }
#if (NET_VERSION >= 21200)
             pphy_api = (NET_PHY_API_ETHER *)pif->Ext_API;
#else
             pphy_api = (NET_PHY_API_ETHER *)pif->Phy_API;
#endif
             if (pphy_api == (void *)0) {
                *perr = NET_IF_ERR_NULL_FNCT;
                 return;
             }
             pphy_api->LinkStateGet(pif, plink_state, perr);
             if (*perr != NET_PHY_ERR_NONE) {
                return;
             }

            *perr = NET_DEV_ERR_NONE;
             break;


        case NET_IF_IO_CTRL_LINK_STATE_UPDATE:
             plink_state = (NET_DEV_LINK_ETHER *)p_data;

             switch (plink_state->Duplex) {                     /* Update duplex setting on device.                     */
                case NET_PHY_DUPLEX_FULL:
                     pdev->MACCR |= (CPU_INT32U)ETH_MACCR_DM;   /* Full Duplex Mode, this will set bit 11.              */
                     break;

                case NET_PHY_DUPLEX_HALF:
                     pdev->MACCR &= ~((CPU_INT32U)ETH_MACCR_DM);/* Half Duplex Mode, this will resset bit 11.           */
                     break;

                default:
                     break;
             }

             switch (plink_state->Spd) {                        /* Update speed setting on device.                */
                case NET_PHY_SPD_10:
                     pdev->MACCR &= ~((CPU_INT32U)ETH_MACCR_FES);
                     break;

                case NET_PHY_SPD_100:
                     pdev->MACCR |= (CPU_INT32U)ETH_MACCR_FES;
                     break;

                default:
                     break;
             }

             *perr = NET_DEV_ERR_NONE;
             break;


        default:
            *perr = NET_IF_ERR_INVALID_IO_CTRL_OPT;
             break;
    }
}


/*$PAGE*/
/*
*********************************************************************************************************
*                                            NetDev_MII_Rd()
*
* Description : Read data over the (R)MII bus from the specified Phy register.
*
* Argument(s) : pif         Pointer to the interface requiring service.
*
*               phy_addr   (R)MII bus address of the Phy requiring service.
*
*               reg_addr    Phy register number to write to.
*
*               p_data      Pointer to variable to store returned register data.
*
*               perr    Pointer to return error code.
*                           NET_DEV_ERR_NULL_PTR            Pointer argument(s) passed NULL pointer(s).
*                           NET_PHY_ERR_NONE                MII write completed successfully.
*                           NET_PHY_ERR_TIMEOUT_REG_RD      Register read time-out.
*
* Return(s)   : none.
*
* Caller(s)   : Various Phy functions.
*
* Note(s)     : (1) All functions that require device register access MUST obtain reference
*                   to the device hardware register space PRIOR to attempting to access
*                   any registers.  Register definitions SHOULD NOT be absolute and SHOULD
*                   use the provided base address within the device configuration structure,
*                   as well as the device register definition structure in order to properly
*                   resolve register addresses during run-time.
*********************************************************************************************************
*/

static  void  NetDev_MII_Rd (NET_IF      *pif,
                             CPU_INT08U   phy_addr,
                             CPU_INT08U   reg_addr,
                             CPU_INT16U  *p_data,
                             NET_ERR     *perr)
{
    NET_DEV_CFG_ETHER  *pdev_cfg;
    NET_DEV            *pdev;
    CPU_INT32U          tmpreg;
    CPU_INT32U          timeout;


                                                                 /* ------- OBTAIN REFERENCE TO DEVICE REGISTERS ------ */
    pdev_cfg = (NET_DEV_CFG_ETHER *)pif->Dev_Cfg;                /* Obtain ptr to the dev cfg struct.                   */
    pdev     = (NET_DEV           *)pdev_cfg->BaseAddr;          /* Overlay dev reg struct on top of dev base addr.     */


    timeout  =  0;

    tmpreg   =  pdev->MACMIIAR;                                  /* Get the ETHERNET MACMIIAR value                     */
    tmpreg  &= ~ETH_MACMIIAR_CR_MASK;                            /* Keep only the CSR Clk Range CR[2:0] bits val.       */

                                                                 /* Prepare the MII address register val.               */
    tmpreg  |= (((CPU_INT32U)phy_addr << 11) & ETH_MACMIIAR_PA); /* Set the PHY device addr.                            */
    tmpreg  |= (((CPU_INT32U)reg_addr <<  6) & ETH_MACMIIAR_MR); /* Set the PHY reg addr.                               */
    tmpreg  &= ~ETH_MACMIIAR_MW;                                 /* Set the read mode.                                  */
    tmpreg  |=  ETH_MACMIIAR_MB;                                 /* Set the MII Busy bit.                               */

    pdev->MACMIIAR = tmpreg;                                     /* Write the result val into the MII addr reg.         */

    do {                                                         /* Check for the Busy flag.                            */
        timeout++;
        tmpreg = pdev->MACMIIAR;
    } while ((tmpreg & ETH_MACMIIAR_MB) && (timeout < (CPU_INT32U)PHY_RD_TO));

    if (timeout == PHY_RD_TO) {                                  /* Return ERROR in case of timeout.                    */
       *perr = NET_PHY_ERR_TIMEOUT_REG_RD;
        return;
    }

    *p_data = (CPU_INT16U)(pdev->MACMIIDR);                      /* Return data reg val.                                */

    *perr = NET_PHY_ERR_NONE;
}


/*$PAGE*/
/*
*********************************************************************************************************
*                                            NetDev_MII_Wr()
*
* Description : Write data over the (R)MII bus to the specified Phy register.
*
* Argument(s) : pif         Pointer to the interface requiring service.
*
*               phy_addr   (R)MII bus address of the Phy requiring service.
*
*               reg_addr    Phy register number to write to.
*
*               data        Data to write to the specified Phy register.
*
*               perr    Pointer to return error code.
*                           NET_PHY_ERR_NONE                MII write completed successfully.
*                           NET_PHY_ERR_TIMEOUT_REG_WR      Register write time-out.
*
* Return(s)   : none.
*
* Caller(s)   : Various Phy functions.
*
* Note(s)     : (1) All functions that require device register access MUST obtain reference
*                   to the device hardware register space PRIOR to attempting to access
*                   any registers.  Register definitions SHOULD NOT be absolute and SHOULD
*                   use the provided base address within the device configuration structure,
*                   as well as the device register definition structure in order to properly
*                   resolve register addresses during run-time.
*********************************************************************************************************
*/

static  void  NetDev_MII_Wr (NET_IF      *pif,
                             CPU_INT08U   phy_addr,
                             CPU_INT08U   reg_addr,
                             CPU_INT16U   data,
                             NET_ERR     *perr)
{
    NET_DEV_CFG_ETHER  *pdev_cfg;
    NET_DEV            *pdev;
    CPU_INT32U          timeout;
    CPU_INT32U          tmpreg;

                                                                  /* ------ OBTAIN REFERENCE TO DEVICE REGISTERS ------ */
    pdev_cfg = (NET_DEV_CFG_ETHER *)pif->Dev_Cfg;                 /* Obtain ptr to the dev cfg struct.                  */
    pdev     = (NET_DEV           *)pdev_cfg->BaseAddr;           /* Overlay dev reg struct on top of dev base addr.    */


    tmpreg   = pdev->MACMIIAR;                                    /* Get the ETHERNET MACMIIAR val.                     */

    tmpreg  &= ~ETH_MACMIIAR_CR_MASK;                             /* Keep only the CSR Clk Range CR[2:0] bits val.      */

                                                                  /* Prepare the MII reg addr val.                      */
    tmpreg  |= (((CPU_INT32U)phy_addr << 11) & ETH_MACMIIAR_PA);  /* Set the PHY device addr.                           */
    tmpreg  |= (((CPU_INT32U)reg_addr <<  6) & ETH_MACMIIAR_MR);  /* Set the PHY reg addr.                              */
    tmpreg  |= ETH_MACMIIAR_MW;                                   /* Set the write mode.                                */
    tmpreg  |= ETH_MACMIIAR_MB;                                   /* Set the MII Busy bit.                              */

    pdev->MACMIIDR = data;                                        /* Give the val to the MII data reg.                  */
    pdev->MACMIIAR = tmpreg;                                      /* Write the result value into the MII addr reg.      */

    timeout = 0u;                                                 /* Init timeout.                                      */
	
    do {                                                          /* Check for the Busy flag.                           */
        timeout++;
        tmpreg = pdev->MACMIIAR;
    } while ((tmpreg & ETH_MACMIIAR_MB) && (timeout < (CPU_INT32U)PHY_WR_TO));

    if (timeout == PHY_WR_TO) {                                   /* Return ERROR in case of timeout.                   */
       *perr = NET_PHY_ERR_TIMEOUT_REG_WR;
        return;
    }

    *perr = NET_PHY_ERR_NONE;
}

