/*
 *****************************************************************************
 * @file:    adi_hae_data.c
 * @brief:   Data declaration for HAE Device Driver
 * @version: $Revision: 22005 $
 * @date:    $Date: 2013-06-25 14:40:02 -0400 (Tue, 25 Jun 2013) $
 *****************************************************************************

Copyright(c) 2013 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_hae_data.c
*
* @brief     This file contains the data definition required by HAE driver.
*
*/

/*
 * Function pointer tables for interrupt mode and DMA mode abstraction.
 *
 * By default interrupt mode table will be used and if HAE is operating
 * only in interrupt mode the DMA mode table and its associated functions
 * will be eliminated by linker.
 */

/* Function pointer table for interrupt mode */
static ADI_HAE_FN_TABLE   gIntFnTable =
{
    &hae_Int_Close,
    &hae_Int_SubmitBuffer,
    &hae_Int_Enable
};

/* Function pointer table for DMA mode */
static ADI_HAE_FN_TABLE   gDMAFnTable =
{
    &hae_DMA_Close,
    &hae_DMA_SubmitBuffer,
    &hae_DMA_Enable
};

/* the device information array */
static ADI_HAE_DEVICE gHaeDevices[] = 
{
    {   
        ADI_HAE_STATE_NOT_OPENED,                           /* HAE device state */
        NULL,                                               /* HAE Tx device data */
        NULL,                                               /* HAE Rx0 device data */
        NULL,                                               /* HAE Rx1 device data */
        false,                                              /* DMA in use flag */
//        ADI_DMA_CHNL_HAE0_TX,                               /* Tx DMA Channel ID */
//        ADI_DMA_CHNL_HAE0_RX_CH0,                           /* Rx0 DMA Channel ID */
//        ADI_DMA_CHNL_HAE0_RX_CH1,                           /* Rx1 DMA Channel ID */
        (uint32_t)ADI_INT_HAE0_STAT,                        /* Status interrupt ID */
        &gIntFnTable,                                       /* Function pointer table */
        NULL,                                               /* Pointer to the device callback */
        NULL,                                               /* Callback parameter */
        {                                                   /* HAE registers */
            pREG_HAE0_RUN,
            pREG_HAE0_COEF_RAM,
            pREG_HAE0_CFG0,
            pREG_HAE0_CFG1,
            pREG_HAE0_CFG2,
            pREG_HAE0_CFG3,
            pREG_HAE0_STAT,
            pREG_HAE0_ISAMPLE,
            pREG_HAE0_VSAMPLE,
            pREG_HAE0_IWAVEFORM,
            pREG_HAE0_VWAVEFORM,
            pREG_HAE0_RESULTS_RAM,
            pREG_HAE0_DATA_RAM,
            pREG_HAE0_CFG4,
            pREG_HAE0_DIDT_GAIN,
            pREG_HAE0_DIDT_COEF,
            pREG_HAE0_VLEVEL,
            pREG_HAE0_H00_INDX,
            pREG_HAE0_H01_INDX,
            pREG_HAE0_H02_INDX,
            pREG_HAE0_H03_INDX,
            pREG_HAE0_H04_INDX,
            pREG_HAE0_H05_INDX,
            pREG_HAE0_H06_INDX,
            pREG_HAE0_H07_INDX,
            pREG_HAE0_H08_INDX,
            pREG_HAE0_H09_INDX,
            pREG_HAE0_H10_INDX,
            pREG_HAE0_H11_INDX
        }
     }
};

