/*********************************************************************************

Copyright(c) 2013 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

#ifndef _ADI_HAE_DEF_H_
#define _ADI_HAE_DEF_H_

/* HAE Controller registers */
typedef struct
{
	volatile uint32_t *pRUN;                    /* HAE0 HAE Run Register */
	volatile uint32_t *pCOEF_RAM;               /* HAE0 Coefficient RAM Register */
	volatile uint32_t *pCFG0;                   /* HAE0 Configuration Register 0 */
	volatile uint32_t *pCFG1;                   /* HAE0 Configuration Register 1 */
	volatile uint32_t *pCFG2;                   /* HAE0 Configuration Register 2 */
	volatile uint32_t *pCFG3;                   /* HAE0 Configuration Register 3 */
	volatile uint32_t *pSTAT;                   /* HAE0 Status Register */
	volatile uint32_t *pISAMPLE;                /* HAE0 MCU I Sample Register */
	volatile uint32_t *pVSAMPLE;                /* HAE0 MCU V Sample Register */
	volatile uint32_t *pIWAVEFORM;              /* HAE0 I Waveform Register */
	volatile uint32_t *pVWAVEFORM;              /* HAE0 V Waveform Register */
	volatile uint32_t *pRESULTS_RAM;            /* HAE0 Results RAM Register */
	volatile uint32_t *pDATA_RAM;               /* HAE0 Data/Configuration RAM Register */
	volatile uint32_t *pCFG4;                   /* HAE0 Configuration Register 4 */
	volatile uint32_t *pDIDT_GAIN;              /* HAE0 DIDT Gain Register */
	volatile uint32_t *pDIDT_COEF;              /* HAE0 DIDT Coefficient Register */
	volatile uint32_t *pVLEVEL;                 /* HAE0 Voltage Level Register */
	volatile uint32_t *pH00_INDX;               /* HAE0 Harmonic Index n Register */
	volatile uint32_t *pH01_INDX;               /* HAE0 Harmonic Index n Register */
	volatile uint32_t *pH02_INDX;               /* HAE0 Harmonic Index n Register */
	volatile uint32_t *pH03_INDX;               /* HAE0 Harmonic Index n Register */
	volatile uint32_t *pH04_INDX;               /* HAE0 Harmonic Index n Register */
	volatile uint32_t *pH05_INDX;               /* HAE0 Harmonic Index n Register */
	volatile uint32_t *pH06_INDX;               /* HAE0 Harmonic Index n Register */
	volatile uint32_t *pH07_INDX;               /* HAE0 Harmonic Index n Register */
	volatile uint32_t *pH08_INDX;               /* HAE0 Harmonic Index n Register */
	volatile uint32_t *pH09_INDX;               /* HAE0 Harmonic Index n Register */
	volatile uint32_t *pH10_INDX;               /* HAE0 Harmonic Index n Register */
	volatile uint32_t *pH11_INDX;               /* HAE0 Harmonic Index n Register */
} ADI_HAE_REG;

/* enumeration of device states */
typedef enum
{
    ADI_HAE_STATE_NOT_OPENED,                   /* The device is not yet opened */
    ADI_HAE_STATE_OPENED,                       /* The device is opened */
    ADI_HAE_STATE_ENABLED                       /* The data flow is enabled. */
} ADI_HAE_STATE;

/* Enumeration of different buffer modes */
typedef enum
{
    ADI_HAE_BUFFER_MODE_NONBLOCKING,            /* Non blocking mode (Submit/Get) */
    ADI_HAE_BUFFER_MODE_BLOCKING,               /* Blocking mode (Read/Write) */
    ADI_HAE_BUFFER_MODE_CALLBACK                /* Callback mode (always non blocking) */
}ADI_HAE_BUFFER_MODE;

/* Enumeration of different data flow states */
typedef enum
{
    ADI_HAE_DATAFLOW_DISABLED,
    ADI_HAE_DATAFLOW_ENABLED,
    ADI_HAE_DATAFLOW_PAUSED
} ADI_HAE_DATAFLOW_STATE;

/* Structure which holds the buffer related information */
typedef struct _ADI_HAE_BUF_INFO_
{
    uint8_t                        *pBuffer;            /* Pointer to the buffer supplied by application */
    uint32_t                        nBufSize;           /* Size of the buffer */
    uint32_t                        index;              /* The current buffer index */
    bool                            bInUse;             /* Flag which indicates whether the buffer is in use by driver */
} ADI_HAE_BUF_INFO;

/* Structure which holds the device data */
typedef struct
{
    /* Buffer management related information */
    ADI_HAE_BUF_INFO                BufferInfo0;
    ADI_HAE_BUF_INFO                BufferInfo1;

    /* Pointer to the active buffer information */
    ADI_HAE_BUF_INFO               *pActiveBuf;

    /* DMA Channel Handle */
    ADI_DMA_CHANNEL_HANDLE          hDMAChnl;

    /* Pointer to the memory required for opening DMA channel */
    void *                          pDMAChnlMemory;

    ADI_DMA_CHANNEL_ID              eDMAChnlID;

    /* Number of bytes transferred in a DMA transfer */
    int32_t                         nBytesPerTransfer;

    /* Data flow state */
    ADI_HAE_DATAFLOW_STATE          eDataFlowState;

    /* Buffer mode */
    ADI_HAE_BUFFER_MODE             eBufferMode;

    /* Event to indicate that the buffer is processed */
    ADI_HAE_EVENT                   eBufProcessedEvent;

    ADI_OSAL_SEM_HANDLE             hSemaphore;                            /* Semaphore handle */
    uint8_t                         SemMemory[ADI_OSAL_MAX_SEM_SIZE_CHAR]; /* Semaphore memory */
} ADI_HAE_CHANNEL;

/* Forward declaration for pointer to HAE device */
typedef struct _ADI_HAE_DEVICE_*   ADI_HAE_PDEVICE;

/* Table to hold the function pointers related interrupt mode and DMA mode */
typedef struct _ADI_HAE_FN_TABLE_
{
    ADI_HAE_RESULT (* pfClose)(ADI_HAE_PDEVICE  pDevice);
    ADI_HAE_RESULT (* pfSubmitBuffer)(ADI_HAE_PDEVICE pDevice, ADI_HAE_CHANNEL *pChannel, void *pBuffer, uint32_t nBufSize, bool bRead);
    ADI_HAE_RESULT (* pfEnable)(ADI_HAE_PDEVICE pDevice, ADI_HAE_CHANNEL *pChannel, bool bEnable);

}ADI_HAE_FN_TABLE;

/* Structure which holds mostly static information */
typedef struct _ADI_HAE_DEVICE_
{   
    ADI_HAE_STATE                  eHaeState;              /* HAE device state */
    ADI_HAE_CHANNEL               *pTxChannel;             /* Tx (result) Channel related data */
    ADI_HAE_CHANNEL               *pRx0Channel;            /* Rx (current) Channel related data */
    ADI_HAE_CHANNEL               *pRx1Channel;            /* Rx (voltage) Channel related data */
    bool                           bDMAMode;               /* DMA in use flag */
    const uint32_t                 nStatusIntID;           /* Status interrupt ID */
    ADI_HAE_FN_TABLE              *pFnTable;               /* Pointer to function pointer table */
    ADI_CALLBACK                   pfCallback;             /* Pointer to the device callback */
    void *                         pCBParam;               /* Callback parameter */
    ADI_HAE_REG                    HaeRegs;                /* HAE registers */
} ADI_HAE_DEVICE;

/* Number of HAE Devices in the processor */
#define ADI_HAE_NUM_DEVICES            (sizeof(gHaeDevices)/sizeof(ADI_HAE_DEVICE))


/*
 * Does the required buffer management and calls the abstracted
 * (DMA and Interrupt mode) functions
 */
static ADI_HAE_RESULT SubmitBuffer(
    ADI_HAE_DEVICE           *pDevice,
    ADI_HAE_CHANNEL          *pChannel,
    void                     *pBuffer,
    uint32_t                  nBufferSize,
    bool                      bRead
);

/*
 * Interrupt mode functions
 */

/* Takes care of cleanup required in interrupt mode */
static ADI_HAE_RESULT hae_Int_Close(
   ADI_HAE_DEVICE       *pDevice
);

/* Takes care of tasks when data flow is enabled in interrupt mode */
static ADI_HAE_RESULT hae_Int_Enable  (
   ADI_HAE_DEVICE       *pDevice,
   ADI_HAE_CHANNEL      *pChannel,
   bool                  bEnable
);

/* Submits a buffer in Interrupt mode */
static ADI_HAE_RESULT hae_Int_SubmitBuffer(
   ADI_HAE_DEVICE      *pDevice,                                             
   ADI_HAE_CHANNEL     *pChannel,
   void                 *pBuffer,
   uint32_t              nBufferSize,
   bool                  bRead
);

/*
 * DMA mode functions
 */

/* Takes care of cleanup required when closing HAE which was in DMA mode */
static ADI_HAE_RESULT hae_DMA_Close(
    ADI_HAE_DEVICE      *pDevice
);


/* Submits a buffer in DMA mode */
static ADI_HAE_RESULT hae_DMA_SubmitBuffer(
    ADI_HAE_DEVICE      *pDevice,                                             
    ADI_HAE_CHANNEL     *pChannel,
    void                 *pBuffer,
    uint32_t              nBufferSize,
    bool                  bRead
);

/* Takes care of tasks when data flow is enabled in interrupt mode */
static ADI_HAE_RESULT hae_DMA_Enable(
    ADI_HAE_DEVICE      *pDevice,
    ADI_HAE_CHANNEL     *pChannel,
    bool                  bEnable
);

/*
 * DMA interrupt handler
 */
static void HaeTxDMAHandler(void *pCBParam, uint32_t Event, void *pArg);
static void HaeRx0DMAHandler(void *pCBParam, uint32_t Event, void *pArg);
static void HaeRx1DMAHandler(void *pCBParam, uint32_t Event, void *pArg);

/* Status interrupt handler */
static void StatusIntHandler(uint32_t iid, void *pCBParam);

static ADI_HAE_RESULT EnableDataFlow(ADI_HAE_DEVICE *pDevice, ADI_HAE_CHANNEL *pChannel, bool bEnable);

#ifdef ADI_DEBUG
/* Validate the device */
static ADI_HAE_RESULT ValidateDevice(ADI_HAE_DEVICE *pDevice);
static bool IsEnabled(ADI_HAE_DEVICE   *pDevice);
#endif /* ADI_DEBUG */

#endif /* _ADI_HAE_DEF_H_ */
