/*
 *****************************************************************************
 * @file:    adi_counter_data_bf6xx.c
 * @brief:   Data declaration for COUNTER Device Driver for BF6XX family processor
 * @version: $Revision: 21988 $
 * @date:    $Date: 2013-06-21 17:01:32 -0400 (Fri, 21 Jun 2013) $
 *****************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_counter_data_b65xx.c
* @brief     This file contains the data decleration required by device driver for
*            the operating  the  COUNTER device
*/

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rules_all:"Suppress all rules for other header files")
#endif

#include <adi_types.h>
#include <stddef.h>
#include <sys/platform.h> 
#include <sys/exception.h>
#include <services/int/adi_int.h>
#include "adi_counter_def.h"

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"Allows portions of code to be commented out")
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_6:"Allow identifier names to be reused in different name spaces")
#pragma diag(suppress:misra_rule_5_7:"Allow identifier names to be reused")
#pragma diag(suppress:misra_rule_14_7:"Allow multiple exits from a function for readability and optimized code")
#pragma diag(suppress:misra_rule_17_4:"Allow pointer arithmetic for MMR address generation")
#pragma diag(suppress:misra_rule_19_7:"Allow macros to be used for readability")
#endif /* _MISRA_RULES */

#if defined(__ADSPBF609_FAMILY__)
/* Stores the information about the specific device instance.*/

#if !defined(ADI_CODE_IN_ROM)
static  ADI_COUNTER_DEVICE_INFO counter_device_info []=
{
     {ADI_COUNTER_STATE_UNINITIALIZED,(uint32_t)INTR_CNT0_STAT,(volatile COUNTER_REGISTERS *)pREG_CNT0_CFG,NULL},
};
#endif

#elif defined(ADI_ADSP_CM40Z)
static  ADI_COUNTER_DEVICE_INFO counter_device_info []=
{
     {ADI_COUNTER_STATE_UNINITIALIZED,(uint32_t)INTR_CNT0_STAT,(volatile COUNTER_REGISTERS *)pREG_CNT0_CFG,NULL},
     {ADI_COUNTER_STATE_UNINITIALIZED,(uint32_t)INTR_CNT1_STAT,(volatile COUNTER_REGISTERS *)pREG_CNT1_CFG,NULL}
};
#elif defined(__ADSPBF548_FAMILY__) || defined(__ADSPBF527_FAMILY__) ||  defined(__ADSPBF526_FAMILY__) ||  defined(__ADSPBF518_FAMILY__)
/* Stores the information about the specific device instance.*/
static  ADI_COUNTER_DEVICE_INFO counter_device_info []=
{
     {ADI_COUNTER_STATE_UNINITIALIZED,(uint32_t)ADI_SID_GP_COUNTER,(volatile COUNTER_REGISTERS *)pCNT_CONFIG,NULL},
};
#elif  defined(__ADSPBF506F_FAMILY__)
/* Stores the information about the specific device instance.*/
static  ADI_COUNTER_DEVICE_INFO counter_device_info []=
{
     {ADI_COUNTER_STATE_UNINITIALIZED,(uint32_t)ADI_SID_GP_COUNTER0,(volatile COUNTER_REGISTERS *)pCNT0_CONFIG,NULL},
     {ADI_COUNTER_STATE_UNINITIALIZED,(uint32_t)ADI_SID_GP_COUNTER1,(volatile COUNTER_REGISTERS *)pCNT1_CONFIG,NULL}
};

#elif defined(__ADSPBF707_FAMILY__)
/* Stores the information about the specific device instance.*/
#if !defined(ADI_CODE_IN_ROM)
static  ADI_COUNTER_DEVICE_INFO counter_device_info []=
{
     {ADI_COUNTER_STATE_UNINITIALIZED,(uint32_t)INTR_CNT0_STAT,(volatile COUNTER_REGISTERS *)pREG_CNT0_CFG,NULL}
};
#endif


#else

#error "The counter device is not supported in  this processor type"

#endif

/* Number of counter Instances */
#if !defined(ADI_CODE_IN_ROM)
#define ADI_NUM_COUNTER_DEVICE_INSTANCES       (sizeof(counter_device_info)/sizeof(ADI_COUNTER_DEVICE_INFO))
#endif

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif
