/*!
 *****************************************************************************
 * @file:    adi_counter_def_bf6xx.h
 * @brief:   General purpose counter (rotary Device) Driver definition for blackfin processor
 * @version: $
 * @date:    $
 *****************************************************************************
Copyright(c) 2011-2013 Analog Devices, Inc. All Rights Reserved.
This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.
*********************************************************************************
     File         :   adi_counter_def.h
     Description  :   Static function and structure definition  needed by the device driver
                      for managing  the rotary  device.
================================================================================ */
#ifndef _DEF_COUNTER_DEF_XXX_H_
#define _DEF_COUNTER_DEF_XXX_H_

#if  defined(__ADSPBF548_FAMILY__) || defined(__ADSPBF527_FAMILY__) ||  defined(__ADSPBF526_FAMILY__) ||  defined(__ADSPBF518_FAMILY__) || defined(__ADSPBF506F_FAMILY__)
#include <services/int/adi_sid.h>
#include <services/int/adi_int.h>
#elif defined(__ADSPBF609_FAMILY__)
#include <services/int/adi_sec.h>
#endif


#define  MAX_PRESCALE_VALUE 0X11U
/* counter device registers */
#pragma pack()
typedef struct _COUNTER_REGISTERS
{
    /* config register for Counter */
    volatile uint16_t  COUNTER_CONFIG;
    /* Padding variable */
    uint16_t nPadding1;
    /* IMASK  register  for Counter */
    volatile uint16_t  COUNTER_IMASK;
    /* Padding variable */
    uint16_t nPadding2;
    /* Status  register  for Counter */
    volatile uint16_t  COUNTER_STATUS;
    /* Padding variable */
    uint16_t nPadding3;
    /*Command Register for Counter */
    volatile uint16_t  COUNTER_COMMAND;
    /* Padding variable */
    uint16_t nPadding4;
    /* Debounce for Counter */
    volatile uint16_t  COUNTER_DEBOUNCE;
    /* Padding variable */
    uint16_t nPadding5;
    /*Counter for Counter */ 
    volatile uint32_t  COUNTER_COUNTER;
    /* Maximum register */
    volatile uint32_t  COUNTER_MAX;
    /* Min register */
    volatile uint32_t  COUNTER_MIN;
} COUNTER_REGISTERS;

/* Enumeration of different COUNTER states */
typedef enum
{
    ADI_COUNTER_STATE_UNINITIALIZED = 0,  /* COUNTER is not yet initialized */

    ADI_COUNTER_STATE_INITIALIZED,        /* COUNTER is initialized */

    ADI_COUNTER_STATE_ACTIVE               /* COUNTER is active */

} ADI_COUNTER_STATE;


typedef struct _ADI_COUNTER_DEVICE_INFO
{
	ADI_COUNTER_STATE eState;                   /* To indicate the state of the device */

    CONST uint32_t nStatusSID;                 /* Status Interrupt ID */

    volatile COUNTER_REGISTERS  *pCounterRegs; /*Base address of the  Counter registers */

    ADI_COUNTER_HANDLE  hDevice;                /* Counter handle */

}ADI_COUNTER_DEVICE_INFO;

/* COUNTER driver instance data */
typedef struct _ADI_COUNTER_DEVICE
{
    /* Pointer to COUNTER Memory Mapped Registers */
    volatile COUNTER_REGISTERS  *pCounterRegs;

    /*  Function pointer for callback function. */
    ADI_CALLBACK             pfCallback;

    /*  Parameter to callback function. */
    void                    *pCBParam;

    ADI_COUNTER_DEVICE_INFO *pDeviceInfo;

} ADI_COUNTER_DEVICE;

static void StatusHandler(uint32_t IID, void *pCBParam);

#ifdef ADI_DEBUG
static ADI_COUNTER_RESULT  ValidateHandle(ADI_COUNTER_DEVICE *pInDevice);
#endif /* ADI_DEBUG */

#endif /* end of ifndef _DEF_COUNTER_DEF_XXX_H_ */


