/*********************************************************************************

Copyright(c) 2012 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

#ifndef _ADI_SINC_DEF_H_
#define _ADI_SINC_DEF_H_

/* ADC Controller registers */
typedef struct
{
	uint32_t CTL;                   /* SINC0 SINC Enables & Interrupt Masks */
	uint32_t STAT;                  /* SINC0 Status bits */
	uint32_t CLK;                   /* SINC0 Modulator Clock Control */
    uint32_t REVID;                 /* Revision ID */
	uint32_t RATE0;                 /* SINC0 Rate Control for Group 0 */
	uint32_t RATE1;                 /* SINC0 Rate Control for Group 1 */
	uint32_t LEVEL0;                /* SINC0 Level Control for Group 0 */
	uint32_t LEVEL1;                /* SINC0 Level Control for Group 1 */
	uint32_t LIMIT0;                /* SINC0 Amplitude Limits for Secondary SINC of Pair 0 */
	uint32_t LIMIT1;                /* SINC0 Amplitude Limits for Secondary SINC of Pair 1 */
	uint32_t LIMIT2;                /* SINC0 Amplitude Limits for Secondary SINC of Pair 2 */
	uint32_t LIMIT3;                /* SINC0 Amplitude Limits for Secondary SINC of Pair 3 */
	uint32_t BIAS0;                 /* SINC0 Primary SINC Output Bias for Group 0 */
	uint32_t BIAS1;                 /* SINC0 Primary SINC Output Bias for Group 1 */
	uint32_t PPTR0;                 /* SINC0 Primary SINC Circular Buffer Pointer for Group 0 */
	uint32_t PPTR1;                 /* SINC0 Primary SINC Circular Buffer Pointer for Group 1 */
	uint32_t PHEAD0;                /* SINC0 Primary SINC Circular Buffer Head for Group 0 */
	uint32_t PHEAD1;                /* SINC0 Primary SINC Circular Buffer Head for Group 1 */
	uint32_t PTAIL0;                /* SINC0 Primary SINC Circular Buffer Tail for Group 0 */
	uint32_t PTAIL1;                /* SINC0 Primary SINC Circular Buffer Tail for Group 1 */
	uint32_t HIS_STAT;              /* SINC0 History Buffer Status */
	uint32_t P0SEC_HIST0;           /* SINC0 Pair0 Secondary History Register n */
	uint32_t P0SEC_HIST1;           /* SINC0 Pair0 Secondary History Register n */
	uint32_t P0SEC_HIST2;           /* SINC0 Pair0 Secondary History Register n */
	uint32_t P0SEC_HIST3;           /* SINC0 Pair0 Secondary History Register n */
	uint32_t P1SEC_HIST0;           /* SINC0 Pair1 Secondary History Register n */
	uint32_t P1SEC_HIST1;           /* SINC0 Pair1 Secondary History Register n */
	uint32_t P1SEC_HIST2;           /* SINC0 Pair1 Secondary History Register n */
	uint32_t P1SEC_HIST3;           /* SINC0 Pair1 Secondary History Register n */
	uint32_t P2SEC_HIST0;           /* SINC0 Pair2 Secondary History Register n */
	uint32_t P2SEC_HIST1;           /* SINC0 Pair2 Secondary History Register n */
	uint32_t P2SEC_HIST2;           /* SINC0 Pair2 Secondary History Register n */
	uint32_t P2SEC_HIST3;           /* SINC0 Pair2 Secondary History Register n */
	uint32_t P3SEC_HIST0;           /* SINC0 Pair3 Secondary History Register n */
	uint32_t P3SEC_HIST1;           /* SINC0 Pair3 Secondary History Register n */
	uint32_t P3SEC_HIST2;           /* SINC0 Pair3 Secondary History Register n */
	uint32_t P3SEC_HIST3;           /* SINC0 Pair3 Secondary History Register n */
} ADI_SINC_REG;

/* enumeration of device states */
typedef enum
{
    ADI_SINC_STATE_NOT_OPENED,                                          /* The device is not yet opened */
    ADI_SINC_STATE_OPENED,                                              /* The device is opened */
    ADI_SINC_STATE_ENABLED                                              /* The data flow is enabled. */
} ADI_SINC_STATE;

/* Structure which holds the buffer related information */
typedef struct _ADI_SINC_BUF_INFO_
{
    uint8_t                        *pBuffer;            /* Pointer to the buffer supplied by application */
    uint32_t                        nBufSize;           /* Size of the buffer */
} ADI_SINC_BUF_INFO;

/* Structure which holds the device data */
typedef struct _ADI_SINC_GROUP_DATA_
{
    /* Buffer management related information */
    ADI_SINC_BUF_INFO               BufferInfo0;
    ADI_SINC_BUF_INFO               BufferInfo1;

    /* Pointer to the active buffer information */
    ADI_SINC_BUF_INFO              *pActiveBuf;

    /* Pointer to the ready buffer information */
    ADI_SINC_BUF_INFO              *pReadyBuf;

    ADI_OSAL_SEM_HANDLE             hSemaphore;                            /* Semaphore handle for group */
    uint8_t                         SemMemory[ADI_OSAL_MAX_SEM_SIZE_CHAR]; /* Semaphore memory for group */

} ADI_SINC_GROUP_DATA;

/* Structure which holds the device data */
typedef struct
{
    /* Group related information */
    ADI_SINC_GROUP_DATA             Group0;
    ADI_SINC_GROUP_DATA             Group1;

    ADI_CALLBACK                    pfStatusCallback;                  /* Pointer to the device status callback */
    void *                          pStatusCBParam;                    /* Callback status parameter */
    
    ADI_CALLBACK                    pfDataCallback;                    /* Pointer to the device data callback */
    void *                          pDataCBParam;                      /* Callback data parameter */
} ADI_SINC_DEV_DATA;

/* forward declaration of SINC device */
typedef struct _ADI_SINC_DEVICE_*   ADI_SINC_DEVICEP;

/* Structure which holds mostly static information */
typedef struct _ADI_SINC_DEVICE_
{   
    ADI_SINC_STATE                  eSincState;                             /* SINC device state */
    volatile ADI_SINC_REG *const    pSincRegs;                              /* Pointer to SINC registers */
    const uint32_t                  nStatusIntID;                           /* Status interrupt ID */
    ADI_SINC_DEV_DATA *             pSincData;                              /* SINC device data */
} ADI_SINC_DEVICE;

/* Number of SINC Devices in the processor */
#define ADI_SINC_NUM_DEVICES            (sizeof(gSincDevices)/sizeof(ADI_SINC_DEVICE))

/*
 *  Internal functions
 */

/* Status interrupt handler */
static void StatusIntHandler(uint32_t iid, void *pCBParam);

/* Data interrupt handler */
static void Data0IntHandler(uint32_t iid, void *pCBParam);
static void Data1IntHandler(uint32_t iid, void *pCBParam);

static ADI_SINC_RESULT CalcOutputBias(ADI_SINC_HANDLE const hDevice, ADI_SINC_GROUP const eGroup);

#ifdef ADI_DEBUG

/* Validate the device */
static ADI_SINC_RESULT ValidateDevice(ADI_SINC_DEVICE     *pDevice);

#endif /* ADI_DEBUG */

#endif /* _ADI_SINC_DEF_H_ */
