/* $Id: adi_spi_data_bf6xx.c 21989 2013-06-21 21:21:03Z skafka $ */
/*********************************************************************************

 Copyright(c) 2010 Analog Devices, Inc. All Rights Reserved.

 This software is proprietary and confidential.  By using this software you agree
 to the terms of the associated Analog Devices License Agreement.

 *********************************************************************************/
/*!
 * @file      adi_spi_data_bf6x.c
 *
 * @brief     SPI driver global API implementation
 *
 * @details
 *            This is a source file which contains the non-exposed definitions of
 *            the adi_spi_bf6x.c driver.
 */

/** @addtogroup SPI_Driver_BF6xx SPI Device Driver
 *  @{
 */

#ifndef __ADI_SPI_DATA_BF6XX_C__
#define __ADI_SPI_DATA_BF6XX_C__

#include "adi_spi_def_bf6xx.h"

#if !defined(ADI_CODE_IN_ROM)

/* SPI instances */
#if defined(__ADSPBF609_FAMILY__)
static ADI_SPI aSpiDevice[] =
{
    /* SPI 0 */
    {   (uint32_t)INTR_SPI0_STAT,                       /* SPI status interrupt ID              			*/
    	(uint32_t)INTR_SPI0_TXDMA,                      /* SPI peripheral DMA Tx interrupt ID         		*/
    	(uint32_t)INTR_SPI0_RXDMA,                     	/* SPI peripheral DMA Rx interrupt ID         		*/
        ADI_DMA_CHNL_SPI0_TX,                           /* SPI peripheral DMA Tx channel ID         		*/
        ADI_DMA_CHNL_SPI0_RX,                           /* SPI peripheral DMA Rx channel ID         		*/
        (ADI_SPI_REGS *)pREG_SPI0_CTL,                  /* Base address of registers            			*/
        NULL                                            /* null pointer for parameters storage  			*/
    },
    /* SPI 1 */
    {   (uint32_t)INTR_SPI1_STAT,                       /* SPI status interrupt ID              			*/
    	(uint32_t)INTR_SPI1_TXDMA,                      /* SPI peripheral DMA Tx interrupt ID         		*/
    	(uint32_t)INTR_SPI1_RXDMA,                     	/* SPI peripheral DMA Rx interrupt ID         		*/
        ADI_DMA_CHNL_SPI1_TX,                           /* SPI peripheral DMA Tx channel ID         		*/
        ADI_DMA_CHNL_SPI1_RX,                           /* SPI peripheral DMA Rx channel ID         		*/
        (ADI_SPI_REGS *)pREG_SPI1_CTL,                  /* Base address of registers            			*/
        NULL                                            /* null pointer for parameters storage  			*/
    }
};
#elif defined(ADI_ADSP_CM40Z)
static ADI_SPI aSpiDevice[] =
{
    /* SPI 0 */
    {   (uint32_t)INTR_SPI0_STAT,                       /* SPI status interrupt ID              			*/
    	(uint32_t)INTR_SPI0_TXDMA,                      /* SPI peripheral DMA Tx interrupt ID         		*/
    	(uint32_t)INTR_SPI0_RXDMA,                     	/* SPI peripheral DMA Rx interrupt ID         		*/
        ADI_DMA_CHNL_SPI0_TX,                           /* SPI peripheral DMA Tx channel ID         		*/
        ADI_DMA_CHNL_SPI0_RX,                           /* SPI peripheral DMA Rx channel ID         		*/
        (ADI_SPI_REGS *)pREG_SPI0_CTL,                  /* Base address of registers            			*/
        NULL                                            /* null pointer for parameters storage  			*/
    },
    /* SPI 1 */
    {   (uint32_t)INTR_SPI1_STAT,                       /* SPI status interrupt ID              			*/
    	(uint32_t)INTR_SPI1_TXDMA,                      /* SPI peripheral DMA Tx interrupt ID         		*/
    	(uint32_t)INTR_SPI1_RXDMA,                     	/* SPI peripheral DMA Rx interrupt ID         		*/
        ADI_DMA_CHNL_SPI1_TX,                           /* SPI peripheral DMA Tx channel ID         		*/
        ADI_DMA_CHNL_SPI1_RX,                           /* SPI peripheral DMA Rx channel ID         		*/
        (ADI_SPI_REGS *)pREG_SPI1_CTL,                  /* Base address of registers            			*/
        NULL                                            /* null pointer for parameters storage  			*/
    },
    /* SPI 2 */
    {   (uint32_t)INTR_SPI2_STAT,                       /* SPI status interrupt ID              			*/
    	(uint32_t)ADI_INT_SPI2_TX,                      /* SPI peripheral non-DMA Tx interrupt ID         		*/
    	(uint32_t)ADI_INT_SPI2_RX,          	        /* SPI peripheral non-DMA Rx interrupt ID         		*/
        NULL,                                           /* SPI peripheral DMA Tx channel ID         		*/
        NULL,                                           /* SPI peripheral DMA Rx channel ID         		*/
        (ADI_SPI_REGS *)pREG_SPI2_CTL,                  /* Base address of registers            			*/
        NULL                                            /* null pointer for parameters storage  			*/
    }
};
#elif defined(__ADSPBF707_FAMILY__)
static ADI_SPI aSpiDevice[] =
{
    /* SPI 0 */
    {   (uint32_t)INTR_SPI0_STAT,                       /* SPI status interrupt ID                          */
        (uint32_t)INTR_SPI0_TXDMA,                      /* SPI peripheral DMA Tx interrupt ID               */
        (uint32_t)INTR_SPI0_RXDMA,                      /* SPI peripheral DMA Rx interrupt ID               */
        ADI_DMA_CHNL_SPI0_TX,                           /* SPI peripheral DMA Tx channel ID                 */
        ADI_DMA_CHNL_SPI0_RX,                           /* SPI peripheral DMA Rx channel ID                 */
        (ADI_SPI_REGS *)pREG_SPI0_CTL,                  /* Base address of registers                        */
        NULL                                            /* null pointer for parameters storage              */
    },
    /* SPI 1 */
    {   (uint32_t)INTR_SPI1_STAT,                       /* SPI status interrupt ID                          */
        (uint32_t)INTR_SPI1_TXDMA,                      /* SPI peripheral DMA Tx interrupt ID               */
        (uint32_t)INTR_SPI1_RXDMA,                      /* SPI peripheral DMA Rx interrupt ID               */
        ADI_DMA_CHNL_SPI1_TX,                           /* SPI peripheral DMA Tx channel ID                 */
        ADI_DMA_CHNL_SPI1_RX,                           /* SPI peripheral DMA Rx channel ID                 */
        (ADI_SPI_REGS *)pREG_SPI1_CTL,                  /* Base address of registers                        */
        NULL                                            /* null pointer for parameters storage              */
    },
    /* SPI 2 */
    {   (uint32_t)INTR_SPI2_STAT,                       /* SPI status interrupt ID                          */
        (uint32_t)INTR_SPI2_TXDMA,                      /* SPI peripheral DMA Tx interrupt ID               */
        (uint32_t)INTR_SPI2_RXDMA,                      /* SPI peripheral DMA Rx interrupt ID               */
        ADI_DMA_CHNL_SPI2_TX,                           /* SPI peripheral DMA Tx channel ID                 */
        ADI_DMA_CHNL_SPI2_RX,                           /* SPI peripheral DMA Rx channel ID                 */
        (ADI_SPI_REGS *)pREG_SPI2_CTL,                  /* Base address of registers                        */
        NULL                                            /* null pointer for parameters storage              */
    }
};


#else  /* device is not supported */

#error "SPI is not supported "
#endif

/* Number of SPI devices */
#define ADI_SPI_NUM_DEVICES (sizeof(aSpiDevice)/sizeof(ADI_SPI))

#endif /* ADI_CODE_IN_ROM */

/*! Lookup table to calculate, the number of times the Tx/Rx Fifo
 *  needs to be written/read to fill/empty it. The number is dependant
 *  on the Spi transfer size and the watermark level of the fifo at the
 *  time of the interrupt
 */

const uint8_t AccessFifo[5][9] =
{
	{0u, 0u, 0u, 0u, 0u, 0u, 0u, 0u, 0u},
	{0u, 1u, 2u, 3u, 4u, 5u, 6u, 7u, 8u},
	{0u, 1u, 1u, 2u, 2u, 3u, 3u, 4u, 4u},
	{0u, 0u, 0u, 0u, 0u, 0u, 0u, 0u, 0u},
	{0u, 1u, 1u, 1u, 1u, 2u, 2u, 2u, 2u}
};




/* status register */
const ADI_SPI_EVENT aSpiStatusEventFields[] =
{
	ADI_SPI_HW_ERR_NONE,
	ADI_SPI_HW_ERR_RX_OVERUN,
	ADI_SPI_HW_ERR_TX_UNDERRUN,
	ADI_SPI_HW_ERR_TX_COLLISION,
	ADI_SPI_HW_ERR_MODE
};

#define ADI_SPI_NUMSTATUSEVENTS (sizeof(aSpiStatusEventFields)/sizeof(uint32_t))



#endif /* __ADI_SPI_DATA_6XX_H__ */


