/*
 *****************************************************************************
 * @file:    adi_sport_data_bf6xx.c
 * @brief:   Data declaration for SPORT Device Driver for BF6XX family processor
 * @version: $Revision: 21988 $
 * @date:    $Date: 2013-06-21 17:01:32 -0400 (Fri, 21 Jun 2013) $
 *****************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_sport_data_bf6xx.c
* @brief     This file contains the data decleration required by device driver for
*            the operating  the  SPORT device in BF6XX family processor.
*/

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rules_all:"Suppress all rules for other header files")
#endif


#include "adi_sport_def.h"

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"Allows portions of code to be commented out")
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_6:"Allow identifier names to be reused in different name spaces")
#pragma diag(suppress:misra_rule_5_7:"Allow identifier names to be reused")
#pragma diag(suppress:misra_rule_14_7:"Allow multiple exits from a function for readability and optimized code")
#pragma diag(suppress:misra_rule_17_4:"Allow pointer arithmetic for MMR address generation")
#pragma diag(suppress:misra_rule_19_7:"Allow macros to be used for readability")
#endif /* _MISRA_RULES */

#if defined(__ADSPBF609_FAMILY__)

#define ADI_SPORT_NUM_INSTANCES         3u


#if !defined(ADI_CODE_IN_ROM)
/* Stores the information about the specific device */
static  ADI_SPORT_DEVICE_INFO sport_device_info [ADI_SPORT_NUM_INSTANCES][2]=
{
    {
        {ADI_SPORT_STATE_UNINITIALIZED,ADI_DMA_CHNL_SPORT0_A,(uint32_t)INTR_SPORT0_A_DMA,(uint32_t)INTR_SPORT0_A_STAT,(volatile SPORT_REGISTERS *)pREG_SPORT0_CTL_A,NULL},
        {ADI_SPORT_STATE_UNINITIALIZED,ADI_DMA_CHNL_SPORT0_B,(uint32_t)INTR_SPORT0_B_DMA,(uint32_t)INTR_SPORT0_B_STAT,(volatile SPORT_REGISTERS *)pREG_SPORT0_CTL_B,NULL},
    },
    {
        {ADI_SPORT_STATE_UNINITIALIZED,ADI_DMA_CHNL_SPORT1_A,(uint32_t)INTR_SPORT1_A_DMA,(uint32_t)INTR_SPORT1_A_STAT,(volatile SPORT_REGISTERS *)pREG_SPORT1_CTL_A,NULL},
        {ADI_SPORT_STATE_UNINITIALIZED,ADI_DMA_CHNL_SPORT1_B,(uint32_t)INTR_SPORT1_B_DMA,(uint32_t)INTR_SPORT1_B_STAT,(volatile SPORT_REGISTERS *)pREG_SPORT1_CTL_B,NULL},
    },
    {
        {ADI_SPORT_STATE_UNINITIALIZED,ADI_DMA_CHNL_SPORT2_A,(uint32_t)INTR_SPORT2_A_DMA,(uint32_t)INTR_SPORT2_A_STAT,(volatile SPORT_REGISTERS *)pREG_SPORT2_CTL_A,NULL},
        {ADI_SPORT_STATE_UNINITIALIZED,ADI_DMA_CHNL_SPORT2_B,(uint32_t)INTR_SPORT2_B_DMA,(uint32_t)INTR_SPORT2_B_STAT,(volatile SPORT_REGISTERS *)pREG_SPORT2_CTL_B,NULL},
    }
};
#endif

#elif defined(__ADSPBF707_FAMILY__)

#define ADI_SPORT_NUM_INSTANCES         2u

/* Stores the information about the specific device */

#ifdef ADI_CODE_IN_ROM
static  ADI_SPORT_DEVICE_INFO sport_device_info [ADI_SPORT_NUM_INSTANCES][2]=
#else
static  ADI_SPORT_DEVICE_INFO sport_device_info [ADI_SPORT_NUM_INSTANCES][2]=
#endif
{
    {
        {ADI_SPORT_STATE_UNINITIALIZED,ADI_DMA_CHNL_SPORT0_A,(uint32_t)INTR_SPORT0_A_DMA,(uint32_t)INTR_SPORT0_A_STAT,(volatile SPORT_REGISTERS *)pREG_SPORT0_CTL_A,NULL},
        {ADI_SPORT_STATE_UNINITIALIZED,ADI_DMA_CHNL_SPORT0_B,(uint32_t)INTR_SPORT0_B_DMA,(uint32_t)INTR_SPORT0_B_STAT,(volatile SPORT_REGISTERS *)pREG_SPORT0_CTL_B,NULL},
    },
    {
        {ADI_SPORT_STATE_UNINITIALIZED,ADI_DMA_CHNL_SPORT1_A,(uint32_t)INTR_SPORT1_A_DMA,(uint32_t)INTR_SPORT1_A_STAT,(volatile SPORT_REGISTERS *)pREG_SPORT1_CTL_A,NULL},
        {ADI_SPORT_STATE_UNINITIALIZED,ADI_DMA_CHNL_SPORT1_B,(uint32_t)INTR_SPORT1_B_DMA,(uint32_t)INTR_SPORT1_B_STAT,(volatile SPORT_REGISTERS *)pREG_SPORT1_CTL_B,NULL},
    }
};
#elif defined(ADI_ADSP_CM40Z)

#define ADI_SPORT_NUM_INSTANCES         2u

/* Stores the information about the specific device */
static  ADI_SPORT_DEVICE_INFO sport_device_info [ADI_SPORT_NUM_INSTANCES][2]=
{
    {
        {ADI_SPORT_STATE_UNINITIALIZED,ADI_DMA_CHNL_SPORT0_A,(uint32_t)INTR_SPORT0_A_DMA,(uint32_t)INTR_SPORT0_A_STAT,(volatile SPORT_REGISTERS *)pREG_SPORT0_CTL_A,NULL},
        {ADI_SPORT_STATE_UNINITIALIZED,ADI_DMA_CHNL_SPORT0_B,(uint32_t)INTR_SPORT0_B_DMA,(uint32_t)INTR_SPORT0_B_STAT,(volatile SPORT_REGISTERS *)pREG_SPORT0_CTL_B,NULL},
    },
    {
        {ADI_SPORT_STATE_UNINITIALIZED,ADI_DMA_CHNL_SPORT1_A,(uint32_t)INTR_SPORT1_A_DMA,(uint32_t)INTR_SPORT1_A_STAT,(volatile SPORT_REGISTERS *)pREG_SPORT1_CTL_A,NULL},
        {ADI_SPORT_STATE_UNINITIALIZED,ADI_DMA_CHNL_SPORT1_B,(uint32_t)INTR_SPORT1_B_DMA,(uint32_t)INTR_SPORT1_B_STAT,(volatile SPORT_REGISTERS *)pREG_SPORT1_CTL_B,NULL},
    }
};

#else
#error "Not supported"
#endif

#if !defined(ADI_CODE_IN_ROM)
/*! Function pointer table for DMA  mode */
const SPORT_FUNCTION_TABLE gSportDMAFuncTable =
{
    /*! closing the device in DMA mode*/
    &adi_pdma_Close,
    /*! Buffer submission function for DMA mode*/
    &SubmitBufferDmaMode,
	/*! function pointer for enabling the DMA */
	&EnableDma,
	/*! function pointer for disabling the DMA */
	&adi_pdma_DisableXfer
};
#endif

/*! Function pointer table for interrupt mode */

const SPORT_FUNCTION_TABLE gSportIntFuncTable =
{
    /*! closing the DMA manager*/
    NULL,
    /*! submitting the buffer */
    &SubmitBufferIntMode,
    /*! function pointer for enabling the DMA*/
    NULL,
    /*! function pointer for disabling the DMA*/
    NULL,
};

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif
