/*!
 *****************************************************************************
 * @file:    adi_sport_def_6xx.h
 * @brief:   SPORT Device Driver definition for  ADSP-BF609 family processor
 * @version: $Revision: 22007 $
 * @date:    $Date: 2013-06-25 15:06:10 -0400 (Tue, 25 Jun 2013) $
 *****************************************************************************
Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.
This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.
*********************************************************************************
     File         :   adi_sport_def_6xx.h
     Description  :   Static function and structure definition  needed by the device driver
                      for managing  the SPORT  device for  ADSP-BF609  family processor.
================================================================================ */
#ifndef _DEF_SPORT_DEF_6XX_H_
#define _DEF_SPORT_DEF_6XX_H_


#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"Identifiers (internal and external) shall not rely on the significance of more than 31 characters")
#pragma diag(suppress:misra_rule_19_4:"C macros shall only expand to a braced initialiser, a constant, a string literal, a parenthesised expression, a type qualifier, a storage class specifier, or a do-while-zero construct")
#endif /* _MISRA_RULES */


#if defined(__ADSPBF609_FAMILY__) || \
    defined(__ADSPBF707_FAMILY__)
#include <services/int/adi_sec.h>
#endif

/* Number of DMA Descriptors */
#define NUM_SPORT_BUFFER                3

#define DEFAULT_SPORT_INTERRUPT_MASK  0X7U

#define DEFAULT_SPORT_CLOCK_VALUE     0X00U

#define ADI_SPORT_NUM_DMA_DESC           2

#define SPORT_MAX_DATA_LENGTH           31U

#define SPORT_MAX_WINDOW_SIZE           15U

#define SPORT_BYTE_TRANSFER_LENGTH       7U

#define SPORT_HALFWORD_TRANSFER_LENGTH  15U

#define SPORT_WORD_TRANSFER_LENGTH      31U

#define SPORT_MAX_NUM_MC_SLOTS          128U

#define SPORT_MAX_WINDOW_OFFSET        896U

#define SPORT_MAX_FRM_DELAY             15U

/* 2DO: NEED TO SET TO DIFFERENT VALUE */
#define DEFAULT_SPORT_CONTROL_REG_VALUE     0X00U
#define DEFAULT_SPORT_CONTROL_REG2_VALUE    0X00U
#define DEFAULT_SPORT_MC_CONTROL_REG_VALUE  0X00U

/* Mask for data config field */
#define BITM_SPORT_DATA_CONFIG   (BITM_SPORT_CTL_DTYPE | \
                                 BITM_SPORT_CTL_LSBF  | \
                                 BITM_SPORT_CTL_PACK  | \
                                 BITM_SPORT_CTL_RJUST | \
                                 BITM_SPORT_CTL_SLEN)
/* Mask for clock  config field */
#define BITM_SPORT_CLOCK_CONFIG  (BITM_SPORT_CTL_ICLK  | \
                                 BITM_SPORT_CTL_CKRE  | \
                                 BITM_SPORT_CTL_GCLKEN)

/* Mask for FS config field */
#define BITM_SPORT_FS_CONFIG     (BITM_SPORT_CTL_FSR   | \
                                 BITM_SPORT_CTL_IFS   | \
                                 BITM_SPORT_CTL_DIFS  | \
                                 BITM_SPORT_CTL_LFS   | \
                                 BITM_SPORT_CTL_LAFS  | \
                                 BITM_SPORT_CTL_FSED)

#define BITM_SPORT_MC_CONFIG     (BITM_SPORT_MCTL_WOFFSET | \
                                 BITM_SPORT_MCTL_MCPDE)

#define BITM_SPORT_MUX_CONFIG    (BITM_SPORT_CTL2_CKMUXSEL | \
	                             BITM_SPORT_CTL2_FSMUXSEL)

#define MAX_MULTI_CHANNEL_NUM        127u

#if defined(__ADSPBLACKFIN__)
#define ALIGN
#define ALIGN4  _Pragma("align(4)")
#elif defined(__ICCARM__)
#define ALIGN   _Pragma("pack()")
#define ALIGN4  _Pragma("pack(4)")
#endif
                                       
/* Enumeration of different SPORT states */
typedef enum
{
    ADI_SPORT_STATE_UNINITIALIZED = 0,  /* SPORT is not yet initialized */

    ADI_SPORT_STATE_INITIALIZED,        /* SPORT is initialized */

    ADI_SPORT_STATE_DATA_FLOW_ENABLED,  /* SPORT Tx  OR Rx data flow is enabled */

    ADI_SPORT_STATE_DATA_FLOW_DISABLED,  /* SPORT Tx  OR Rx data flow is disabled */

    ADI_SPORT_STATE_PAUSED
} ADI_SPORT_STATE;


/* Function pointer typedef for the function which  closes the sport device */
typedef ADI_DMA_RESULT (*SPORT_CLOSE) (ADI_DMA_CHANNEL_HANDLE hChannel, bool bEnable);

/* Function pointer typedef for the function which  submit the buffer */
typedef ADI_SPORT_RESULT (*SPORT_BUFFER_SUBMIT) (ADI_SPORT_HANDLE hDevice,void *pBuffer, uint32_t nsize);

/* Function prototype for the  submitting the buffer interrupt mode*/
typedef ADI_DMA_RESULT (*SPORT_DMA_ENABLE)(ADI_DMA_CHANNEL_HANDLE hChannel,ADI_DMA_MSIZE eMsize, ADI_DMA_PSIZE ePsize );

/* Function prototype for the  submitting the buffer interrupt mode*/
typedef ADI_DMA_RESULT (*SPORT_DMA_DISABLE)(ADI_DMA_CHANNEL_HANDLE hChannel);


/* Structure for  Function table */
typedef struct _SPORT_FUNCTION_TABLE
{

    SPORT_CLOSE          pfDMAClose;        /* function pointer for closing the device */

    SPORT_BUFFER_SUBMIT  pfSubmitBuffer;    /* function pointer for submitting the buffer */

    SPORT_DMA_ENABLE     pfEnableDMA;       /* function pointer for enabling the DMA */

    SPORT_DMA_DISABLE    pfDisableDMA;      /* function pointer for disabling the DMA */

}SPORT_FUNCTION_TABLE;



/* Descriptor structure for Interrupt mode */
typedef struct _ADI_SPORT_BUFF_LIST
{

    void    *pStartAddress;                 /* pointer to a buffer*/

    uint32_t nCount;                        /* Size of the buffer in bytes */

    uint32_t nIndex;                        /* variable for indexing the next word for placing in FIFO */

    bool bInUse;                            /* useage flag */

    struct _ADI_SPORT_BUFF_LIST  *pNext;    /* Pointer to next descriptor */

}ADI_SPORT_BUFF_LIST;


/* SPORT registers */
typedef struct _SPORT_REGISTERS
{
    volatile uint32_t  SPORT_CTL;           /* SPORT control Register */
    volatile uint32_t  SPORT_DIV;           /* SPORT Clock Division register */
    volatile uint32_t  SPORT_MCTL;          /* SPORT Multi-channel control register */
    volatile uint32_t  SPORT_CS0;           /* SPORT Channel select register-0 */
    volatile uint32_t  SPORT_CS1;           /* SPORT Channel select register-1 */
    volatile uint32_t  SPORT_CS2;           /* SPORT Channel select register-2 */
    volatile uint32_t  SPORT_CS3;           /* SPORT Channel select register-3 */
    volatile uint32_t  SPORT_CNT;           /* SPORT current status of counters of the FS divider and internal clock generators */
    volatile uint32_t  SPORT_ERRCTL;        /* SPORT Error Control Register */
    volatile uint32_t  SPORT_MSTAT;         /* SPORT Multi-channel status register */
    volatile uint32_t  SPORT_CTL2;          /* SPORT Control Register 2 */
    volatile uint32_t  SPORT_PADDING1;      /* SPORT padding variable -1 */
    volatile uint32_t  SPORT_PADDING2;      /* SPORT padding variable -2 */
    volatile uint32_t  SPORT_PADDING3;      /* SPORT padding variable -3 */
    volatile uint32_t  SPORT_PADDING4;      /* SPORT padding variable -4 */
    volatile uint32_t  SPORT_PADDING5;      /* SPORT padding variable -5 */
    volatile uint32_t  SPORT_TXA;           /* SPORT Primary Transmit Shift Register */
    volatile uint32_t  SPORT_RXA;           /* SPORT Primary Receive  Shift Register */
    volatile uint32_t  SPORT_TXB;           /* SPORT Secondary Transmit Counter Register */
    volatile uint32_t  SPORT_RXB;           /* SPORT Secondary Receive Counter Register */
    volatile uint32_t  SPORT_REVID;         /* SPORT Revision ID register */
} SPORT_REGISTERS;
/* SPORT registers */


typedef struct _ADI_SPORT_DEVICE_INFO
{
    ADI_SPORT_STATE           eState;       /* To indicate the state of the device */
    CONST ADI_DMA_CHANNEL_ID  eDMAChnlID;   /* DMA channel ID */
    CONST uint32_t            nDMAIntID;    /* DMA Interrupt ID */
    CONST uint32_t            nStatusIntID; /* Status Interrupt ID */
    volatile SPORT_REGISTERS *pSportRegs;   /*Base address of the  SPORT registers */
    ADI_SPORT_HANDLE          hDevice;      /* SPORT handle */
}ADI_SPORT_DEVICE_INFO;


/* SPORT driver instance data */
typedef struct _ADI_SPORT_DEVICE
{
    ADI_SPORT_DIRECTION       eDirection;   /* Direction in which the SPORT is opened */

    ADI_SPORT_MODE            eMode;        /* Mode in which SPORT device operates */

    bool                      bDMAMode;     /* Boolean flag to indicate whether enable DMA when the associated SPORT is enabled */

    void                     *pDMAChnlMem;  /* Pointer to  the DMA manager memory  */

    ADI_DMA_CHANNEL_HANDLE    hDMAChnl;     /* DMA Channel Handle */

    volatile SPORT_REGISTERS *pSPORTRegs;   /* Pointer to SPORT Memory Mapped Registers */

    ADI_SPORT_BUFF_LIST       BufInfo[NUM_SPORT_BUFFER];     /* Buffer management related information */

    ADI_SPORT_BUFF_LIST      *pCurrDesc;    /* Points  to  current position of queue  */

    ADI_SPORT_BUFF_LIST      *pActiveDesc;  /* Points to an active buffer(From which data being processed) poition in the queue */

    ADI_SPORT_BUFF_LIST      *pProcDesc;    /* Points to the "free" buffer position in the queue */

    ADI_SPORT_DEVICE_INFO    *pSportInfo;   /* pointer to the structure which stores the information about the sport instances.*/

    SPORT_FUNCTION_TABLE     *pFunTable;    /* Pointer to Sport function table */

    ADI_CALLBACK              pfCallback;   /*  Function pointer for callback function. */

    void                     *pCBParam;     /*  Parameter to callback function. */

    ADI_OSAL_SEM_HANDLE       hSemaphore;   /* Semaphoer Handle  */

    volatile uint32_t         nHwError;     /* variable to store the hardware status */

    ADI_DMA_MSIZE             eDmaWordSize;  /* DMA transfer size */

    ADI_DMA_PSIZE             ePWordSize;    /* DMA transfer size */

    int32_t                   nXmodify;      /* X modify for the data transfer. */

    uint16_t                  nShiftValue;    /* Shift value to calculate the transfer count. */
#if defined (__CC_ARM) 
    /* Memory required for creating Semaphore */
    uint8_t                   SemMemory[ADI_OSAL_MAX_SEM_SIZE_CHAR] __attribute__ ((aligned));
#else
ALIGN4
    /* Memory required for creating Semaphore */
    uint8_t                   SemMemory[ADI_OSAL_MAX_SEM_SIZE_CHAR];
ALIGN
#endif
} ADI_SPORT_DEVICE;

/* Initializes the DMA descriptors */
static void InitDevice (ADI_SPORT_DEVICE *pDevice);

static void sport_DataHandler(uint32_t IID ,void * pCBParam);

 /* SPORT  Status interrupt handler */
static void SPORTStatusHandler(uint32_t IID  ,void * pCBParam);

 /* SPORT DMA interrupt handler */
static void SPORTDMAHandler(void *pCBParam, uint32_t Event, void *pArg);

/* Function prototype for the  submitting the buffer in DMA mode */
static ADI_SPORT_RESULT SubmitBufferDmaMode(ADI_SPORT_HANDLE hDevice,void *pBuffer, uint32_t nSize);

/* Function prototype for the  submitting the buffer in  interrupt mode*/
static ADI_SPORT_RESULT SubmitBufferIntMode(ADI_SPORT_HANDLE hDevice,void *pBuffer, uint32_t nSize);

/* Function prototype for the  managing the buffer after it is processed*/
static void sport_mange_buffer(ADI_SPORT_DEVICE *pDevice,void *pProcessedBuff);

/* Function prototype for the  enabling the DMA operation */
static ADI_DMA_RESULT EnableDma(ADI_DMA_CHANNEL_HANDLE hChannel, ADI_DMA_MSIZE eMsize, ADI_DMA_PSIZE ePsize);
#if  defined(ADI_ADSP_CM40Z)
static ADI_SPORT_RESULT adi_sec_ClearPendStatus(uint32_t SID);
#endif

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

#endif	/* end of ifndef _DEF_SPORT_DEF_6XX_H_ */


