/******************************************************************************

Copyright (c) 2013 Analog Devices.  All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.
*******************************************************************************/

#ifndef __ADI_TWI_IMP_H__
#define __ADI_TWI_IMP_H__

#include <drivers/twi/adi_twi.h>

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_8_2:"When an array is declared with external linkage, its size shall be stated explicitly or defined implicitly by initialisation")
#pragma diag(suppress:misra_rule_8_12:"When an array is declared with external linkage, its size shall be stated explicitly or defined implicitly by initialisation.")
#pragma diag(suppress:misra_rule_16_4:"The identifiers used in the declaration and definition of a function shall be identical")
#endif


/******************************************************
 Internal data structures:
 *******************************************************/

/*
 * The blocking mode is set depending on the specific API's called
 * by the application.
 */
typedef enum
{
    BLOCK_MODE_NONE,
    BLOCK_MODE_BLOCKING,
    BLOCK_MODE_NON_BLOCKING
} BLOCK_MODE;

/*
 * TWI register structure
 */
typedef volatile struct {
	uint16_t clkdiv;
	uint16_t pad1;
	uint16_t ctl;
	uint16_t pad2;
	uint16_t slvctl;
	uint16_t pad3;
	uint16_t slvstat;
	uint16_t pad4;
	uint16_t slvaddr;
	uint16_t pad5;
	uint16_t mstrctl;
	uint16_t pad6;
	uint16_t mstrstat;
	uint16_t pad7;
	uint16_t mstraddr;
	uint16_t pad8;
	uint16_t istat;
	uint16_t pad9;
	uint16_t imsk;
	uint16_t pad10;
	uint16_t fifoctl;
	uint16_t pad11;
	uint16_t fifostat;
} ADI_TWI_REGISTER1_STRUCT;

/* use two structs due to gap between TWI registers */

typedef volatile struct {
	uint16_t txdata8;
	uint16_t pad1;
	uint16_t txdata16;
	uint16_t pad2;
	uint16_t rxdata8;
	uint16_t pad3;
	uint16_t rxdata16;
} ADI_TWI_REGISTER2_STRUCT;

/*
 * TWI configuration structure
 */
typedef struct {
	ADI_TWI_MODE mode;       /* dataflow mode/method (Control call) */
	uint16_t frequency;      /* the frequency of the twi in kHz (default=100) */
	uint16_t prescale;       /* the prescale value */
	uint16_t duty_cycle;     /* the duty cycle of the twi in % (default=50) */
	uint8_t fifo;            /* byte or word per interrupt, set by ADI_TWI_CMD_FIFO Control */
	uint8_t lostarbLimit;    /* error count limits (default=3) */
	uint8_t anakLimit;       /* (default=1) */
	uint8_t dnakLimit;       /* (default=1) */
} ADI_TWI_CONFIG_STRUCT;

/*
 * TWI buffer structure
 */
typedef struct {
	uint8_t *Data;
	uint32_t Count;
	uint32_t Index;

	/* counters to keep track of the number of errors */
	uint8_t lostarb;
	uint8_t anak;
	uint8_t dnak;

	uint8_t pendRx;  /* next read should move data from TWI FIFO */
	uint8_t pendTx;  /* next write should move data to TWI FIFO */
	uint8_t sdir;    /* a copy of SDIT at SINIT */
	uint8_t gcall;   /* general call is being received */

	uint8_t Direction;
	bool Restart;
	bool Complete;
	bool bError;
} ADI_TWI_BUFFER_STRUCT;

/*
 * TWI data structure
 */
typedef struct {
	ADI_TWI_CONFIG_STRUCT config;
	ADI_TWI_BUFFER_STRUCT buffer1;
	ADI_TWI_BUFFER_STRUCT buffer2;

	ADI_CALLBACK Callback;
	void *pCallParam;
	bool bInUse;
	uint32_t sid;
	BLOCK_MODE eBlockMode;
	ADI_TWI_BUFFER_STRUCT *activeBuffer;
	ADI_OSAL_SEM_HANDLE hSemHandle;

} ADI_TWI_DATA_STRUCT;

#define DATA_MEM_SIZE (sizeof(ADI_TWI_DATA_STRUCT) + sizeof(ADI_TWI_CONFIG_STRUCT) \
	+ 2u*sizeof(ADI_TWI_BUFFER_STRUCT))

/*
 *  TWI Device Structure
 */


typedef struct
{
	ADI_TWI_REGISTER1_STRUCT *  pReg1;
	ADI_TWI_REGISTER2_STRUCT *  pReg2;
	ADI_TWI_DATA_STRUCT      *  pData;
} ADI_TWI_DRIVER_STRUCT;

/* alias for the actual device structure */
typedef ADI_TWI_DRIVER_STRUCT ADI_TWI_DRIVER;

#if defined(ADI_CODE_IN_ROM)
extern ADI_TWI_DRIVER adi_twi_Device[];
#endif

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

#endif
