/*!
 *****************************************************************************
 * @file:    adi_uart_def.h
 * @brief:   UART Device Driver definition for blackfin processor
 * @version: $Revision: 15037 $
 * @date:    $Date: 2012-06-26 05:41:32 -0400 (Tue, 26 Jun 2012) $
 *****************************************************************************
Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.
This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.
*********************************************************************************
     File         :   adi_uart_def.h
     Description  :   Static function and structure definition  needed by the device driver
                      managing  the UART  device.
================================================================================ */
#ifndef _DEF_UART_DEF_H_
#define _DEF_UART_DEF_H_

#if defined( __ADSPBF506F_FAMILY__)  || defined( __ADSPBF548_FAMILY__)

#define  ADI_UART_SET_UART_DALB
#define  ADI_UART_CLEAR_UART_DALB

#define  ADI_UART_RBR_GET()       (pDevice->pUARTRegs->UARTREG_RBR)
#define  ADI_UART_THR_SET(Value)  (pDevice->pUARTRegs->UARTREG_THR = (Value))

#define  ADI_UART_DISABLE_TX()     (pDevice->pUARTRegs->UARTREG_IERCLR |=(uint16_t)  (ENUM_UART_IER_ETBEI_EN))
#define  ADI_UART_ENABLE_TX()      (pDevice->pUARTRegs->UARTREG_IERSET |=(uint16_t)  (ENUM_UART_IER_ETBEI_EN ))

#define  ADI_UART_DISABLE_RX()     (pDevice->pUARTRegs->UARTREG_IERCLR |=(uint16_t)  (ENUM_UART_IER_ERBFI_EN))
#define  ADI_UART_ENABLE_RX()      (pDevice->pUARTRegs->UARTREG_IERSET |=(uint16_t)  (ENUM_UART_IER_ERBFI_EN ))

#define HW_FLOW_CONFIG_FIELDS     ( ENUM_UART_MCR_ACTS_EN | ENUM_UART_MCR_ARTS_EN | ENUM_UART_MCR_FCPOL_POS )

#else

#define  ADI_UART_SET_UART_DALB    (pDevice->pUARTRegs->UARTREG_LCR |=(uint16_t) ENUM_UART_CTL_DLH_DLL)
#define  ADI_UART_CLEAR_UART_DALB  (pDevice->pUARTRegs->UARTREG_LCR &=(uint16_t) ~(ENUM_UART_CTL_DLH_DLL))

#define  ADI_UART_RBR_GET()        (pDevice->pUARTRegs->UARTREG_DLL)
#define  ADI_UART_THR_SET(Value)   (pDevice->pUARTRegs->UARTREG_DLL =(uint16_t) (Value))

#define  ADI_UART_DISABLE_TX()     (pDevice->pUARTRegs->UARTREG_DLH &=(uint16_t)~(ENUM_UART_IER_ETBEI_EN))
#define  ADI_UART_ENABLE_TX()      (pDevice->pUARTRegs->UARTREG_DLH |=(uint16_t) (ENUM_UART_IER_ETBEI_EN))

#define  ADI_UART_DISABLE_RX()     (pDevice->pUARTRegs->UARTREG_DLH &=(uint16_t) ~(ENUM_UART_IER_ERBFI_EN))
#define  ADI_UART_ENABLE_RX()      (pDevice->pUARTRegs->UARTREG_DLH |=(uint16_t)  (ENUM_UART_IER_ERBFI_EN))

#endif

/*!  X-Modification for DMA operation since transfer size is one byte. */
#define UART_DATA_XMODIFY 1

/*!  Configure the DMA operation for one byte transfer. */
#define ADI_UART_DMA_MSIZE    (ADI_DMA_MSIZE_1BYTE)

/*!  Number of bits for autobaud detection.Default character for autobaud detection is ' '. i.e Space character. */
#define NUM_OF_BITS_FOR_AUTOBAUD 6U

/*!  Timer device number for autobaud. */
#define TIMER_NUM_FOR_AUTOBAUD 0U

/*! Default IER register configuration. */
#define DEFAULT_IER_CONFIG  (ENUM_UART_IER_ELSI_EN)

/*! Number of descriptors to manage the buffers*/
#define ADI_NUM_UART_BUFFER    3U

#define DEFAULT_PRE_SCALE_VALUE 16.0

/*! Function pointer typedef for the function which  closes the uart device */
typedef ADI_UART_RESULT (*UART_CLOSE) (ADI_UART_HANDLE hDevice);

/*! Function pointer typedef for the function which  submit the buffer */
typedef ADI_UART_RESULT (*UART_BUFFER_SUBMIT) (ADI_UART_HANDLE hDevice,void *const 	pBuffer, uint32_t nsize,ADI_UART_DIRECTION eDirection);

/*! Function prototype for enabling DMA associated with the device*/
typedef ADI_DMA_RESULT (*UART_DMA_ENABLE)(ADI_DMA_CHANNEL_HANDLE const hChannel);

/*! Function prototype for disabling the DMA associated with the device*/
typedef ADI_DMA_RESULT (*UART_DMA_DISABLE)(ADI_DMA_CHANNEL_HANDLE hChannel);

/*! typedef  of a function which enable/disable the specified channel */
typedef ADI_UART_RESULT (*UART_ENABLE) (ADI_UART_HANDLE hDevice,bool bEnable);

/*!
 * \enum  ADI_UART_MODE
 *
 *  Enum for Mode of operation .
 *
 */
typedef enum
{
    ADI_UART_MODE_NORMAL      = 0,     /*!< Regular UART Mode */

    ADI_UART_MODE_IRDA                 /*!< IrDA Mode */
} ADI_UART_MODE;

/*!
 * \enum  ADI_UART_TRANSFER_MODE
 *
 *  enumeration for Data transfer mode.
 */
typedef enum
{
    /*! Mode of data transfer is not selected.*/
    ADI_UART_DATA_TRANSFER_MODE_NONE,

    /*! Blocking mode. Only call to adi_uart_Read/adi_uart_write is allowed.
    Complete data of specified size is transfered before returning from the function. */
    ADI_UART_DATA_TRANSFER_MODE_BLOCKING,

    /*!Non-Blocking mode. Only call to adi_uart_SubmitRxBuffer/adi_uart_SubmitTxBuffer is allowed.
    Data transfer happens in the background and notified by posting a semaphore.*/
    ADI_UART_DATA_TRANSFER_MODE_NONBLOCKING

} ADI_UART_TRANSFER_MODE;

/*!
 * \enum  CHANNEL_DATAFLOW_STATUS
 *
 *!  Enumeration of channel data flow status
 */
typedef enum
{
    CHANNEL_STATE_DATA_FLOW_DISABLED,   /*!< data flow is disabled for the channel*/

    CHANNEL_STATE_DATA_FLOW_ENABLED,    /*!< data flow is enabled for the channel*/

    CHANNEL_STATE_DATA_FLOW_PAUSED      /*!< data flow is paused due to no availability  of buffer*/

}CHANNEL_DATAFLOW_STATUS;

/*!
 * \struct _UART_REGISTERS
 *!   Structure representing the UART registers in memory. This helps to access
 *   UART device registers. Members of this structure  MUST be in  same order as
 *   they defined.  UART device defined for "__ADSPBF506F_FAMILY__" and "__ADSPBF548_FAMILY__"
 *   is different from UART devices defined for other  blackfin family processor
 */
#if defined( __ADSPBF506F_FAMILY__)  || defined( __ADSPBF548_FAMILY__)

typedef  struct _UART_REGISTERS
{
	volatile uint16_t    UARTREG_DLL;    /*!<  DLL registers */
	uint16_t             Padding0;       /*!< Padding to advance the address */

    volatile uint16_t    UARTREG_DLH;    /*!< DLH  registers */
    uint16_t             Padding1;       /*!< Padding to advance the address */

    volatile uint16_t    UARTREG_GCTL;   /*!< Global control register */
    uint16_t             Padding2;       /*!< Padding to advance the address */

    volatile uint16_t    UARTREG_LCR;    /*!< line control register */
    uint16_t             Padding3;       /*!< Padding to advance the address */

    volatile uint16_t    UARTREG_MCR;    /*!<  modem control register */
    uint16_t             Padding4;       /*!< Padding to advance the address */

    volatile uint16_t    UARTREG_LSR;    /*!<  line status register */
    uint16_t             Padding5;       /*!< Padding to advance the address */

    volatile uint16_t    UARTREG_MSR;    /*!<  modem status register */
    uint16_t             Padding6;       /*!< Padding to advance the address */

    volatile uint16_t    UARTREG_SCR;    /*!<  scratch register */
    uint16_t             Padding7;       /*!< Padding to advance the address */

    volatile uint16_t    UARTREG_IERSET; /*!<  IER set register */
    uint16_t             Padding8;       /*!< Padding to advance the address */

    volatile uint16_t    UARTREG_IERCLR; /*!<  IER clear register */
    uint16_t             Padding9;       /*!< Padding to advance the address */

    volatile uint16_t    UARTREG_THR;    /*!< THR registers */
    uint16_t             Padding10;      /*!< Padding to advance the address */

    volatile uint16_t    UARTREG_RBR;   /*!<  RBR  registers */
    uint16_t             Padding11;     /*!< Padding to advance the address */

} UART_REGISTERS;
#else
typedef struct _UART_REGISTERS
{
    volatile uint16_t    UARTREG_DLL; 	/*!< THR, RBR and DLL registers */
    uint16_t             Padding0;      /*!< Padding to advance the address */

    volatile uint16_t    UARTREG_DLH;   /*!< DLH and IER registers */
    uint16_t             Padding1;      /*!< Padding to advance the address */

    volatile uint16_t    UARTREG_IIR;   /*!< interrupt ID register */
    uint16_t             Padding2;      /*!< Padding to advance the address */

    volatile uint16_t    UARTREG_LCR;   /*!< line control register */
    uint16_t             Padding3;      /*!< Padding to advance the address */

    volatile uint16_t    UARTREG_MCR;   /*!<  modem control register */
    uint16_t             Padding4;      /*!< Padding to advance the address */

    volatile uint16_t    UARTREG_LSR;   /*!<  line status register */
    uint16_t             Padding5;      /*!< Padding to advance the address */

    uint16_t             Padding6;      /*!< padding to advance the address */
    uint16_t             Padding7;      /*!< Padding to advance the address */

    volatile uint16_t    UARTREG_SCR;   /*!<  scratch register */
    uint16_t             Padding8;      /*!< Padding to advance the address */

    uint16_t             Padding9;      /*!< padding to advance the address */
    uint16_t             Padding10;     /*!< Padding to advance the address */

    volatile uint16_t    UARTREG_GCTL;  /*!< global control register */

} UART_REGISTERS;
#endif
/*!
 * \struct _ADI_UART_BUFF_INFO
 *
 *  structure for  managing the submitted  the buffers.
 */
typedef struct _ADI_UART_BUFF_INFO
{
    void    *pStartAddress;              /*!< pointer to a buffer*/

    uint32_t nCount;                     /*!< Size of the buffer in bytes */

    uint32_t nIndex;                     /*!< variable for indexing the next word for placing in FIFO */

    bool     bInUse;                     /*!< useage flag */

    struct _ADI_UART_BUFF_INFO  *pNext;  /*!< Pointer to next descriptor */

}ADI_UART_BUFF_INFO;

/*!
 * \enum  ADI_UART_STATE
 *
 *  Enumeration of different UART states
 */
typedef enum
{
    ADI_UART_STATE_UNINITIALIZED = 0,    /*!< UART is not yet initialized */

    ADI_UART_STATE_INITIALIZED,          /*!< UART is initialized */

    ADI_UART_STATE_DATA_FLOW_ENABLED,    /*!< UART Tx  OR Rx data flow is enabled */

    ADI_UART_STATE_DATA_FLOW_DISABLED,   /*!< UART Tx  OR Rx data flow is disabled */

    ADI_UART_STATE_AUTOBAUD_IN_PROGRESS  /*!< Started the Autobaud detection */

}ADI_UART_STATE;

/*!
 * \struct  _ADI_UART_DEVICE_INFO
 *
 *  Structre for storing the basic device information such as state, address, interrupt ID etc.
 *
 */
typedef struct _ADI_UART_DEVICE_INFO
{

    ADI_UART_STATE               eState;     /*!<  To indicate the state of the device */

    const ADI_DMA_PERIPHERAL_MAP eDMAMapRx;  /*!<  Peripheral map for Rx */

    const ADI_DMA_PERIPHERAL_MAP eDMAMapTx;  /*!<  Peripheral map for Tx */

    const uint32_t               nIntID;     /*!<  Status Interrupt ID */

    volatile UART_REGISTERS     *pUartRegs;  /*!<  Base address of the  UART registers */

    ADI_UART_HANDLE              hDevice;    /*!<  Handle for the instance */

}ADI_UART_DEVICE_INFO;

/*!
 * \struct _UART_DATA_CHANNEL
 *
 *  structure to manage the data transfer for a given channel.
 *  One instance of this  structure will be created for managing the
 *  data transfer in each direction.
 */
typedef struct _UART_DATA_CHANNEL
{

    void                  *pDMAChnlMem;      /*!< Pointer to  the DMA manager memory*/

    ADI_DMA_CHANNEL_HANDLE hDMAChnl;         /*!<  DMA Channel Handle  */

    ADI_UART_BUFF_INFO    oBufferInfo[ADI_NUM_UART_BUFFER];  /*!< Number of buffer nodes */

    ADI_UART_BUFF_INFO    *pCurrDesc;        /*!<  Pointer to the current buffer info */

    ADI_UART_BUFF_INFO    *pProcDesc;        /*!<  Pointer to the queue of the processed Buffers */

    ADI_UART_BUFF_INFO    *pFreeDesc;        /*!<  Pointer to the queue of the free descriptor*/

    ADI_UART_TRANSFER_MODE eDataTranferMode; /*!<  Stores the Data transfer mode */

    uint32_t               nActiveBufferCount;/*!<  Active buffer count */

    uint32_t               nUARTDataSID;     /*!<  UART  data SID */

    ADI_DMA_CHANNEL_ID     eDMAChannelID; 	 /*!<  UART DMA channel ID */

    ADI_OSAL_SEM_HANDLE    hSemaphore;       /*!<  Semaphore Handle  */

    CHANNEL_DATAFLOW_STATUS  eChannelStatus; /*!<  indicate the data flow status of the channel */

#pragma align(4)
    uint8_t               SemMemory[ADI_OSAL_MAX_SEM_SIZE_CHAR];     /*!< Memory required for creating Semaphore */

}UART_DATA_CHANNEL;

/*!
 * \struct __DEVICE_FUNCTION_TABLE
 *
 *  structure to manage the function table.Two function tables are
 *  defined to manage the UART device in Interrupt mode and DMA mode.
 *
 */
typedef struct __DEVICE_FUNCTION_TABLE
{

    UART_CLOSE          pfuart_Close;        /*!< function pointer for closing the device */

    UART_BUFFER_SUBMIT  pfuart_SubmitBuffer; /*!< function pointer for submitting the buffer */

    UART_DMA_ENABLE     pfuart_EnableDMA;    /*!< function pointer for enabling the DMA */

    UART_DMA_DISABLE    pfuart_DisableDMA;   /*!< function pointer for disabling the DMA */
}DEVICE_FUNCTION_TABLE;

/*!
 * \struct  _ADI_UART_DEVICE
 *  Structre for managing the UART device. This structure contain the data for
 *  managing the UART device.
 */
#pragma pack(4)
typedef struct _ADI_UART_DEVICE
{
    ADI_UART_DIRECTION     eDirection;       /*!< Direction in which the UART is opened */

    ADI_UART_MODE          eMode;            /*!< Mode in which UART device operates */

    bool                   bDMAMode;         /*!< Boolean flag to indicate whether enable DMA when the associated UART is enabled */

    ADI_UART_DEVICE_INFO  *pUartInfo;        /*!< pointer to the structure which stores the information about the uart instances.*/

    DEVICE_FUNCTION_TABLE *pFunTable;        /*!< Pointer to uart function table */

    ADI_CALLBACK           pfCallback;       /*!<  Function pointer for callback function. */

    void                  *pCBParam;         /*!<  Parameter to callback function. */

    uint32_t               nBaudRate;        /*!< Baudrate of the device. */

    uint32_t               nTimerDevNum;     /*!< Timer Device number for  Autobaud */

    uint32_t               nDivisorBits;     /*!< Number of divisorBits */

    ADI_TMR_HANDLE         hTimer;           /*!< Timer handle for Autobaud */

    volatile UART_REGISTERS  *pUARTRegs;     /*!< Pointer to UART Memory Mapped Registers */

    uint8_t aTmrMemory[ADI_TMR_MEMORY_SIZE];      /*!< Buffer for managing the timer for autobaud */

    volatile uint32_t      nHwError;         /*!< To store the last error */
    
    float32_t              nPreScale;        /*!<  Prescaler value */

    UART_DATA_CHANNEL     *pChannelTx;       /*!< Pointer for managing the Tx channel */

    UART_DATA_CHANNEL     *pChannelRx;       /*!< Pointer for managing the Rx channel */
} ADI_UART_DEVICE;


/*! Initializes the Nodes which contains the buffer informations */
static void uart_InitDesc ( uint32_t NumDesc,ADI_UART_BUFF_INFO *pBuffDesc);

/*! Init function for the uart device.Initializes all the registers. */
static void uart_init(ADI_UART_DEVICE  *pDevice);

/*! Channel initialization function prototype */
static ADI_UART_RESULT uart_channel_init(ADI_UART_DEVICE  *pDevice,UART_DATA_CHANNEL *pChannel,ADI_DMA_PERIPHERAL_MAP eDMAMap,ADI_INT_HANDLER_PTR pfDataHandler);

/* Common get buffer function prototype. This function can be called from either Tx channel or Rx channel */
static ADI_UART_RESULT uart_getbuffer(ADI_UART_DEVICE  *pDevice,UART_DATA_CHANNEL *pChannel,void **ppBuffer);

/*! Function which pends for the compeletion of the buffer processing. */
static ADI_UART_RESULT uart_pendForBuffer(ADI_UART_DEVICE  *pDevice ,UART_DATA_CHANNEL *pChannel,void *const pBuffer,UART_ENABLE pfEnable);

/*! Common function for managing the buffer. This function is being called for both Tx and Rx channel . */
static void uart_mange_buffer(ADI_UART_DEVICE *pDevice,UART_DATA_CHANNEL *pChannel,ADI_UART_EVENT eEvent,void *pArg);

/*! Function prototype for the  submitting the buffer in DMA mode */
static ADI_UART_RESULT uart_SubmittBufferDMAmode(ADI_UART_HANDLE hDevice,void *const pBuffer, uint32_t nSize,ADI_UART_DIRECTION eDirection);

/*! Function prototype for the  submitting the buffer in  interrupt mode*/
static ADI_UART_RESULT uart_SubmittBufferIntmode(ADI_UART_HANDLE hDevice,void *const pBuffer, uint32_t nSize,ADI_UART_DIRECTION eDirection);

/*! Function prototype for the  closing the uart in DMA mode*/
static ADI_UART_RESULT uart_CloseDMAmode(ADI_DMA_CHANNEL_HANDLE hChannel);

/*! Common function for UART enable for both TX and RX */
static ADI_UART_RESULT uart_Enable(ADI_UART_HANDLE  hDevice,bool bEnable, ADI_DMA_CHANNEL_HANDLE hChannel);

/*! UART  RX DMA  handler */
static void uart_RxDMAHandler(void *pCBParam,uint32_t Event,void *pArg);

/*! UART  TX DMA  handler */
static void uart_TxDMAHandler(void *pCBParam,uint32_t Event,void *pArg);

/*! UART  Status interrupt handler */
static void uart_StatusHandler(uint32_t IID ,void * pCBParam);

/*! Function for data handling in transmit mode */
static void uart_TxDataHandler(uint32_t IID, void *pCBParam);

/*! Function for data handling in receive mode */
static void uart_RxDataHandler(uint32_t IID, void *pCBParam);

/*! Function for timer call-back for autobaud  */
static void TimerCallback(void *pCBParam, uint32_t nEvent, void *pArg);

/*! Function for closing the RX or TX channel  */
static ADI_UART_RESULT uart_CloseChannel(UART_DATA_CHANNEL *pChannel, bool bDMAMode);
#endif	/* end of ifndef _DEF_UART_DEF_H_ */


