/*********************************************************************************

Copyright(c) 2012 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential. By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_usbd_dev_bsp_musbmhdrc.c ($Rev: 21593 $)
*
* @brief     USB Device Driver (MUSBMHDRC) BSP implementation.
*
* @details
*            This is a source file which contains the register and DMA channel definitions
*            for Mentor Graphics MUSBMHDRC Controller.
*/

#include "adi_usbd_dev_musbmhdrc_local.h"

#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC) || (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC)

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_1:"Assembly language shall be encapsulated and isolated.")
#pragma diag(suppress:misra_rule_19_1:"")
#pragma diag(suppress:misra_rule_8_1:"")
#pragma diag(suppress:misra_rule_19_4:"C macros shall only expand to a braced initialiser, a constant, a string literal, a parenthesised expression, a type qualifier, a storage class specifier, or a do-while-zero construct.")
#pragma diag(suppress:misra_rule_19_7:"")
#pragma diag(suppress:misra_rule_19_14:"")
#endif /* _MISRA_RULES */

#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC)

/*
 * Register address assignments
 */
ADI_USBD_DEV_REGS_DEF ADI_USBD_Registers[ADI_USBD_DEV_MAX_DEVICES] = {
    {
        (volatile uint8_t*)REG_USB0_FADDR,
        (volatile uint8_t*)REG_USB0_POWER,
        (volatile uint8_t*)REG_USB0_DEV_CTL,
        (volatile uint16_t*)REG_USB0_VBUS_CTL,
        (volatile uint16_t*)REG_USB0_PLL_OSC,
        (volatile uint8_t*)REG_USB0_PHY_CTL,

        (volatile uint16_t*)REG_USB0_INTRTX,
        (volatile uint16_t*)REG_USB0_INTRRX,
        (volatile uint16_t*)REG_USB0_INTRTXE,
        (volatile uint16_t*)REG_USB0_INTRRXE,
        (volatile uint8_t*)REG_USB0_IRQ,
        (volatile uint8_t*)REG_USB0_IEN,
        (volatile uint16_t*)REG_USB0_FRAME,
        (volatile uint8_t*)REG_USB0_INDEX,
        (volatile uint8_t*)REG_USB0_TXFIFOSZ,
        (volatile uint8_t*)REG_USB0_RXFIFOSZ,

        (volatile uint8_t*)REG_USB0_TESTMODE,

        (volatile uint8_t*)REG_USB0_EPI_TXTYPE0,
        (volatile uint16_t*)REG_USB0_EPI_TXMAXP0,
        (volatile uint8_t*)REG_USB0_EPI_TXINTERVAL0,
        (volatile uint16_t*)REG_USB0_EPI_TXCSR_P0,

        (volatile uint8_t*)REG_USB0_EPI_RXTYPE0,
        (volatile uint16_t*)REG_USB0_EPI_RXMAXP0,
        (volatile uint8_t*)REG_USB0_EPI_RXINTERVAL0,
        (volatile uint16_t*)REG_USB0_EPI_RXCSR_P0,
        (volatile uint16_t*)REG_USB0_EPI_RXCNT0,

        (volatile uint8_t*)REG_USB0_EP0I_TYPE0,
        (volatile uint16_t*)REG_USB0_EP0I_CSR0_P,
        (volatile uint16_t*)REG_USB0_EP0I_CNT0,
        (volatile uint8_t*)REG_USB0_EP0I_NAKLIMIT0,

        (volatile uint8_t*)REG_USB0_EP0I_CFGDATA0,

        (volatile uint8_t*)REG_USB0_DMA_IRQ,
        {
          (volatile uint16_t*)REG_USB0_DMA0_CTL,
          (volatile uint16_t*)REG_USB0_DMA1_CTL,
          (volatile uint16_t*)REG_USB0_DMA2_CTL,
          (volatile uint16_t*)REG_USB0_DMA3_CTL,
          (volatile uint16_t*)REG_USB0_DMA4_CTL,
          (volatile uint16_t*)REG_USB0_DMA5_CTL,
          (volatile uint16_t*)REG_USB0_DMA6_CTL,
          (volatile uint16_t*)REG_USB0_DMA7_CTL
        },
        {
          (volatile uint32_t*)REG_USB0_DMA0_ADDR,
          (volatile uint32_t*)REG_USB0_DMA1_ADDR,
          (volatile uint32_t*)REG_USB0_DMA2_ADDR,
          (volatile uint32_t*)REG_USB0_DMA3_ADDR,
          (volatile uint32_t*)REG_USB0_DMA4_ADDR,
          (volatile uint32_t*)REG_USB0_DMA5_ADDR,
          (volatile uint32_t*)REG_USB0_DMA6_ADDR,
          (volatile uint32_t*)REG_USB0_DMA7_ADDR
        },
        {
          (volatile uint32_t*)REG_USB0_DMA0_CNT,
          (volatile uint32_t*)REG_USB0_DMA1_CNT,
          (volatile uint32_t*)REG_USB0_DMA2_CNT,
          (volatile uint32_t*)REG_USB0_DMA3_CNT,
          (volatile uint32_t*)REG_USB0_DMA4_CNT,
          (volatile uint32_t*)REG_USB0_DMA5_CNT,
          (volatile uint32_t*)REG_USB0_DMA6_CNT,
          (volatile uint32_t*)REG_USB0_DMA7_CNT
        },
        /* FIFO Addresses */
        {
          (volatile uint32_t*)REG_USB0_FIFO0,
          (volatile uint32_t*)REG_USB0_FIFO1,
          (volatile uint32_t*)REG_USB0_FIFO2,
          (volatile uint32_t*)REG_USB0_FIFO3,
          (volatile uint32_t*)REG_USB0_FIFO4,
          (volatile uint32_t*)REG_USB0_FIFO5,
          (volatile uint32_t*)REG_USB0_FIFO6,
          (volatile uint32_t*)REG_USB0_FIFO7,
          (volatile uint32_t*)REG_USB0_FIFO8,
          (volatile uint32_t*)REG_USB0_FIFO9,
          (volatile uint32_t*)REG_USB0_FIFO10,
          (volatile uint32_t*)REG_USB0_FIFO11
        }
    }

};

/*
 * DMA Channel table
 */
#if (ADI_USBD_DEV_USE_DMA == ADI_USBD_DEV_YES)
ADI_USBD_DEV_DMA_DEF ADI_USBD_DMAChannels[ADI_USBD_DEV_MAX_DEVICES][ADI_USBD_DEV_MAX_DMA_CHANNELS] = {
    {
        { true, (uint8_t)BITM_USB_DMA_IRQ_D0 },
        { true, (uint8_t)BITM_USB_DMA_IRQ_D1 },
        { true, (uint8_t)BITM_USB_DMA_IRQ_D2 },
        { true, (uint8_t)BITM_USB_DMA_IRQ_D3 },
        { true, (uint8_t)BITM_USB_DMA_IRQ_D4 },
        { true, (uint8_t)BITM_USB_DMA_IRQ_D5 },
        { true, (uint8_t)BITM_USB_DMA_IRQ_D6 },
        { true, (uint8_t)BITM_USB_DMA_IRQ_D7 }
    }
};
#endif

/* FIXIT: replace macro with AduCM350 macro */
#elif defined(__ADSPARM_FAMILY__)

/*
 * Register address assignments
 */
#if defined(ADI_ADSP_CM40Z)
ADI_USBD_DEV_REGS_DEF ADI_USBD_Registers[ADI_USBD_DEV_MAX_DEVICES] = {
    {
        (volatile uint8_t*)REG_USB0_FADDR,
        (volatile uint8_t*)REG_USB0_POWER,
        (volatile uint8_t*)REG_USB0_DEV_CTL,
        (volatile uint16_t*)REG_USB0_VBUS_CTL,
        (volatile uint16_t*)REG_USB0_PLL_OSC,
        (volatile uint8_t*)REG_USB0_PHY_CTL,

        (volatile uint16_t*)REG_USB0_INTRTX,
        (volatile uint16_t*)REG_USB0_INTRRX,
        (volatile uint16_t*)REG_USB0_INTRTXE,
        (volatile uint16_t*)REG_USB0_INTRRXE,
        (volatile uint8_t*)REG_USB0_IRQ,
        (volatile uint8_t*)REG_USB0_IEN,
        (volatile uint16_t*)REG_USB0_FRAME,
        (volatile uint8_t*)REG_USB0_INDEX,
        (volatile uint8_t*)REG_USB0_TXFIFOSZ,
        (volatile uint8_t*)REG_USB0_RXFIFOSZ,

        (volatile uint8_t*)REG_USB0_TESTMODE,

        (volatile uint8_t*)REG_USB0_EPI_TXTYPE0,
        (volatile uint16_t*)REG_USB0_EPI_TXMAXP0,
        (volatile uint8_t*)REG_USB0_EPI_TXINTERVAL0,
        (volatile uint16_t*)REG_USB0_EPI_TXCSR_P0,

        (volatile uint8_t*)REG_USB0_EPI_RXTYPE0,
        (volatile uint16_t*)REG_USB0_EPI_RXMAXP0,
        (volatile uint8_t*)REG_USB0_EPI_RXINTERVAL0,
        (volatile uint16_t*)REG_USB0_EPI_RXCSR_P0,
        (volatile uint16_t*)REG_USB0_EPI_RXCNT0,

        (volatile uint8_t*)REG_USB0_EP0I_TYPE0,
        (volatile uint16_t*)REG_USB0_EP0I_CSR0_P,
        (volatile uint16_t*)REG_USB0_EP0I_CNT0,
        (volatile uint8_t*)REG_USB0_EP0I_NAKLIMIT0,

        (volatile uint8_t*)REG_USB0_EP0I_CFGDATA0,

        (volatile uint8_t*)REG_USB0_DMA_IRQ,
        {
          (volatile uint16_t*)REG_USB0_DMA_CTL0,
          (volatile uint16_t*)REG_USB0_DMA_CTL1,
          (volatile uint16_t*)REG_USB0_DMA_CTL2,
          (volatile uint16_t*)REG_USB0_DMA_CTL3,
          (volatile uint16_t*)REG_USB0_DMA_CTL4,
          (volatile uint16_t*)REG_USB0_DMA_CTL5,
          (volatile uint16_t*)REG_USB0_DMA_CTL6,          
        },
        {
          (volatile uint32_t*)REG_USB0_DMA_ADDR0,
          (volatile uint32_t*)REG_USB0_DMA_ADDR1,
          (volatile uint32_t*)REG_USB0_DMA_ADDR2,
          (volatile uint32_t*)REG_USB0_DMA_ADDR3,
          (volatile uint32_t*)REG_USB0_DMA_ADDR4,
          (volatile uint32_t*)REG_USB0_DMA_ADDR5,
          (volatile uint32_t*)REG_USB0_DMA_ADDR6,
        },
        {
          (volatile uint32_t*)REG_USB0_DMA_CNT0,
          (volatile uint32_t*)REG_USB0_DMA_CNT1,
          (volatile uint32_t*)REG_USB0_DMA_CNT2,
          (volatile uint32_t*)REG_USB0_DMA_CNT3,
          (volatile uint32_t*)REG_USB0_DMA_CNT4,
          (volatile uint32_t*)REG_USB0_DMA_CNT5,
          (volatile uint32_t*)REG_USB0_DMA_CNT6,          
        },
        /* FIFO Addresses */
        {
          (volatile uint32_t*)REG_USB0_FIFO0,
          (volatile uint32_t*)REG_USB0_FIFO1,
          (volatile uint32_t*)REG_USB0_FIFO2,
          (volatile uint32_t*)REG_USB0_FIFO3,          
        }
    }

};
#else
ADI_USBD_DEV_REGS_DEF ADI_USBD_Registers[ADI_USBD_DEV_MAX_DEVICES] = {
    {
        (volatile uint8_t*)REG_USB0_FADDR,
        (volatile uint8_t*)REG_USB0_POWER,
        (volatile uint8_t*)REG_USB0_DEV_CTL,
        (volatile uint16_t*)REG_USB0_VBUS_CTL,
        (volatile uint16_t*)REG_USB0_PLL_OSC,
        (volatile uint8_t*)REG_USB0_PHY_CTL,

        (volatile uint16_t*)REG_USB0_INTRTX,
        (volatile uint16_t*)REG_USB0_INTRRX,
        (volatile uint16_t*)REG_USB0_INTRTXE,
        (volatile uint16_t*)REG_USB0_INTRRXE,
        (volatile uint8_t*)REG_USB0_IRQ,
        (volatile uint8_t*)REG_USB0_IEN,
        (volatile uint16_t*)REG_USB0_FRAME,
        (volatile uint8_t*)REG_USB0_INDEX,
        (volatile uint8_t*)REG_USB0_TXFIFOSZ,
        (volatile uint8_t*)REG_USB0_RXFIFOSZ,

        (volatile uint8_t*)REG_USB0_TESTMODE,

        (volatile uint8_t*)REG_USB0_EPI_TXTYPE0,
        (volatile uint16_t*)REG_USB0_EPI_TXMAXP0,
        (volatile uint8_t*)REG_USB0_EPI_TXINTERVAL0,
        (volatile uint16_t*)REG_USB0_EPI_TXCSR_P0,

        (volatile uint8_t*)REG_USB0_EPI_RXTYPE0,
        (volatile uint16_t*)REG_USB0_EPI_RXMAXP0,
        (volatile uint8_t*)REG_USB0_EPI_RXINTERVAL0,
        (volatile uint16_t*)REG_USB0_EPI_RXCSR_P0,
        (volatile uint16_t*)REG_USB0_EPI_RXCNT0,

        (volatile uint8_t*)REG_USB0_EP0I_TYPE0,
        (volatile uint16_t*)REG_USB0_EP0I_CSR0_P,
        (volatile uint16_t*)REG_USB0_EP0I_CNT0,
        (volatile uint8_t*)REG_USB0_EP0I_NAKLIMIT0,

        (volatile uint8_t*)REG_USB0_EP0I_CFGDATA0,

        (volatile uint8_t*)REG_USB0_DMA_IRQ,
        {
          (volatile uint16_t*)REG_USB0_DMA_CTL0,
          (volatile uint16_t*)REG_USB0_DMA_CTL1,
        },
        {
          (volatile uint32_t*)REG_USB0_DMA_ADDR0,
          (volatile uint32_t*)REG_USB0_DMA_ADDR1,
        },
        {
          (volatile uint32_t*)REG_USB0_DMA_CNT0,
          (volatile uint32_t*)REG_USB0_DMA_CNT1,
        },
        /* FIFO Addresses */
        {
          (volatile uint32_t*)REG_USB0_FIFO0,
          (volatile uint32_t*)REG_USB0_FIFO1,
          (volatile uint32_t*)REG_USB0_FIFO2,
          (volatile uint32_t*)REG_USB0_FIFO3,
        }
    }

};

#endif
/*
 * DMA Channel table
 */
#if (ADI_USBD_DEV_USE_DMA == ADI_USBD_DEV_YES)
ADI_USBD_DEV_DMA_DEF ADI_USBD_DMAChannels[ADI_USBD_DEV_MAX_DEVICES][ADI_USBD_DEV_MAX_DMA_CHANNELS] = {
    {
        { true, BITM_USB_DMA_IRQ_D0 },
        { true, BITM_USB_DMA_IRQ_D1 }
    }
};
#endif

#endif


#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC)

#include <cycle_count_bf.h>

#elif (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC)

#define REG_CYCLE_COUNT 0xE0001004
#define pREG_CYCLE_COUNT  ((volatile CYCLE_CNT_TYPE *)(REG_CYCLE_COUNT))
#define __PROCESSOR_SPEED__ 16000000

#define _GET_CYCLE_COUNT(C) ((C) = *pREG_CYCLE_COUNT)
#endif

#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC)

static void WaitOneMillisec(void)
{
    CYCLE_CNT_TYPE CurCycleCount, EndCycleCount;
    _GET_CYCLE_COUNT(CurCycleCount);
    EndCycleCount= CurCycleCount + ((CYCLE_CNT_TYPE)__PROCESSOR_SPEED__/(CYCLE_CNT_TYPE)1000);

    while (CurCycleCount < EndCycleCount) {
        _GET_CYCLE_COUNT(CurCycleCount);
    }

}
#endif

/*!
 * @brief       The adi_usbd_dev_bsp_WaitMilliSec API delays the core for the given number of milliseconds.
 *
 * @param[in]   Millisec number of milliseconds to delay
 *
 * @return		void.
 */
void adi_usbd_dev_bsp_WaitMilliSec(uint32_t Millisec)
{
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC)
    /* 0x10000 loops per 'Millisec' provide enough of a delay for SET ADDRESS
     * to complete without resulting in NAKs. (debug build)
    */
    //int i = 0x10*Millisec;
    //int i = 0x100*Millisec;
    int i = 0x10000*Millisec;
    for (; i>0; i--);
#else
    while (Millisec != (uint32_t)0) {
           WaitOneMillisec();
           Millisec--;
    }
#endif
}

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

#endif /* ADI_USBD_DEV_BF_MUSBMHDRC or ADI_USBD_DEV_ARM_MUSBMHDRC */
