/*********************************************************************************

Copyright(c) 2012 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential. By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/**
* \file      adi_usbd_dev_musbmhdrc.c
* \brief     USB Device Driver (MUSBMHDRC) global API implementation.
* \version   $Rev: 21530 $
* \date      $Date: 2013-05-20 16:17:19 -0400 (Mon, 20 May 2013) $
*
* \details
*            This is a source file which contains the implementation of the global
*            API functions for the device driver for Mentor Graphics MUSBMHDRC
*            Controller.
*/

/** @addtogroup USBD_Driver_BF6xx USBD Driver
 *  @{

The MUSBMHDRC USB Controller device mode driver provides support, with the appropriate stack and class drivers, to
enable an embedded application to operate as a USB device. The controller supports both Full and High Speed endpoints
for Control, Interrupt, Bulk and Isochronous transfers.
 */

#define __ADI_USBD_DEV_API_MODULE__
#define _ADI_USBD_DEV_MODULE_

#include "adi_usbd_dev_musbmhdrc_local.h"

/*
 * disable misra diagnostics as necessary
 */
#ifdef _MISRA_RULES
#pragma diag(push)

#pragma diag(suppress:misra_rule_2_2:"")
#pragma diag(suppress:misra_rule_2_4:"")

#pragma diag(suppress:misra_rule_5_1:"")
#pragma diag(suppress:misra_rule_5_6:"")

#pragma diag(suppress:misra_rule_10_1_a:"The value of an expression of integer type shall not be implicitly converted to a different underlying type if it is not a conversion to a wider integer type of the same signedness.")
#pragma diag(suppress:misra_rule_10_1_b:"The value of an expression of integer type shall not be implicitly converted to a different underlying type if the expression is complex. ('unsigned char' and 'unsigned short'.")
#pragma diag(suppress:misra_rule_10_5:"If the bitwise operators ~ and << are applied to an operand of underlying type unsigned char or unsigned short, the result shall be immediately cast to the underlying type of the operand.")

#pragma diag(suppress:misra_rule_11_1:"")
#pragma diag(suppress:misra_rule_11_3:"")
#pragma diag(suppress:misra_rule_11_4:"")

#pragma diag(suppress:misra_rule_12_1:"")
#pragma diag(suppress:misra_rule_12_6:"")
#pragma diag(suppress:misra_rule_12_7:"Bitwise operators shall not be applied to operands whose underlying type is signed")
#pragma diag(suppress:misra_rule_12_8:"")

#pragma diag(suppress:misra_rule_13_2:"")
#pragma diag(suppress:misra_rule_13_7:"")

#pragma diag(suppress:misra_rule_14_3:"Before preprocessing, a null statement shall only occur on a line by itself; it may be followed by a comment provided that the first character following the null statement is a white-space character.")
#pragma diag(suppress:misra_rule_14_7:"A function shall have a single point of exit at the end of the function.")

#pragma diag(suppress:misra_rule_15_3:"")

#pragma diag(suppress:misra_rule_16_7:"")
#pragma diag(suppress:misra_rule_16_9:"A function identifier shall only be used with either a preceding &, or with a parenthesised parameter list.")
#pragma diag(suppress:misra_rule_16_10:"If a function returns error information, then that error information shall be tested.")

#pragma diag(suppress:misra_rule_17_4:"Array indexing shall be the only allowed form of pointer arithmetic.")

#pragma diag(suppress:misra_rule_19_5:"")
#pragma diag(suppress:misra_rule_19_7:"")
#pragma diag(suppress:misra_rule_19_11:"")
#pragma diag(suppress:misra_rule_19_14:"The defined preprocessor operator shall only be used in one of the two standard forms.")
#pragma diag(suppress:misra_rule_10_3:"")
#pragma diag(suppress:misra_rule_14_9:"")
#pragma diag(suppress:misra_rule_10_1_c:"")



#endif /* _MISRA_RULES */

#define USBD_NULL ((void*)0)
#define GET_EP_DIR(P) ( ((P)->basic.EpAddr) & 0x80 )

#if defined(__ADSPBLACKFIN__) && !defined(WA_16000030)
#warning "WA_16000030 is not defined"
#endif

/*
*********************************************************************************************************
*                                      USB DEVICE CONTROLLER DRIVER API FUNCTIONS
*********************************************************************************************************
*/

/* Connection Functions */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_Open(
        uint32_t                   DeviceNumber,
        void *const                pDeviceMemory,
        uint32_t                   MemorySize,
        uint8_t                    nEndpointsInUse,
        ADI_USBD_DEV_CB_FN         pfBusCallback,
        void                       *CallbackHandle,
        ADI_USBD_DEV_HANDLE        *phDevice
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_Close(
                                ADI_USBD_DEV_HANDLE        hDevice
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_SetCallbackHandle(
                                ADI_USBD_DEV_HANDLE hDevice,
                                void *CallbackHandle
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EnableUSB(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                bool                       bEnable
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_Connect(
                                ADI_USBD_DEV_HANDLE        hDevice
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_Disconnect(
                                ADI_USBD_DEV_HANDLE        hDevice
);

/* Control Functions */
ADI_USBD_DEV_RESULT ADI_USBD_DEV_SetAddress(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                uint8_t                    DevAddr
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_SetSpeed(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                ADI_USBD_DEV_SPEED         Speed
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_GetSpeed(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                ADI_USBD_DEV_SPEED         *pSpeed
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_GetFrameNumber(
        ADI_USBD_DEV_HANDLE        hDevice,
        uint16_t                   *pnFrameNumber
);

/* Endpoint Functions */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_BindEndpoint(
        ADI_USBD_DEV_HANDLE             hDevice,
        uint8_t                         EpAddr,
        uint8_t                         EpType,
        uint16_t                        nMaxPktSize,
        uint8_t                         nHighCapacityMultiplier,
        ADI_USBD_DEV_EP_DEF ** const    pEndpoint
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_ReleaseEndpoint(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                uint8_t     EpAddr
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpRegisterCallback(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                uint8_t     EpAddr,
                                ADI_USBD_DEV_CB_FN         pfCallback,
                                void                       *pCBParam
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpSubmitRxBuffer(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                uint8_t     EpAddr,
                                void                       *pBuffer,
                                uint32_t                   nBufSize
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpGetRxBuffer(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                uint8_t     EpAddr,
                                void ** const              ppBuffer,
                                uint32_t                   *pnBytes
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_IsRxBufferAvailable(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                uint8_t     EpAddr,
                                bool * const               pbAvailable
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpRead(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                uint8_t     EpAddr,
                                void                       *pBuffer,
                                uint32_t                   nBufSize
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpReadZLP(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                uint8_t     EpAddr
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpSubmitTxBuffer(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                uint8_t     EpAddr,
                                void                       *pBuffer,
                                uint32_t                   nBufSize
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpGetTxBuffer(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                uint8_t     EpAddr,
                                void ** const              ppBuffer,
                                uint32_t                   *pnBytes
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_IsTxBufferAvailable(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                uint8_t     EpAddr,
                                bool * const               pbAvailable
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpWrite(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                uint8_t     EpAddr,
                                void                       *pBuffer,
                                uint32_t                   nBufSize
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpWriteZLP(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                uint8_t     EpAddr
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpSetStall(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                uint8_t     EpAddr
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpClearStall(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                uint8_t     EpAddr
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpAbortTransfer(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                uint8_t     EpAddr
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpGetMaxRxTransferSize(
                                     ADI_USBD_DEV_HANDLE        hDevice,
                                     uint8_t                    EpAddr,
                                     uint8_t                    *pData,
                                     uint32_t                   *pMaxTransferSize
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpGetMaxTxTransferSize(
                                     ADI_USBD_DEV_HANDLE        hDevice,
                                     uint8_t                    EpAddr,
                                     uint8_t                    *pData,
                                     uint32_t                   *pMaxTransferSize
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_SetConfig(
                                                      ADI_USBD_DEV_HANDLE    hDevice,
                                                      uint8_t                            ConfigID
);

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_ClearConfig(
                                                     ADI_USBD_DEV_HANDLE     hDevice,
                                                     uint8_t                             ConfigID
);
/*
 * API functions are assigned to a global structure
 */
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC) || (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC)
ADI_USBD_DEV_API ADI_USBD_DEV_BF_MUSBMHDRC_API = {
#elif (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC)
ADI_USBD_DEV_API ADI_USBD_DEV_ARM_MUSBMHDRC_API = {
#endif
        ADI_USBD_DEV_Open,
        ADI_USBD_DEV_Close,
        ADI_USBD_DEV_EnableUSB,
        ADI_USBD_DEV_Connect,
        ADI_USBD_DEV_Disconnect,
        ADI_USBD_DEV_SetAddress,
        ADI_USBD_DEV_SetSpeed,
        ADI_USBD_DEV_GetSpeed,
        ADI_USBD_DEV_GetFrameNumber,
        ADI_USBD_DEV_BindEndpoint,
        ADI_USBD_DEV_ReleaseEndpoint,
        ADI_USBD_DEV_EpRegisterCallback,
        ADI_USBD_DEV_EpSubmitRxBuffer,
        ADI_USBD_DEV_EpGetRxBuffer,
        ADI_USBD_DEV_IsRxBufferAvailable,
        ADI_USBD_DEV_EpRead,
        ADI_USBD_DEV_EpReadZLP,
        ADI_USBD_DEV_EpSubmitTxBuffer,
        ADI_USBD_DEV_EpGetTxBuffer,
        ADI_USBD_DEV_IsTxBufferAvailable,
        ADI_USBD_DEV_EpWrite,
        ADI_USBD_DEV_EpWriteZLP,
        ADI_USBD_DEV_EpSetStall,
        ADI_USBD_DEV_EpClearStall,
        ADI_USBD_DEV_EpAbortTransfer,
        ADI_USBD_DEV_EpGetMaxRxTransferSize,
        ADI_USBD_DEV_EpGetMaxTxTransferSize,
        ADI_USBD_DEV_SetCallbackHandle,
        ADI_USBD_DEV_SetConfig,
        ADI_USBD_DEV_ClearConfig
};


#if defined(ADI_DEBUG)
static bool CheckMemorySize(
        uint32_t                   MemorySize,
        uint8_t                    nEndpointsInUse
);
#endif

/*
*********************************************************************************************************
*                                             Static functions
*********************************************************************************************************
*/
static void ConfigureEndPoint(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef);
#if (ADI_USBD_DEV_DYN_FIFO == ADI_USBD_DEV_YES)
static void SetDyamicRxFifoSize(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, uint16_t Size, bool bDoubleBuffer);
static void SetDyamicTxFifoSize(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, uint16_t Size, bool bDoubleBuffer);
static bool SetDynamicFifoAddress(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
                                    ADI_USBD_DEV_EP_DEF_EX  *pEpDef,
                                    bool bDoubleBuffer);
#endif
static ADI_USBD_DEV_RESULT SubmitBuffer(
                ADI_USBD_DEV_MUSBMHDRC_DEF  *pDevice,
                ADI_USBD_DEV_EP_DEF_EX      *pEpDef,
                void                        *pData,
                uint32_t                    DataLen
);
static uint32_t SetBytesThisPass(ADI_USBD_DEV_EP_DEF_EX *pEpDef, bool bMultiPacket);
static ADI_USBD_DEV_EP_TYPE SetEpType(uint8_t EpType);

static void CtrlRxStatusPhase(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef);
static void CtrlTxStatusPhase(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef);
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpGetMaxTransferSize(
                                     ADI_USBD_DEV_HANDLE        hDevice,
                                     uint8_t                    EpAddr,
                                     uint8_t                    *pData,
                                     ADI_USBD_DEV_EP_DIR        Direction,
                                     uint32_t                   *pMaxTransferSize
);

#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)
static void set_dma_ctrl(uint32_t ep_num, uint16_t data);
static void set_dma_addr(uint32_t ep_num,uint16_t addr_low, uint16_t addr_high);
static void set_dma_count(uint32_t ep_num,uint16_t count_low, uint16_t count_high);
#endif

/*
*********************************************************************************************************
*                                        GLOBAL API FUNCTIONS
*********************************************************************************************************
*/

/*!
 * \brief       Opens the specified USB Controller device & initiates the data structures;
 *              Passes back the device handle upon successful completion.
 *
 * \param[in]   DeviceNumber    The USB controller instance number to be opened.
 * \param[in]   pDeviceMemory   The pointer to the device instance memory passed by application.
 * \param[in]   MemorySize      The memory size passed by application.
 * \param[in]   nEndpointsInUse The maximum number of endpoints to be used
 * \param[in]   pfBusCallback   Callback function for bus events
 * \param[in]   CallbackHandle  Handle to pass back as first argument in bus callback
 * \param[out]  phDevice        The pointer to a location where the handle to the opened device is written.
 *
 * \return
 *              - #ADI_USBD_DEV_RESULT_SUCCESS      Device successfully opened
 *              - #ADI_USBD_DEV_RESULT_NO_MEMORY    Insufficient memory for device instance
 *
 * \note        The amount, MemorySize, of memory supplied  at pDeviceMemory must be sufficient for the
 *              the maximum number of endpoints that may be used, regardless of whether they are all
 *              in use at the same time:
 *              MemorySize &;ge  #ADI_USBD_DEV_DEF_DATA_SIZE
 *                               + sizeof(#ADI_USBD_DEV_EP_TABLE_DEF)
 *                               + nEndpointsInUse * sizeof(#ADI_USBD_DEV_EP_DEF *)
 *                               + nEndpointsInUse * #ADI_USBD_DEV_EP_DATA_SIZE
 *                               + nEndpointsInUse * (sizeof(#ADI_USBD_DEV_EP_BUFFER_LIST)
 *                                                    + #ADI_USBD_DEV_MAX_BUFS*sizeof(#ADI_USBD_DEV_EP_BUFFER*)
 *                                                    + #ADI_USBD_DEV_MAX_BUFS*sizeof(#ADI_USBD_DEV_EP_BUFFER)
 *                                                   )
 *
 * \sa          ADI_USBD_DEV_Close().
 */

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_Open(
                uint32_t                   DeviceNumber,
                void *const                pDeviceMemory,
                uint32_t                   MemorySize,
                uint8_t                    nEndpointsInUse,
                ADI_USBD_DEV_CB_FN         pfBusCallback,
                void                       *CallbackHandle,
                ADI_USBD_DEV_HANDLE        *phDevice
)
{
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice;
    uint8_t *pDataPos = pDeviceMemory;
    uint8_t *pEpDataStart;
    uint8_t EpNum;

    ADI_USBD_DEV_EP_DEF_EX *pEndPointArray;

#if defined(ADI_DEBUG)
    if (!CheckMemorySize(MemorySize,nEndpointsInUse))
    {
        return ADI_USBD_DEV_RESULT_NO_MEMORY;
    }
#endif

    /* Assign Device definition pointer and handle to return */
    pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)pDataPos;
    *phDevice = (ADI_USBD_DEV_HANDLE)pDevice;
    pDataPos +=  sizeof(ADI_USBD_DEV_MUSBMHDRC_DEF);

    pDevice->nEndpointsInUse = nEndpointsInUse;
    pDevice->pfBusCallback = pfBusCallback;
    pDevice->CallbackHandle = CallbackHandle;

#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC) || (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC)
#if (ADI_USBD_DEV_USE_DMA == ADI_USBD_DEV_YES)
    pDevice->pDMAChannels = &ADI_USBD_DMAChannels[pDevice->DeviceNumber][0];
#endif
#endif

#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC)
    /* first 64 bytes are reserved for EP0 */
    pDevice->FifoRamAdress    = USB_DYNFIFO_START_ADDR; /* 64 */
    pDevice->FifoRamTotalSize = USB_DYNFIFO_TOTAL_SIZE; /* 16K */
#endif

    /*
     * Initialize Endpoint lookup Table
     */
    adi_usbd_dev_EpCreateTable( nEndpointsInUse, pDataPos, &pDevice->pEpTable, sizeof(ADI_USBD_DEV_EP_DEF_EX));
    pEpDataStart = pDataPos + sizeof(ADI_USBD_DEV_EP_TABLE_DEF) + nEndpointsInUse*sizeof(ADI_USBD_DEV_EP_DEF*);
    pEndPointArray = (ADI_USBD_DEV_EP_DEF_EX *)pEpDataStart;
    pDataPos = pEpDataStart + (nEndpointsInUse * sizeof(ADI_USBD_DEV_EP_DEF_EX));

    /*
     * Initialize Endpoint Buffer lists
     */
    for (EpNum=0; EpNum<nEndpointsInUse; EpNum++)
    {
        adi_usbd_dev_EpCreateBufferList( ADI_USBD_DEV_MAX_BUFS, pDataPos, &(pEndPointArray[EpNum].pBufferList));
        pDataPos += sizeof(ADI_USBD_DEV_EP_BUFFER_LIST) + ADI_USBD_DEV_MAX_BUFS*(sizeof(ADI_USBD_DEV_EP_BUFFER*)+ sizeof(ADI_USBD_DEV_EP_BUFFER));
    }

    /* Assign Device Number */
    pDevice->DeviceNumber = DeviceNumber;

    /* assign register mappings */
    pDevice->pRegs= &ADI_USBD_Registers[pDevice->DeviceNumber];

    /* Set Device State as OPEN */
    pDevice->DevState = ADI_USBD_DEV_STATE_OPEN;

#if defined(__ADSPARM_FAMILY__) && !defined(RELOCATE_IVT)
    _adi_usbd_dev_os_init((ADI_USBD_DEV_HANDLE)pDevice);
#endif

    /* In case USB application is already flashed and you are trying to load it from debugger
     * as well, as soon as reset is asserted the one in the flash starts running leaving USB
     * interrupts active. We not only reset the interrupts at the USB level but also the latched
     * interrupts at the NVIC level. Below code acknowledges the pending USB interrupts at NVIC.
     */
#if defined(ADI_ADSP_CM40Z)
      if ( NVIC_GetPendingIRQ((IRQn_Type)ADI_INT_USB0_STAT ) )
      {                              
          NVIC_ClearPendingIRQ((IRQn_Type)ADI_INT_USB0_STAT); 
      }
      if ( NVIC_GetPendingIRQ((IRQn_Type)ADI_INT_USB0_DATA) ) 
      {                               
          NVIC_ClearPendingIRQ((IRQn_Type)ADI_INT_USB0_DATA);   
      }
#endif

    return ADI_USBD_DEV_RESULT_SUCCESS;
}

/*!
 * \brief       The ADI_USBD_DEV_Close API closes a specified USB Controller device.
 *
 * \param[in]   hDevice         Handle to device instance.
 *
 * \result      Status:
 *              - - #ADI_USBD_DEV_RESULT_SUCCESS    successful completion
 *
 * Closes the specified USB Controller device.
 *
 * \sa          ADI_USBD_DEV_Open().
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_Close(
                                ADI_USBD_DEV_HANDLE        hDevice
)
{
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;

    /* If device already enabled then disable */
    if (pDevice->DevState == ADI_USBD_DEV_STATE_ENABLED)
    {
        ADI_USBD_DEV_EnableUSB(hDevice, false);
    }

    /* Set Device State as IDLE */
    pDevice->DevState = ADI_USBD_DEV_STATE_IDLE;

    return (ADI_USBD_DEV_RESULT)ADI_USBD_DEV_RESULT_SUCCESS;
}

/*!
 * \brief       The ADI_USBD_DEV_SetCallbackHandle API Assigns the client specific handle
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[in]   CallbackHandle  Required Client Handle to be assigned
 *
 * \result      Status:
 *              - - #ADI_USBD_DEV_RESULT_SUCCESS    Controller enabled OK
 *
 * \note        This handle is first assigned in ADI_USBD_DEV_Open. However, in some
 *              circumstances this may require changing without closing the driver.
 *
 * \sa          ADI_USBD_DEV_Open().
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_SetCallbackHandle(
                                ADI_USBD_DEV_HANDLE hDevice,
                                void *CallbackHandle
)
{
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;

    pDevice->CallbackHandle = CallbackHandle;

    return (ADI_USBD_DEV_RESULT)ADI_USBD_DEV_RESULT_SUCCESS;
}

/*!
 * \brief       The ADI_USBD_DEV_EnableUSB API enables the USB controller for use
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[in]   bEnable         Boolean flag: true to enable controller; false to disable
 *
 * \result      Status:
 *              - - #ADI_USBD_DEV_RESULT_SUCCESS    Controller enabled OK
 *              - - #ADI_USBD_DEV_RESULT_DEVICE_NOT_OPEN Device not open
 *
 * \note        Configures/resets the controller, hook/unhooks the interrupt handlers.
 *
 * \sa          ADI_USBD_DEV_Open().
 */

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EnableUSB(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                bool                       bEnable
)
{
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;

    if (bEnable) {

#if defined(ADI_DEBUG)
        if (pDevice->DevState != ADI_USBD_DEV_STATE_OPEN) {
            return ADI_USBD_DEV_RESULT_DEVICE_NOT_OPEN;
        }
#endif

#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)
    /* Configure PLL oscillator register */
    SET_DEVICE_REG_VALUE(pDevice, pPLL_OSC,
            (0xF000 | BITM_USB_PLL_OSC_PLLMSEL | BITM_USB_PLL_OSC_PLLMSEL),
            (0x3000 | (0x14<<BITP_USB_PLL_OSC_PLLM) | BITM_USB_PLL_OSC_PLLMSEL)
            );
    ssync();

    /* set restore bit - puts out of hibernate */
    SET_DEVICE_REG_VALUE(pDevice, pPHY_CTL, BITM_USB_PHY_CTL_RESTORE, BITM_USB_PHY_CTL_RESTORE);

    /* set USB clock div in case we can get the current system clock value */
    adi_usbd_dev_bsp_SetClockDiv();

#elif (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC)
        /* setting bit 7 of PHY_CTL to enable the PHY */
        SET_DEVICE_REG_VALUE(pDevice, pPHY_CTL,  (1<<7),  (1<<7)  );
#endif

        /* NOTE
         * Therefore the only thing that seems to make a difference in Srinivas' set up is
         * setting bit 7 of PHY_CTL. This bit is undcoumented. This is also specific to the
         * ADSP_BF60x implemententation.
         */
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)
        /*
         * Route all interrupts through USB_INT0
         */
        *pUSB_GLOBINTR = 0x7;
        /*
         * Install interrupt handlers
         */
        adi_int_InstallHandler( ADI_SID_USB_INT0,   (ADI_INT_HANDLER_PTR)_adi_usbd_dev_USBinterruptHandler, pDevice, false );
        adi_int_InstallHandler( ADI_SID_USB_DMAINT, (ADI_INT_HANDLER_PTR)_adi_usbd_dev_DMAinterruptHandler, pDevice, false );

#elif (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC) || ((ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC) && defined(RELOCATE_IVT))
        adi_int_InstallHandler(INTR_USB0_STAT, (ADI_INT_HANDLER_PTR)_adi_usbd_dev_USBinterruptHandler, pDevice, true);
        adi_int_InstallHandler(INTR_USB0_DATA, (ADI_INT_HANDLER_PTR)_adi_usbd_dev_DMAinterruptHandler, pDevice, true);
#endif

#if (ADI_USBD_DEV_USE_DMA == ADI_USBD_DEV_YES)
        /* Initialize DMA channels */
        adi_usbd_dev_musbmhdrc_InitDMAChannels(pDevice);
#endif

        /* Enable Software connection */
        SET_DEVICE_REG_VALUE(pDevice, pPOWER,  BITM_USB_POWER_SOFTCONN,  ENUM_USB_POWER_SOFTCONN  );
        /* Enable Reset and other connection interrupts */
        SET_DEVICE_REG_VALUE(pDevice, pIEN,
                             (BITM_USB_IEN_RSTBABBLE|BITM_USB_IEN_DISCON|BITM_USB_IEN_SUSPEND),
                             (ENUM_USB_IEN_RSTBABBLEEN|ENUM_USB_IEN_DISCONEN|ENUM_USB_IEN_SUSPENDEN)
                             );


        /* Set Device State as ENABLED */
        pDevice->DevState = ADI_USBD_DEV_STATE_ENABLED;

    /* Enable USB interrupts */
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)
        /*
         * Enable USB and the data endpoint logic by setting appropriate bits in Global Control register
         * With the execution of below statement we are ready to receive and transmit setup packets on EP0
         */
        *pUSB_GLOBAL_CTL = (USB_GLOBAL_ENABLE_B | USB_GLOBAL_CTRL_ALL_TX_RX_B );

        adi_int_EnableInt(ADI_SID_USB_INT0, true);
        adi_int_EnableInt(ADI_SID_USB_DMAINT, true);

#elif (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC) || ((ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC) && defined(RELOCATE_IVT))
        adi_int_EnableInt(INTR_USB0_STAT, true);
        adi_int_EnableInt(INTR_USB0_DATA, true);
#elif (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC) || ((ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC) && !defined(RELOCATE_IVT))
        _adi_usbd_dev_os_EnableInt(INTR_USB0_STAT, true);
        _adi_usbd_dev_os_EnableInt(INTR_USB0_DATA, true);
#endif

    }
    else {

        /*
         * Disable the USB
         */

#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)

        /*
         * Disable USB interrupts
         */
        adi_int_EnableInt(ADI_SID_USB_INT0, false);
        adi_int_EnableInt(ADI_SID_USB_DMAINT, false);

        /*
         * uninstall the handlers
         */
        adi_int_UninstallHandler(ADI_SID_USB_INT0);
        adi_int_UninstallHandler(ADI_SID_USB_DMAINT);

#elif (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC) || ((ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC) && defined(RELOCATE_IVT))
        /* Disable USB interrupts */
        adi_int_EnableInt(INTR_USB0_STAT, false);

        /*
         * uninstall the handlers
         */
        adi_int_UninstallHandler(INTR_USB0_STAT);
        adi_int_UninstallHandler(INTR_USB0_DATA);
#elif (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC) || ((ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC) && !defined(RELOCATE_IVT))
        _adi_usbd_dev_os_EnableInt(INTR_USB0_STAT, false);
#endif

        /* Revert Device State to OPEN */
        pDevice->DevState = ADI_USBD_DEV_STATE_OPEN;

        /* Disable Software connection */
        CLR_DEVICE_REG_VALUE(pDevice, pPOWER,  BITM_USB_POWER_SOFTCONN );

        /* Disable HS mode */
#if (ADI_USBD_DEV_DRVR != ADI_USBD_DEV_ARM_MUSBMHDRC)
        CLR_DEVICE_REG_VALUE(pDevice, pPOWER,  BITM_USB_POWER_HSEN  );
#endif
        /* Enable Reset and other connection interrupts */
        CLR_DEVICE_REG_VALUE(pDevice, pIEN,
                             (BITM_USB_IEN_RSTBABBLE|BITM_USB_IEN_DISCON|BITM_USB_IEN_SUSPEND));

    }
    return ADI_USBD_DEV_RESULT_SUCCESS;
}


/*!
 * \brief       The ADI_USBD_DEV_Connect API is called to connect the USB device.
 *
 * \param[in]   hDevice         Handle to device instance.
 *
 * \result      Status:
 *              - - #ADI_USBD_DEV_RESULT_SUCCESS    Success
 *
 * Not currently implemented
 *
 * \sa          ADI_USBD_DEV_Disconnect().
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_Connect(
                                ADI_USBD_DEV_HANDLE        hDevice
)
{
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;

    /* Enable Software connection */
    SET_DEVICE_REG_VALUE(pDevice, pPOWER,  BITM_USB_POWER_SOFTCONN,  ENUM_USB_POWER_SOFTCONN  );

    return ADI_USBD_DEV_RESULT_SUCCESS;
}


/*!
 * \brief       The ADI_USBD_DEV_Disconnect API is called to disconnect the USB device.
 *
 * \param[in]   hDevice         Handle to device instance.
 *
 * \result      Status:
 *              - - #ADI_USBD_DEV_RESULT_SUCCESS    Success
 *
 * Not currently implemented
 *
 * \sa          ADI_USBD_DEV_Connect().
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_Disconnect(
                                ADI_USBD_DEV_HANDLE        hDevice
)
{
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;

    /* Disable Software connection */
    CLR_DEVICE_REG_VALUE(pDevice, pPOWER,  BITM_USB_POWER_SOFTCONN );

    return ADI_USBD_DEV_RESULT_SUCCESS;
}


/*!
 * \brief       The ADI_USBD_DEV_SetAddress sets the device address
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[in]   DevAddr         Address value.
 *
 * \result      Status:
 *              - - #ADI_USBD_DEV_RESULT_SUCCESS    Success
 *
 */

ADI_USBD_DEV_RESULT ADI_USBD_DEV_SetAddress(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                uint8_t                    DevAddr
)
{
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;

    ADI_USBD_ENTER_CRITICAL();

    /* Set address recovery interval, USB 2.0 specification 9.2.6.3
     * I'm not sure this is needed, as the host will hold off for 2ms
     * */
     adi_usbd_dev_bsp_WaitMilliSec(2);
     *pDEVICE_REG(pDevice,pFADDR) = (DevAddr&0x7F);

    ADI_USBD_EXIT_CRITICAL();

    return ADI_USBD_DEV_RESULT_SUCCESS;
}


/*!
 * \brief       The ADI_USBD_DEV_SetSpeed API sets the device speed.
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[in]   Speed           Device speed
 *                              #ADI_USBD_DEV_SPEED_FS  High speed (USB 1.1)
 *                              #ADI_USBD_DEV_SPEED_HS  High speed (USB 2.0)
 * \result      Status:
 *              - - #ADI_USBD_DEV_RESULT_SUCCESS    Success
 *
 * \sa          ADI_USBD_DEV_GetSpeed()
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_SetSpeed(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                ADI_USBD_DEV_SPEED         Speed
)
{
#if (ADI_USBD_DEV_DRVR != ADI_USBD_DEV_ARM_MUSBMHDRC)
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;
#endif
    ADI_USBD_ENTER_CRITICAL();
#if (ADI_USBD_DEV_HS_ENABLE == 1)
    if (Speed == ADI_USBD_DEV_SPEED_HS) {
        SET_DEVICE_REG_VALUE(pDevice, pPOWER,  BITM_USB_POWER_HSEN,  ENUM_USB_POWER_HSEN  );
    }
    else
    {
        CLR_DEVICE_REG_VALUE(pDevice, pPOWER, BITM_USB_POWER_HSEN);
    }
#else
#if (ADI_USBD_DEV_DRVR != ADI_USBD_DEV_ARM_MUSBMHDRC)
    CLR_DEVICE_REG_VALUE(pDevice, pPOWER, BITM_USB_POWER_HSEN);
#endif
#endif
    ADI_USBD_EXIT_CRITICAL();

    return ADI_USBD_DEV_RESULT_SUCCESS;
}

/*!
 * \brief       The ADI_USBD_DEV_GetSpeed API returns the device speed.
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[out]  pSpeed          Location to write device speed
 *                              #ADI_USBD_DEV_SPEED_FS  High speed (USB 1.1)
 *                              #ADI_USBD_DEV_SPEED_HS  High speed (USB 2.0)
 * \result      Status:
 *              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 *
 * \sa          ADI_USBD_DEV_SetSpeed()
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_GetSpeed(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                ADI_USBD_DEV_SPEED         *pSpeed
)
{
#if (ADI_USBD_DEV_HS_ENABLE == 1)
    uint8_t HSMode;
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;
    ADI_USBD_ENTER_CRITICAL();
    HSMode = GET_DEVICE_REG_VALUE(pDevice,pPOWER,BITM_USB_POWER_HSMODE);
    ADI_USBD_EXIT_CRITICAL();
    if (HSMode) {
        *pSpeed = ADI_USBD_DEV_SPEED_HS;
    }
    else {
        *pSpeed = ADI_USBD_DEV_SPEED_FS;
    }
#else
    *pSpeed = ADI_USBD_DEV_SPEED_FS;
#endif

    return ADI_USBD_DEV_RESULT_SUCCESS;
}

/*!
 * \brief       The ADI_USBD_DEV_GetFrameNumber API returns the current frame number.
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[out]  pnFrameNumber   Location to write frame number.
 *
 * \result      Status:
 *              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_GetFrameNumber(
        ADI_USBD_DEV_HANDLE        hDevice,
        uint16_t                   *pnFrameNumber
)
{
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;
    ADI_USBD_ENTER_CRITICAL();
    *pnFrameNumber = GET_DEVICE_REG_VALUE(pDevice,pFRAME,BITM_USB_FRAME_VALUE);
    ADI_USBD_EXIT_CRITICAL();

    return ADI_USBD_DEV_RESULT_SUCCESS;
}


/*!
 * \brief       The ADI_USBD_DEV_BindEndpoint API binds a physical endpoint to the logical endpoint.
 *
 * \param[in]   hDevice                     Handle to device instance.
 * \param[in]   EpAddr                      Logical Endpoint Address
 * \param[in]   EpType                      Endpoint type:
 *                                          - #ADI_USBD_DEV_EP_TYPE_CTRL        Control
 *                                          - #ADI_USBD_DEV_EP_TYPE_ISOC        Isochronous
 *                                          - #ADI_USBD_DEV_EP_TYPE_BULK        Bulk
 *                                          - #ADI_USBD_DEV_EP_TYPE_INTR        Interrupt
 *
 * \param[in]   nMaxPktSize                 Maximum Packet Size
 * \param[in]   nHighCapacityMultiplier     High Capacity Multiplier (Isochronous/Interrupt)
 * \param[out]  pEndpoint                   Location to store pointer to Endpoint definition structure
 *
 * \return      Status:
 *              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 *              - #ADI_USBD_DEV_RESULT_INVALID_EP   Invalid Endpoint address
 *
 * \sa          ADI_USBD_DEV_ReleaseEndpoint().
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_BindEndpoint(
        ADI_USBD_DEV_HANDLE             hDevice,
        uint8_t                         EpAddr,
        uint8_t                         EpType,
        uint16_t                        nMaxPktSize,
        uint8_t                         nHighCapacityMultiplier,
        ADI_USBD_DEV_EP_DEF ** const    pEndpoint
)
{
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;
    ADI_USBD_DEV_EP_DEF_EX     *pEpDef;
    ADI_USBD_DEV_RESULT        Result = ADI_USBD_DEV_RESULT_INVALID_EP;
    uint8_t                    LogEpNum;

    ADI_USBD_ENTER_CRITICAL();

    if ( (EpAddr&0x7F) == 0 )
    {
        pEpDef = (ADI_USBD_DEV_EP_DEF_EX *)adi_usbd_dev_EpGetEntry(pDevice->pEpTable,(EpAddr&0x7F));
        /* if EP 0 already registered then return with previous pointer */
        if (pEpDef) {
            *pEndpoint = (ADI_USBD_DEV_EP_DEF *)pEpDef;
            pDevice->pEp0Def = pEpDef;
            ADI_USBD_EXIT_CRITICAL();
            return ADI_USBD_DEV_RESULT_SUCCESS;
        }

    }
    if ((EpAddr&0x7F) == 0 ) {
        LogEpNum = EpAddr&0x7F;
    } else {
        LogEpNum = EpAddr;
    }
    pEpDef = (ADI_USBD_DEV_EP_DEF_EX *)adi_usbd_dev_EpAddEntry(pDevice->pEpTable);

    if (pEpDef)
    {
        pEpDef->basic.EpAddr = LogEpNum;
        pEpDef->EpNum = EpAddr&0x7F;
        pEpDef->EpType = SetEpType(EpType);
        pEpDef->bConfigured = false;

        /* EP 0 is IN and OUT */
        if ( (EpAddr&0x7F) == 0 )
        {
            pEpDef->EpDir = ADI_USBD_DEV_EP_DIR_EP0;
        }
        else if ( (EpAddr&0x80)==0x80 )
        {
            /* IN direction = TX */
            pEpDef->EpDir = ADI_USBD_DEV_EP_DIR_TX;
        }
        else {
            /* OUT direction = RX */
            pEpDef->EpDir = ADI_USBD_DEV_EP_DIR_RX;
        }

        pEpDef->nMaxPktSize = nMaxPktSize;

        pEpDef->nHighCapacityMultiplier = nHighCapacityMultiplier;

        if ( (EpAddr&0x7F) != 0 ) {
            ConfigureEndPoint(pDevice,pEpDef);
            pEpDef->pfEpStateHandler = adi_usbd_dev_musbmhdrc_DataEpStateHandler;
        }
        else {
            pEpDef->pfEpStateHandler = adi_usbd_dev_musbmhdrc_Ep0StateHandler;
        }

        /* Initialize state to IDLE */
        pEpDef->EpState = ADI_USBD_EP_STATE_IDLE;

        /* set return value */
        *pEndpoint = (ADI_USBD_DEV_EP_DEF *)pEpDef;

        Result = ADI_USBD_DEV_RESULT_SUCCESS;
    }
    else {
        *pEndpoint = (ADI_USBD_DEV_EP_DEF *)0;
    }

    ADI_USBD_EXIT_CRITICAL();

    return Result;
}

/*!
 * \brief       The ADI_USBD_DEV_ReleaseEndpoint API releases physical endpoint bound to the logical endpoint.
 *
 * \param[in]   hDevice                     Handle to device instance.
 * \param[in]   EpAddr                      Logical Endpoint Address
 *
 * \return      Status:
 *              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 *              - #ADI_USBD_DEV_RESULT_INVALID_EP   Invalid Endpoint address
 *
 * \sa          ADI_USBD_DEV_BindEndpoint().
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_ReleaseEndpoint(
        ADI_USBD_DEV_HANDLE             hDevice,
        uint8_t                         EpAddr
)
{
    ADI_USBD_DEV_RESULT         Result;
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;
    ADI_USBD_DEV_EP_DEF_EX     *pEpDef;

    ADI_USBD_ENTER_CRITICAL();

    pEpDef = adi_usbd_dev_musbmhdrc_GetEpDef(pDevice, EpAddr);

    if (pEpDef) {
        adi_usbd_dev_EpRemoveEntry(pDevice->pEpTable,EpAddr);

        adi_usbd_dev_musbmhdrc_ResetEndpoint(pDevice,pEpDef);
        if ( pEpDef->EpNum == 0 )
        {
            pDevice->pEp0Def = USBD_NULL;
        }
        Result = ADI_USBD_DEV_RESULT_SUCCESS;
    }

    /* This is the second call to release EP 0 - so we just ignore and report success
    */
    else if ((EpAddr&0x7F) == 0 )
    {
        Result = ADI_USBD_DEV_RESULT_SUCCESS;
    }
    else {
        Result = ADI_USBD_DEV_RESULT_INVALID_EP;
    }

    ADI_USBD_EXIT_CRITICAL();

    return Result;
}


/*!
 * \brief       The ADI_USBD_DEV_EpRegisterCallback API registers an Endpoint callback .
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[in]   EpAddr          Logical Endpoint Address
 * \param[in]   pfCallback      Address of callback function
 * \param[in]   pCBParam        Parameter to passed as first argument to callback function
 *
 * \return      Status:
 *              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 *              - #ADI_USBD_DEV_RESULT_INVALID_EP   Invalid Endpoint address
 *
 * Registers a callback function and associated client parameter with an opened endpoint.
 *
 * \sa          ().
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpRegisterCallback(
                    ADI_USBD_DEV_HANDLE     hDevice,
                    uint8_t                 EpAddr,
                    ADI_USBD_DEV_CB_FN      pfCallback,
                    void                    *pCBParam
)
{
    ADI_USBD_DEV_RESULT         Result;
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;
    ADI_USBD_DEV_EP_DEF_EX     *pEpDef;

    ADI_USBD_ENTER_CRITICAL();

    pEpDef = adi_usbd_dev_musbmhdrc_GetEpDef(pDevice, EpAddr);
    if (pEpDef) {
        /* NOTE: Overwrites first EP 0 registration values */
        pEpDef->pfCallback = pfCallback;
        pEpDef->CallbackHandle = pCBParam;
        Result = ADI_USBD_DEV_RESULT_SUCCESS;
    }
    else {
        Result = ADI_USBD_DEV_RESULT_INVALID_EP;
    }

    ADI_USBD_EXIT_CRITICAL();

    return Result;
}


/*!
 * \brief       The ADI_USBD_DEV_EpSubmitRxBuffer API submits a data buffer to receive data from the host.
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[in]   EpAddr          Logical Endpoint Address
 * \param[in]   pBuffer         Address of data buffer
 * \param[in]   nBufSize        Number of bytes to transmit to host
 *
 * \return      Status:
 *              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 *              - #ADI_USBD_DEV_RESULT_INVALID_EP   Invalid Endpoint address
 *
 *              Non-blocking function to read data from host
 *
 * \sa          ADI_USBD_DEV_EpGetRxBuffer().
 * \sa          ADI_USBD_DEV_IsRxBufferAvailable().
 *
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpSubmitRxBuffer(
                    ADI_USBD_DEV_HANDLE        hDevice,
                    uint8_t                    EpAddr,
                    void                       *pBuffer,
                    uint32_t                   nBufSize
)
{
    ADI_USBD_DEV_MUSBMHDRC_DEF  *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;
    ADI_USBD_DEV_EP_DEF_EX      *pEpDef;
    ADI_USBD_DEV_RESULT         Result;

    ADI_USBD_ENTER_CRITICAL();

    pEpDef = adi_usbd_dev_musbmhdrc_GetEpDef(pDevice, EpAddr);

    if ( pEpDef && ((pEpDef->EpDir == ADI_USBD_DEV_EP_DIR_RX) || (pEpDef->EpDir == ADI_USBD_DEV_EP_DIR_EP0)) )
    {
        Result = SubmitBuffer(pDevice,pEpDef,pBuffer,nBufSize);
        /*
         * For EP 0, we disabled the EP0 interrupt on the setup packet to ensure buffer
         * was submitted before interrupt taken for next packet
         */
        if (pEpDef->EpNum == 0)
        {
            adi_usbd_dev_musbhdrc_EnableTxInterrupt(pDevice,pEpDef);
        }
    }
    else {
        Result = ADI_USBD_DEV_RESULT_INVALID_EP;
    }

    ADI_USBD_EXIT_CRITICAL();

    return Result;
}

/*!
 * \brief       The ADI_USBD_DEV_EpGetRxBuffer API gets the data buffer of the last RX transfer
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[in]   EpAddr          Logical Endpoint Address
 * \param[out]  ppBuffer        Location to store address of data buffer
 * \param[out]  pnBytes         Location to sore the number of bytes received
 *
 * \return      Status:
 *              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 *              - #ADI_USBD_DEV_RESULT_INVALID_EP   Invalid Endpoint address
 *
 * \sa          ADI_USBD_DEV_EpSubmitRxBuffer().
 * \sa          ADI_USBD_DEV_IsRxBufferAvailable().
 *
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpGetRxBuffer(
                ADI_USBD_DEV_HANDLE        hDevice,
                uint8_t                    EpAddr,
                void ** const              ppBuffer,
                uint32_t                   *pnBytes
)
{
    ADI_USBD_DEV_RESULT         Result = ADI_USBD_DEV_RESULT_INVALID_EP;
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;
    ADI_USBD_DEV_EP_DEF_EX     *pEpDef;

    pEpDef = adi_usbd_dev_musbmhdrc_GetEpDef(pDevice, EpAddr);
    if (pEpDef) {
        /*
         * FIXIT: we should strictly block on a semaphore, but this may not be necessary
         */
        *pnBytes = pEpDef->nBytesProcessed;
        *ppBuffer = pEpDef->pProcessedDataBuffer;

        Result = ADI_USBD_DEV_RESULT_SUCCESS;
    }

    return Result;
}


/*!
 * \brief       ADI_USBD_DEV_IsRxBufferAvailable ().
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[in]   EpAddr          Logical Endpoint Address
 * \param[out]  pbAvailable     Location to store boolean
 *
 * \return      Status:
 *              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 *              - #ADI_USBD_DEV_RESULT_INVALID_EP   Invalid Endpoint address
 *
 * Not currently supported
 *
 * \sa          ADI_USBD_DEV_EpSubmitRxBuffer().
 * \sa          ADI_USBD_DEV_EpGetRxBuffer
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_IsRxBufferAvailable(
                ADI_USBD_DEV_HANDLE     hDevice,
                uint8_t                 EpAddr,
                bool * const            pbAvailable
)
{
#if 1
    /* not supported (yet) */
    return ADI_USBD_DEV_RESULT_FAIL;
#else
    ADI_USBD_DEV_RESULT         Result = ADI_USBD_DEV_RESULT_INVALID_EP;
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;
    ADI_USBD_DEV_EP_DEF_EX     *pEpDef;

    pEpDef = adi_usbd_dev_musbmhdrc_GetEpDef(pDevice, EpAddr);
    if (pEpDef) {

        *pbAvailable = pEpDef->bProcessedFlag;

        Result = ADI_USBD_DEV_RESULT_SUCCESS;
    }

    return Result;
#endif
}


/*!
 * \brief       The ADI_USBD_DEV_EpRead API reads data from host.
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[in]   EpAddr          Logical Endpoint Address
 * \param[in]   pBuffer         Address of data buffer
 * \param[in]   nBufSize        Number of bytes to transmit to host
 *
 * \return      Status:
 *              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 *              - #ADI_USBD_DEV_RESULT_INVALID_EP   Invalid Endpoint address
 *
 *              Blocking function to read data from host.
 *              Not currently supported

 * \sa          ADI_USBD_DEV_EpSubmitRxBuffer().
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpRead(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                uint8_t                    EpAddr,
                                void                       *pBuffer,
                                uint32_t                   nBufSize
)
{
    return ADI_USBD_DEV_RESULT_FAIL;
}


/*!
 * \brief       The ADI_USBD_DEV_EpReadZLP API reads a zero length packet from the host.
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[in]   EpAddr          Logical Endpoint Address
 *
 * \return      Status:
 *              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 *              - #ADI_USBD_DEV_RESULT_INVALID_EP   Invalid Endpoint address
 *
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpReadZLP(
                                ADI_USBD_DEV_HANDLE     hDevice,
                                uint8_t                 EpAddr
)
{
    ADI_USBD_DEV_MUSBMHDRC_DEF  *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;
    ADI_USBD_DEV_EP_DEF_EX      *pEpDef;
    ADI_USBD_DEV_RESULT         Result;

    ADI_USBD_DBG_EP("MUSBMHDRC ADI_USBD_DEV_EpReadZLP",EpAddr);

    ADI_USBD_ENTER_CRITICAL();

    pEpDef = adi_usbd_dev_musbmhdrc_GetEpDef(pDevice, EpAddr);

    if (pEpDef) {
        /*
         * IF Control EP and then this is either the status phase, or the end of
         *    a Control RX transfer
         */
        if (pEpDef->EpNum == 0)
        {
            if (pEpDef->EpState == ADI_USBD_EP_STATE_TX)
            {
                /*
                 * status phase of Control TX transfer
                 */
                CtrlTxStatusPhase(pDevice, pEpDef);
            }
            /*
             * IF this is the end of a Control RX transaction then
             * state remains the same and we await next EP0 interrupt
             */
        }
        else {
            /*
             * We can simply consume the ZLP at the next RX interrupt, so here we
             * just move to status phase
             */
            pEpDef->EpState = ADI_USBD_EP_STATE_RX_STATUS;
            pEpDef->pfEpStateHandler(pDevice, pEpDef);
        }

        Result = ADI_USBD_DEV_RESULT_SUCCESS;
    }
    else {
        Result = ADI_USBD_DEV_RESULT_INVALID_EP;
    }

    ADI_USBD_EXIT_CRITICAL();

    return Result;
}


/*!
 * \brief       The ADI_USBD_DEV_EpSubmitTxBuffer API submits a data buffer to transmit data to the host.
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[in]   EpAddr          Logical Endpoint Address
 * \param[in]   pBuffer         Address of data buffer
 * \param[in]   nBufSize        Number of bytes to transmit to host
 *
 * \return      Status:
 *              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 *              - #ADI_USBD_DEV_RESULT_INVALID_EP   Invalid Endpoint address
 *
 *              Non-blocking function to write data to host
 *
 * \sa          ADI_USBD_DEV_EpGetTxBuffer().
 * \sa          ADI_USBD_DEV_IsTxBufferAvailable().
 *
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpSubmitTxBuffer(
                    ADI_USBD_DEV_HANDLE        hDevice,
                    uint8_t                    EpAddr,
                    void                       *pBuffer,
                    uint32_t                   nBufSize
)
{
    ADI_USBD_DEV_MUSBMHDRC_DEF  *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;
    ADI_USBD_DEV_EP_DEF_EX      *pEpDef;
    ADI_USBD_DEV_RESULT         Result = ADI_USBD_DEV_RESULT_INVALID_EP;

    ADI_USBD_ENTER_CRITICAL();

    pEpDef = adi_usbd_dev_musbmhdrc_GetEpDef(pDevice, EpAddr);

    /* If Zero Length Packet */
    if (pEpDef && ((pEpDef->EpDir == ADI_USBD_DEV_EP_DIR_TX) || (pEpDef->EpDir == ADI_USBD_DEV_EP_DIR_EP0)))
    {
        Result = SubmitBuffer(pDevice,pEpDef,pBuffer,nBufSize);
    }

    ADI_USBD_EXIT_CRITICAL();

    return Result;
}

/*!
 * \brief       The ADI_USBD_DEV_EpGetTxBuffer API gets the data buffer of the last TX transfer
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[in]   EpAddr          Logical Endpoint Address
 * \param[out]  ppBuffer        Location to store address of data buffer
 * \param[out]  pnBytes         Location to sore the number of bytes transmitted
 *
 * \return      Status:
 *              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 *              - #ADI_USBD_DEV_RESULT_INVALID_EP   Invalid Endpoint address
 *
 * Not currently supported
 *
 * \sa          ADI_USBD_DEV_EpSubmitTxBuffer().
 * \sa          ADI_USBD_DEV_IsTxBufferAvailable().
 *
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpGetTxBuffer(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                uint8_t                    EpAddr,
                                void ** const              ppBuffer,
                                uint32_t                   *pnBytes
)
{
#if 1
    /* not supported (yet) */
    return ADI_USBD_DEV_RESULT_FAIL;
#else
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;
    ADI_USBD_DEV_EP_DEF_EX     *pEpDef;

    /* Look up endpoint definition */
    pEpDef = adi_usbd_dev_musbmhdrc_GetEpDef(pDevice, EpAddr);
    if (!pEpDef) {
        return ADI_USBD_DEV_RESULT_INVALID_EP;
    }

    ADI_USBD_DBG_EP("MUSBMHDRC ADI_USBD_DEV_EpGetTxBuffer",pEpDef->EpNum);

    /* Block on completion */
    while (!pEpDef->bProcessedFlag);

    /* Assign data pointer */
    **ppBuffer = pEpDef->pCurrentBuffer->pData;

    /* Assign bytes transmitted */
    *pnBytes = pEpDef->pCurrentBuffer->nBytesProcessed;

    /* Free current buffer */
    adi_usbd_dev_EpFreeBuffer(pEpDef->pBufferList, pEpDef->pCurrentBuffer);

    return ADI_USBD_DEV_RESULT_SUCCESS;
#endif
}


/*!
 * \brief       ADI_USBD_DEV_IsTxBufferAvailable ().
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[in]   EpAddr          Logical Endpoint Address
 * \param[out]  pbAvailable     Location to store boolean
 *
 * \return      Status:
 *              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 *              - #ADI_USBD_DEV_RESULT_INVALID_EP   Invalid Endpoint address
 *
 * Not currently supported
 *
 * \sa          ADI_USBD_DEV_EpSubmitTxBuffer().
 * \sa          ADI_USBD_DEV_EpGetTxBuffer()
 */
static
ADI_USBD_DEV_RESULT ADI_USBD_DEV_IsTxBufferAvailable(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                uint8_t                    EpAddr,
                                bool * const               pbAvailable
)
{
#if 1
    /* not supported (yet) */
    return ADI_USBD_DEV_RESULT_FAIL;
#else
    ADI_USBD_DEV_RESULT         Result = ADI_USBD_DEV_RESULT_INVALID_EP;
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;
    ADI_USBD_DEV_EP_DEF_EX     *pEpDef;

    pEpDef = adi_usbd_dev_musbmhdrc_GetEpDef(pDevice, EpAddr);
    if (pEpDef) {

        *pbAvailable = pEpDef->bProcessedFlag;

        Result = ADI_USBD_DEV_RESULT_SUCCESS;
    }

    return Result;
#endif
}


/*!
 * \brief       The ADI_USBD_DEV_EpWrite API writes data from host.
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[in]   EpAddr          Logical Endpoint Address
 * \param[in]   pBuffer         Address of data buffer
 * \param[in]   nBufSize        Number of bytes to transmit to host
 *
 * \return      Status:
 *              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 *              - #ADI_USBD_DEV_RESULT_INVALID_EP   Invalid Endpoint address
 *
 *              Blocking function to read data from host.
 *              Not currently supported

 * \sa          ADI_USBD_DEV_EpSubmitTxBuffer().
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpWrite(
                                ADI_USBD_DEV_HANDLE     hDevice,
                                uint8_t                 EpAddr,
                                void                    *pBuffer,
                                uint32_t                nBufSize
)
{
#if 1
    /* not supported (yet) */
    return ADI_USBD_DEV_RESULT_FAIL;
#else
    ADI_USBD_DEV_RESULT         Result = ADI_USBD_DEV_RESULT_INVALID_EP;
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;
    ADI_USBD_DEV_EP_DEF_EX     *pEpDef;

    /* Look up endpoint definition */
    pEpDef = adi_usbd_dev_musbmhdrc_GetEpDef(pDevice, EpAddr);
    if (pEpDef)
    {
        ADI_USBD_DBG_EP("MUSBMHDRC ADI_USBD_DEV_EpWrite",pEpDef->EpNum);

        Result = ADI_USBD_DEV_EpSubmitTxBuffer(hDevice, EpAddr, pBuffer, nBufSize);
    }

    return Result;
#endif
}


/*!
 * \brief       The ADI_USBD_DEV_EpReadZLP API reads a zero length packet from the host.
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[in]   EpAddr          Logical Endpoint Address
 *
 * \return      Status:
 *              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 *              - #ADI_USBD_DEV_RESULT_INVALID_EP   Invalid Endpoint address
 *
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpWriteZLP(
            ADI_USBD_DEV_HANDLE     hDevice,
            uint8_t                 EpAddr
)
{
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;
    ADI_USBD_DEV_EP_DEF_EX     *pEpDef;
    ADI_USBD_DEV_RESULT Result = ADI_USBD_DEV_RESULT_INVALID_EP;

    ADI_USBD_ENTER_CRITICAL();

    pEpDef = adi_usbd_dev_musbmhdrc_GetEpDef(pDevice, EpAddr);

    if (pEpDef) {
        /*
         * FIXIT: will nBytesThisPass be set to zero already?
         */
        ADI_USBD_DBG_EP("MUSBMHDRC ADI_USBD_DEV_EpWriteZLP",pEpDef->basic.EpAddr);

        if ( pEpDef->EpNum == 0)
        {
            switch (pEpDef->EpState) {
                case ADI_USBD_EP_STATE_IDLE:
                    /*
                     * Status phase of Zero data phase
                     */
                case ADI_USBD_EP_STATE_RX:
                    /*
                     * status phase of Control RX transfer
                     */
                    CtrlRxStatusPhase(pDevice, pEpDef);
                    break;

                case ADI_USBD_EP_STATE_TX:
                    /*
                     * Last packet in Control TX data phase
                     */
                    SetBytesThisPass(pEpDef, false);
                    adi_usbd_dev_musbmhdrc_SetTxPktRdy(pDevice, pEpDef);
                    break;

                default:
                    break;
            }
        }
        else {
            SetBytesThisPass(pEpDef, false);
            adi_usbd_dev_musbmhdrc_SetTxPktRdy(pDevice, pEpDef);
        }
        Result = ADI_USBD_DEV_RESULT_SUCCESS;
    }

    ADI_USBD_EXIT_CRITICAL();

    return Result;
}


/*!
 * \brief       The ADI_USBD_DEV_EpSetStall API stalls the given endpoint.
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[in]   EpAddr          Logical Endpoint Address
 *
 * \return      Status:
 *              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 *              - #ADI_USBD_DEV_RESULT_INVALID_EP   Invalid Endpoint address
 *
 * \sa          ADI_USBD_DEV_EpClearStall().
 */

static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpSetStall(
                    ADI_USBD_DEV_HANDLE     hDevice,
                    uint8_t                 EpAddr
)
{
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;
    ADI_USBD_DEV_EP_DEF_EX     *pEpDef;
    ADI_USBD_DEV_RESULT Result;

    pEpDef = adi_usbd_dev_musbmhdrc_GetEpDef(pDevice, EpAddr);
    if (pEpDef)
    {
        ADI_USBD_DBG_EP("MUSBMHDRC ADI_USBD_DEV_EpSetStall",pEpDef->basic.EpAddr);

        if (pEpDef->EpNum == 0)
        {
            *pDEVICE_REG(pDevice,pINDEX) = 0;
            SET_DEVICE_REG_VALUE(pDevice, pEP0_CSR0, BITM_USB_EP0_CSR_P_SENDSTALL, ENUM_USB_EP0_CSR_P_STALL);
            Result = ADI_USBD_DEV_RESULT_SUCCESS;
        }
        else
        {
            ADI_USBD_ENTER_CRITICAL();
            /*
             * Set INDEX register */
            *pDEVICE_REG(pDevice,pINDEX) = pEpDef->EpNum;

            adi_usbd_dev_musbmhdrc_FlushEndpoint(pDevice,pEpDef);

            /* Set the send stall bit  */
            if (pEpDef->EpDir==ADI_USBD_DEV_EP_DIR_RX)
            {
                SET_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EP_RXCSR_P_SENDSTALL, ENUM_USB_EP_RXCSR_P_STALL);
            }
            else {
                SET_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EP_TXCSR_P_SENDSTALL, ENUM_USB_EP_TXCSR_P_STALL);
            }
            Result = ADI_USBD_DEV_RESULT_SUCCESS;
            ADI_USBD_EXIT_CRITICAL();
        }
    }
    else {
        Result = ADI_USBD_DEV_RESULT_INVALID_EP;
    }

    return Result;
}


/*!
 * \brief       The ADI_USBD_DEV_EpClearStall API clears the stall on the given endpoint.
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[in]   EpAddr          Logical Endpoint Address
 *
 * \return      Status:
 *              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 *              - #ADI_USBD_DEV_RESULT_INVALID_EP   Invalid Endpoint address
 *
 * \sa          ADI_USBD_DEV_EpSetStall().
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpClearStall(
                                ADI_USBD_DEV_HANDLE        hDevice,
                                uint8_t     EpAddr
)
{
    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;
    ADI_USBD_DEV_EP_DEF_EX     *pEpDef;
    ADI_USBD_DEV_RESULT Result;

    pEpDef = adi_usbd_dev_musbmhdrc_GetEpDef(pDevice, EpAddr);
    if (pEpDef)
    {
        ADI_USBD_DBG_EP("MUSBMHDRC ADI_USBD_DEV_EpClearStall",pEpDef->basic.EpAddr);

        if (pEpDef->EpNum == 0)
        {
            Result = ADI_USBD_DEV_RESULT_SUCCESS;
        }
        else
        {
            pEpDef = adi_usbd_dev_musbmhdrc_GetEpDef(pDevice, EpAddr);
            if (pEpDef)
            {
                ADI_USBD_ENTER_CRITICAL();
                /*
                 * Set INDEX register */
                *pDEVICE_REG(pDevice,pINDEX) = pEpDef->EpNum;

                /*
                 * Set Clear Data Toggle Bit & clear stall
                 */
                if (pEpDef->EpDir==ADI_USBD_DEV_EP_DIR_RX)
                {
                    SET_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_CLRDATATGL, ENUM_USB_EPI_RXCSR_P_CLRTGL);
                    CLR_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EP_RXCSR_P_SENDSTALL);
                }
                else {
                    SET_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_CLRDATATGL, ENUM_USB_EPI_TXCSR_P_CLRTGL);
                    CLR_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EP_TXCSR_P_SENDSTALL);
                }
                Result = ADI_USBD_DEV_RESULT_SUCCESS;
                ADI_USBD_EXIT_CRITICAL();
           }
        }
    }
    else {
        Result = ADI_USBD_DEV_RESULT_INVALID_EP;
    }

    return Result;
}


/*!
 * \brief       The ADI_USBD_DEV_EpAbortTransfer API aborts the current transfer on the given endpoint.
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[in]   EpAddr          Logical Endpoint Address
 *
 * \return      Status:
 *              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 *              - #ADI_USBD_DEV_RESULT_INVALID_EP   Invalid Endpoint address
 *
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpAbortTransfer(
                ADI_USBD_DEV_HANDLE     hDevice,
                uint8_t                 EpAddr
)
{
    ADI_USBD_DEV_MUSBMHDRC_DEF  *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;
    ADI_USBD_DEV_EP_DEF_EX      *pEpDef;
    ADI_USBD_DEV_RESULT         Result;

    pEpDef = adi_usbd_dev_musbmhdrc_GetEpDef(pDevice, EpAddr);
    if (pEpDef)
    {
        ADI_USBD_ENTER_CRITICAL();
        /* Free Buffer */
        adi_usbd_dev_EpFreeBuffer(pEpDef->pBufferList, pEpDef->pCurrentBuffer);
        pEpDef->pCurrentBuffer = 0;
        ADI_USBD_EXIT_CRITICAL();

        /* Flush Endpoint */
        adi_usbd_dev_musbmhdrc_FlushEndpoint(pDevice, pEpDef);

        Result = ADI_USBD_DEV_RESULT_SUCCESS;
    }
    else {
        Result = ADI_USBD_DEV_RESULT_INVALID_EP;
    }

    return Result;
}

/*!
 * \brief       The ADI_USBD_DEV_EpGetMaxRxTransferSize returns the maximum transfer size that can be
 *              received in a single transfer
 *
 * \param[in]   hDevice             Handle to device instance.
 * \param[in]   EpAddr              Logical Endpoint Address
 * \param[in]   pData               Address of data buffer
 * \param[out]  pMaxTransferSize    Location to store the max transfer size
 *
 * \return      Status:
 *              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 *              - #ADI_USBD_DEV_RESULT_INVALID_EP   Invalid Endpoint address
 *
 *              The maximum transfer size will depend on:
 *              1) The endpoint type - only bulk endpoints can transfer multipackets
 *              2) The buffer alignment - if not aligned to 4 bytes then PIO mode restricts size to
 *                 the endpoints MaxPktSize.
 *              3) Compilation options - the programmer may choose to restrict transfers to
 *                 single packet mode
 *
 * \sa          ADI_USBD_DEV_EpSubmitRxBuffer().
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpGetMaxRxTransferSize(
                                     ADI_USBD_DEV_HANDLE        hDevice,
                                     uint8_t                    EpAddr,
                                     uint8_t                    *pData,
                                     uint32_t                   *pMaxTransferSize
)
{
    return ADI_USBD_DEV_EpGetMaxTransferSize(hDevice,EpAddr,pData,ADI_USBD_DEV_EP_DIR_RX,pMaxTransferSize);
}

/*!
 * \brief       The ADI_USBD_DEV_EpGetMaxTxTransferSize returns the maximum transfer size that can be
 *              transmitted in a single transfer
 *
 * \param[in]   hDevice             Handle to device instance.
 * \param[in]   EpAddr              Logical Endpoint Address
 * \param[in]   pData               Address of data buffer
 * \param[out]  pMaxTransferSize    Location to store the max transfer size
 *
 * \return      Status:
 *              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 *              - #ADI_USBD_DEV_RESULT_INVALID_EP   Invalid Endpoint address
 *
 *              The maximum transfer size will depend on:
 *              1) The endpoint type - only bulk endpoints can transfer multipackets
 *              2) The buffer alignment - if not aligned to 4 bytes then PIO mode restricts size to
 *                 the endpoints MaxPktSize.
 *              3) Compilation options - the programmer may choose to restrict transfers to
 *                 single packet mode
 *
 * \sa          ADI_USBD_DEV_EpSubmitTxBuffer().
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpGetMaxTxTransferSize(
                 ADI_USBD_DEV_HANDLE        hDevice,
                 uint8_t                    EpAddr,
                 uint8_t                    *pData,
                 uint32_t                   *pMaxTransferSize
)
{
    return ADI_USBD_DEV_EpGetMaxTransferSize(hDevice,EpAddr,pData,ADI_USBD_DEV_EP_DIR_TX,pMaxTransferSize);
}

/*
 * ADI_USBD_DEV_EpGetMaxTransferSize()
 *
 * called by ADI_USBD_DEV_EpGetMaxRxTransferSize & ADI_USBD_DEV_EpGetMaxTxTransferSize to do
 * the actual determination of the max transfer size
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_EpGetMaxTransferSize(
                                     ADI_USBD_DEV_HANDLE        hDevice,
                                     uint8_t                    EpAddr,
                                     uint8_t                    *pData,
                                     ADI_USBD_DEV_EP_DIR        Direction,
                                     uint32_t                   *pMaxTransferSize
)
{
    ADI_USBD_DEV_MUSBMHDRC_DEF  *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;
    ADI_USBD_DEV_EP_DEF_EX      *pEpDef;
    ADI_USBD_DEV_RESULT         Result = ADI_USBD_DEV_RESULT_INVALID_EP;
#if (ADI_USBD_DEV_USE_PIO != ADI_USBD_DEV_YES)
    uint32_t                    nShortPkt;
#endif
    pEpDef = adi_usbd_dev_musbmhdrc_GetEpDef(pDevice, EpAddr);
    if (pEpDef)
    {

#if (ADI_USBD_DEV_USE_PIO == ADI_USBD_DEV_YES)
        *pMaxTransferSize = pEpDef->nMaxPktSize;
#else
#if defined(__ADSPBLACKFIN__) && (_ADI_USBD_ALGNMNT_CHECK_ == ADI_USBD_DEV_YES)
        /* if buffer is not aligned to 32 byte boundary PIO only can be used */
        if ( (adi_usbd_dev_IsDataCacheEnabled()) && (((uint32_t)pData & 0x1F) != 0) )
        {
            *pMaxTransferSize = pEpDef->nMaxPktSize;
        }
        else
#endif
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC) || (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC)
            /* if buffer is not aligned to 4 byte boundary PIO only can be used */
            if ( ((uint32_t)pData & 0x3) != 0 )
            {
                *pMaxTransferSize = pEpDef->nMaxPktSize;
            }
            else
#endif
            {
                if (pEpDef->EpType != ADI_USBD_DEV_EP_TYPE_BULK  ) {
                    *pMaxTransferSize = pEpDef->nMaxPktSize;
                }
                else {
                    if (Direction == ADI_USBD_DEV_EP_DIR_RX) {
                        *pMaxTransferSize = 0xFFFFFFFF;
                    }
                    else {
                        nShortPkt = (*pMaxTransferSize) % pEpDef->nMaxPktSize;
                        if (nShortPkt && ((*pMaxTransferSize) > pEpDef->nMaxPktSize) ) {
                            *pMaxTransferSize -= nShortPkt;
                        }
                        else {
                            *pMaxTransferSize = 0xFFFFFFFF;
                        }
                    }
                }
            }
#endif
        Result = ADI_USBD_DEV_RESULT_SUCCESS;
    }
    return Result;
}

/*!
 * \brief       The ADI_USBD_DEV_SetConfig API selects a configuration
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[in]   ConfigID        Configuration ID
 *
 * \return              Status:
 *                              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 *
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_SetConfig(
                                ADI_USBD_DEV_HANDLE     hDevice,
                                uint8_t                 ConfigID
)
{
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC)
    ADI_USBD_DEV_MUSBMHDRC_DEF  *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;

    /* when new configuration is set we will make sure the FIFO addresses are reset */
    pDevice->FifoRamAdress = USB_DYNFIFO_START_ADDR;
#endif

    return(ADI_USBD_DEV_RESULT_SUCCESS);
}

/*!
 * \brief       The ADI_USBD_DEV_ClearConfig API clears a configuration
 *
 * \param[in]   hDevice         Handle to device instance.
 * \param[in]   ConfigID        Configuration ID
 *
 * \return              Status:
 *                              - #ADI_USBD_DEV_RESULT_SUCCESS  Success
 *
 */
static ADI_USBD_DEV_RESULT ADI_USBD_DEV_ClearConfig(
                                ADI_USBD_DEV_HANDLE     hDevice,
                                uint8_t                 ConfigID
)
{
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC)
    ADI_USBD_DEV_MUSBMHDRC_DEF  *pDevice = (ADI_USBD_DEV_MUSBMHDRC_DEF *)hDevice;

    /* configuration switch we have to reset the fifo addresses */
    pDevice->FifoRamAdress = USB_DYNFIFO_START_ADDR;
#endif

    return (ADI_USBD_DEV_RESULT_SUCCESS);
}

/*
 * adi_usbd_dev_musbmhdrc_ReceiveDataBuffer()
 *
 * Initiates receipt of data from the host
 *
 */
uint32_t adi_usbd_dev_musbmhdrc_ReceiveDataBuffer(
                                        ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
                                        ADI_USBD_DEV_EP_DEF_EX     *pEpDef
)
{
    uint32_t nBytes = 0;

    if ((pEpDef->pCurrentBuffer) && (pEpDef->pCurrentBuffer->nBytesRemaining))
    {
        adi_usbd_dev_musbmhdrc_SetTransferMode(pDevice, pEpDef, ADI_USBD_DEV_EP_DIR_RX);
        pEpDef->pCurrentBuffer->bInUse = true;

        /* configure the Endpoint */
        if (pEpDef->EpNum != 0) {
            adi_usbd_dev_musbmhdrc_SetupEndPoint(pDevice, pEpDef);
        }

        if (pEpDef->EpNum == 0) {
            pEpDef->pCurrentBuffer->nBytesThisPass = adi_usbd_dev_musbhdrc_GetRxCount(pDevice, pEpDef);
        }

        /* schedule the transfer */
        switch((pEpDef->TransferMode))
        {
#if (ADI_USBD_DEV_USE_DMA == ADI_USBD_DEV_YES)
            case ADI_USBD_DEV_EP_MODE_DMA_1:
                pEpDef->pCurrentBuffer->nBytesThisPass = pEpDef->pCurrentBuffer->nBytesRemaining;
                nBytes = pEpDef->pCurrentBuffer->nBytesThisPass;

                adi_usbd_dev_musbhdrc_EnableRxInterrupt(pDevice, pEpDef);
                adi_usbd_dev_musbmhdrc_ScheduleDMA(pDevice, pEpDef, ADI_USBD_DMA_MODE_1);
                /* Command flow transferred to DMA/RX Interrupt handler */
                break;

            case ADI_USBD_DEV_EP_MODE_DMA_0:
                SetBytesThisPass(pEpDef, false);
                nBytes = pEpDef->pCurrentBuffer->nBytesThisPass;
                /* receive packet from Host */
                adi_usbd_dev_musbhdrc_EnableRxInterrupt(pDevice, pEpDef);
                /* Command flow transferred to RX Interrupt handler */
                break;
#endif
            case ADI_USBD_DEV_EP_MODE_PIO:
                SetBytesThisPass(pEpDef, false);
                nBytes = pEpDef->pCurrentBuffer->nBytesThisPass;
                /*
                 * Even though the request is for multiple packets, PIO can only
                 * transfer a single packet which is what pEpDef->pCurrentBuffer->nBytesThisPass
                 * is set to. In order that the GetRxBuffer() call is handled properly we must limit
                 * the transfer in this pass to a single packet.
                 */
                pEpDef->pCurrentBuffer->nBytesRemaining = pEpDef->pCurrentBuffer->nBytesThisPass;

                /* receive packet from Host */
                adi_usbd_dev_musbhdrc_EnableRxInterrupt(pDevice, pEpDef);
                /* Command flow transferred to RX Interrupt handler */
                break;

            default:
                break;

        }
    }

    return nBytes;
}


/*
 * adi_usbd_dev_musbmhdrc_TransmitDataBuffer()
 *
 * Initiates transmission of data to the host
 *
 */
uint32_t adi_usbd_dev_musbmhdrc_TransmitDataBuffer(
                                        ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
                                        ADI_USBD_DEV_EP_DEF_EX     *pEpDef
                          )
{

    uint32_t nBytes=0;
    bool bTxPktRdySet = true;

    if ((pEpDef->pCurrentBuffer) && (pEpDef->pCurrentBuffer->nBytesRemaining))
    {
        pEpDef->pCurrentBuffer->bInUse = true;

//#define __ADI_USBD_BF609_WORKAROUND_FOR_DMA_MODE_1_ISSUE__
#ifdef __ADI_USBD_BF609_WORKAROUND_FOR_DMA_MODE_1_ISSUE__
        {
            /* FIXIT: Workaround for two consecutive write */
            bool bTxPktReadyBitSet = true;
            /* Set the index to TxEndpoint */
            *pDEVICE_REG(pDevice,pINDEX) = pEpDef->EpNum;
            /* Read the TxPktRdy bit */
            bTxPktReadyBitSet = GET_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_TXPKTRDY);
            /* If EP FIFO is not free, wait until data is transmitted to host */
            while(bTxPktReadyBitSet)
            {
                /*
                 * This should never be reached.
                 */
                bTxPktReadyBitSet = GET_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_TXPKTRDY);
            }
        }
#endif

        /* configure the Endpoint */
        adi_usbd_dev_musbmhdrc_SetTransferMode(pDevice, pEpDef, ADI_USBD_DEV_EP_DIR_TX);
        if (pEpDef->EpNum != 0) {
            adi_usbd_dev_musbmhdrc_SetupEndPoint(pDevice, pEpDef);
        }

        /* schedule the transfer */
        switch(pEpDef->TransferMode)
        {
            case ADI_USBD_DEV_EP_MODE_PIO:
                /* Determine the number of bytes this pass */
                nBytes = SetBytesThisPass(pEpDef, false);

                /* Load the FIFO */
                adi_usbd_dev_musbmhdrc_LoadFifoPIO(pDevice, pEpDef);
                /* send packet to Host */
                bTxPktRdySet = adi_usbd_dev_musbmhdrc_SetTxPktRdy(pDevice, pEpDef);
                if (!bTxPktRdySet) {
                    /*
                     * With EP 0 last TX packet, we cannot set TxPktRdy until the status phase
                     */
                    pEpDef->pfEpStateHandler(pDevice, pEpDef);
                }

                /* Command flow transferred to TX Interrupt handler */
                break;

#if (ADI_USBD_DEV_USE_DMA == ADI_USBD_DEV_YES)
            case ADI_USBD_DEV_EP_MODE_DMA_0:
                /* Determine the number of bytes this pass */
                nBytes = SetBytesThisPass(pEpDef, false);

                /* Schedule DMA Mode 0 transfer */
                adi_usbd_dev_musbmhdrc_ScheduleDMA(pDevice, pEpDef, ADI_USBD_DMA_MODE_0);

                /* Command flow transferred to DMA Interrupt handler */
                break;

            case ADI_USBD_DEV_EP_MODE_DMA_1:
                /* Determine the number of bytes this pass */
                nBytes = SetBytesThisPass(pEpDef, true);

                /* Schedule DMA Mode 1 transfer */
                adi_usbd_dev_musbmhdrc_ScheduleDMA(pDevice, pEpDef, ADI_USBD_DMA_MODE_1);

                /* Command flow transferred to DMA Interrupt handler */
                break;

            default:
                break;
#endif
        }
    }
    return nBytes;
}

/*
 * adi_usbd_dev_musbmhdrc_GetEpDef()
 *
 * Locates the Endpoint definition structure
 *
 */
ADI_USBD_DEV_EP_DEF_EX *adi_usbd_dev_musbmhdrc_GetEpDef(
                                ADI_USBD_DEV_MUSBMHDRC_DEF  *pDevice,
                                uint8_t                     EpAddr
)
{
    ADI_USBD_DEV_EP_DEF_EX      *pEpDef;

    if ( (EpAddr&0x7F) == 0 ){
        pEpDef = (ADI_USBD_DEV_EP_DEF_EX *)adi_usbd_dev_EpGetEntry(pDevice->pEpTable,(EpAddr&0x7F));
    }
    else {
        pEpDef = (ADI_USBD_DEV_EP_DEF_EX *)adi_usbd_dev_EpGetEntry(pDevice->pEpTable,EpAddr);
    }

    return pEpDef;
}



/*
 * adi_usbd_dev_musbmhdrc_ReadFIFO()
 *
 * Unloads FIFO with appropriate alignment
 */
void adi_usbd_dev_musbmhdrc_ReadFifo(ADI_USBD_DEV_FIFO_ENTRY *pFifo, ADI_USBD_DEV_FIFO_ENTRY *pData, uint32_t nBufSize)
{
    uint32_t BytesRemaining = nBufSize;
#if WA_16000030
    uint32_t intStatus = cli();
#endif

#if (ADI_USBD_DEV_DRVR != ADI_USBD_DEV_BF_MUSBHDRC)
    while ((BytesRemaining >= 4) && (((uint32_t)pData & 0x3) == 0) )
    {
        pData->dword = pFifo->dword;
        pData++;
        BytesRemaining -= 4;
    }
#endif
    while ((BytesRemaining >= 2)  && (((uint32_t)pData & 0x1) == 0) )
    {
        pData->word = pFifo->word;
        pData = (ADI_USBD_DEV_FIFO_ENTRY *)((uint8_t *)pData+2);
        BytesRemaining -= 2;
    }
    while (BytesRemaining >= 1)
    {
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)
        pData->word = pFifo->word;
        pData->byte = (uint8_t)(pData->word & 0x00FF);
#else
        pData->byte = pFifo->byte;
#endif
        pData = (ADI_USBD_DEV_FIFO_ENTRY *)((uint8_t *)pData+1);
        BytesRemaining--;
    }
#if WA_16000030
    sti(intStatus);
#endif
}

/*
 * adi_usbd_dev_musbmhdrc_UnloadFifoPIO()
 *
 * Unloads FIFO with appropriate alignment
 */
void adi_usbd_dev_musbmhdrc_UnloadFifoPIO(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef)
{
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)
    /*
     * We need to use the special byte addressing FIFO for EP0 for the trailing bytes if not a whole 32-bit word
     */
    uint32_t TrailingBytes;
    if (pEpDef->EpNum == 0) {
        TrailingBytes = pEpDef->pCurrentBuffer->nBytesThisPass % 4;
    }
#endif
    adi_usbd_dev_musbmhdrc_ReadFifo(
                                (ADI_USBD_DEV_FIFO_ENTRY *)pDevice->pRegs[pDevice->DeviceNumber].pEP_FIFO[pEpDef->EpNum],
                                (ADI_USBD_DEV_FIFO_ENTRY *)pEpDef->pCurrentBuffer->pData,
                                pEpDef->pCurrentBuffer->nBytesThisPass
                            );

}


#if 1 /*(ADI_USBD_DEV_USE_PIO == ADI_USBD_DEV_YES) */
/*
 * adi_usbd_dev_musbhdrc_LoadFifoPIO()
 *
 * Loads the endpoint FIFO
 */
void adi_usbd_dev_musbmhdrc_LoadFifoPIO(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef)
{
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)
    uint32_t imask=cli();
    uint32_t SavedEpNum = *pDEVICE_REG(pDevice,pINDEX);
    *pDEVICE_REG(pDevice,pINDEX) = pEpDef->EpNum;
    *pUSB_TXCOUNT = pEpDef->pCurrentBuffer->nTransferSize;
    *pDEVICE_REG(pDevice,pINDEX) = SavedEpNum;
    sti(imask);
#endif
    adi_usbd_dev_musbmhdrc_WriteFifo(
                                (ADI_USBD_DEV_FIFO_ENTRY *)pDevice->pRegs[pDevice->DeviceNumber].pEP_FIFO[pEpDef->EpNum],
                                (ADI_USBD_DEV_FIFO_ENTRY *)pEpDef->pCurrentBuffer->pData,
                                pEpDef->pCurrentBuffer->nTransferSize
                            );
}
/*
 * adi_usbd_dev_musbhdrc_WriteFifo()
 *
 * Loads FIFO with appropriate alignment
 */

void adi_usbd_dev_musbmhdrc_WriteFifo(ADI_USBD_DEV_FIFO_ENTRY *pFifo, ADI_USBD_DEV_FIFO_ENTRY *pData, uint32_t nBufSize)
{
    uint32_t BytesRemaining = nBufSize;

#if WA_16000030
    uint32_t intStatus = cli();
#endif

#if (ADI_USBD_DEV_DRVR != ADI_USBD_DEV_BF_MUSBHDRC)
    while ((BytesRemaining >= 4) && (((uint32_t)pData & 0x3) == 0) )
    {
        pFifo->dword = pData->dword;
        pData++;
        BytesRemaining -= 4;
    }
#endif
    while ((BytesRemaining >= 2)  && (((uint32_t)pData & 0x1) == 0) )
    {
        pFifo->word = pData->word;
        pData = (ADI_USBD_DEV_FIFO_ENTRY *)(((uint8_t *)pData)+2);
        BytesRemaining -= 2;
    }
    while (BytesRemaining >= 1)
    {
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)
        pFifo->word = pData->word & 0x00FF;
#else
        pFifo->byte = pData->byte;
#endif
        pData = (ADI_USBD_DEV_FIFO_ENTRY *)(((uint8_t *)pData)+1);
        BytesRemaining--;
    }
#if WA_16000030
    sti(intStatus);
#endif
}
#endif

#if (ADI_USBD_DEV_USE_DMA == ADI_USBD_DEV_YES)

/*
 * adi_usbd_dev_musbmhdrc_ScheduleDMA()
 *
 * Schedules DMA transfers
 */
ADI_USBD_DEV_RESULT adi_usbd_dev_musbmhdrc_ScheduleDMA(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef, uint32_t DMAMode)
{
    uint8_t DMAChannel;
    ADI_USBD_DEV_RESULT Result;
    uint32_t StartAddress = (uint32_t)(&((uint8_t*)pEpDef->pCurrentBuffer->pData)[pEpDef->pCurrentBuffer->nBytesProcessed]);

#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC) || (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC)
    /* select a DMA channel */
    if (adi_usbd_dev_musbmhdrc_GetDMAChannel(pDevice, &DMAChannel))
    {
        pEpDef->DMAChannel = DMAChannel;

        /* Set EP number (without direction bit) */
        SET_DEVICE_REG_VALUE(pDevice, pDMA_CTL[DMAChannel], BITM_USB_DMA_CTL_EP, ((pEpDef->EpNum)<<BITP_USB_DMA_CTL_EP) );

        /* set DMA mode  */
        if (DMAMode == ADI_USBD_DMA_MODE_0)
        {
            CLR_DEVICE_REG_VALUE(pDevice, pDMA_CTL[DMAChannel], BITM_USB_DMA_CTL_MODE );
        }
        else {
            SET_DEVICE_REG_VALUE(pDevice, pDMA_CTL[DMAChannel], BITM_USB_DMA_CTL_MODE, ENUM_USB_DMA_CTL_DMAMODE1 );
         }

        /* Enable DMA Interrupt */
        SET_DEVICE_REG_VALUE(pDevice, pDMA_CTL[DMAChannel], BITM_USB_DMA_CTL_IE,  ENUM_USB_DMA_CTL_DMAINTEN  );

        /* Set Direction bit */
        if (pEpDef->EpDir == ADI_USBD_DEV_EP_DIR_TX) {
            SET_DEVICE_REG_VALUE(pDevice, pDMA_CTL[DMAChannel], BITM_USB_DMA_CTL_DIR, ENUM_USB_DMA_CTL_DMADIR_TX );
        }
        else {
            CLR_DEVICE_REG_VALUE(pDevice, pDMA_CTL[DMAChannel], BITM_USB_DMA_CTL_DIR);
        }

        /* Set Data start address */
        *pDevice->pRegs[pDevice->DeviceNumber].pDMA_ADDR[DMAChannel] = StartAddress;

        /* Set Data count */
        *pDevice->pRegs[pDevice->DeviceNumber].pDMA_CNT[DMAChannel]  = (uint32_t)pEpDef->pCurrentBuffer->nBytesThisPass;
        ADI_USBD_DBG_EP_ARG("MUSBMHDRC DMA",pEpDef->basic.EpAddr, pEpDef->pCurrentBuffer->nBytesThisPass);

        /* Start DMA transfer */
        SET_DEVICE_REG_VALUE(pDevice, pDMA_CTL[DMAChannel], BITM_USB_DMA_CTL_EN, ENUM_USB_DMA_CTL_DMAEN );

        /* Command flow is transferred to DMA Interrupt handler */
        Result = ADI_USBD_DEV_RESULT_SUCCESS;
    }
    else {
        Result = ADI_USBD_DEV_RESULT_NO_DMA;
    }
#elif (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)
    /* reset any previous value */
    set_dma_ctrl(pEpDef->EpNum,(uint16_t)0);

    /* setup start address */
    set_dma_addr(pEpDef->EpNum,(uint16_t)ADDR_LOW(StartAddress),(uint16_t)ADDR_HIGH(StartAddress));

    /* set up dma count */
    set_dma_count(pEpDef->EpNum,ADDR_LOW((uint16_t)pEpDef->pCurrentBuffer->nBytesThisPass),
                                   ADDR_HIGH((uint16_t)pEpDef->pCurrentBuffer->nBytesThisPass));

    /* DMA mode 0 */
    if (DMAMode == ADI_USBD_DEV_EP_MODE_DMA_0)
    {
        /* setup dma control register */
        if (pEpDef->EpDir == ADI_USBD_DEV_EP_DIR_TX)
        {
            set_dma_ctrl(pEpDef->EpNum,(uint16_t)(ENUM_USB_DMA_CTL_DMADIR_TX |
                                                        ENUM_USB_DMA_CTL_DMAINTEN  |
                                                        ENUM_USB_DMA_CTL_DMAEN     |
                                                        ((pEpDef->EpNum &0xf) << 4)
                                                       ));
        }
        else
        {
            set_dma_ctrl(pEpDef->EpNum,(uint16_t)(ENUM_USB_DMA_CTL_DMADIR_RX |
                                                        ENUM_USB_DMA_CTL_DMAINTEN  |
                                                        ENUM_USB_DMA_CTL_DMAEN     |
                                                        ((pEpDef->EpNum &0xf) << 4)
                                                        ));
         }
    }
    else /* DMA MODE 1 */
    {
            /* setup dma control register */
         if (pEpDef->EpDir == ADI_USBD_DEV_EP_DIR_TX)
         {
             set_dma_ctrl(pEpDef->EpNum,(uint16_t)(ENUM_USB_DMA_CTL_DMADIR_TX |
                                                         ENUM_USB_DMA_CTL_DMAMODE1  |
                                                         ENUM_USB_DMA_CTL_DMAINTEN  |
                                                         ENUM_USB_DMA_CTL_DMAEN     |
                                                         ((pEpDef->EpNum &0xf) << 4)
                                                         ));
         }
         else
         {
             set_dma_ctrl(pEpDef->EpNum,(uint16_t)(ENUM_USB_DMA_CTL_DMADIR_RX  |
                                                         ENUM_USB_DMA_CTL_DMAMODE1   |
                                                         ENUM_USB_DMA_CTL_DMAINTEN   |
                                                         ENUM_USB_DMA_CTL_DMAEN      |
                                                         ((pEpDef->EpNum &0xf) << 4)
                                                         ));
         }
    }

    /* Command flow is transferred to DMA Interrupt handler */
    Result = ADI_USBD_DEV_RESULT_SUCCESS;
#endif
    return Result;
}


#endif


/*
 * SubmitBuffer()
 *
 * Called by API functions, ADI_USBD_DEV_EpSubmitRxBuffer & ADI_USBD_DEV_EpSubmitTxBuffer to
 * queue the buffer with the endpoint
 */
static ADI_USBD_DEV_RESULT SubmitBuffer(
                ADI_USBD_DEV_MUSBMHDRC_DEF  *pDevice,
                ADI_USBD_DEV_EP_DEF_EX      *pEpDef,
                void                        *pData,
                uint32_t                    DataLen
)
{
    ADI_USBD_DEV_RESULT         Result = ADI_USBD_DEV_RESULT_INVALID_EP;

    /*
     * Submit to Buffer Queue
     */
    if (adi_usbd_dev_EpQueueBuffer(pEpDef->pBufferList, pData, DataLen))
    {
        /*
         * If Enpoint is currently processing a buffer, then the just queued buffer will be
         * processed on completion of the current buffer.
         * Otherwise it is processed now.
         */

        if (!pEpDef->pCurrentBuffer)
        {
            /*
             * Get Queued buffer (might not be the same as the one just queued)
             */
            pEpDef->pCurrentBuffer = adi_usbd_dev_EpGetQueuedBuffer(pEpDef->pBufferList);

            /* Set State according to transfer direction */
            if (pEpDef->EpDir == ADI_USBD_DEV_EP_DIR_RX) {
                pEpDef->EpState = ADI_USBD_EP_STATE_RX;
            }
            else {
                pEpDef->EpState = ADI_USBD_EP_STATE_TX;
            }
#if defined(__ADSPBLACKFIN__) && (ADI_USBD_DEV_USE_DMA == ADI_USBD_DEV_YES)
            adi_usbd_dev_FlushCachedBuffer(pEpDef->EpDir, pData, DataLen);
#endif
#define _QUICK_FIX_FOR_TX_STATE_MODEL_
#if defined(_QUICK_FIX_FOR_TX_STATE_MODEL_)
/* FIXIT need to review EP0 state model handling */
            if ((pEpDef->EpNum == 0)  && (pEpDef->EpDir  == ADI_USBD_DEV_EP_DIR_TX))
            {
                pEpDef->EpState = ADI_USBD_EP_STATE_TX;
            }
#endif
            /* Process state */
            if (/*(pEpDef->EpNum == 0) && */(pEpDef->EpState == ADI_USBD_EP_STATE_TX))
            {
                adi_usbd_dev_musbmhdrc_TransmitDataBuffer(pDevice, pEpDef);
            }
            else {

                adi_usbd_dev_musbmhdrc_ReceiveDataBuffer(pDevice, pEpDef);
                //pEpDef->pfEpStateHandler(pDevice, pEpDef);
            }

        }
        /* either way, it was a successfull submission */
        Result = ADI_USBD_DEV_RESULT_SUCCESS;
    }

    return Result;
}

/*
 * ConfigureEndPoint()
 *
 * Configures the Endpoint upon binding. Called from ADI_USBD_DEV_BindEndpoint()
 *
 */
static void ConfigureEndPoint(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef)
{
    adi_usbd_dev_musbhdrc_setEpIndex(pDevice,pEpDef->EpNum);

    if (pEpDef->EpDir == ADI_USBD_DEV_EP_DIR_TX)
    {
        *pDEVICE_REG(pDevice,pEPI_TXMAXP) = pEpDef->nMaxPktSize;

/* IF (dynamic FIFO) */
#if (ADI_USBD_DEV_DYN_FIFO == ADI_USBD_DEV_YES)
        /* Set FIFO size */
        SetDynamicFifoAddress(pDevice,pEpDef,true);

/* IF (single buffering) */
#if (ADI_USBD_DEV_TX_SNGL_BUFFER == ADI_USBD_DEV_YES)
        SetDyamicTxFifoSize(pDevice, pEpDef->nMaxPktSize, false);
/* ELSE (double buffering */
#else
        SetDyamicTxFifoSize(pDevice, pEpDef->nMaxPktSize, true);
/* ENDIF */
#endif

/* ELSE (static FIFO) */
#else

/* IF (single buffering) */
#if (ADI_USBD_DEV_TX_SNGL_BUFFER == ADI_USBD_DEV_YES)

/* IF (ARM implementation) */
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC)
        *pREG_USB0_TXDPKTBUFDIS |= (1<<pEpDef->EpNum);
/* ELSE IF (BF52x/BF54x implementation) */
#elif (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)
        *pUSB_TX_MAX_PACKET = 1024;
/* ENDIF */
#endif

/* ENDIF */
#endif

/* ENDIF */
#endif

        /*
         * Set Clear Data Toggle Bit
         */
        SET_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_CLRDATATGL, ENUM_USB_EPI_TXCSR_P_CLRTGL);

        /*
         * Flush the FIFO if required (twice for double buffering)
         */
        if ( GET_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_TXPKTRDY)
            || GET_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_NEFIFO)
        )
        {
            SET_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_FLUSHFIFO, ENUM_USB_EPI_TXCSR_P_FLUSH);
            SET_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_FLUSHFIFO, ENUM_USB_EPI_TXCSR_P_FLUSH);
        }

        /*
         * Reset all
         */
        CLR_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, 0xFFFF);

        /*
         * Isochronous endpoint
         */
        if (pEpDef->EpType == ADI_USBD_DEV_EP_TYPE_ISOC)
        {
            SET_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, ENUM_USB_EPI_TXCSR_P_ISOEN, ENUM_USB_EPI_TXCSR_P_ISOEN);
        }

        if (pEpDef->EpType == ADI_USBD_DEV_EP_TYPE_INTR)
        {
            /* Force data toggle */

            SET_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_FRCDATATGL, ENUM_USB_EPI_TXCSR_P_FRCTGL);
        }
    }
    else {
        /*
         * Set Maxpacket Size
         */
        *pDEVICE_REG(pDevice,pEPI_RXMAXP) = pEpDef->nMaxPktSize;

/* IF (dynamic FIFO) */
#if (ADI_USBD_DEV_DYN_FIFO == ADI_USBD_DEV_YES)
        /* Set FIFO size */
        SetDynamicFifoAddress(pDevice,pEpDef,true);

/* IF (single buffering) */
#if (ADI_USBD_DEV_RX_SNGL_BUFFER == ADI_USBD_DEV_YES)
        SetDyamicRxFifoSize(pDevice, pEpDef->nMaxPktSize, false);
/* ELSE (double buffering */
#else
        SetDyamicRxFifoSize(pDevice, pEpDef->nMaxPktSize, true);
/* ENDIF */
#endif

/* ELSE (static FIFO) */
#else

/* IF (single buffering) */
#if (ADI_USBD_DEV_RX_SNGL_BUFFER == ADI_USBD_DEV_YES)

/* IF (ARM implementation) */
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC)
        *pREG_USB0_RXDPKTBUFDIS |= (1<<pEpDef->EpNum);
/* ELSE IF (BF52x/BF54x implementation) */
#elif (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)
        *pUSB_RX_MAX_PACKET = 1024;
/* ENDIF */
#endif

/* ENDIF */
#endif

/* ENDIF */
#endif

        /*
         * Flush the FIFO if required (twice for double buffering)
         */
        if ( GET_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_RXPKTRDY)
            || GET_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_FIFOFULL)
        )
        {
            SET_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_FLUSHFIFO, ENUM_USB_EPI_RXCSR_P_FLUSH);
            SET_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_FLUSHFIFO, ENUM_USB_EPI_RXCSR_P_FLUSH);
        }

        /*
         * Set Clear Data Toggle Bit
         */
        SET_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_CLRDATATGL, ENUM_USB_EPI_RXCSR_P_CLRTGL);

        /*
         * Reset all
         */
        CLR_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, 0xFFFF);

        /*
         * Isochronous endpoint
         */
        if (pEpDef->EpType == ADI_USBD_DEV_EP_TYPE_ISOC)
        {
            SET_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_ISO, ENUM_USB_EPI_RXCSR_P_ISOEN);
        }

        if (pEpDef->EpType == ADI_USBD_DEV_EP_TYPE_INTR)
        {
            /* Disable NYET */
#if (ADI_USBD_DEV_HS_ENABLE == ADI_USBD_DEV_YES)
            SET_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_DNYETPERR, ENUM_USB_EPI_RXCSR_P_DNYTERRDIS);
#endif
        }
    }
}

/*
 * adi_usbd_dev_musbmhdrc_SetupEndPoint()
 *
 * Sets up the endpoint ahead of current transfer.
 * Called from adi_usbd_dev_musbmhdrc_ReceiveDataBuffer() and adi_usbd_dev_musbmhdrc_TransmitDataBuffer
 *
 */
void adi_usbd_dev_musbmhdrc_SetupEndPoint(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef)
{
    /*
     * NOTE: the Mentor PG says that the DmaReqMode bit cannot be cleared in same clock cycle
     */
    /*
     * we only need to set up the endpoint if transfer mode has changed from last use
     */
    if (pEpDef->bConfigured) {
        return;
    }

    adi_usbd_dev_musbhdrc_setEpIndex(pDevice,pEpDef->EpNum);

    /*
     * IF (RX (OUT) Endpoint)
     */
    if (pEpDef->EpDir == ADI_USBD_DEV_EP_DIR_RX)
    {
        if (pEpDef->TransferMode == ADI_USBD_DEV_EP_MODE_DMA_1)
        {
            SET_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_DMAREQEN,   ENUM_USB_EPI_RXCSR_P_DMAREQEN);
            SET_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_AUTOCLR,    ENUM_USB_EPI_RXCSR_P_AUTOCLR);
            SET_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_DMAREQMODE, ENUM_USB_EPI_RXCSR_P_DMARQMODE1);
#if (ADI_USBD_DEV_HS_ENABLE == ADI_USBD_DEV_YES)
            CLR_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_DNYETPERR );
#endif
        }
        else if (pEpDef->TransferMode == ADI_USBD_DEV_EP_MODE_DMA_0)
        {
            CLR_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_DMAREQEN );
            CLR_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_AUTOCLR );
            CLR_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_DMAREQMODE );
#if (ADI_USBD_DEV_HS_ENABLE == ADI_USBD_DEV_YES)
            CLR_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_DNYETPERR );
#endif
        }
        else /* Programmed I/O */
        {
            CLR_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_DMAREQEN );
            CLR_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_AUTOCLR );
            CLR_DEVICE_REG_VALUE(pDevice, pEPI_RXCSR, BITM_USB_EPI_RXCSR_P_DMAREQMODE );
        }
    }
    /*
     * ELSE (Tx (IN) Endpoint)
     */
    else
    {
        if (pEpDef->TransferMode == ADI_USBD_DEV_EP_MODE_DMA_1)
        {

            SET_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_DMAREQEN,   ENUM_USB_EPI_TXCSR_P_DMAREQEN  );
            SET_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_AUTOSET,    ENUM_USB_EPI_TXCSR_P_AUTOSET  );
            SET_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_DMAREQMODE, ENUM_USB_EPI_TXCSR_P_DMARQMODE1  );
            CLR_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_FRCDATATGL);
        }
        else if (pEpDef->TransferMode == ADI_USBD_DEV_EP_MODE_DMA_0)
        {
            CLR_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_DMAREQEN);
            CLR_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_AUTOSET);
            CLR_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_DMAREQMODE);
            CLR_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_FRCDATATGL);
        }
        else /* Programmed I/O */
        {
            CLR_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_DMAREQEN);
            CLR_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_AUTOSET);
            CLR_DEVICE_REG_VALUE(pDevice, pEPI_TXCSR, BITM_USB_EPI_TXCSR_P_DMAREQMODE);
        }
    }

    /*
     * Set flag to show EP is configured
     */
    pEpDef->bConfigured = true;
}
#if (ADI_USBD_DEV_DYN_FIFO == ADI_USBD_DEV_YES)
/*
* SetDyamicRxFifoSize()
*
* Sets up the EP FIFO size for RX transfers
*/
static void SetDyamicRxFifoSize(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, uint16_t Size, bool bDoubleBuffer)
{
    switch(Size) {
    case 8:
        SET_DEVICE_REG_VALUE(pDevice, pRXFIFOSZ, BITM_USB_RXFIFOSZ_SZ, ENUM_USB_RXFIFOSZ_SZ8);
        break;
    case 64:
        SET_DEVICE_REG_VALUE(pDevice, pRXFIFOSZ, BITM_USB_RXFIFOSZ_SZ, ENUM_USB_RXFIFOSZ_SZ64);
        break;
    case 512:
        SET_DEVICE_REG_VALUE(pDevice, pRXFIFOSZ, BITM_USB_RXFIFOSZ_SZ, ENUM_USB_RXFIFOSZ_SZ512);
        break;
    case 1024:
        SET_DEVICE_REG_VALUE(pDevice, pRXFIFOSZ, BITM_USB_RXFIFOSZ_SZ, ENUM_USB_RXFIFOSZ_SZ1024);
        break;
   default:
       break;
   }

   if (bDoubleBuffer) {
       SET_DEVICE_REG_VALUE(pDevice, pRXFIFOSZ, BITM_USB_RXFIFOSZ_DPB, ENUM_USB_RXFIFOSZ_DPBEN);
   }
}

/*
* SetDyamicTxFifoSize()
*
* Sets up the EP FIFO size for TX transfers
*/
static void SetDyamicTxFifoSize(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, uint16_t Size, bool bDoubleBuffer)
{
    switch(Size) {
    case 8:
        SET_DEVICE_REG_VALUE(pDevice, pTXFIFOSZ, BITM_USB_TXFIFOSZ_SZ, ENUM_USB_TXFIFOSZ_SZ8);
        break;
    case 64:
        SET_DEVICE_REG_VALUE(pDevice, pTXFIFOSZ, BITM_USB_TXFIFOSZ_SZ, ENUM_USB_TXFIFOSZ_SZ64);
        break;
    case 512:
        SET_DEVICE_REG_VALUE(pDevice, pTXFIFOSZ, BITM_USB_TXFIFOSZ_SZ, ENUM_USB_TXFIFOSZ_SZ512);
        break;
    case 1024:
        SET_DEVICE_REG_VALUE(pDevice, pTXFIFOSZ, BITM_USB_TXFIFOSZ_SZ, ENUM_USB_TXFIFOSZ_SZ1024);
        break;
   default:
       break;
   }
   if (bDoubleBuffer) {
       SET_DEVICE_REG_VALUE(pDevice, pTXFIFOSZ, BITM_USB_TXFIFOSZ_DPB, ENUM_USB_TXFIFOSZ_DPBEN);
   }
}

/*
 * This function sets the dynamic FIFO addresses linerarly and returns error incase it can not allocate requested memory
 * Once memory is allocated for an endpoint FIFO it can not be re-used or re-assigned for another endpoint.
 */
static bool SetDynamicFifoAddress(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX  *pEpDef,bool bDoubleBuffer)
{
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC)
    uint32_t EpSavedIndex;
    uint32_t EpFifoSize = (bDoubleBuffer ? (pEpDef->nMaxPktSize << 1) : pEpDef->nMaxPktSize);

    /* FIFO ram is organized as multiples of 8 with respect to index so round it to nearest multiple of 8 */
    EpFifoSize = ((EpFifoSize + 7) & ~7u);

    /* we want to be in critical region when we check/modify the fifo address */
    ADI_USBD_ENTER_CRITICAL();

    /* check whether we have enough memory avaialble */
    if( (pDevice->FifoRamAdress + EpFifoSize) > pDevice->FifoRamTotalSize)
    {
        ADI_USBD_DBG_EP("MUSBMHDRC - No FIFO memory available",pDevice->FifoRamTotalSize);
        return (false);
    }

    EpSavedIndex     = *pREG_USB0_INDEX;

    /* Set the index */
     *pREG_USB0_INDEX = pEpDef->EpNum;

    if (pEpDef->EpDir == ADI_USBD_DEV_EP_DIR_TX) {
        /* fifo ram goes by multiples of 8, ex: addr 64 means we have to set 8 */
        *pREG_USB0_TXFIFOADDR |= ((pDevice->FifoRamAdress >> 3) & 0x1FFF);
        ADI_USBD_DBG_EP("SetDynamicFifoAddress - FIFO address index",*pREG_USB0_TXFIFOADDR);
    }
    else {
        /* fifo ram goes by multiples of 8, ex: addr 64 means we have to set 8 */
        *pREG_USB0_RXFIFOADDR |= ((pDevice->FifoRamAdress >> 3) & 0x1FFF);
        ADI_USBD_DBG_EP("SetDynamicFifoAddress - FIFO address index",*pREG_USB0_RXFIFOADDR);
    }

     /* increment the ram address */
     pDevice->FifoRamAdress += EpFifoSize;

     /* restore back the ep index */
     *pREG_USB0_INDEX = EpSavedIndex;

     ADI_USBD_EXIT_CRITICAL();
#endif
    return(true);
}
#endif
/*
* adi_usbd_dev_musbmhdrc_SetTransferMode()
 *
 * Sets teh transfer mode of the transfer
 */
void adi_usbd_dev_musbmhdrc_SetTransferMode(
        ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
        ADI_USBD_DEV_EP_DEF_EX     *pEpDef,
        ADI_USBD_DEV_EP_DIR        Direction
)
{
    ADI_USBD_DEV_EP_MODE CurrentTransferMode = pEpDef->TransferMode;

#if (ADI_USBD_DEV_USE_PIO == ADI_USBD_DEV_YES)
    pEpDef->TransferMode = ADI_USBD_DEV_EP_MODE_PIO;
#else
#if defined(__ADSPBF609_FAMILY__) && (_ADI_USBD_ALGNMNT_CHECK_ == ADI_USBD_DEV_YES)
    /* if buffer is not aligned to 32 byte boundary PIO only can be used */
    if ( (adi_usbd_dev_IsDataCacheEnabled()) && (((uint32_t)pEpDef->pCurrentBuffer->pData & 0x1F) != 0) )
    {
        ADI_USBD_DBG_EP_ARG("MUSBMHDRC ALIGNMENT FAULT!",pEpDef->basic.EpAddr,(uint32_t)pEpDef->pCurrentBuffer->pData);
        pEpDef->TransferMode = ADI_USBD_DEV_EP_MODE_PIO;
     }
    else
#endif
    /* if buffer is not aligned to 4 byte boundary PIO only can be used */
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC) || (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC)
    if ( ((uint32_t)pEpDef->pCurrentBuffer->pData & 0x3) != 0 )
    {
        pEpDef->TransferMode = ADI_USBD_DEV_EP_MODE_PIO;
        ADI_USBD_DBG_EP_ARG("MUSBMHDRC ALIGNMENT FAULT!",pEpDef->basic.EpAddr,(uint32_t)pEpDef->pCurrentBuffer->pData);
    }
    else
#endif
    {
        if (
            ((Direction == ADI_USBD_DEV_EP_DIR_RX) && (ADI_USBD_DEV_RX_DMA_MODE0 == ADI_USBD_DEV_YES) ) ||
            ((Direction == ADI_USBD_DEV_EP_DIR_TX) && (ADI_USBD_DEV_TX_DMA_MODE0 == ADI_USBD_DEV_YES) )
            )
        {
            /* DMA Mode 0 has been enforced */
            pEpDef->TransferMode = ADI_USBD_DEV_EP_MODE_DMA_0;
        }
        else {
            /* DMA Mode 1 can only be used for multi-packet Bulk transfers */
            if ((pEpDef->EpType == ADI_USBD_DEV_EP_TYPE_BULK) && (pEpDef->pCurrentBuffer->nBytesRemaining > pEpDef->nMaxPktSize))
            {
                pEpDef->TransferMode = ADI_USBD_DEV_EP_MODE_DMA_1;
            }
            else {
                pEpDef->TransferMode = ADI_USBD_DEV_EP_MODE_DMA_0;
            }
        }
    }
#endif
    /* We only need to reconfigure the EP if transfer mode has changed */
    if (pEpDef->TransferMode != CurrentTransferMode) {
        pEpDef->bConfigured = false;
    }
}

/*
 * SetBytesThisPass()
 *
 * Sets the number of bytes to be transferred in the current pass
 *
 */
static uint32_t SetBytesThisPass(ADI_USBD_DEV_EP_DEF_EX *pEpDef, bool bMultiPacket)
{
    uint32_t nShortPacket;

    if (bMultiPacket) {
        nShortPacket = pEpDef->pCurrentBuffer->nBytesRemaining % pEpDef->nMaxPktSize;
        pEpDef->pCurrentBuffer->nBytesThisPass = pEpDef->pCurrentBuffer->nBytesRemaining - nShortPacket;
    }
    else if (pEpDef->pCurrentBuffer->nBytesRemaining > pEpDef->nMaxPktSize)
    {
        pEpDef->pCurrentBuffer->nBytesThisPass = pEpDef->nMaxPktSize;
    }
    else {
        pEpDef->pCurrentBuffer->nBytesThisPass = pEpDef->pCurrentBuffer->nBytesRemaining;
    }
    return pEpDef->pCurrentBuffer->nBytesThisPass;
}

/*
 *
 * SetEpType()
 *
 * Converts the Endpoint type from the numerical value to the ADI_USBD_DEV_EP_TYPE value
 *
 */
static ADI_USBD_DEV_EP_TYPE SetEpType(uint8_t EpType)
{
    ADI_USBD_DEV_EP_TYPE type;

    if (EpType > 3) {
        type = ADI_USBD_DEV_EP_TYPE_INVALID;
    }
    else {
        type = (ADI_USBD_DEV_EP_TYPE)EpType;
    }
    return type;
}

/*
 * CtrlRxStatusPhase()
 *
 * Executes the Status Phase of an RX control transfer
 *
 */
static void CtrlRxStatusPhase(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef)
{
    *pDEVICE_REG(pDevice,pINDEX) = pEpDef->EpNum;
     SET_DEVICE_REG_VALUE(pDevice, pEP0_CSR0,
                         (BITM_USB_EP0_CSR_P_SPKTRDY|BITM_USB_EP0_CSR_P_DATAEND),
                         (ENUM_USB_EP0_CSR_P_SPKTRDY|ENUM_USB_EP0_CSR_P_DATAEND)
                        );

    /* State is set to TX STATUS */
    pEpDef->EpState = ADI_USBD_EP_STATE_TX_STATUS;
    ADI_USBD_DBG_EP("MUSBMHDRC Control RX Status",pEpDef->basic.EpAddr);
    /* call EP0 state handler */
    /*
     * FIXIT: should we not just wait for EP0 Interrupt?
     */
    pEpDef->pfEpStateHandler(pDevice, pEpDef);
}

/*
 * CtrlTxStatusPhase()
 *
 * Executes the Status Phase of a TX control transfer
 *
 */
static void CtrlTxStatusPhase(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef)
{
    /* transmit packet if not already done so */
    adi_usbd_dev_musbmhdrc_SetTxPktRdy(pDevice, pEpDef);

    pEpDef->EpState = ADI_USBD_EP_STATE_RX_STATUS;
    ADI_USBD_DBG_EP("MUSBMHDRC Control TX Status",pEpDef->basic.EpAddr);
    pEpDef->pfEpStateHandler(pDevice, pEpDef);

    /* Await TX interrupt on clearing TxPktRdy */
    pEpDef->EpState = ADI_USBD_EP_STATE_TX_PKT_SENT;
    pEpDef->NextEpState = ADI_USBD_EP_STATE_IDLE;
}



#define GET_DEVICE_REG8_VALUE_RAW(R,M) ((*(volatile uint8_t*)(R))&(M))
#define GET_DEVICE_REG16_VALUE_RAW(R,M) ((*(volatile uint16_t*)(R))&(M))

/*!
 * \brief       The adi_usbd_dev_musbmhdrc_GetHardwareConfig API provides the information
 *              about the implementation of the MUSBMHDRC controller.
 *
 * \param[out]   pHwInfo    Pointer to ADI_USBD_DEV_MUSBMHDRC_HWINFO structure to store
 *                          the results
 *
 * \return      void.
 *
 */
void adi_usbd_dev_musbmhdrc_GetHardwareConfig(ADI_USBD_DEV_MUSBMHDRC_HWINFO *pHwInfo)
{
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC)
    #define BITP_USB_HWVERS_RC              15                                  /* Release Candidate */
    #define BITP_USB_HWVERS_MAJOR           10                                  /* Major version number */
    #define BITP_USB_HWVERS_MINOR           0                                   /* Minor version number */

    pHwInfo->HWVERS_RC       = GET_DEVICE_REG16_VALUE_RAW(REG_USB0_HWVERS, BITM_USB_HWVERS_RC   ) >> BITP_USB_HWVERS_RC;
    pHwInfo->HWVERS_Major    = GET_DEVICE_REG16_VALUE_RAW(REG_USB0_HWVERS, BITM_USB_HWVERS_MAJOR) >> BITP_USB_HWVERS_MAJOR;
    pHwInfo->HWVERS_Minor    = GET_DEVICE_REG16_VALUE_RAW(REG_USB0_HWVERS, BITM_USB_HWVERS_MINOR) >> BITP_USB_HWVERS_MINOR;

    pHwInfo->NumRxEndpoints  = GET_DEVICE_REG8_VALUE_RAW(pREG_USB0_EPINFO, BITM_USB_EPINFO_RXEP ) >> BITP_USB_EPINFO_RXEP;
    pHwInfo->NumTxEndpoints  = GET_DEVICE_REG8_VALUE_RAW(pREG_USB0_EPINFO, BITM_USB_EPINFO_TXEP ) >> BITP_USB_EPINFO_TXEP;

    pHwInfo->NumDMAChannels  = GET_DEVICE_REG8_VALUE_RAW(pREG_USB0_RAMINFO, BITM_USB_RAMINFO_DMACHANS) >> BITP_USB_RAMINFO_DMACHANS;
    pHwInfo->RamAddressWidth = GET_DEVICE_REG8_VALUE_RAW(pREG_USB0_RAMINFO, BITM_USB_RAMINFO_RAMBITS ) >> BITP_USB_RAMINFO_RAMBITS;
#endif
}

#if defined(ADI_DEBUG)
/*
 * CheckMemorySize()
 *
 * Checks that the memory assigned is sufficient.
 */
static bool CheckMemorySize(
        uint32_t                   MemorySize,
        uint8_t                    nEndpointsInUse
)
{
    uint32_t MinMemorySize;

    bool Result = 1;

    MinMemorySize = sizeof(ADI_USBD_DEV_MUSBMHDRC_DEF);
    MinMemorySize += sizeof(ADI_USBD_DEV_EP_TABLE_DEF) + (nEndpointsInUse * sizeof(ADI_USBD_DEV_EP_DEF *));
    MinMemorySize += nEndpointsInUse * sizeof(ADI_USBD_DEV_EP_DEF_EX);
    MinMemorySize += nEndpointsInUse * (sizeof(ADI_USBD_DEV_EP_BUFFER_LIST) + ADI_USBD_DEV_MAX_BUFS*sizeof(ADI_USBD_DEV_EP_BUFFER));

    if (MemorySize < MinMemorySize)
    {
        Result = 0;
    }

    return Result;
}
#endif

#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)

/*
 * Sets DMA control register with the passed data and for the specified endpoint
 */
static void set_dma_ctrl(uint32_t ep_num, uint16_t data)
{
    uint16_t volatile *paddr = (uint16_t*)(USB_DMA_CTRL_BASE + (ep_num * USB_DMA_ADDR_MULT));
    *paddr = data;
    ssync();
}

/*
 * Sets DMA address register with the passed data and for the specified endpoint
 */
static void set_dma_addr(uint32_t ep_num,uint16_t addr_low, uint16_t addr_high)
{
    uint16_t volatile *paddr = (uint16_t*)(USB_DMA_ADDR_LOW_BASE + (ep_num * USB_DMA_ADDR_MULT));
    *paddr = addr_low;
    ssync();
    paddr = (uint16_t*)(USB_DMA_ADDR_HIGH_BASE + (ep_num * USB_DMA_ADDR_MULT));
    *paddr = addr_high;
    ssync();
}
/*
 * Sets DMA count register with the passed data and for the specified endpoint
 */
static void set_dma_count(uint32_t ep_num,uint16_t count_low, uint16_t count_high)
{
    uint16_t volatile *paddr = (uint16_t*)(USB_DMA_COUNT_LOW_BASE + (ep_num * USB_DMA_ADDR_MULT));
    *paddr = count_low;
    ssync();
    paddr = (uint16_t*)(USB_DMA_COUNT_HIGH_BASE + (ep_num * USB_DMA_ADDR_MULT));
    *paddr = count_high;
    ssync();
}

/*
 * Gets DMA address register with the passed data and for the specified endpoint
 */
void adi_usbd_dev_get_dma_addr(uint32_t ep_num,uint32_t *addr)
{
    uint16_t volatile *paddr = (uint16_t*)(USB_DMA_ADDR_LOW_BASE + (ep_num * USB_DMA_ADDR_MULT));
    *addr = (uint32_t)*paddr;
    ssync();
    paddr = (uint16_t*)(USB_DMA_ADDR_HIGH_BASE + (ep_num * USB_DMA_ADDR_MULT));
    *addr |= (uint32_t)((uint16_t)*paddr << 16);
    ssync();
}

/*
 * returns the DMA control register contents
 */
uint16_t adi_usbd_dev_get_dma_ctrl(uint32_t ep_num)
{
    uint16_t volatile *paddr = (uint16_t*)(USB_DMA_CTRL_BASE + (ep_num * USB_DMA_ADDR_MULT));
    ssync();
    return(*paddr);
}

#endif
/*@}*/

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */


