
#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_8_5:"There shall be no definitions of objects or functions in a header file.")
#pragma diag(suppress:misra_rule_13_7:"Allow to add asserts for memory size macros.")
#pragma diag(suppress:misra_rule_12_7:"Bitwise operators shall not be applied to operands whose underlying type is signed.")
#pragma diag(suppress:misra_rule_14_7:"A function shall have a single point of exit at the end of the function.")
#pragma diag(suppress:misra_rule_18_4:"Unions shall not be used.")
#pragma diag(suppress:misra_rule_19_4:"C macros shall only expand to a braced initialiser, a constant, a string literal, a parenthesised expression, a type qualifier, a storage class specifier, or a do-while-zero construct.")
#pragma diag(suppress:misra_rule_19_10:"In the definition of a function-like macro each instance of a parameter shall be enclosed in parentheses unless it is used as the operand of # and ##.")
#pragma diag(suppress:misra_rule_19_7:"")
#pragma diag(suppress:misra_rule_19_14:"")
#pragma diag(suppress:misra_rule_5_7:"")
#pragma diag(suppress:misra_rule_5_1:"")
#endif /* _MISRA_RULES */

#ifndef __ADI_USBD_DEV_MUSBMHDRC_LOCAL_H__
#define __ADI_USBD_DEV_MUSBMHDRC_LOCAL_H__
/*
*********************************************************************************************************
*                                              INCLUDE FILES
*********************************************************************************************************
*/

#include <drivers/usb/controller/device/adi_usbd_dev_os.h>

#define __EP_DEBUG
/*
*********************************************************************************************************
*                                              LOCAL DEFINES
*********************************************************************************************************
*/


#define DEVICE_REGS(P) ( (P)->pRegs[(P)->DeviceNumber])
#define pDEVICE_REG(P,R) ( (P)->pRegs[(P)->DeviceNumber].R)
#define SET_DEVICE_REG_VALUE(P,R,M,V) ( *((P)->pRegs[(P)->DeviceNumber].R) |= (M)&(V))
#define CLR_DEVICE_REG_VALUE(P,R,M) ( *((P)->pRegs[(P)->DeviceNumber].R) &= ~(M))
#define GET_DEVICE_REG_VALUE(P,R,M) ( *((P)->pRegs[(P)->DeviceNumber].R)&(M))

#if defined(__ADSPBF609_FAMILY__) || defined(__ADSPARM_FAMILY__)
#define CLEAR_INTERRUPTS(P,R,V)
#else
#define CLEAR_INTERRUPTS(P,R,V) SET_DEVICE_REG_VALUE((P),R,(V),(V))
#endif

#define READ_INT_REG(V,P,R,M)   (V) = GET_DEVICE_REG_VALUE(P,R,(M)); \
                              CLEAR_INTERRUPTS((P),R,(V));

#define IS_INTRPT(M,V)  ( ((V) & (M)) )

#define SET_EP_INDEX(P,N)  (*((P)->pRegs[(P)->DeviceNumber].pINDEX) = (N))

#define GET_SETUP_RQ_DIR(P) ( (P)[0] & 0x80 )
#define GET_SETUP_RQ_LEN(P) ( (*((uint16_t*)&(P)[6])) )

#ifdef ADI_DEBUG
#include "stdio.h"
void __adi_usbd_dev_assert(char *msg, int line, char *file);
#define USB_ASSERT(E,M) if ( E ) {__adi_usbd_dev_assert(M, __LINE__, __FILE__); }

#else
#define USB_ASSERT(E,M)
#endif

/*
*********************************************************************************************************
*                                            LOCAL DATA TYPES
*********************************************************************************************************
*/

/* Device State
*/
typedef enum {
    ADI_USBD_DEV_STATE_IDLE,            /* */
    ADI_USBD_DEV_STATE_OPEN,            /* */
    ADI_USBD_DEV_STATE_ENABLED,         /* */
    ADI_USBD_DEV_STATE_DEFAULT,         /* */
    ADI_USBD_DEV_STATE_ADDRESS,         /* */
    ADI_USBD_DEV_STATE_AWAIT_ENUM,      /* */
    ADI_USBD_DEV_STATE_ENUMERATING,     /* */
    ADI_USBD_DEV_STATE_READY            /* */
} ADI_USBD_DEV_STATE;


/*! Enumeration for the Endpoint Transfer Modes */
typedef enum {
    ADI_USBD_DEV_EP_MODE_DMA_0,         /*!< Single Packet DMA Mode (Mode 0) */
    ADI_USBD_DEV_EP_MODE_DMA_1,         /*!< Multi Packet DMA Mode (Mode 1)  */
    ADI_USBD_DEV_EP_MODE_PIO            /*!< Programmed I/O mode             */
} ADI_USBD_DEV_EP_MODE;

/*! Enumeration for the Endpoint State */
typedef enum {
    ADI_USBD_EP_STATE_IDLE,             /*!< Endpoint Idle                          */
    ADI_USBD_EP_STATE_RQST,             /*!< Endpoint request phase                 */
    ADI_USBD_EP_STATE_RX,               /*!< Endpoint receive data phase            */
    ADI_USBD_EP_STATE_RX_STATUS,        /*!< Endpoint receive status phase          */
    ADI_USBD_EP_STATE_TX,               /*!< Endpoint transmit data phase           */
    ADI_USBD_EP_STATE_TX_STATUS,        /*!< Endpoint transmit status phase         */
    ADI_USBD_EP_STATE_TX_PKT_SENT       /*!< Endpoint transmit packet sent phase    */
} ADI_USBD_DEV_EP_STATE;

typedef void (*ADI_USBD_DEV_STATE_FN)(void *, void *);


enum {
    ADI_USBD_DMA_MODE_0,
    ADI_USBD_DMA_MODE_1
};

/*! Endpoint definition structure (not exposed to public API) */
typedef struct {
    /* information about EP descriptor */
    ADI_USBD_DEV_EP_DEF         basic;                      /*< Basic information (can be shared with caller functions) */

    uint8_t                     EpNum;                      /*< Physical Endpoint number (no direction bit)             */
    uint32_t                    nBytesProcessed;            /*< Number of bytes transferred in last transfer            */
    ADI_USBD_DEV_EP_TYPE        EpType;                     /*< Endpoint type                                           */
    ADI_USBD_DEV_EP_DIR         EpDir;                      /*< Endpoint direction                                      */

    uint16_t                    nMaxPktSize;                /*< Maximum packet size for endpoint                        */
    uint8_t                     nHighCapacityMultiplier;    /*< High capacity (interrupt * isochronous) multiplier      */

    ADI_USBD_DEV_EP_MODE        TransferMode;               /*< Transfer mode for the current transfer operation        */
    uint8_t                     DMAChannel;                 /*< DMA channel number assigned for the current transfer    */

    ADI_USBD_DEV_EP_BUFFER      *pCurrentBuffer;            /*< Pointer to buffer table entry of current transfer       */
    void                        *pProcessedDataBuffer;      /*< Pointer to data buffer of previously completed transfer */

    /* EP State definition */
    ADI_USBD_DEV_EP_STATE       EpState;                    /*< Endpoint state                                          */
    ADI_USBD_DEV_EP_STATE       NextEpState;                /*< Next endpoint state                                     */
    ADI_USBD_DEV_STATE_FN       pfEpStateHandler;           /*< Address of function handling the state model            */

    ADI_USBD_DEV_CB_FN          pfCallback;                 /*< Callback function for endpoint status reports           */
    void                        *CallbackHandle;            /*< Handle to pass back as first argument in callback       */

    bool                        bConfigured;                /*< Flag: true if endpoint is configured; false if not      */

    /* Buffer management */
    ADI_USBD_DEV_EP_BUFFER_LIST *pBufferList;               /*< Buffer entry table                                      */

} ADI_USBD_DEV_EP_DEF_EX;

/*! MUSBMHDRC Device Driver instance data Structure */
typedef struct {
    ADI_USBD_DEV_EP_TABLE_DEF   *pEpTable;                  /*< Pointer to Endpoint lookup table                        */
    uint32_t                    DeviceNumber;               /*< Device number of controller                             */
    ADI_USBD_DEV_REGS_DEF       *pRegs;                     /*< Pointer to Register mapping structure                   */

    ADI_USBD_DEV_STATE          DevState;                   /*< Device state                                            */
    uint8_t                     nEndpointsInUse;            /*< Number of endpoints in use                              */

    ADI_USBD_DEV_CB_FN          pfBusCallback;              /*< Callback function for bus events                        */
    void                        *CallbackHandle;            /*<  Handle to pass back as first argument in bus callback  */

#ifdef __ECC__
#pragma align(4)
#endif
    uint8_t                     SetupPacket[8];             /*< Array to store Setup request packet                     */
    ADI_USBD_DEV_EP_DEF_EX      *pEp0Def;                   /*< Pointer to Endpoint Zero definition structure           */
#if (ADI_USBD_DEV_USE_DMA == ADI_USBD_DEV_YES)
    ADI_USBD_DEV_DMA_DEF        *pDMAChannels;              /*< Pointer to DMA channel table                            */
#endif
#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC)
    uint32_t                    FifoRamAdress;              /*< Current FIFO address                                    */
    uint32_t                    FifoRamTotalSize;           /*< Total ram space in bytes                                */
#endif

} ADI_USBD_DEV_MUSBMHDRC_DEF;

#define EP0_IN  0x80
#define EP0_OUT 0x00

#if (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBHDRC)

#define CYCLE_CNT_TYPE uint64_t
extern void adi_usbd_dev_bsp_SetClockDiv(void);

#elif (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_BF_MUSBMHDRC)

#define CYCLE_CNT_TYPE uint64_t
#define USB_DYNFIFO_START_ADDR  (64)
#define USB_DYNFIFO_TOTAL_SIZE  (16*1024)

#elif (ADI_USBD_DEV_DRVR == ADI_USBD_DEV_ARM_MUSBMHDRC)

#define CYCLE_CNT_TYPE uint32_t
#endif
/*
 * Internal data
 */
extern ADI_USBD_DEV_REGS_DEF    ADI_USBD_Registers[ADI_USBD_DEV_MAX_DEVICES];
#if (ADI_USBD_DEV_USE_DMA == ADI_USBD_DEV_YES)
extern ADI_USBD_DEV_DMA_DEF     ADI_USBD_DMAChannels[ADI_USBD_DEV_MAX_DEVICES][ADI_USBD_DEV_MAX_DMA_CHANNELS];
#endif

/*
 * internal functions
 */
extern void adi_usbd_dev_musbmhdrc_ResetPhysicalEndPoints(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice);


ADI_USBD_DEV_RESULT adi_usbd_dev_musbmhdrc_ScheduleDMA(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef, uint32_t DMAMode);
void adi_usbd_dev_musbmhdrc_UnloadFifoPIO(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef);
void adi_usbd_dev_musbmhdrc_LoadFifoPIO(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef);
void adi_usbd_dev_musbmhdrc_ReadFifo(ADI_USBD_DEV_FIFO_ENTRY *pFifo, ADI_USBD_DEV_FIFO_ENTRY *pData, uint32_t nBufSize);
void adi_usbd_dev_musbmhdrc_WriteFifo(ADI_USBD_DEV_FIFO_ENTRY *pFifo, ADI_USBD_DEV_FIFO_ENTRY *pData, uint32_t nBufSize);

extern void adi_usbd_dev_musbmhdrc_DeviceModeResetToDefault(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice);
bool adi_usbd_dev_musbmhdrc_SetTxPktRdy(
                    ADI_USBD_DEV_MUSBMHDRC_DEF  *pDevice,
                    ADI_USBD_DEV_EP_DEF_EX      *pEpDef
);

extern CYCLE_CNT_TYPE GetCycleCount(void);
uint32_t adi_usbd_dev_musbmhdrc_TransmitDataBuffer(
                                        ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
                                        ADI_USBD_DEV_EP_DEF_EX     *pEpDef
);
uint32_t adi_usbd_dev_musbmhdrc_ReceiveDataBuffer(
                                        ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
                                        ADI_USBD_DEV_EP_DEF_EX     *pEpDef
);

void adi_usbd_dev_musbmhdrc_SetupEndPoint(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, ADI_USBD_DEV_EP_DEF_EX *pEpDef);

void adi_usbd_dev_musbhdrc_EnableRxInterrupt(
                                ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
                                ADI_USBD_DEV_EP_DEF_EX      *pEpDef
);

void adi_usbd_dev_musbhdrc_EnableTxInterrupt(
                                ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
                                ADI_USBD_DEV_EP_DEF_EX      *pEpDef
);

void adi_usbd_dev_musbhdrc_DisableTxInterrupt(
                                ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
                                ADI_USBD_DEV_EP_DEF_EX      *pEpDef
);

void adi_usbd_dev_musbhdrc_DisableRxInterrupt(
                                ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
                                ADI_USBD_DEV_EP_DEF_EX      *pEpDef
);
uint16_t adi_usbd_dev_musbhdrc_GetRxCount(
                                ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
                                ADI_USBD_DEV_EP_DEF_EX      *pEpDef
);
void adi_usbd_dev_musbhdrc_setEpIndex(
                                ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
                                uint8_t                    EpNum
);

void adi_usbd_dev_musbmhdrc_ClearRxPktRdy(
                    ADI_USBD_DEV_MUSBMHDRC_DEF  *pDevice,
                    ADI_USBD_DEV_EP_DEF_EX      *pEpDef
);


void adi_usbd_dev_musbmhdrc_Ep0StateHandler(void *hDevice, void *hEndpoint);
void adi_usbd_dev_musbmhdrc_DataEpStateHandler( void *hDevice, void *hEndpoint);

#if (ADI_USBD_DEV_USE_DMA == ADI_USBD_DEV_YES)
void adi_usbd_dev_musbmhdrc_InitDMAChannels(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice);
bool adi_usbd_dev_musbmhdrc_GetDMAChannel(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, uint8_t *pDMAChannel);
uint8_t adi_usbd_dev_musbmhdrc_DetermineDMAChannel(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, uint8_t IntrDMA);
void adi_usbd_dev_musbmhdrc_FreeDMAChannel(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice, uint8_t DMAChannel);
#endif

void adi_usbd_dev_musbmhdrc_SetTransferMode(ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,ADI_USBD_DEV_EP_DEF_EX *pEpDef, ADI_USBD_DEV_EP_DIR Direction);

void adi_usbd_dev_musbmhdrc_ScheduleBuffer(
    ADI_USBD_DEV_MUSBMHDRC_DEF  *pDevice,
    ADI_USBD_DEV_EP_DEF_EX     *pEpDef,
    ADI_USBD_DEV_EP_BUFFER      *pBuffer
);

void _adi_usbd_dev_USBinterruptHandler(
	    uint32_t            iid,
        void*               handlerArg
);
void _adi_usbd_dev_DMAinterruptHandler(
	    uint32_t            iid,
        void*               handlerArg
);


ADI_USBD_DEV_EP_DEF_EX *adi_usbd_dev_musbmhdrc_GetEpDef(
                                ADI_USBD_DEV_MUSBMHDRC_DEF  *pDevice,
                                uint8_t                     EpAddr
);

void adi_usbd_dev_musbmhdrc_FlushEndpoint (
                    ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
                    ADI_USBD_DEV_EP_DEF_EX     *pEpDef
);

void adi_usbd_dev_musbmhdrc_ResetEndpoint (
                            ADI_USBD_DEV_MUSBMHDRC_DEF *pDevice,
                            ADI_USBD_DEV_EP_DEF_EX     *pEpDef
);

/*
 * BSP functions (located in adi_usbd_dev_bsp_musbmhdrc.c)
 */
void adi_usbd_musbmhdrc_SetPLL(void);
void adi_usbd_dev_bsp_WaitMilliSec(uint32_t Millisec);


#if 0
extern void adi_usbd_Debug(const char *pStr);
extern void adi_usbd_Debug_EP(const char *pStr, uint8_t EpAddr);
extern void adi_usbd_Debug_EParg(const char *pStr, uint8_t EpAddr, uint32_t value);


#define ADI_USBD_DBG(msg) adi_usbd_Debug( (msg) )
#define ADI_USBD_DBG_EP(msg,ep) adi_usbd_Debug_EP( (msg), (ep) )
#define ADI_USBD_DBG_EP_ARG(msg,ep,arg) adi_usbd_Debug_EParg( (msg), (ep), (arg) )

#else

#define ADI_USBD_DBG(msg)
#define ADI_USBD_DBG_EP(msg,ep)
#define ADI_USBD_DBG_EP_ARG(msg,ep,arg)

#endif

#endif /* __ADI_USBD_DEV_MUSBMHDRC_LOCAL_H__ */

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */


