/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
 * @file      adi_dma_data_cm40z.c
 * @brief     ADSP-CM40z processor specific data for DMA Manager.
 * @version:  $Revision: 18384 $
 * @date:     $Date: 2013-04-04 17:17:34 -0400 (Thu, 04 Apr 2013) $
 *
 * @details
 *            This is an additional source file for DMA Manager Service
 *            for ADSP-CM40z processor. This file contains processor specific
 *            data for ADSP-CM40z processor family.
 *
 */

/** @addtogroup DMA_Manager ADSP-CM40z processor specific data.
 *  @{
 */

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_8_5:"Objects defined in this source files are common to MDMA and PDMA source files")
#pragma diag(suppress:misra_rule_8_7:"Allow objects to be defined in a common file")
#pragma diag(suppress:misra_rule_9_2:" ")
#pragma diag(suppress:misra_rule_10_1_a:" ")
#endif /* _MISRA_RULES */

/*=============  P R O C E S S O R    S P E C I F I C     D A T A  =============*/

/* ADSP-CM402/CM403/CM407/CM408 processors */
#if defined(ADI_ADSP_CM40Z)

/* Interrupt ID for Reserved DMA channel */
#define ADI_DMA_RSV_INTR_ID                     (255u)
/* Total number of DMA Channels available in this processor family */
#define ADI_DMA_TOTAL_NUM_CHANNELS              (21u)

/* DMA Channel specific information */
static ADI_DMA_INFO     ChannelInfo[] =
{
   { INTR_SPORT0_A_DMA,       INTR_SPORT0_A_DMA_ERR,       (volatile ADI_DMA_REGS *) REG_DMA0_DSCPTR_NXT,  NULL }, /* DMA  0 - SPORT0 Channel A */
   { INTR_SPORT0_B_DMA,       INTR_SPORT0_B_DMA_ERR,       (volatile ADI_DMA_REGS *) REG_DMA1_DSCPTR_NXT,  NULL }, /* DMA  1 - SPORT0 Channel B */
   { INTR_SPI0_TXDMA,         INTR_SPI0_TXDMA_ERR,         (volatile ADI_DMA_REGS *) REG_DMA2_DSCPTR_NXT,  NULL }, /* DMA  2 - SPI0 TX */
   { INTR_SPI0_RXDMA,         INTR_SPI0_RXDMA_ERR,         (volatile ADI_DMA_REGS *) REG_DMA3_DSCPTR_NXT,  NULL }, /* DMA  3 - SPI0 RX */
   { INTR_UART0_TXDMA,        INTR_UART0_TXDMA_ERR,        (volatile ADI_DMA_REGS *) REG_DMA4_DSCPTR_NXT,  NULL }, /* DMA  4 - UART0 TX */
   { INTR_UART0_RXDMA,        INTR_UART0_RXDMA_ERR,        (volatile ADI_DMA_REGS *) REG_DMA5_DSCPTR_NXT,  NULL }, /* DMA  5 - UART0 RX */
   { INTR_SPORT1_A_DMA,       INTR_SPORT1_A_DMA_ERR,       (volatile ADI_DMA_REGS *) REG_DMA6_DSCPTR_NXT,  NULL }, /* DMA  6 - SPORT1 Channel A */
   { INTR_SPORT1_B_DMA,       INTR_SPORT1_B_DMA_ERR,       (volatile ADI_DMA_REGS *) REG_DMA7_DSCPTR_NXT,  NULL }, /* DMA  7 - SPORT1 Channel B */
   { INTR_SPI1_TXDMA,         INTR_SPI1_TXDMA_ERR,         (volatile ADI_DMA_REGS *) REG_DMA8_DSCPTR_NXT,  NULL }, /* DMA  8 - SPI1 TX */
   { INTR_SPI1_RXDMA,         INTR_SPI1_RXDMA_ERR,         (volatile ADI_DMA_REGS *) REG_DMA9_DSCPTR_NXT,  NULL }, /* DMA  9 - SPI1 RX */
   { INTR_UART1_TXDMA,        INTR_UART1_TXDMA_ERR,        (volatile ADI_DMA_REGS *) REG_DMA10_DSCPTR_NXT, NULL }, /* DMA 10 - UART1 TX */
   { INTR_UART1_RXDMA,        INTR_UART1_RXDMA_ERR,        (volatile ADI_DMA_REGS *) REG_DMA11_DSCPTR_NXT, NULL }, /* DMA 11 - UART1 RX */
   { INTR_UART2_TXDMA,        INTR_UART2_TXDMA_ERR,        (volatile ADI_DMA_REGS *) REG_DMA12_DSCPTR_NXT, NULL }, /* DMA 12 - UART2 TX */
   { INTR_UART2_RXDMA,        INTR_UART2_RXDMA_ERR,        (volatile ADI_DMA_REGS *) REG_DMA13_DSCPTR_NXT, NULL }, /* DMA 13 - UART2 RX */
   { INTR_HAE0_RXDMA_CH0,     INTR_HAE0_RXDMA_CH0_ERR,     (volatile ADI_DMA_REGS *) REG_DMA14_DSCPTR_NXT, NULL }, /* DMA 14 - Harmonic Analysis Engine (HAE) RX DMA Channel 0 */
   { INTR_HAE0_RXDMA_CH1,     INTR_HAE0_RXDMA_CH1_ERR,     (volatile ADI_DMA_REGS *) REG_DMA15_DSCPTR_NXT, NULL }, /* DMA 15 - Harmonic Analysis Engine (HAE) RX DMA Channel 1 */
   { INTR_HAE0_TXDMA,         INTR_HAE0_TXDMA_ERR,         (volatile ADI_DMA_REGS *) REG_DMA16_DSCPTR_NXT, NULL }, /* DMA 16 - Harmonic Analysis Engine (HAE) TX DMA */
   { INTR_MDMA0_SRC,          INTR_MDMA0_SRC_CRC0_IN_ERR,  (volatile ADI_DMA_REGS *) REG_DMA17_DSCPTR_NXT, NULL }, /* DMA 17 - MDMA0 Source/CRC0 Input */
   { INTR_MDMA0_DST,          INTR_MDMA0_DST_CRC0_OUT_ERR, (volatile ADI_DMA_REGS *) REG_DMA18_DSCPTR_NXT, NULL }, /* DMA 18 - MDMA0 Destination/CRC0 Output */
   { INTR_MDMA1_SRC,          INTR_MDMA1_SRC_ERR,          (volatile ADI_DMA_REGS *) REG_DMA19_DSCPTR_NXT, NULL }, /* DMA 19 - MDMA1 Source */
   { INTR_MDMA1_DST,          INTR_MDMA1_DST_ERR,          (volatile ADI_DMA_REGS *) REG_DMA20_DSCPTR_NXT, NULL }  /* DMA 20 - MDMA1 Destination */
};

/* Memory DMA Stream specific information */
static const ADI_DMA_MEM_STREAM_INFO    StreamInfo[] =
{
    { ADI_DMA_CHNL_MEMDMA_D0, ADI_DMA_CHNL_MEMDMA_S0 },  /* MDMA Stream 0 Source & Destination channel IDs */
    { ADI_DMA_CHNL_MEMDMA_D1, ADI_DMA_CHNL_MEMDMA_S1 }   /* MDMA Stream 1 Source & Destination channel IDs */
};

#endif /* ADI_ADSP_CM40Z */

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

/*****/

/*@}*/

