/*********************************************************************************

Copyright(c) 2012 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_gpio_data_cm40z.c
* @brief     This is the pin interrupt data file for the GPIO Driver.
* @version:  $Revision: 18384 $
* @date:     $Date: 2013-04-04 17:17:34 -0400 (Thu, 04 Apr 2013) $
*
*/

#include <services/int/adi_int.h>
#include <adi_osal.h>
#include <stddef.h>

typedef uint32_t REGSIZE;

#define __ADI_GPIO_HAS_PINT__

typedef struct {
	volatile REGSIZE *Fer;                  /* gpio pin control */
	volatile REGSIZE *Data;                 /* gpio data register */
	volatile REGSIZE *Set;                  /* set gpio register (W1C) */
	volatile REGSIZE *Clear;                /* clear gpio register (W1C) */
	volatile REGSIZE *DirSet;               /* gpio direction register */
	volatile REGSIZE *DirClear;             /* gpio direction register */
	volatile REGSIZE *InputEnable;          /* gpio input enable register */
	volatile REGSIZE *Toggle;               /* gpio toggle register */
	volatile REGSIZE *Polar;                /* gpio inversion register */
	volatile REGSIZE *PolarSet;             /* gpio set inversion register */
	volatile REGSIZE *PolarClear;           /* gpio clear inversion register */
	volatile REGSIZE *Lock;                 /* gpio lock register */
} ADI_GPIO_PORT_INFO;

typedef struct {
	ADI_GPIO_PIN_INTERRUPT pinintId;         /* pin interrupt ID */
	volatile uint32_t *MaskSet;              /* set mask register (W1S) */
	volatile uint32_t *MaskClear;            /* clear mask register (W1C) */
	volatile uint32_t *Irq;                  /* interrupt request register (W1C) */
	volatile uint32_t *Assign;               /* interrupt assignment register (r/w) */
	volatile uint32_t *EdgeSet;              /* set edge interrupts (W1S) */
	volatile uint32_t *EdgeClear;            /* clear edge interrupts (make level) (W1C) */
	volatile uint32_t *InvertSet;            /* set inverted polarity (W1S) */
	volatile uint32_t *InvertClear;          /* clear inverted polarity (W1C) */
	volatile uint32_t *PinState;             /* pin state register (RO) */
	volatile uint32_t *Latch;                /* interrupt latch register (r/w) */
	ADI_GPIO_CALLBACK  pfCallback;           /* user installed callback function */
	int16_t pid;                             /* peripheral ID */
} ADI_GPIO_PIN_INT_INFO;

static ADI_GPIO_PORT_INFO PortInfo[] = {
		{ /* port a */
			pREG_PORTA_FER,
			pREG_PORTA_DATA,
			pREG_PORTA_DATA_SET,
			pREG_PORTA_DATA_CLR,
			pREG_PORTA_DIR_SET,
			pREG_PORTA_DIR_CLR,
			pREG_PORTA_INEN,
			pREG_PORTA_DATA_TGL,
			pREG_PORTA_POL,
			pREG_PORTA_POL_SET,
			pREG_PORTA_POL_CLR,
			pREG_PORTA_LOCK,
		},
		{ /* port b */
			pREG_PORTB_FER,
			pREG_PORTB_DATA,
			pREG_PORTB_DATA_SET,
			pREG_PORTB_DATA_CLR,
			pREG_PORTB_DIR_SET,
			pREG_PORTB_DIR_CLR,
			pREG_PORTB_INEN,
			pREG_PORTB_DATA_TGL,
			pREG_PORTB_POL,
			pREG_PORTB_POL_SET,
			pREG_PORTB_POL_CLR,
			pREG_PORTB_LOCK,
		},
		{ /* port c */
			pREG_PORTC_FER,
			pREG_PORTC_DATA,
			pREG_PORTC_DATA_SET,
			pREG_PORTC_DATA_CLR,
			pREG_PORTC_DIR_SET,
			pREG_PORTC_DIR_CLR,
			pREG_PORTC_INEN,
			pREG_PORTC_DATA_TGL,
			pREG_PORTC_POL,
			pREG_PORTC_POL_SET,
			pREG_PORTC_POL_CLR,
			pREG_PORTC_LOCK,
		},
		{ /* port d */
			pREG_PORTD_FER,
			pREG_PORTD_DATA,
			pREG_PORTD_DATA_SET,
			pREG_PORTD_DATA_CLR,
			pREG_PORTD_DIR_SET,
			pREG_PORTD_DIR_CLR,
			pREG_PORTD_INEN,
			pREG_PORTD_DATA_TGL,
			pREG_PORTD_POL,
			pREG_PORTD_POL_SET,
			pREG_PORTD_POL_CLR,
			pREG_PORTD_LOCK,
		},
		{ /* port e */
			pREG_PORTE_FER,
			pREG_PORTE_DATA,
			pREG_PORTE_DATA_SET,
			pREG_PORTE_DATA_CLR,
			pREG_PORTE_DIR_SET,
			pREG_PORTE_DIR_CLR,
			pREG_PORTE_INEN,
			pREG_PORTE_DATA_TGL,
			pREG_PORTE_POL,
			pREG_PORTE_POL_SET,
			pREG_PORTE_POL_CLR,
			pREG_PORTE_LOCK,
		},
		{ /* port f */
			pREG_PORTF_FER,
			pREG_PORTF_DATA,
			pREG_PORTF_DATA_SET,
			pREG_PORTF_DATA_CLR,
			pREG_PORTF_DIR_SET,
			pREG_PORTF_DIR_CLR,
			pREG_PORTF_INEN,
			pREG_PORTF_DATA_TGL,
			pREG_PORTF_POL,
			pREG_PORTF_POL_SET,
			pREG_PORTF_POL_CLR,
			pREG_PORTF_LOCK,
		},
};

static ADI_GPIO_PIN_INT_INFO PinIntInfo[] = {
		{
			ADI_GPIO_PIN_INTERRUPT_0,
			(volatile uint32_t *)pREG_PINT0_MSK_SET,
			(volatile uint32_t *)pREG_PINT0_MSK_CLR,
			(volatile uint32_t *)pREG_PINT0_REQ,
			(volatile uint32_t *)pREG_PINT0_ASSIGN,
			(volatile uint32_t *)pREG_PINT0_EDGE_SET,
			(volatile uint32_t *)pREG_PINT0_EDGE_CLR,
			(volatile uint32_t *)pREG_PINT0_INV_SET,
			(volatile uint32_t *)pREG_PINT0_INV_CLR,
			(volatile uint32_t *)pREG_PINT0_PINSTATE,
			(volatile uint32_t *)pREG_PINT0_LATCH,
    		(void*)0,
    		(uint16_t)INTR_PINT0_BLOCK
		},
		{
			ADI_GPIO_PIN_INTERRUPT_1,
			(volatile uint32_t *)pREG_PINT1_MSK_SET,
			(volatile uint32_t *)pREG_PINT1_MSK_CLR,
			(volatile uint32_t *)pREG_PINT1_REQ,
			(volatile uint32_t *)pREG_PINT1_ASSIGN,
			(volatile uint32_t *)pREG_PINT1_EDGE_SET,
			(volatile uint32_t *)pREG_PINT1_EDGE_CLR,
			(volatile uint32_t *)pREG_PINT1_INV_SET,
			(volatile uint32_t *)pREG_PINT1_INV_CLR,
			(volatile uint32_t *)pREG_PINT1_PINSTATE,
			(volatile uint32_t *)pREG_PINT1_LATCH,
    		(void*)0,
    		(uint16_t)INTR_PINT1_BLOCK
		},
		{
			ADI_GPIO_PIN_INTERRUPT_2,
			(volatile uint32_t *)pREG_PINT2_MSK_SET,
			(volatile uint32_t *)pREG_PINT2_MSK_CLR,
			(volatile uint32_t *)pREG_PINT2_REQ,
			(volatile uint32_t *)pREG_PINT2_ASSIGN,
			(volatile uint32_t *)pREG_PINT2_EDGE_SET,
			(volatile uint32_t *)pREG_PINT2_EDGE_CLR,
			(volatile uint32_t *)pREG_PINT2_INV_SET,
			(volatile uint32_t *)pREG_PINT2_INV_CLR,
			(volatile uint32_t *)pREG_PINT2_PINSTATE,
			(volatile uint32_t *)pREG_PINT2_LATCH,
    		(void*)0,
    		(uint16_t)INTR_PINT2_BLOCK
		},
		{
			ADI_GPIO_PIN_INTERRUPT_3,
			(volatile uint32_t *)pREG_PINT3_MSK_SET,
			(volatile uint32_t *)pREG_PINT3_MSK_CLR,
			(volatile uint32_t *)pREG_PINT3_REQ,
			(volatile uint32_t *)pREG_PINT3_ASSIGN,
			(volatile uint32_t *)pREG_PINT3_EDGE_SET,
			(volatile uint32_t *)pREG_PINT3_EDGE_CLR,
			(volatile uint32_t *)pREG_PINT3_INV_SET,
			(volatile uint32_t *)pREG_PINT3_INV_CLR,
			(volatile uint32_t *)pREG_PINT3_PINSTATE,
			(volatile uint32_t *)pREG_PINT3_LATCH,
    		(void*)0,
    		(uint16_t)INTR_PINT3_BLOCK
		}
		,
		{
			ADI_GPIO_PIN_INTERRUPT_4,
			(volatile uint32_t *)pREG_PINT4_MSK_SET,
			(volatile uint32_t *)pREG_PINT4_MSK_CLR,
			(volatile uint32_t *)pREG_PINT4_REQ,
			(volatile uint32_t *)pREG_PINT4_ASSIGN,
			(volatile uint32_t *)pREG_PINT4_EDGE_SET,
			(volatile uint32_t *)pREG_PINT4_EDGE_CLR,
			(volatile uint32_t *)pREG_PINT4_INV_SET,
			(volatile uint32_t *)pREG_PINT4_INV_CLR,
			(volatile uint32_t *)pREG_PINT4_PINSTATE,
			(volatile uint32_t *)pREG_PINT4_LATCH,
    		(void*)0,
    		(uint16_t)INTR_PINT4_BLOCK
		},
};
