/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_tmr_data_bf6xx.c
*
* @brief     Timer service global API implementation.
*
* @details
*            This is a source file which contains data related to timer service
*            for BF6XX processor family.
*/

#ifndef __ADI_TMR_DATA_BF6XX__
#define __ADI_TMR_DATA_BF6XX__

#include "adi_tmr_def_bf6xx.h"

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_8_5:"Allow variables to be defined in the source file.  \
                                      Misra treats this as header file as it is included \
                                      in other source file")
#pragma diag(suppress:misra_rule_11_4:"Allow tapecasting between different pointer types")
#pragma diag(suppress:misra_rule_2_4:"Allows doxygen addtogroup block")
#endif

/** @addtogroup Timer_Service_BF6xx Timer Service
*  @{
*/


/* Table of available timers in the processor */
static ADI_TMR_DEV  gaTimers[] =
{
    /* Timer 0 */
    {
        (ADI_TMR_REGS_PTR) pREG_TIMER0_TMR0_CFG,    /* Timer register address base */
        (uint32_t)INTR_TIMER0_TMR0,                 /* Timer data interrupt SID */
        0u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    },
    /* Timer 1 */
    {
        (ADI_TMR_REGS_PTR) pREG_TIMER0_TMR1_CFG,    /* Timer register address base */
        (uint32_t)INTR_TIMER0_TMR1,                 /* Timer data interrupt SID */
        1u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    },
    /* Timer 2 */
    {
        (ADI_TMR_REGS_PTR) pREG_TIMER0_TMR2_CFG,    /* Timer register address base */
        (uint32_t)INTR_TIMER0_TMR2,                 /* Timer data interrupt SID */
        2u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    },
    /* Timer 3 */
    {
        (ADI_TMR_REGS_PTR) pREG_TIMER0_TMR3_CFG,    /* Timer register address base */
        (uint32_t)INTR_TIMER0_TMR3,                 /* Timer data interrupt SID */
        3u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    },
    /* Timer 4 */
    {
        (ADI_TMR_REGS_PTR) pREG_TIMER0_TMR4_CFG,    /* Timer register address base */
        (uint32_t)INTR_TIMER0_TMR4,                 /* Timer data interrupt SID */
        4u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    },
    /* Timer 5 */
    {
        (ADI_TMR_REGS_PTR) pREG_TIMER0_TMR5_CFG,    /* Timer register address base */
        (uint32_t)INTR_TIMER0_TMR5,                 /* Timer data interrupt SID */
        5u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    },
    /* Timer 6 */
    {
        (ADI_TMR_REGS_PTR) pREG_TIMER0_TMR6_CFG,    /* Timer register address base */
        (uint32_t)INTR_TIMER0_TMR6,                 /* Timer data interrupt SID */
        6u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    },
    /* Timer 7 */
    {
       (ADI_TMR_REGS_PTR) pREG_TIMER0_TMR7_CFG,     /* Timer register address base */
       (uint32_t)INTR_TIMER0_TMR7,                  /* Timer data interrupt SID */
        7u,                                         /* Timer ID */
        NULL                                        /* Pointer to timer instance Data */
    }
};

/*@}*/

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

#endif /* __ADI_TMR_DATA_BF6XX__ */
