/*********************************************************************************

Copyright(c) 2011 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
* @file      adi_tmr_def_bf6xx.h
*
* @brief     Timer service local header file
*
* @details
*            This is a local header file which contains the defenitions required
*            for the GP timer service implementation.
*/

#ifndef _ADI_TMR_DEF_BF6XX_H_
#define _ADI_TMR_DEF_BF6XX_H_

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_2_4:"Allows doxygen addtogroup block")
#pragma diag(suppress:misra_rule_5_6:"Allow identifier names to be reused in different name spaces")
#pragma diag(suppress:misra_rule_5_7:"Allow identifier names to be reused in different name spaces")
#endif

/** @addtogroup Timer_Service_BF6xx Timer Service
*  @{
*/

#if defined(__ADSPBF707_FAMILY__)
typedef uint32_t    TMR_REG_TYPE;
#else
typedef uint16_t    TMR_REG_TYPE;
#endif

/*********************************************************************

Declarations

*********************************************************************/

/* Register specific to each timer */
typedef struct _ADI_TMR_REGS
{
    volatile TMR_REG_TYPE   CFG;
#if defined(__ADSPBF609_FAMILY__) || defined(ADI_ADSP_CM40Z)
    volatile uint16_t       RES1;
#endif
    volatile uint32_t       CNT;
    volatile uint32_t       PER;
    volatile uint32_t       WID;
    volatile uint32_t       DLY;
} ADI_TMR_REGS;

typedef volatile ADI_TMR_REGS*  ADI_TMR_REGS_PTR;

/* Timer instance data. */
typedef struct _ADI_TMR_DATA
{

    ADI_CALLBACK        pfCallback;     /* Application supplied callback. */
    void               *pCBParam;       /* Application supplied callback parameter. */

    /* Add other fields required */
} ADI_TMR_DATA;


/* Timer device related data. */
typedef struct _ADI_TMR_DEV
{
    ADI_TMR_REGS_PTR        const pTmrRegs;     /* Pointer to timer MMRs, specific to each timer. */
    uint32_t                const nDataIntSID;  /* SID for data interrupt */
    uint32_t                const nTimerID;     /* Timer number */
    ADI_TMR_DATA                * pDevData;     /* Pointer to timer instance data. */

    /* Add other fields required */
} ADI_TMR_DEV;

/* Timer state */
typedef enum
{
    ADI_TMR_STATE_UNKNOWN,                      /* Timer sate is unknown (not initialized) */
    ADI_TMR_STATE_INITIALIZED                   /* Timer service initialized */

    /* Add more states here */
} ADI_TMR_STATE;

/* Number of GP timers supported by the processor */
#define ADI_NUM_TIMERS  (sizeof(gaTimers)/sizeof(ADI_TMR_DEV))


/*=============== Local functions =======================*/

/* Status interrupt handler */
static void StatusIntHandler(
    uint32_t            IID,
    void               *pCBParam
);

/* Data interrupt handler */
static void DataIntHandler(
    uint32_t            IID,
    void               *pCBParam
);


/* Enables/disables group of timers */
static void EnableTimers(
    uint16_t nTmrMask,
    bool     bEnable
);

/* Timer service initialization function */
static ADI_TMR_RESULT  adi_tmr_Init(
    void
);

#ifdef ADI_DEBUG
/* Validates the device handle */
static ADI_TMR_RESULT ValidateDeviceHandle (
    ADI_TMR_DEV     *const pDevice
);
#endif


/*@}*/

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif

#endif /* _ADI_TMR_DEF_BF6XX_H_ */
