/* -----------------------------------------------------------------------------
 * Copyright (c) 2014 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty. 
 * In no event will the authors be held liable for any damages arising from 
 * the use of this software. Permission is granted to anyone to use this 
 * software for any purpose, including commercial applications, and to alter 
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not 
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be 
 *    appreciated but is not required. 
 * 
 * 2. Altered source versions must be plainly marked as such, and must not be 
 *    misrepresented as being the original software. 
 * 
 * 3. This notice may not be removed or altered from any source distribution.
 *   
 *
 * $Date:        15. April 2014
 * $Revision:    V1.00
 *  
 * Project:      Flash Device Description for AnalogDevices CM40x Flashes
 * --------------------------------------------------------------------------- */

/* History:
 *  Version 1.00
 *    Initial release
 */ 

#include "..\FlashOS.H"         // FlashOS Structures


#ifdef FLASH_MEM
#ifdef CM40x_256
struct FlashDevice const FlashDevice  =  {
   FLASH_DRV_VERS,              // Driver Version, do not modify!
   "CM40x 256kB Flash",         // Device Name
   ONCHIP,                      // Device Type
   0x18000000,                  // Device Start Address
   0x00040000,                  // Device Size in Bytes (256kB)
   256,                         // Programming Page Size
   0,                           // Reserved, must be 0
   0xFF,                        // Initial Content of Erased Memory
   300,                         // Program Page Timeout 300 mSec
   3000,                        // Erase Sector Timeout 3000 mSec

// Specify Size and Address of Sectors
   0x1000, 0x000000,            // Sector Size 4KB ( 64 Sectors)
   SECTOR_END
};
#endif

#ifdef CM40x_512
struct FlashDevice const FlashDevice  =  {
   FLASH_DRV_VERS,              // Driver Version, do not modify!
   "CM40x 512kB Flash",         // Device Name
   ONCHIP,                      // Device Type
   0x18000000,                  // Device Start Address
   0x00080000,                  // Device Size in Bytes (512kB)
   256,                         // Programming Page Size
   0,                           // Reserved, must be 0
   0xFF,                        // Initial Content of Erased Memory
   300,                         // Program Page Timeout 300 mSec
   3000,                        // Erase Sector Timeout 3000 mSec

// Specify Size and Address of Sectors
   0x1000, 0x000000,            // Sector Size 4KB (128 Sectors)
   SECTOR_END
};
#endif

#ifdef CM40x_1024
struct FlashDevice const FlashDevice  =  {
   FLASH_DRV_VERS,              // Driver Version, do not modify!
   "CM40x 1024kB Flash",        // Device Name
   ONCHIP,                      // Device Type
   0x18000000,                  // Device Start Address
   0x00100000,                  // Device Size in Bytes (1024kB)
   256,                         // Programming Page Size
   0,                           // Reserved, must be 0
   0xFF,                        // Initial Content of Erased Memory
   300,                         // Program Page Timeout 300 mSec
   3000,                        // Erase Sector Timeout 3000 mSec

// Specify Size and Address of Sectors
   0x1000, 0x000000,            // Sector Size 4KB (256 Sectors)
   SECTOR_END
};
#endif

#ifdef CM40x_2048
struct FlashDevice const FlashDevice  =  {
   FLASH_DRV_VERS,              // Driver Version, do not modify!
   "CM40x 2048kB Flash",       // Device Name
   ONCHIP,                      // Device Type
   0x18000000,                  // Device Start Address
   0x00200000,                  // Device Size in Bytes (2048kB)
   256,                         // Programming Page Size
   0,                           // Reserved, must be 0
   0xFF,                        // Initial Content of Erased Memory
   300,                         // Program Page Timeout 300 mSec
   3000,                        // Erase Sector Timeout 3000 mSec

// Specify Size and Address of Sectors
   0x1000, 0x000000,            // Sector Size 4KB (512 Sectors)
   SECTOR_END
};
#endif

#endif /* FLASH_MEM */
